﻿class BioActivePower extends BioActive
    config(Game)
    native;

enum EBioPowerState
{
    BIO_POWER_DONE,
    BIO_POWER_CASTING,
    BIO_POWER_RELEASE,
    BIO_POWER_FINISHING,
};

var transient Vector m_vLocationToAimAt;

var transient Actor m_oTargetToAimAt;

var transient int m_nTag;

var transient AnimSet m_oAnimSet;

var transient float m_fCastingBeamTime;

var transient float m_fCurrentWaitTimeForAnimation;

var transient config float m_fMaxWaitTimeForAnimation;

var transient float m_fPowerStateElapsed;

var transient ECoverAction m_ePreviousCoverAction;

var transient EBioPowerState m_ePowerState;

var transient Rotator m_rOriginalCameraRotation;

var transient Vector m_vOriginalCameraLocation;

var transient int m_nReservationID;

var transient bool m_bIgnoreSuppression;

var transient bool m_bPlayerOrderedPowerUse;

var transient bool m_bDisplayingCastingBeam;

var transient bool m_bShowPowerImpactText;

var transient bool m_bEnablePowerCooldown;

var transient bool m_bShowPowerAiming;

var transient bool m_bWaitingForAnimationToStart;

var transient bool m_bReleaseAnimEnded;

var transient bool m_bAnimationReadyForPowerRelease;

var transient bool m_bPlayAnimations;

var transient bool m_bInstantPowerUse;

var transient bool m_bUsePowerReservation;

var transient bool m_bManualIgnoreCooldown;

var transient config bool m_bSkipPowerCooldown;

var transient bool m_bCasterIsHenchman;

var transient BioPlayerController m_oCasterAsPlayerController;

var transient BioEpicPawnBehavior m_oCasterBehavior;

var transient BioPawn m_oCasterAsBioPawn;

var transient Pawn m_oCaster;

var transient BioPower m_oPower;

native final function AnimationReadyForPowerRelease();

native final function bool CancelPower(bool bOnlyCancelIfCasting);

native final function SetTargetActor(Actor oTarget);

native final function SetTargetLocation(Vector vLocation);

native final function UpdatePowerOrder(bool bOrderedByPlayer, optional bool bIgnoreSuppression);

native final function OnAnimCastingBegin();

native final function OnAnimCastingEnd();

native final function OnAnimReleaseBegin();

native final function OnAnimReleaseEnd();

event bool AimProjectilePower()
{
    local Vector HitLocation;
    local Vector HitNormal;
    local Actor HitActor;
    local BioWorldInfo WorldInfo;
    local Vector StartLocation;
    if (m_oCasterAsPlayerController != None)
    {
        return true;
    }
    if (m_oPower == None || m_oCaster == None)
    {
        return false;
    }
    WorldInfo = BioWorldInfo(m_oCaster.WorldInfo);
    if (WorldInfo == None || WorldInfo.m_oPowerManager == None)
    {
        return false;
    }
    WorldInfo.m_oPowerManager.GetStartLocationForLOSCheck(m_oCaster, StartLocation);
    if (m_bShowPowerAiming)
    {
        m_oCaster.DrawDebugLine(StartLocation, m_vLocationToAimAt, 0, 255, 0, true);
    }
    HitActor = m_oCaster.Trace(HitLocation, HitNormal, m_vLocationToAimAt, StartLocation, true, vect(0,0,0));
    if (HitActor != None && HitActor != m_oTargetToAimAt)
    {
        if (VSize(HitLocation - StartLocation) < VSize(m_vLocationToAimAt - StartLocation) * 0,5)
        {
            return false;
        }
    }
    return true;
}

event Projectile SpawnProjectile()
{
    local SFXProjectile_Power Projectile;
    local Vector ProjectileLocation;
    local Rotator ProjectileRotation;
    local Vector HitLocation;
    local Vector HitNormal;
    local Actor HitActor;
    local BioWorldInfo WorldInfo;
    local SFXAI_Core AIController;
    if (m_oPower == None || m_oCaster == None)
    {
        return None;
    }
    if (m_oPower.GetProjectileAttachPoint(m_oCaster, ProjectileLocation) == false)
    {
        ProjectileLocation = m_oCaster.Location;
    }
    if (m_oCasterAsPlayerController != None)
    {
        HitActor = m_oCaster.Trace(HitLocation, HitNormal, m_vLocationToAimAt, ProjectileLocation, true, vect(0,0,0));
        if (HitActor != None && HitActor != m_oTargetToAimAt)
        {
            ProjectileLocation = m_oCaster.Location + m_oCaster.Rotation * 30;
            ProjectileLocation.Z += 30;
            if (m_oCaster.Trace(HitLocation, HitNormal, m_vLocationToAimAt, ProjectileLocation, true, vect(0,0,0)) != None)
            {
                WorldInfo = BioWorldInfo(m_oCaster.WorldInfo);
                if (WorldInfo != None && WorldInfo.m_oPowerManager != None)
                {
                    WorldInfo.m_oPowerManager.GetStartLocationForLOSCheck(m_oCaster, ProjectileLocation);
                }
            }
        }
    }
    ProjectileRotation = m_vLocationToAimAt - ProjectileLocation;
    Projectile = SFXProjectile_Power(SFXGame(m_oCaster.WorldInfo.Game).ObjectPool.GetProjectile(m_oPower.NewPower.ProjectileClass, m_oCaster, m_oCaster.Instigator, ProjectileLocation, ProjectileRotation));
    if (Projectile == None)
    {
        return None;
    }
    Projectile.bUsePowerReservation = m_bUsePowerReservation;
    Projectile.ReservationID = m_nReservationID;
    Projectile.TargetActor = m_oTargetToAimAt;
    Projectile.TargetLocation = m_vLocationToAimAt;
    Projectile.CameraLocation = m_vOriginalCameraLocation;
    Projectile.CameraRotation = m_rOriginalCameraRotation;
    if (Projectile.InitializePowerProjectile(m_oCaster, m_oPower.GetProjectileSpeed(), m_oPower.GetProjectileRadius(), m_oPower) == false)
    {
        return None;
    }
    AIController = SFXAI_Core(m_oCaster.Controller);
    if (AIController != None)
    {
        AIController.m_bPowerProjectileReleased = true;
    }
    return Projectile;
}

event TriggerPowerVocalization()
{
    if (m_oPower.NewPower != None && m_oPower.NewPower.VocalizationEvent != 0)
    {
        SFXGame(m_oCaster.WorldInfo.Game).TriggerVocalizationEvent(m_oPower.NewPower.VocalizationEvent, BioPawn(m_oCaster), BioPawn(m_oTargetToAimAt), 0);
    }
}

event ReleasedOnActor()
{
    local int idx;
    local BioSeqEvt_AffectedByPower powerEvent;
    if (m_oTargetToAimAt != None)
    {
        idx = 0;
        while (idx < m_oTargetToAimAt.GeneratedEvents.Length)
        {
            powerEvent = BioSeqEvt_AffectedByPower(m_oTargetToAimAt.GeneratedEvents[idx]);
            if (powerEvent != None)
            {
                powerEvent.HandleActivation(m_oTargetToAimAt, m_oCaster, m_oPower.Script.Class);
            }
            idx++;
        }
    }
}

event OnCastAt(Actor Target, Actor Caster, BioPower Power)
{
    if (BioPawn(Target) != None)
    {
        BioPawn(Target).OnCastAt(Pawn(Caster), Power.NewPower);
    }
    else
    {
        if (BioArtPlaceable(Target) != None)
        {
            BioArtPlaceable(Target).OnCastAt(Pawn(Caster), Power.NewPower);
        }
        else
        {
            if (SFXPointOfInterest(Target) != None)
            {
                SFXPointOfInterest(Target).OnCastAt(Pawn(Caster), Power.NewPower);
            }
        }
    }
}

function ClearPreviousCoverAction()
{
    m_ePreviousCoverAction = 12;
}

defaultproperties
{
    m_fMaxWaitTimeForAnimation=5
}
