﻿class BioArtPlaceableBehavior extends BioActorBehavior
    config(Game)
    native
    editinlinenew;

const Bio_SkillState = 'SkillState';
const Bio_Physics = 'Physics';
const Bio_Opened = 'Opened';
const Bio_Closed = 'Closed';
var transient array<AnimSet> m_aBackupAnimSets;

var transient bool m_bInMatinee;

var transient Name m_nmLastState;

var transient int m_nPrevPlotIndex;

var transient int m_nPrevRegionIndex;

var String m_sVarName;

var int m_nIndex;

var BioWp_DoorMarker m_oDoorMarker;

var transient Vector m_vLastImpactVector;

var(Placeable) String m_sConnectedObjectB;

var(Placeable) String m_sConnectedObjectA;

var transient Name m_nmCurrentActivity;

var transient float m_fSkillProgress;

var transient BioWorldInfo bioWorld;

var(Plot) EBioAutoSet Conditional;

var(Plot) EBioPlotAutoSet Plot;

var(Plot) EBioRegionAutoSet Region;

var(UseCaseOverride) EBioSkillGameDifficulty m_SkillDifficulty;

var(UseCaseOverride) eBioAPLUseCase m_eUseCase;

var(UseCaseOverride) Name m_State1;

var(UseCaseOverride) Name m_State0;

var bool m_bSkillGameXpAwarded;

var bool m_bSkillGameFailed;

var(Placeable) bool bNoStateSave;

var bool m_bMonitorDoor;

var(Placeable) bool m_bDontInvalidatePaths;

var(Placeable) bool m_bUseCustomCollisionSettings;

var(Placeable) bool m_bPhysicsAffectable;

var(Placeable) bool m_bPhysicsAffectableOverride;

var bool m_bInPhysics;

var(Placeable) bool Enabled;

var(UseCaseOverride) bool m_bPlayerOnly;

var(UseCaseOverride) bool m_bUseCaseOverride;

var(Placeable) bool m_bStartingStateOverride;

var(Placeable) Name StartingState;

var Name m_nCurrentMaterialVar;

var(Appearance) transient eMateralVariation MaterialVariation;

native final private function InitializeActionFramework();

native function bool IsMeshPhysicsCapable();

native function PlaySoundForState(out Name oState);

native function PreloadVFX();

native function ApplyVFXForState(Name NewState, optional bool bForce);

native function VerifyNotEditor();

native function bool RequiresProximityTimer();

native function bool SetState(Name ToState, optional Name NamedTransition, optional bool IgnoreTransition, optional bool bRightNow, optional bool bApplyVFX);

native function bool HasTransition(Name ToState, Name Transition);

native function DoDoorFinished(Name ToState);

function Name GetStartingState()
{
    if (m_bStartingStateOverride)
    {
        return StartingState;
    }
    return BioArtPlaceableType(Owner.ActorType).DefaultStartingState;
}

function PostBeginPlay()
{
    local BioArtPlaceable Placeable;
    local BioArtPlaceableType APType;
    PostBeginPlay();
    InitializeActionFramework();
    if (Owner.bDeleteMe)
    {
        return;
    }
    PreloadVFX();
    APType = BioArtPlaceableType(Owner.ActorType);
    if (APType != None && GetStartingState() == APType.m_nmDestroyedState)
    {
        SetState(APType.m_nmDestroyedState, 'None', false, false, false);
        return;
    }
    Placeable = BioArtPlaceable(Owner);
    if (Placeable != None && Placeable.FSM != None)
    {
        Placeable.FSM.OnTransition = OnTransition;
        SetState(GetStartingState(), 'None', true, true);
        if (APType != None && APType.m_oApperance.AnimSets.Length == 0)
        {
            Placeable.FSM.TransitionComplete();
        }
    }
}

function Name GetState()
{
    if (BioArtPlaceable(Owner).FSM.CurrentState == None)
    {
        return 'Error';
    }
    return BioArtPlaceable(Owner).FSM.CurrentState.StateName;
}

event bool IsInteractive()
{
    local Name nmInert;
    if (IsTransitioning())
    {
        return false;
    }
    if (m_bInPhysics == true)
    {
        return false;
    }
    if (BioArtPlaceable(Owner).FSM == None)
    {
        return false;
    }
    if (BioArtPlaceable(Owner).FSM.CurrentState != None)
    {
        nmInert = BioArtPlaceableType(Owner.ActorType).DefaultInertState;
        if (nmInert != 'None' && nmInert != 'None')
        {
            if (BioArtPlaceable(Owner).FSM.CurrentState.StateName == nmInert)
            {
                return false;
            }
        }
    }
    if (!IsDefaultActionPossible())
    {
        return false;
    }
    return true;
}

event OnArtPlaceableUsed(Actor User, Name ToState)
{
    local int I;
    I = 0;
    while (I < Owner.GeneratedEvents.Length)
    {
        if (ClassIsChildOf(Owner.GeneratedEvents[I].Class, Class'BioSeqEvt_ArtPlaceableUsed'))
        {
            Owner.GeneratedEvents[I].SetNameVars("ToState", ToState);
            Owner.GeneratedEvents[I].SetObjectVars("UsedObject", Owner);
            Owner.GeneratedEvents[I].CheckActivate(Owner, User);
        }
        I++;
    }
}

event OnArtPlaceablePlotFail(Actor oUsingActor, Name nmToState)
{
    local int nCount;
    nCount = 0;
    while (nCount < Owner.GeneratedEvents.Length)
    {
        if (ClassIsChildOf(Owner.GeneratedEvents[nCount].Class, Class'BioSeqEvt_ArtPlaceablePlotFail'))
        {
            Owner.GeneratedEvents[nCount].SetNameVars("ToState", nmToState);
            Owner.GeneratedEvents[nCount].CheckActivate(Owner, oUsingActor);
        }
        nCount++;
    }
}

event OnTransition(Name FromState, Name ToState, optional bool bCallCPP = true)
{
    local int I;
    I = 0;
    while (I < Owner.GeneratedEvents.Length)
    {
        if (ClassIsChildOf(Owner.GeneratedEvents[I].Class, Class'BioSeqEvt_ArtPlaceableTransition'))
        {
            Owner.GeneratedEvents[I].SetNameVars("FromState", FromState);
            Owner.GeneratedEvents[I].SetNameVars("ToState", ToState);
            Owner.GeneratedEvents[I].CheckActivate(Owner, Owner);
        }
        I++;
    }
    if (bCallCPP)
    {
        OnTransitionCPP(FromState, ToState);
    }
}

event ReactToImpulse(Vector vMomentum, Vector vHitLocation)
{
    local BioArtPlaceable oPlaceable;
    local BioBaseAppearancePlaceable pAppearance;
    local SkeletalMeshComponent oSkeletal;
    local StaticMeshComponent oStatic;
    m_vLastImpactVector = vMomentum;
    oPlaceable = BioArtPlaceable(Owner);
    pAppearance = BioArtPlaceableType(Owner.ActorType).m_oApperance;
    if (IsPhysicsAffectable() == true)
    {
        oSkeletal = SkeletalMeshComponent(oPlaceable.PlaceableMesh);
        oStatic = StaticMeshComponent(oPlaceable.PlaceableMesh);
        if (oSkeletal != None && oSkeletal.SkeletalMesh != None)
        {
            oSkeletal.AddImpulse(vMomentum, vHitLocation, pAppearance.m_nmPhysicsImpactBone);
        }
        else
        {
            if (oStatic != None && oStatic.StaticMesh != None)
            {
                oStatic.AddImpulse(vMomentum, vHitLocation, pAppearance.m_nmPhysicsImpactBone);
            }
        }
    }
}

event Name GetCurrentState()
{
    if (BioArtPlaceable(Owner).FSM == None || BioArtPlaceable(Owner).FSM.CurrentState == None)
    {
        return 'None';
    }
    return BioArtPlaceable(Owner).FSM.CurrentState.StateName;
}

function SetCollision(bool bCollide)
{
    BioArtPlaceable(Owner).SetCollision(bCollide, bCollide);
}

native function ChangeStateMesh(out Name nmState);

native function OnTransitionCPP(Name FromState, Name ToState);

native function bool IsTransitioning();

native function Name GetStateSuccess();

native function Name GetStateFailure();

native function bool IsPhysicsAffectable();

function SetWaitingForMove(bool bVal)
{
    m_bMonitorDoor = bVal;
}

function bool ProceedWithMove(Pawn oPawn)
{
    if (m_oDoorMarker != None)
    {
        return m_oDoorMarker.ProceedWithMove(oPawn);
    }
    return true;
}

static function class<BioBaseSaveObject> GetSaveObjectClass()
{
    return Class'BioArtPlaceableBehaviorSaveObject';
}

event ReadSaveObject(BioBaseSaveObject Save)
{
    if (bNoStateSave == false)
    {
        ReadSaveObject(Save);
    }
}

event BioBaseSaveObject WriteSaveObject(Object InOuter, optional BioBaseSaveObject Save)
{
    local BioArtPlaceableType Type;
    Type = BioArtPlaceableType(Owner.ActorType);
    if (m_nmLastState != Type.m_nmDestroyedState)
    {
        return WriteSaveObject(InOuter, Save);
    }
    else
    {
        return None;
    }
}

defaultproperties
{
    Enabled=true
    m_sVarName="Conditional"
    m_bTargetable=true
    ActorGameName="ArtPlaceable"
    m_oAppearanceType=BioInterface_Appearance_ArtPlaceable'oAppearance'
    m_fInteractionRange=600
    m_aTargetTipDefaults=System.Byte[]
    m_nMaxTargetTipStrings=13
    // child object oAppearance of type BioInterface_Appearance_ArtPlaceable
}
