﻿class BioBaseSquad extends BioSquadAPI
    hidecategories(Navigation,Actor,Collision,Display,Movement,Physics,RigidBody)
    config(Game)
    abstract
    native;

enum eATTACK_TYPE
{
    AI_ATTACK_TYPE_NONE,
    AI_ATTACK_TYPE_SNIPER,
    AI_ATTACK_TYPE_ACCURATE,
    AI_ATTACK_TYPE_INACCURATE,
    AI_ATTACK_TYPE_SUPPRESSING,
    AI_ATTACK_TYPE_POWER,
    AI_ATTACK_TYPE_MELEE,
};

enum ESwitchToWeapon
{
    STW_Pistol,
    STW_ShotGun,
    STW_AssaultRifle,
    STW_SniperRifle,
    STW_None,
};

enum ESquadIcons
{
    BIO_SQUADICON_INVALID,
    BIO_SQUADICON_HOLD,
    BIO_SQUADICON_MOVETO,
    BIO_SQUADICON_FOLLOW,
    BIO_SQUADICON_DEAD,
    BIO_SQUADICON_INCOVER,
};

enum EBioCapMode
{
    BIO_CAPMODE_WEAPON,
    BIO_CAPMODE_BIOTICS,
    BIO_CAPMODE_TECH,
    BIO_CAPMODE_COMBAT,
    BIO_CAPMODE_GRENADES,
    BIO_CAPMODE_ERROR,
};

enum EBioTacticalAction
{
    BIO_TACACT_NONE,
    BIO_TACACT_SHOOT,
    BIO_TACACT_GRENADE,
    BIO_TACACT_POWER,
    BIO_TACACT_EQUIP,
};

struct native MemberData
{
    var Pawn SquadMember;

    var BioTacticalMoveToIndicator oDestinationIndicator;

    var bool bMoveOrderAssigned;

    var Name nmPower;

    var Vector vTarget;

    var Vector vHoldPosition;

    var Actor oOrderedAttackTarget;

}

var(Squad) float m_fThreatModifier;

var BioBaseSquad m_pHackedSquad;

var() int ObjVersion;

var const native Map_Mirror m_aSquadPlayPen;

var Color SquadMemberLineColour;

var Color PlaypenSubtractiveVolumeColour;

var Color PlaypenAdditiveVolumeColour;

var(Squad) array<BioPlaypenVolume> PlaypenVolumes;

var transient array<BioBaseSquad> m_aPerceivedSquads;

var(Squad) class<BioFaction> SquadFaction;

var bool m_bEngagedHostileAction;

var bool m_bAllowVehicles;

var bool m_bAllowPawns;

var(Squad) bool bSquadEnabled;

var array<MemberData> Members;

enum ePawnValidType
{
    PVT_Invalid,
    PVT_ValidPawn,
    PVT_ValidCar,
};

native final function bool IsInSquad(Pawn pPawn);

native function int GetSquadSize();

native final function int GetHackedSquadSize();

native function Pawn GetSquadLeader();

native final function Pawn GetMember(int nIndex);

native function int GetMemberIndex(Pawn pPawn);

native function ClearSquad();

native function bool RemoveMember(Pawn pPawn);

native function bool RemoveMemberByIndex(int nIndex);

native final function BioTacticalMoveToIndicator GetMemberMoveIndicator(int nIndex);

native final function SetMemberMoveIndicator(int nIndex, BioTacticalMoveToIndicator oIndicator);

native final function AddSquadToPerception(BioBaseSquad pSquad, optional Pawn pSource, optional Pawn pTarget);

native final function bool IsSquadPerceived(BioBaseSquad pSquad);

native final function bool IsPawnPerceived(Pawn pPawn);

native function RemoveSquadFromPerception(BioBaseSquad pSquad);

native final function bool CanShootExposed(Pawn oSource, Actor oTarget, out int bExposed);

native final function bool CanShoot(Pawn oSource, Actor oTarget);

native final iterator function SquadMembers(out BioAiController oController);

native final iterator function PerceivedPawns(out Pawn oPawn);

native function bool IsPositionInPlaypen(const out Vector vLocation);

native function bool IsPositionInSubtractivePlaypen(const out Vector vLocation);

native function bool IsActorInPlaypen(Actor oActor);

native function bool IsActorInSubtractivePlaypen(Actor oActor);

native function bool HasPlaypen();

native function AddVolumeToPlaypen(BioPlaypenVolume Volume);

native function RemoveVolumeFromPlaypen(BioPlaypenVolume Volume);

native function bool IsVolumeInPlaypen(BioPlaypenVolume Volume);

native function UpdatePlaypen();

native function Actor GetPlaypenNavOrigin(optional Pawn oPawn);

native function Actor GetPlaypenReturnPoint(Pawn oPawn);

event NotifyEnemyPerceived();

event NotifyNoEnemiesPerceived();

function NotifyPlaypenChanged()
{
    local BioAiController AI;
    local SFXAI_Core oSFXAI;
    foreach SquadMembers(AI)
    {
        oSFXAI = SFXAI_Core(AI);
        if (oSFXAI != None)
        {
            oSFXAI.NotifyPlaypenChanged();
        }
    }
}

function bool Died(Pawn Member, Controller Killer)
{
    local int idx;
    local BioAiController oMember;
    foreach SquadMembers(oMember)
    {
        if (oMember.Pawn != None && oMember.Pawn != Member && oMember.Pawn.IsDead() == false)
        {
            if (GetSquadSize() == 2)
            {
                SFXGame(WorldInfo.Game).TriggerVocalizationEvent(29, BioPawn(oMember.Pawn));
                oMember.OnLastManStanding();
            }
        }
    }
    idx = 0;
    while (idx < GeneratedEvents.Length)
    {
        if (ClassIsChildOf(GeneratedEvents[idx].Class, Class'SeqEvent_Death') != false)
        {
            if (Killer != None)
            {
                GeneratedEvents[idx].SetObjectVars("Killer", Killer.Pawn);
            }
            GeneratedEvents[idx].CheckActivate(self, Member);
        }
        idx++;
    }
    return true;
}

native function Pawn FindSquadMemberByTag(Name nmTag);

native function BioBaseSquad GetSquad(Actor oActor);

native function BioEpicPawnBehavior GetPawnBehavior(Actor oActor);

native function int GetNumberOfLiveMembers();

event int AddMember(Pawn Pawn, optional bool bCheckPlaypens = true)
{
    local BioBaseSquad OldSquad;
    local int SquadIdx;
    local BioAiController AI;
    local SFXAI_NativeBase oSFXAI;
    local BioPawn BP;
    if (Pawn == None)
    {
        return -1;
    }
    OldSquad = BioPawn(Pawn).Squad;
    if (OldSquad != None)
    {
        OldSquad.RemoveMember(Pawn);
    }
    BP = BioPawn(Pawn);
    if (BP != None)
    {
        BP.Squad = self;
        BP.OldFactionClass = BP.FactionClass;
        BP.FactionClass = self.SquadFaction;
        BP.m_oBehavior.Squad = self;
    }
    SquadIdx = 0;
    while (SquadIdx < Members.Length)
    {
        if (Members[SquadIdx].SquadMember == Pawn)
        {
            return SquadIdx;
        }
        SquadIdx++;
    }
    SquadIdx = Members.Length;
    Members.Length = Members.Length + 1;
    Members[SquadIdx].SquadMember = Pawn;
    TriggerEventClass(Class'BioSeqEvt_MemberAdded', Pawn);
    AI = BioAiController(Pawn.Controller);
    if (AI != None && HasPlaypen() && bCheckPlaypens)
    {
        if (IsPositionInPlaypen(Pawn.Location) == false)
        {
            AI.OnLeftPlaypen();
        }
        else
        {
            AI.OnEnteredPlaypen();
        }
    }
    oSFXAI = SFXAI_NativeBase(Pawn.Controller);
    if (oSFXAI != None)
    {
        oSFXAI.EnableAI(bSquadEnabled, 4);
    }
    return SquadIdx;
}

function RemoveDyingMember(Pawn oPawn)
{
    RemoveMember(oPawn);
}

event MemberRemoved(Pawn oPawn)
{
    local BioPawn Pawn;
    Pawn = BioPawn(oPawn);
    if (Pawn != None)
    {
        Pawn.Squad = None;
    }
}

event GenerateProximityAlarmEvent(Pawn oMember, Actor oTarget, int nRelationship)
{
    local int nIndex;
    local array<int> ActivateIndices;
    // unknown bytecode 54 ;
}

function ePawnValidType IsBioPawnValid(Actor oTestActor, optional bool bRequiresTargetFlag = false)
{
    local BioPawn oPawn;
    local BioVehicleBase oCar;
    oPawn = BioPawn(oTestActor);
    oCar = BioVehicleBase(oTestActor);
    if (oTestActor == None || oTestActor.bStasis == true)
    {
        return 0;
    }
    if (oPawn == None && oCar == None)
    {
        return 0;
    }
    if (oPawn != None)
    {
        if (oPawn.m_oBehavior == None)
        {
            return 0;
        }
        if (GetSquad(oPawn) == None)
        {
            return 0;
        }
        if (oPawn.IsDead() == true)
        {
            return 0;
        }
        return 1;
    }
    else
    {
        if (oCar != None)
        {
            if (oCar.m_oBehavior == None)
            {
                return 0;
            }
            if (GetSquad(oCar) == None)
            {
                return 0;
            }
            if (oCar.IsDead() == true)
            {
                return 0;
            }
            return 2;
        }
    }
    return 0;
}

native final function BioBaseSquad MakeHackable();

static function class<BioBaseSaveObject> GetSaveObjectClass()
{
    return Class'BioBaseSquadSaveObject';
}

native final function int GetMaxHazardLevel();

event EnableSquad()
{
    local BioAiController AI;
    local SFXAI_NativeBase oSFXAI;
    if (bSquadEnabled == false)
    {
        foreach SquadMembers(AI)
        {
            oSFXAI = SFXAI_NativeBase(AI);
            if (oSFXAI != None)
            {
                oSFXAI.EnableAI(true, 4);
            }
        }
        bSquadEnabled = true;
    }
}

event DisableSquad()
{
    local BioAiController AI;
    local SFXAI_NativeBase oSFXAI;
    if (bSquadEnabled == true)
    {
        foreach SquadMembers(AI)
        {
            oSFXAI = SFXAI_NativeBase(AI);
            if (oSFXAI != None)
            {
                oSFXAI.EnableAI(false, 4);
            }
        }
        bSquadEnabled = false;
    }
}

defaultproperties
{
    bSquadEnabled=true
    m_bAllowPawns=true
    SquadFaction=class'BioFaction_BaseSquad'
    PlaypenAdditiveVolumeColour=(B=0,G=255,R=0,A=255)
    PlaypenSubtractiveVolumeColour=(B=0,G=0,R=255,A=255)
    SquadMemberLineColour=(B=255,G=0,R=0,A=255)
    Components=None
    0=System.Byte[]
    // child object CollisionCylinder of type CylinderComponent
    // child object SquadLines of type BioSquadLinesComponent
}
