﻿class BioCheatManager extends CheatManager
    within BioPlayerController
    config(Game)
    native;

enum EProfileType
{
    Profile_None,
    Profile_AI,
    Profile_Camera,
    Profile_Combat,
    Profile_SysDesign,
    Profile_CombatStats,
    Profile_Difficulty,
    Profile_Angst,
    Profile_Cooldown,
    Profile_Damage,
    Profile_Pawn,
    Profile_Power,
    Profile_Tech,
    Profile_Treasure,
    Profile_Locomotion,
    Profile_AnimTree,
    Profile_Ticket,
    Profile_Vehicle,
    Profile_Henchmen,
    Profile_Settings,
    Profile_Effects,
    Profile_Scaleform,
    Profile_Anim,
    Profile_Cover,
    Profile_Door,
    Profile_Conversation,
    Profile_ConversationBug,
    Profile_Gestures,
    Profile_DesignBudgetGeneral,
    Profile_Bonuses,
    Profile_Multipliers,
    Profile_LookAt,
};

struct native ProfileData
{
    var Name Keyword;

    var String Header;

    var String Description;

    var bool bNoTarget;

    var delegate<ProfileHandler> Func;

    var delegate<ProfileUtility> Utility;

}

var transient array<String> m_aPowerList;

var transient bool m_bShowPowerList;

var transient bool m_bShowPowerImpactText;

var transient bool m_bEnablePowerCooldown;

var transient bool m_bShowPowerAiming;

var transient bool bShowAllProfiles;

var transient float ProfilesDisplayTime;

var transient float ProfilesTime;

var config bool m_bShowAngstIcons;

var config bool bProfileHidesGUI;

var Color ProfileHighlightColor;

var Color ProfileTextColor;

var Color ProfileHeaderColor;

var Color ProfileTitleColor;

var float ColumnWidth;

var int CurrentColumn;

var Vector2D TopLeft;

var Name ProfileSubTarget;

var Actor ProfileTarget;

var EProfileType CurrentProfile;

var array<ProfileData> AllProfiles;

delegate ProfileHandler();

delegate ProfileUtility();

function SFXHeavyWeapon GetHeavyWeapon()
{
    local SFXInventoryManager InvManager;
    local BioPlayerController Controller;
    local SFXHeavyWeapon HWeapon;
    local SFXHeavyWeapon FoundWeapon;
    Controller = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController();
    InvManager = SFXInventoryManager(Controller.Pawn.InvManager);
    foreach InvManager.InventoryActors(Class'SFXHeavyWeapon', HWeapon)
    {
        if (HWeapon != None && HWeapon.AmmoUsedCount > 0)
        {
            FoundWeapon = HWeapon;
        }
    }
    return FoundWeapon;
}

exec function IncrementGrinder(EAchievementID Id)
{
    Outer.UpdateAchievementProgression(Id);
}

exec function ResetGrinder(EAchievementID Id)
{
    Outer.ProfileSettings.GrinderAchievementReset(Id, Outer);
}

exec function SloMo(float T)
{
    SFXGame(Outer.WorldInfo.Game).TimeDilationOverride = FMax(T, 0,01);
}

exec function Profiles(optional float TimeToDisplay)
{
    if (TimeToDisplay == 0)
    {
        TimeToDisplay = 10;
    }
    bShowAllProfiles = true;
    ProfilesDisplayTime = TimeToDisplay;
    ProfilesTime = Outer.WorldInfo.TimeSeconds;
}

function DrawProfiles()
{
    local int idx;
    if (Outer.WorldInfo.IsConsoleBuild())
    {
        TopLeft.X = Outer.myHUD.Canvas.SizeX * BioHUD(Outer.myHUD).SafeAreaRatioX;
        TopLeft.Y = Outer.myHUD.Canvas.SizeY * BioHUD(Outer.myHUD).SafeAreaRatioY;
    }
    else
    {
        TopLeft.X = 2;
        TopLeft.Y = 0;
    }
    SetProfileColumn(0);
    DrawProfileHeaderText("Syntax: ");
    DrawProfileText("profile <keyword> <self/target/camera>");
    idx = 0;
    while (idx < AllProfiles.Length)
    {
        DrawProfileHeaderText("Keyword: ");
        DrawProfileText(AllProfiles[idx].Keyword $ "    " @ AllProfiles[idx].Description);
        idx++;
    }
    if (Outer.TimeSince(ProfilesTime) > ProfilesDisplayTime)
    {
        bShowAllProfiles = false;
    }
}

exec function Profile(Name Keyword, optional String Target, optional Name SubTarget)
{
    local int idx;
    local ProfileData Profile;
    idx = 0;
    while (idx < AllProfiles.Length)
    {
        Profile = AllProfiles[idx];
        if (Profile.Keyword == Keyword)
        {
            if (Keyword != 'None')
            {
                if (Profile.bNoTarget == false)
                {
                    ProfileTarget = GetActorFromString(Target);
                    if (ProfileTarget == None)
                    {
                        CurrentProfile = 0;
                        return;
                    }
                }
                else
                {
                    ProfileTarget = Outer.Pawn;
                }
                if (bProfileHidesGUI && CurrentProfile == 0)
                {
                    Outer.ConsoleCommand("show scaleform");
                }
                CurrentProfile = idx;
                ProfileSubTarget = SubTarget;
                return;
            }
            CurrentProfile = 0;
            ProfileTarget = None;
            if (bProfileHidesGUI)
            {
                Outer.ConsoleCommand("show scaleform");
            }
            return;
        }
        idx++;
    }
}

function DrawCurrentProfile()
{
    local delegate<ProfileHandler> ProfileFunc;
    local delegate<ProfileUtility> UtilityFunc;
    if (ProfileTarget != None && CurrentProfile > 0 && CurrentProfile < 32)
    {
        DrawProfileTitle();
        ProfileFunc = AllProfiles[CurrentProfile].Func;
        if (ProfileFunc != None)
        {
            ProfileFunc.ProfileHandler();
        }
        UtilityFunc = AllProfiles[CurrentProfile].Utility;
        if (UtilityFunc != None)
        {
            UtilityFunc.ProfileUtility();
        }
    }
}

function DrawProfileTitle()
{
    local float XL;
    local float YL;
    local Canvas Canvas;
    Canvas = Outer.myHUD.Canvas;
    if (Canvas == None)
    {
        return;
    }
    Canvas.bCenter = true;
    Canvas.DrawColor = ProfileTitleColor;
    Canvas.Font = Class'Engine'.GetSmallFont();
    Canvas.TextSize("A", XL, YL);
    if (Outer.WorldInfo.IsConsoleBuild())
    {
        Canvas.SetPos(BioHUD(Outer.myHUD).SafeAreaRatioX * Canvas.SizeX, BioHUD(Outer.myHUD).SafeAreaRatioY * Canvas.SizeY);
    }
    else
    {
        Canvas.SetPos(2, 0);
    }
    if (AllProfiles[CurrentProfile].bNoTarget == false)
    {
        Canvas.DrawText(AllProfiles[CurrentProfile].Header @ "<<" @ ProfileTarget @ ">>" @ "(" $ ProfileTarget.Tag $ ")");
    }
    else
    {
        Canvas.DrawText(AllProfiles[CurrentProfile].Header);
    }
    Canvas.DrawText(AllProfiles[CurrentProfile].Description);
    Canvas.Draw2DLine(0, Canvas.CurY, Canvas.ClipX, Canvas.CurY, Canvas.DrawColor);
    if (Outer.WorldInfo.IsConsoleBuild())
    {
        TopLeft.X = BioHUD(Outer.myHUD).SafeAreaRatioX * Canvas.SizeX;
    }
    else
    {
        TopLeft.X = 2;
    }
    TopLeft.Y = Canvas.CurY + YL;
    SetProfileColumn(0);
    Canvas.bCenter = false;
}

final event DrawProfileHeaderText(coerce String Text)
{
    local Canvas Canvas;
    local float OldCurY;
    Canvas = Outer.myHUD.Canvas;
    if (Canvas != None)
    {
        Canvas.DrawColor = ProfileHeaderColor;
        OldCurY = Canvas.CurY;
        Canvas.DrawText(Text, false);
        Canvas.CurY = OldCurY;
    }
}

final event DrawProfileText(coerce String Text, optional bool Highlight = false)
{
    local Canvas Canvas;
    Canvas = Outer.myHUD.Canvas;
    if (Canvas != None)
    {
        Canvas.DrawColor = Highlight ? ProfileHighlightColor : ProfileTextColor;
        Canvas.DrawText(Text);
        Canvas.CurX = GetProfileColumnCoord();
    }
}

final event float GetProfileColumnCoord()
{
    return TopLeft.X + CurrentColumn * ColumnWidth;
}

final event SetProfileColumn(int ColumnIdx)
{
    CurrentColumn = ColumnIdx;
    if (Outer.myHUD.Canvas != None)
    {
        if (ColumnIdx == 0)
        {
            Outer.myHUD.Canvas.SetPos(TopLeft.X, TopLeft.Y);
        }
        else
        {
            Outer.myHUD.Canvas.SetPos(GetProfileColumnCoord(), TopLeft.Y);
        }
    }
}

native final function String GetLatentAction(Controller AI);

native final function int GetNumStates(Controller AI);

native final function String GetStateNameByIdx(Controller AI, int idx);

exec function AT(Name newArea, optional Name StartPoint)
{
    BioWorldInfo(Outer.WorldInfo).MoveToArea(newArea, StartPoint);
}

function RenderHUD(BioHUD myHUD)
{
    if (m_bShowPowerImpactText)
    {
        DisplayPowerImpactText(myHUD);
    }
    if (m_bShowPowerList)
    {
        DisplayPowerList(myHUD);
    }
}

exec function KillTarget()
{
    local Actor MyTarget;
    MyTarget = Outer.m_oPlayerSelection.m_oCurrentSelectionTarget;
    if (Pawn(MyTarget) != None)
    {
        Pawn(MyTarget).Died(Outer, Class'SFXDamageType_CheatKill', MyTarget.Location);
    }
}

exec function KillEnemies()
{
    local BioPawn ChkPawn;
    foreach Outer.WorldInfo.AllPawns(Class'BioPawn', ChkPawn)
    {
        if (ChkPawn.IsHostile(Outer.Pawn))
        {
            ChkPawn.Died(Outer, Class'SFXDamageType_CheatKill', ChkPawn.Location);
        }
    }
}

exec function KillParty()
{
    local BioPawn ChkPawn;
    foreach Outer.WorldInfo.AllPawns(Class'BioPawn', ChkPawn)
    {
        if (ChkPawn.Squad == BioPawn(Outer.Pawn).Squad && ChkPawn != Outer.Pawn)
        {
            ChkPawn.Died(Outer, Class'SFXDamageType_Suicide', ChkPawn.Location);
        }
    }
}

exec function KillSelf()
{
    Outer.Pawn.Died(Outer, Class'SFXDamageType_Suicide', Outer.Location);
}

exec function Damage(String Target, float Amount)
{
    local Actor MyTarget;
    MyTarget = GetActorFromString(Target);
    if (MyTarget != None)
    {
        MyTarget.TakeDamage(Amount, None, MyTarget.Location, vect(0,0,0), Class'SFXDamageType_Default');
    }
}

exec function StasisParty(bool bValue)
{
    local BioPawn ChkPawn;
    local BioAiController AIController;
    foreach Outer.WorldInfo.AllActors(Class'BioPawn', ChkPawn)
    {
        if (ChkPawn.Squad == BioPawn(Outer.Pawn).Squad && ChkPawn != Outer.Pawn)
        {
            ChkPawn.bStasis = bValue;
            AIController = BioAiController(ChkPawn.Controller);
            if (AIController != None)
            {
                if (ChkPawn.bStasis)
                {
                    AIController.OnEnteringStasis();
                }
                else
                {
                    AIController.OnLeavingStasis();
                }
            }
        }
    }
}

exec function GetLocation()
{
    local String sLocation;
    sLocation = "PlayerLocation: (" $ Outer.Pawn.Location.X $ ", " $ Outer.Pawn.Location.Y $ ", " $ Outer.Pawn.Location.Z $ ").  Consider using ShowLocation.";
    Outer.ClientMessage(sLocation);
}

native exec function OutputBugReportXML(bool bCreateSavegame, bool bCreateScreenshot);

native exec function SetLocation(String sDestination);

native exec function SetGCDebugPackage(String sPackageName);

native exec function ForceEnableCCD(bool bValue);

native exec function BioClearCrossLevelReferences(Level pLevel);

native exec function SetVehicleCamRadiusPoint(int nIndex, float fDegVal, float fScaleVal);

exec function TestConditionals()
{
    local BioGlobalVariableTable gv;
    local int I;
    local float fTime;
    gv = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    gv.SetBool(0, true);
    gv.SetBool(1, false);
    gv.SetBool(2, true);
    gv.SetBool(3, true);
    gv.SetInt(0, 4);
    gv.SetInt(1, 8);
    Outer.Clock(fTime);
    BioWorldInfo(Outer.WorldInfo).CheckConditional(0);
    Outer.UnClock(fTime);
    Outer.Clock(fTime);
    BioWorldInfo(Outer.WorldInfo).CheckConditional(7);
    Outer.UnClock(fTime);
    Outer.Clock(fTime);
    I = 0;
    while (I < 100)
    {
        BioWorldInfo(Outer.WorldInfo).CheckConditional(6);
        I++;
    }
    Outer.UnClock(fTime);
    Outer.Clock(fTime);
    I = 0;
    while (I < 300)
    {
        BioWorldInfo(Outer.WorldInfo).CheckConditional(0);
        BioWorldInfo(Outer.WorldInfo).CheckConditional(7);
        BioWorldInfo(Outer.WorldInfo).CheckConditional(6);
        I++;
    }
    Outer.UnClock(fTime);
    Outer.Clock(fTime);
    BioWorldInfo(Outer.WorldInfo).CheckConditional(5);
    Outer.UnClock(fTime);
    Outer.Clock(fTime);
    I = 0;
    while (I < 1000)
    {
        BioWorldInfo(Outer.WorldInfo).CheckConditional(5);
        I++;
    }
    Outer.UnClock(fTime);
}

exec function GiveXP(float XPValue)
{
    Outer.GrantXP(XPValue);
}

exec function DebugVocSystem(bool bEnable)
{
    SFXGame(Outer.WorldInfo.Game).VocManager.bDebugging = bEnable;
}

exec function DebugVocLine(String Instigator, String Recipient, String eventStr)
{
    local BioPawn Inst;
    local BioPawn Recp;
    local ESFXVocalizationEventID EventID;
    local SFXVocalizationEvent Voc;
    EventID = GetEnumIndex(Enum'ESFXVocalizationEventID', eventStr);
    Inst = BioPawn(GetActorFromString(Instigator));
    Recp = BioPawn(GetActorFromString(Recipient));
    Voc.DebugIndex = -1;
    Voc.DelayTimeRemainingSec = 0;
    Voc.Instigator = Inst;
    Voc.Recipient = Recp;
    Voc.Id = EventID;
    SFXGame(Outer.WorldInfo.Game).VocManager.HandleSFXVocalizationEvent(Voc);
}

exec function ShowVocInfo(String PawnTag, String eventStr)
{
    local BioPawn Pawn;
    local ESFXVocalizationEventID EventID;
    local int RoleIdx;
    local int VariationIdx;
    local String outputstring;
    EventID = GetEnumIndex(Enum'ESFXVocalizationEventID', eventStr);
    Pawn = BioPawn(GetActorFromString(PawnTag));
    RoleIdx = 0;
    while (RoleIdx < Pawn.PawnType.VocalizationBank.Vocalizations[EventID].Roles.Length)
    {
        appScreenDebugMessage("Event: " $ GetEnum(Enum'ESFXVocalizationRole', RoleIdx));
        VariationIdx = 0;
        while (VariationIdx < Pawn.PawnType.VocalizationBank.Vocalizations[EventID].Roles[RoleIdx].Variations.Length)
        {
            if (Pawn.PawnType.VocalizationBank.Vocalizations[EventID].Roles[RoleIdx].Variations[VariationIdx].SpecificType.Length > 0)
            {
                outputstring = "" $ Pawn.PawnType.VocalizationBank.Vocalizations[EventID].Roles[RoleIdx].Variations[VariationIdx].SpecificType[0];
                outputstring $= ":::" $ Pawn.PawnType.VocalizationBank.Vocalizations[EventID].Roles[RoleIdx].Variations[VariationIdx].SpecificValue[0];
            }
            else
            {
                outputstring = "default";
            }
            appScreenDebugMessage(outputstring $ " ---> " $ Pawn.PawnType.VocalizationBank.Vocalizations[EventID].Roles[RoleIdx].Variations[VariationIdx].Sound);
            VariationIdx++;
        }
        RoleIdx++;
    }
}

exec function GiveTalentPoints(int nNumPoints)
{
    local BioPawn Player;
    local BioPawn SquadMember;
    local int nIndex;
    Player = BioPawn(Outer.Pawn);
    if (Player != None && Player.Squad != None)
    {
        nIndex = 0;
        while (nIndex < Player.Squad.GetSquadSize())
        {
            SquadMember = BioPawn(Player.Squad.GetMember(nIndex));
            if (SquadMember != None)
            {
                SquadMember.AddTalentPoints(nNumPoints);
            }
            nIndex++;
        }
    }
    Outer.ClientMessage(nNumPoints $ " talent points given to each squad member");
}

exec function GiveBonusTalent(int bonusIdentifier)
{
    local BioPawn Player;
    Player = BioPawn(Outer.Pawn);
    if (Player != None)
    {
        Player.m_oBehavior.m_Talents.AddBonusTalents(bonusIdentifier);
    }
}

exec function IncreaseTalentMaxRank(int TalentID, int MaxRank)
{
    local BioPawn Player;
    Player = BioPawn(Outer.Pawn);
    if (Player != None)
    {
        Player.m_oBehavior.m_Talents.IncreaseTalentMaxRank(TalentID, MaxRank);
    }
}

exec function IncreaseTalentMaxRankByIncrement(int TalentID, int MaxRankIncrement)
{
    local BioPawn Player;
    Player = BioPawn(Outer.Pawn);
    if (Player != None)
    {
        Player.m_oBehavior.m_Talents.IncreaseTalentMaxRankByIncrement(TalentID, MaxRankIncrement);
    }
}

exec function ScaleTalentByIncrement(int TalentID, int rankIncrement)
{
    local BioPawn Player;
    Player = BioPawn(Outer.Pawn);
    if (Player != None)
    {
        Player.m_oBehavior.m_Talents.ScaleTalentByIncrement(TalentID, rankIncrement);
    }
}

exec function GiveSpectreTalents()
{
    GiveBonusTalent(259);
    IncreaseTalentMaxRankByIncrement(108, 3);
    ScaleTalentByIncrement(108, 1);
    IncreaseTalentMaxRankByIncrement(109, 3);
    ScaleTalentByIncrement(109, 1);
}

native exec function BioSaveState(int nSaveStateSlot);

native exec function BioLoadState(int nSaveStateSlot);

native exec function BioTransition(int nTransition, int nParam);

exec function TestPrefab(int nTestNum)
{
    DebugSpawnPrefab(nTestNum);
}

native function DebugSpawnPrefab(int nTestNum);

native exec function TestRagdoll(coerce String sWhitespaceDelimitedArguments);

exec function TestFalling(Name nmTarget)
{
    local BioPlayerController oController;
    local BioPawn oPawn;
    local Vector Location;
    oController = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController();
    oPawn = BioPawn(oController.Pawn);
    if (nmTarget == 'Target')
    {
        oPawn = BioPawn(BioPlayerController(Outer.Pawn.Controller).m_oPlayerSelection.m_oCurrentSelectionTarget);
        if (oPawn == None)
        {
            oPawn = BioPawn(oController.Pawn);
        }
    }
    Location = oPawn.Location;
    Location.Z += 500;
    oPawn.SetLocation(Location);
    oPawn.SetPhysics(2);
    oPawn.Falling();
}

native final function Actor GetActorFromString(String Str);

native final function Actor GetUIWorldActor();

exec function EnableDamage(bool B)
{
    local BioRemoteLogger GLogger;
    SFXGame(Outer.WorldInfo.Game).EnableDamage = B;
    Outer.ClientMessage("Damage" @ B ? "Enabled" : "Disabled");
    GLogger = Class'BioRemoteLogger'.GetLogger();
    if (GLogger != None)
    {
        GLogger.SendInvalidPlaythrough("EnableDamage");
    }
}

exec function InitResource(EInventoryResourceTypes eResourceType, int nAmt)
{
    local int nAmtIncrease;
    local BioPawn oPlayerPawn;
    local SFXInventoryManager oInventory;
    oPlayerPawn = BioPawn(Outer.Pawn);
    if (oPlayerPawn == None || oPlayerPawn.InvManager == None)
    {
        return;
    }
    oInventory = SFXInventoryManager(oPlayerPawn.InvManager);
    if (oInventory == None)
    {
        return;
    }
    nAmtIncrease = nAmt - oInventory.GetResource(eResourceType);
    oInventory.AdjustResource(eResourceType, nAmtIncrease, true, true);
}

exec function InitCredits(int nCredits)
{
    InitResource(0, nCredits);
}

exec function InitPalladium(int nResource)
{
    InitResource(6, nResource);
}

exec function InitEezo(int nResource)
{
    InitResource(4, nResource);
}

exec function InitIridium(int nResource)
{
    InitResource(5, nResource);
}

exec function InitSalvage(int nSalvage)
{
    InitResource(2, nSalvage);
}

exec function InitPlatinum(int nResource)
{
    InitResource(7, nResource);
}

exec function InitMedigel(int nMedigel)
{
    InitResource(1, nMedigel);
}

exec function InitProbes(int nResource)
{
    InitResource(8, nResource);
}

exec function InitAmmo(int nAmmo)
{
    local SFXInventoryManager InvManager;
    local BioPlayerController Controller;
    local SFXHeavyWeapon HWeapon;
    local SFXWeapon Weapon;
    HWeapon = GetHeavyWeapon();
    if (HWeapon != None)
    {
        HWeapon.AddHeavyAmmo(nAmmo);
    }
    Controller = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController();
    InvManager = SFXInventoryManager(Controller.Pawn.InvManager);
    foreach InvManager.InventoryActors(Class'SFXWeapon', Weapon)
    {
        Weapon.AddAmmo(nAmmo);
    }
}

exec function InitFuel(float fAmt)
{
    local SFXInventoryManager oInventory;
    oInventory = SFXInventoryManager(BioPawn(Outer.Pawn).InvManager);
    oInventory.CurrentFuel = FMin(oInventory.CurrentFuel + fAmt, oInventory.GetMaxFuel());
}

exec function InitMaxFuel(float FMax)
{
    local SFXInventoryManager oInventory;
    oInventory = SFXInventoryManager(BioPawn(Outer.Pawn).InvManager);
    oInventory.SetMaxFuel(FMax);
    oInventory.CurrentFuel = FMin(oInventory.CurrentFuel, oInventory.GetMaxFuel());
}

exec function InitFuelEfficiency(float fEff)
{
    local SFXInventoryManager oInventory;
    oInventory = SFXInventoryManager(BioPawn(Outer.Pawn).InvManager);
    oInventory.FuelEfficiency = fEff;
}

exec function BasicWeapons()
{
    local BioPlayerController oController;
    local SFXInventoryManager oInventory;
    local Weapon oWeapon;
    oController = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController();
    oInventory = SFXInventoryManager(BioPawn(oController.Pawn).InvManager);
    foreach oInventory.InventoryActors(Class'Weapon', oWeapon)
    {
        oInventory.RemoveFromInventory(oWeapon);
    }
    GiveItem("self", 'AssaultRifle');
    GiveItem("self", 'Shotgun');
    GiveItem("self", 'HeavyPistol');
    GiveItem("self", 'SniperRifle');
    GiveItem("self", 'GrenadeLauncher');
}

final private function class<SFXWeapon> FindWeaponClass(String WeaponClassName)
{
    local class<SFXWeapon> FoundClass;
    FoundClass = Class<SFXWeapon>(FindObject(WeaponClassName, Class'Class'));
    if (FoundClass == None && Class'SFXEngine'.IsSeekFreeObjectSupported(WeaponClassName))
    {
        FoundClass = Class<SFXWeapon>(Class'SFXEngine'.LoadSeekFreeObject(WeaponClassName, Class'Class'));
    }
    if (FoundClass == None)
    {
        FoundClass = Class<SFXWeapon>(DynamicLoadObject(WeaponClassName, Class'Class'));
    }
    return FoundClass;
}

exec function ResetWeapons(String sTarget)
{
    local SFXPawn Pawn;
    Pawn = SFXPawn(GetActorFromString(sTarget));
    if (Pawn == None)
    {
        return;
    }
    Pawn.CreateWeapons(Pawn.GetPawnLoadout());
}

exec function GiveItem(String sTarget, Name nmItemLabel)
{
    local BioPlayerController oController;
    local BioPawn oPawn;
    local class<SFXWeapon> cWeapon;
    oController = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController();
    oPawn = BioPawn(oController.Pawn);
    oPawn = BioPawn(GetActorFromString(sTarget));
    if (oPawn == None)
    {
        return;
    }
    cWeapon = FindWeaponClass(nmItemLabel);
    if (cWeapon == None)
    {
        cWeapon = FindWeaponClass("SFXGameContent_Inventory." $ nmItemLabel);
    }
    if (cWeapon == None)
    {
        cWeapon = FindWeaponClass("SFXGameContent_Inventory.SFXWeapon_" $ nmItemLabel);
    }
    if (cWeapon == None)
    {
        cWeapon = FindWeaponClass("SFXGameContent_Inventory.SFXHeavyWeapon_" $ nmItemLabel);
    }
    if (cWeapon == None)
    {
        return;
    }
    oPawn.CreateWeapon(cWeapon);
}

native function ShowMarketplaceUI();

exec function DownloadContent()
{
    ShowMarketplaceUI();
}

exec function ToggleSFHUD()
{
    local MassEffectGuiManager oManager;
    oManager = MassEffectGuiManager(Outer.GetScaleFormManager());
    oManager.ReleaseHudCache();
}

exec function Min1Health(bool B)
{
    Outer.ClientMessage("Min1Health set to" @ B);
    SetPlayerSquadMin1Health(B);
}

exec function SetPlayerSquadMin1Health(bool B)
{
    local BioBaseSquad pSquad;
    local int I;
    local BioActorBehavior pBehavior;
    local BioRemoteLogger GLogger;
    pSquad = BioEpicPawnBehavior(Outer.Pawn.oBioComponent).Squad;
    I = 0;
    while (pSquad.GetSquadSize() != I)
    {
        pBehavior = BioActorBehavior(pSquad.GetMember(I).oBioComponent);
        pBehavior.m_bMin1HealthOverride = true;
        pBehavior.m_bMin1Health = B;
        ++I;
    }
    GLogger = Class'BioRemoteLogger'.GetLogger();
    if (GLogger != None)
    {
        GLogger.SendInvalidPlaythrough("SetPlayerSquadMin1Health");
    }
}

exec function GiveSuperGun()
{
    local BioPlayerController oController;
    local BioPawn oPlayer;
    local SFXWeapon oWeapon;
    oController = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController();
    oPlayer = BioPawn(oController.Pawn);
    oWeapon = SFXWeapon(oPlayer.Weapon);
    oWeapon.bSuperDamage = true;
    oWeapon.MagSize.Value = 9999;
}

native exec function SkynetToggleQAPaths();

native exec function SkynetToggleSilentMode();

native exec function SkynetToggleVerboseMode();

native exec function SkynetToggleSessionDisplay();

native exec function SkynetToggleNucleusTelemetry();

native exec function NotFun();

native exec function float GetPowerField(BioPawnBehavior targetBehavior, Name PowerName, Name FieldName);

native exec function SkynetScreenshot(String sFilename);

native exec function SkynetSavegame(String sFilename);

exec function TogglePowerAiming()
{
    m_bShowPowerAiming = !m_bShowPowerAiming;
}

exec function FeignDeath(Name nmPawn, bool B)
{
    local BioPawn pTargetPawn;
    pTargetPawn = BioPawn(GetActorFromString(nmPawn));
    pTargetPawn.m_oBehavior.FeignDeath(B);
}

native exec function JournalAudit();

native exec function CodexAudit();

native exec function DisplayStringID(bool i_bDisplay);

native exec function HideStrings(bool i_bHide);

exec function UsePower(Name nmPawn, Name nmPower)
{
    local Pawn oPawn;
    local BioPlayerSquad oPlayerSquad;
    if (nmPawn == 'Player' || nmPawn == 'Self')
    {
        oPawn = Outer.Pawn;
    }
    else
    {
        foreach Outer.AllActors(Class'Pawn', oPawn)
        {
            if (nmPawn == oPawn.Name)
            {
                break;
            }
        }
    }
    if (oPawn == None)
    {
        return;
    }
    oPlayerSquad = BioPlayerSquad(BioPawn(Outer.Pawn).Squad);
    if (oPlayerSquad != None)
    {
        Outer.SquadOrderUsePower(nmPower, oPawn);
        Outer.ApplyTacticalOrders();
    }
}

exec function EnterCombatStasis(Name nmPawn, float fDuration, bool bAllowDamage)
{
    local Pawn oPawn;
    local BioActorBehavior oBehavior;
    if (nmPawn == 'Player' || nmPawn == 'Self')
    {
        oPawn = Outer.Pawn;
    }
    else
    {
        foreach Outer.AllActors(Class'Pawn', oPawn)
        {
            if (nmPawn == oPawn.Name)
            {
                break;
            }
        }
    }
    if (oPawn == None)
    {
        return;
    }
    oBehavior = BioActorBehavior(oPawn.oBioComponent);
    if (oBehavior == None)
    {
        return;
    }
    oBehavior.EnterCombatStasis(fDuration, bAllowDamage// unknown bytecode 4F ;
}

exec function EnablePowerCooldown(bool bEnable)
{
    local BioRemoteLogger GLogger;
    m_bEnablePowerCooldown = bEnable;
    if (bEnable == false)
    {
        GLogger = Class'BioRemoteLogger'.GetLogger();
        if (GLogger != None)
        {
            GLogger.SendInvalidPlaythrough("EnablePowerCooldown");
        }
    }
}

exec function DumpPlotManagerValueByIndex(int nIndex, String sVariableType)
{
    local BioGlobalVariableTable gv;
    local int nValue;
    local float fValue;
    local bool bValue;
    gv = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    if (sVariableType == "bool")
    {
        bValue = gv.GetBool(nIndex);
    }
    else
    {
        if (sVariableType == "int")
        {
            nValue = gv.GetInt(nIndex);
        }
        else
        {
            if (sVariableType == "float")
            {
                fValue = gv.GetFloat(nIndex);
            }
            else
            {
                bValue = gv.GetBool(nIndex);
                nValue = gv.GetInt(nIndex);
                fValue = gv.GetFloat(nIndex);
            }
        }
    }
}

exec function InitPlotManagerValueByIndex(int nIndex, String sVariableType, float fValue)
{
    local BioGlobalVariableTable gv;
    local int nValue;
    local bool bValue;
    gv = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    if (sVariableType == "bool")
    {
        bValue = true;
        if (fValue == 0)
        {
            bValue = false;
        }
        gv.SetBool(nIndex, bValue);
    }
    else
    {
        if (sVariableType == "int")
        {
            nValue = fValue;
            gv.SetInt(nIndex, nValue);
        }
        else
        {
            if (sVariableType == "float")
            {
                gv.SetFloat(nIndex, fValue);
                jump 229;
            }
        }
    }
    return;
}

exec function TogglePowerImpact()
{
    m_bShowPowerImpactText = !m_bShowPowerImpactText;
}

function DisplayPowerImpactText(BioHUD oHUD)
{
    local Actor oActor;
    local BioActorBehavior oBehavior;
    local Vector vLocation;
    if (oHUD == None || oHUD.Canvas == None)
    {
        return;
    }
    oHUD.Canvas.SetDrawColor(255, 255, 255, 255);
    foreach Outer.AllActors(Class'Actor', oActor)
    {
        oBehavior = BioActorBehavior(oActor.oBioComponent);
        if (oBehavior != None)
        {
            if (oBehavior.m_fPowerAffectingActorTime > 0)
            {
                vLocation = oHUD.Canvas.Project(oActor.Location);
                oHUD.Canvas.SetPos(vLocation.X - 10, vLocation.Y - 10);
                oHUD.Canvas.DrawRect(20, 20);
                oHUD.Canvas.SetPos(vLocation.X - 40, vLocation.Y - 30);
                oHUD.Canvas.DrawText(oBehavior.m_nmPowerAffectingActor);
            }
        }
    }
}

final exec function ToggleEdgeAA(Name nmPanel)
{
    local BioSFManager oManager;
    local BioSFPanel oPanel;
    local bool bAll;
    local int I;
    bAll = nmPanel == 'All';
    oManager = Outer.GetScaleFormManager();
    I = 0;
    while (I < oManager.m_lstPanels.Length)
    {
        oPanel = oManager.m_lstPanels[I];
        if (bAll || nmPanel == oPanel.nmTag)
        {
            oPanel.SetUseEdgeAA(!oPanel.GetUseEdgeAA());
        }
        I++;
    }
}

final exec function SetCurvePixelError(Name nmPanel, float fError)
{
    local BioSFManager oManager;
    local BioSFPanel oPanel;
    local bool bAll;
    local int I;
    bAll = nmPanel == 'All';
    oManager = Outer.GetScaleFormManager();
    I = 0;
    while (I < oManager.m_lstPanels.Length)
    {
        oPanel = oManager.m_lstPanels[I];
        if (bAll || nmPanel == oPanel.nmTag)
        {
            oPanel.SetCurvePixelError(fError);
        }
        I++;
    }
}

exec function SetVehicleExitLog(bool bStatus)
{
    local BioVehicleBase oVeh;
    oVeh = BioVehicleBase(Outer.Pawn);
    if (oVeh != None)
    {
        oVeh.m_bDebugVehicleEnterExitSequence = bStatus;
        jump 61;
    }
    return;
}

exec function TogglePowerList()
{
    m_bShowPowerList = !m_bShowPowerList;
}

exec function ClearPowerList()
{
    m_aPowerList.Length = 0;
}

function DisplayPowerList(BioHUD oHUD)
{
    local float fX;
    local float fY;
    local int nIndex;
    if (oHUD == None || oHUD.Canvas == None)
    {
        return;
    }
    fX = oHUD.Canvas.SizeX - oHUD.Canvas.SizeX * oHUD.SafeAreaRatioX - 400;
    fY = oHUD.Canvas.SizeY * 0,5;
    oHUD.Canvas.SetPos(fX, fY);
    oHUD.Canvas.SetDrawColor(255, 255, 255, 255);
    nIndex = m_aPowerList.Length - 1;
    while (nIndex >= 0)
    {
        oHUD.Canvas.DrawText(m_aPowerList[nIndex]);
        fY -= 15;
        oHUD.Canvas.SetPos(fX, fY);
        nIndex--;
    }
}

event AddPowerInfoToList(String sPowerInfo)
{
    if (m_aPowerList.Length == 10)
    {
        m_aPowerList.Remove(0, 1);
    }
    m_aPowerList[m_aPowerList.Length] = sPowerInfo;
}

exec function GameOver()
{
    MassEffectGuiManager(Outer.GetScaleFormManager()).ShowGameOverGui();
}

exec function ClearDebugLines()
{
    if (Outer.Pawn != None)
    {
        Outer.Pawn.FlushPersistentDebugLines();
    }
}

native exec function Mark(String sComment);

exec function SetVehicleCOMOffsetZ(float fOffsetZ)
{
    local SVehicle oVeh;
    oVeh = SVehicle(Outer.Pawn);
    if (oVeh != None)
    {
        oVeh.COMOffset.Z = fOffsetZ;
        jump 70;
    }
    return;
}

exec function SetAutoRightUpForceAndTorque(float fUpLiftForce, float fUprightTorque)
{
    local SVehicle oVeh;
    oVeh = SVehicle(Outer.Pawn);
    if (oVeh != None)
    {
        oVeh.UprightLiftStrength = fUpLiftForce;
        oVeh.UprightTorqueStrength = fUprightTorque;
        jump 79;
    }
    return;
}

native exec function LogAllGameProperties();

native exec function LogAllInstantGameProperties();

exec function AILog(String nmPawn, bool bEnable)
{
    local BioPawn oPawn;
    local BioAiController oController;
    oPawn = BioPawn(GetActorFromString(nmPawn));
    if (oPawn == None)
    {
        return;
    }
    oController = BioAiController(oPawn.Controller);
    if (oController == None)
    {
        return;
    }
    oController.bAILogging = bEnable;
    oPawn.bWeaponDebug_Accuracy = bEnable;
    if (bEnable)
    {
        jump 129;
    }
    return;
}

exec function AddAIFilter(Name nmFilter)
{
    local BioAiController oController;
    local int nIndex;
    local bool bFound;
    foreach Outer.AllActors(Class'BioAiController', oController)
    {
        if (oController != None)
        {
            bFound = false;
            nIndex = 0;
            while (nIndex < oController.AILogFilter.Length)
            {
                if (oController.AILogFilter[nIndex] == nmFilter)
                {
                    bFound = true;
                    break;
                }
                nIndex++;
            }
            if (bFound == false)
            {
                oController.AILogFilter[oController.AILogFilter.Length] = nmFilter;
            }
        }
    }
}

exec function RemoveAIFilter(Name nmFilter)
{
    local BioAiController oController;
    local int nIndex;
    foreach Outer.AllActors(Class'BioAiController', oController)
    {
        if (oController != None)
        {
            nIndex = 0;
            while (nIndex < oController.AILogFilter.Length)
            {
                if (oController.AILogFilter[nIndex] == nmFilter)
                {
                    oController.AILogFilter.Remove(nIndex, 1);
                    break;
                }
                nIndex++;
            }
        }
    }
}

native exec function AddResourceToRoot(String sName);

exec function DebugVoc()
{
    local BioHUD HUD;
    HUD = BioHUD(Outer.myHUD);
    if (HUD != None && SFXGame(Outer.WorldInfo.Game).VocManager != None)
    {
        HUD.SetDebugDraw(SFXGame(Outer.WorldInfo.Game).VocManager.DebugDraw);
        Outer.ClientMessage("DebugFriction" @ HUD.IsDrawing(Outer.AimAssist.DebugDraw_Friction));
    }
}

exec function DebugZoomSnap()
{
    local BioHUD HUD;
    HUD = BioHUD(Outer.myHUD);
    if (HUD != None && Outer.AimAssist != None)
    {
        HUD.SetDebugDraw(Outer.AimAssist.DebugDraw_ZoomSnap);
        Outer.ClientMessage("DebugZoomSnap" @ HUD.IsDrawing(Outer.AimAssist.DebugDraw_ZoomSnap));
    }
}

exec function DebugHealth()
{
    local BioHUD HUD;
    local BioPawn BP;
    HUD = BioHUD(Outer.myHUD);
    BP = BioPawn(BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController().Pawn);
    if (HUD != None)
    {
        HUD.SetDebugDraw(DebugDraw_Health);
        BP.GetModule(Class'SFXModule_DamagePlayer').bDebugHealthBarEnabled = true;
        Outer.ClientMessage("DebugHealth" @ HUD.IsDrawing(DebugDraw_Health));
    }
}

function DebugDraw_Health(BioHUD HUD)
{
    local BioPawn BP;
    local float fShields;
    local float fHealth;
    local SFXModule_DamagePlayer oDmg;
    BP = BioPawn(Outer.Pawn);
    if (BP == None || BP.m_bInConversation)
    {
        return;
    }
    oDmg = BP.GetModule(Class'SFXModule_DamagePlayer');
    fShields = oDmg.fDebugShieldIndicator;
    fHealth = oDmg.fDebugHealthIndicator;
    if (fShields > 0,01 && fShields < 0,99)
    {
        Outer.myHUD.Canvas.DrawColor = MakeColor(100, 100, 255, 255);
        Outer.myHUD.Canvas.SetPos(Outer.myHUD.Canvas.SizeX * 0,25, Outer.myHUD.Canvas.SizeY * 0,85);
        Outer.myHUD.Canvas.DrawRect(Outer.myHUD.Canvas.SizeX * 0,5 * fShields, Outer.myHUD.Canvas.SizeY * 0,03);
    }
    else
    {
        if (fHealth > 0 && fShields <= 0,01)
        {
            Outer.myHUD.Canvas.DrawColor = MakeColor(255, 0, 0, 255);
            Outer.myHUD.Canvas.SetPos(Outer.myHUD.Canvas.SizeX * 0,25, Outer.myHUD.Canvas.SizeY * 0,85);
            Outer.myHUD.Canvas.DrawRect(Outer.myHUD.Canvas.SizeX * 0,5 * fHealth, Outer.myHUD.Canvas.SizeY * 0,03);
        }
    }
    if (oDmg.fDebugCooldownIndicator > 0,01)
    {
        if (oDmg.fDebugCooldownIndicator > 0,1)
        {
            Outer.myHUD.Canvas.DrawColor = MakeColor(255, 0, 0, 100);
            Outer.myHUD.Canvas.SetPos(Outer.myHUD.Canvas.SizeX * 0,5 - 0,0025 - 0,002 - oDmg.fDebugCooldownIndicator * 0,025, Outer.myHUD.Canvas.SizeY * 0,485);
            Outer.myHUD.Canvas.DrawRect(Outer.myHUD.Canvas.SizeX * 0,002, Outer.myHUD.Canvas.SizeY * 0,03);
            Outer.myHUD.Canvas.SetPos(Outer.myHUD.Canvas.SizeX * 0,5 - 0,0025 + 0,002 + oDmg.fDebugCooldownIndicator * 0,025, Outer.myHUD.Canvas.SizeY * 0,485);
            Outer.myHUD.Canvas.DrawRect(Outer.myHUD.Canvas.SizeX * 0,002, Outer.myHUD.Canvas.SizeY * 0,03);
        }
        else
        {
            Outer.myHUD.Canvas.DrawColor = MakeColor(0, 255, 0, 100);
            Outer.myHUD.Canvas.SetPos(Outer.myHUD.Canvas.SizeX * 0,5 - Outer.myHUD.Canvas.SizeY * 0,015, Outer.myHUD.Canvas.SizeY * 0,5 - 0,015);
            Outer.myHUD.Canvas.DrawRect(Outer.myHUD.Canvas.SizeY * 0,03, Outer.myHUD.Canvas.SizeY * 0,03);
        }
    }
}

exec function DebugBioticCharge()
{
    local BioHUD HUD;
    local SFXPower Power;
    local BioPowerScript Script;
    HUD = BioHUD(Outer.myHUD);
    Power = BioPawn(Outer.Pawn).PowerManager.GetPower('BioticCharge');
    if (HUD != None && BioPawn(Outer.Pawn).PowerManager.GetPower('BioticCharge') != None)
    {
        Script = Power.GetPowerScript();
        if (Script != None)
        {
            HUD.SetDebugDraw(Script.DebugDraw_Power);
            Outer.ClientMessage("DebugBioticCharge" @ HUD.IsDrawing(Script.DebugDraw_Power));
            return;
        }
    }
    Outer.ClientMessage("Couldn't debug biotic charge.");
}

exec function DebugAdhesion()
{
    local BioHUD HUD;
    HUD = BioHUD(Outer.myHUD);
    if (HUD != None && Outer.AimAssist != None)
    {
        HUD.SetDebugDraw(Outer.AimAssist.DebugDraw_Adhesion);
        Outer.ClientMessage("DebugAdhesion" @ HUD.IsDrawing(Outer.AimAssist.DebugDraw_Adhesion));
    }
}

exec function DebugFriction()
{
    local BioHUD HUD;
    HUD = BioHUD(Outer.myHUD);
    if (HUD != None && Outer.AimAssist != None)
    {
        HUD.SetDebugDraw(Outer.AimAssist.DebugDraw_Friction);
        Outer.ClientMessage("DebugFriction" @ HUD.IsDrawing(Outer.AimAssist.DebugDraw_Friction));
    }
}

exec function DebugWeapon()
{
    if (BioPawn(Outer.Pawn) != None)
    {
        BioPawn(Outer.Pawn).bWeaponDebug_Accuracy = !BioPawn(Outer.Pawn).bWeaponDebug_Accuracy;
        Outer.ClientMessage("DebugWeapon" @ BioPawn(Outer.Pawn).bWeaponDebug_Accuracy);
    }
}

exec function DebugAI()
{
    local BioHUD HUD;
    HUD = BioHUD(Outer.myHUD);
    if (HUD != None)
    {
        HUD.SetDebugDraw(DebugDraw_AI);
        Outer.ClientMessage("DebugAI" @ HUD.IsDrawing(DebugDraw_AI));
    }
}

function DebugDraw_AI(BioHUD HUD)
{
    local Vector CamLoc;
    local Rotator CamRot;
    local BioAiController AI;
    HUD.Canvas.SetDrawColor(255, 255, 255);
    Outer.GetPlayerViewPoint(CamLoc, CamRot);
    foreach Outer.WorldInfo.AllControllers(Class'BioAiController', AI)
    {
        if (AI.Pawn != None && AI.Pawn.Location - CamLoc Dot CamRot > 0)
        {
            AI.DrawDebug(HUD);
        }
    }
}

exec function DebugLOD()
{
    local BioHUD HUD;
    HUD = BioHUD(Outer.myHUD);
    if (HUD != None)
    {
        HUD.SetDebugDraw(DebugDraw_LOD);
        Outer.ClientMessage("DebugLOD" @ HUD.IsDrawing(DebugDraw_LOD));
    }
}

function DebugDraw_LOD(BioHUD HUD)
{
    local Vector CamLoc;
    local Vector RenderLoc;
    local Rotator CamRot;
    local Actor chkActor;
    local Pawn ChkPawn;
    local SkeletalMeshActor SMA;
    local SkeletalMeshComponent Mesh;
    local float X;
    local float Y;
    local float YL;
    local Vector ScreenCoords;
    HUD.Canvas.SetDrawColor(255, 255, 255);
    Outer.GetPlayerViewPoint(CamLoc, CamRot);
    foreach Outer.DynamicActors(Class'Actor', chkActor)
    {
        Mesh = None;
        ChkPawn = Pawn(chkActor);
        if (ChkPawn != None && ChkPawn.Location - CamLoc Dot CamRot > 0 && ChkPawn.TimeSince(ChkPawn.LastRenderTime) < 1)
        {
            if (BioPawn(ChkPawn) == None || BioPawn(ChkPawn).m_oBehavior.bActive)
            {
                RenderLoc = ChkPawn.Location;
                Mesh = ChkPawn.Mesh;
            }
        }
        if (Mesh == None)
        {
            SMA = SkeletalMeshActor(chkActor);
            if (SMA != None && SMA.Location - CamLoc Dot CamRot > 0 && SMA.TimeSince(SMA.LastRenderTime) < 1)
            {
                RenderLoc = SMA.Location;
                RenderLoc.Z += SMA.SkeletalMeshComponent.Bounds.BoxExtent.Z;
                Mesh = SMA.SkeletalMeshComponent;
            }
        }
        if (Mesh != None)
        {
            X = HUD.Canvas.CurX;
            Y = HUD.Canvas.CurY;
            YL = HUD.Canvas.CurYL;
            ScreenCoords = HUD.Canvas.Project(RenderLoc);
            HUD.Canvas.CurX = ScreenCoords.X;
            HUD.Canvas.CurY = ScreenCoords.Y;
            HUD.Canvas.DrawText("Actor:" @ chkActor, true);
            HUD.Canvas.DrawText("FoLod:" @ Mesh.ForcedLodModel, true);
            HUD.Canvas.DrawText("MinLOD:" @ Mesh.MinAutoLODLevel, true);
            HUD.Canvas.DrawText("CurLOD:" @ Mesh.PredictedLODLevel);
            HUD.Canvas.CurX = X;
            HUD.Canvas.CurY = Y;
            HUD.Canvas.CurYL = YL;
        }
    }
}

exec function DebugDistance()
{
    local BioHUD HUD;
    HUD = BioHUD(Outer.myHUD);
    if (HUD != None)
    {
        HUD.SetDebugDraw(DebugDraw_Distance);
        Outer.ClientMessage("DebugDistance" @ HUD.IsDrawing(DebugDraw_Distance));
    }
}

function DebugDraw_Distance(BioHUD HUD)
{
    local Vector CamLoc;
    local Vector RenderLoc;
    local Rotator CamRot;
    local Actor chkActor;
    local Pawn ChkPawn;
    local SkeletalMeshComponent Mesh;
    local float X;
    local float Y;
    local float YL;
    local Vector ScreenCoords;
    HUD.Canvas.SetDrawColor(255, 255, 255);
    Outer.GetPlayerViewPoint(CamLoc, CamRot);
    foreach Outer.DynamicActors(Class'Actor', chkActor)
    {
        Mesh = None;
        ChkPawn = Pawn(chkActor);
        if (ChkPawn != None && ChkPawn.Location - CamLoc Dot CamRot > 0 && ChkPawn.TimeSince(ChkPawn.LastRenderTime) < 1)
        {
            if (BioPawn(ChkPawn) == None || BioPawn(ChkPawn).m_oBehavior.bActive)
            {
                RenderLoc = ChkPawn.Location;
                Mesh = ChkPawn.Mesh;
            }
        }
        if (Mesh != None)
        {
            X = HUD.Canvas.CurX;
            Y = HUD.Canvas.CurY;
            YL = HUD.Canvas.CurYL;
            ScreenCoords = HUD.Canvas.Project(RenderLoc);
            HUD.Canvas.CurX = ScreenCoords.X;
            HUD.Canvas.CurY = ScreenCoords.Y;
            HUD.Canvas.DrawText("Actor:" @ chkActor, true);
            HUD.Canvas.DrawText("Distance:" @ PrettyFloat(VSize(chkActor.Location - Outer.Pawn.Location) / 100, 2) @ "m");
            HUD.Canvas.CurX = X;
            HUD.Canvas.CurY = Y;
            HUD.Canvas.CurYL = YL;
        }
    }
}

exec function DebugAmmo()
{
    local BioHUD HUD;
    HUD = BioHUD(Outer.myHUD);
    if (HUD != None)
    {
        HUD.SetDebugDraw(DebugDraw_Ammo);
        Outer.ClientMessage("DebugAmmo" @ HUD.IsDrawing(DebugDraw_Ammo));
    }
}

function DebugDraw_Ammo(BioHUD HUD)
{
    local float OldX;
    local float OldY;
    local Font OldFont;
    local SFXWeapon Weapon;
    OldX = HUD.Canvas.CurX;
    OldY = HUD.Canvas.CurY;
    OldFont = HUD.Canvas.Font;
    Weapon = SFXWeapon(Outer.Pawn.Weapon);
    HUD.Canvas.SetDrawColor(255, 255, 255);
    HUD.Canvas.Font = Class'Engine'.GetSmallFont();
    HUD.Canvas.CurX = 340;
    HUD.Canvas.CurY = 640;
    if (Weapon != None)
    {
        HUD.Canvas.DrawText("GetCurrentTotalAmmo():" @ Weapon.GetCurrentTotalAmmo());
        HUD.Canvas.DrawText("AmmoUsedCount:" @ Weapon.AmmoUsedCount);
        HUD.Canvas.DrawText("GetCurrentSpareAmmo():" @ Weapon.GetCurrentSpareAmmo());
        HUD.Canvas.DrawText("GetAmmoCountInMagazine():" @ Weapon.GetAmmoCountInMagazine());
        HUD.Canvas.DrawText("GetMagazineSize():" @ Weapon.GetMagazineSize());
    }
    else
    {
        HUD.Canvas.DrawText("Total Ammo: (no weapon)");
    }
    HUD.Canvas.Font = OldFont;
    HUD.Canvas.CurX = OldX;
    HUD.Canvas.CurY = OldY;
}

exec function DebugMoveToCover(Name nmPawn, Name nmLink, int nSlot)
{
    local Pawn oPawn;
    local SFXAI_Core oController;
    local CoverLink oLink;
    local CoverInfo oCoverInfo;
    foreach Outer.AllActors(Class'Pawn', oPawn)
    {
        if (nmPawn == oPawn.Name)
        {
            break;
        }
    }
    if (oPawn == None)
    {
        return;
    }
    oController = SFXAI_Core(oPawn.Controller);
    if (oController == None)
    {
        return;
    }
    foreach Outer.AllActors(Class'CoverLink', oLink)
    {
        if (nmLink == oLink.Name)
        {
            break;
        }
    }
    if (oLink == None)
    {
        return;
    }
    if (nSlot >= oLink.Slots.Length)
    {
        return;
    }
    oCoverInfo.Link = oLink;
    oCoverInfo.SlotIdx = nSlot;
    if (oController.DebugMoveToCover(oCoverInfo))
    {
        jump 273;
    }
    return;
}

exec function DebugMoveToGoal(Name nmPawn, Name nmGoalActor, optional float fOffsetDistance)
{
    local Pawn oPawn;
    local SFXAI_Core oController;
    local Actor oGoalActor;
    foreach Outer.AllActors(Class'Pawn', oPawn)
    {
        if (nmPawn == oPawn.Name)
        {
            break;
        }
    }
    if (oPawn == None)
    {
        return;
    }
    oController = SFXAI_Core(oPawn.Controller);
    if (oController == None)
    {
        return;
    }
    foreach Outer.AllActors(Class'Actor', oGoalActor)
    {
        if (nmGoalActor == oGoalActor.Name)
        {
            break;
        }
    }
    if (oGoalActor == None)
    {
        return;
    }
    if (oController.DebugMoveToGoal(oGoalActor, fOffsetDistance))
    {
        jump 208;
    }
    return;
}

exec function SetPowerPercent(Name nmPawn, float fPercent)
{
    local Pawn oPawn;
    local BioPawnBehavior oBehavior;
    foreach Outer.AllActors(Class'Pawn', oPawn)
    {
        if (nmPawn == oPawn.Name)
        {
            break;
        }
    }
    if (oPawn == None)
    {
        return;
    }
    oBehavior = BioPawnBehavior(oPawn.oBioComponent);
    if (oBehavior == None)
    {
        return;
    }
    if (fPercent < 0 || fPercent > 1)
    {
        return;
    }
    oBehavior.m_bPowerUsePercentOverride = true;
    oBehavior.m_fPowerUsePercent = fPercent;
}

exec function EnableAI(Name nmPawn, bool bEnable)
{
    local Pawn oPawn;
    local SFXAI_Core oController;
    if (nmPawn == 'All')
    {
        EnableAllAI(bEnable);
        return;
    }
    foreach Outer.AllActors(Class'Pawn', oPawn)
    {
        if (nmPawn == oPawn.Name)
        {
            break;
        }
    }
    if (oPawn == None)
    {
        return;
    }
    oController = SFXAI_Core(oPawn.Controller);
    if (oController == None)
    {
        return;
    }
    oController.EnableAI(bEnable, 1);
    if (bEnable)
    {
        jump 180;
    }
    return;
}

exec function EnableAllAI(bool bEnable)
{
    local SFXAI_Core oController;
    local BioRemoteLogger GLogger;
    foreach Outer.AllActors(Class'SFXAI_Core', oController)
    {
        if (oController != None)
        {
            oController.EnableAI(bEnable, 1);
        }
    }
    if (bEnable)
    {
    }
    else
    {
        GLogger = Class'BioRemoteLogger'.GetLogger();
        if (GLogger != None)
        {
            GLogger.SendInvalidPlaythrough("EnableAllAI");
        }
    }
}

exec function Distance(String nmActor1, String nmActor2)
{
    local Actor oActor1;
    local Actor oActor2;
    oActor1 = GetActorFromString(nmActor1);
    if (oActor1 == None)
    {
        return;
    }
    oActor2 = GetActorFromString(nmActor2);
    if (oActor2 == None)
    {
        return;
    }
    Outer.ClientMessage("The distance is " $ VSize(oActor1.Location - oActor2.Location));
    Outer.DrawDebugLine(oActor1.Location, oActor2.Location, 0, 0, 255, true);
}

exec function AwardTreasure(int nTreasureId)
{
    local SFXPlotTreasure oTreasure;
    oTreasure = BioWorldInfo(Outer.WorldInfo).m_oTreasure;
    oTreasure.AwardTreasure(nTreasureId, true);
}

exec function SetIntByName(Name nmVar, int nValue)
{
    local BioGlobalVariableTable oGV;
    local SFXPlotTreasure oTreasure;
    local STech stTech;
    oGV = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    oTreasure = BioWorldInfo(Outer.WorldInfo).m_oTreasure;
    if (oGV != None && nmVar != 'None')
    {
        stTech = oTreasure.tech(nmVar);
        if (stTech.nmTech != 'None')
        {
            oGV.SetIntByName(nmVar, nValue - 1);
            oTreasure.AwardTech(nmVar);
        }
        else
        {
            oGV.SetIntByName(nmVar, nValue);
        }
        SFXGame(BioWorldInfo(Outer.WorldInfo).Game).BonusList.AwardTechBonus(nmVar, BioWorldInfo(Outer.WorldInfo));
    }
}

exec function SetBoolByName(Name nmVar, bool nValue)
{
    local BioGlobalVariableTable oGV;
    oGV = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    if (oGV != None && nmVar != 'None')
    {
        oGV.SetBoolByName(nmVar, nValue);
    }
}

exec function SetBoolByID(int PlotID, bool nValue)
{
    local BioGlobalVariableTable oGV;
    oGV = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    if (oGV != None)
    {
        oGV.SetBool(PlotID, nValue);
    }
}

exec function Playpens()
{
    local BioPlaypenVolume oPlaypen;
    local int nRed;
    local int nGreen;
    local Vector vOrigin;
    local Vector vExtent;
    local Vector vStart;
    local Vector vEnd;
    foreach Outer.AllActors(Class'BioPlaypenVolume', oPlaypen)
    {
        if (oPlaypen != None && oPlaypen.CollisionComponent != None)
        {
            if (oPlaypen.bSubtractive)
            {
                nRed = 255;
                nGreen = 0;
            }
            else
            {
                nRed = 0;
                nGreen = 255;
            }
            vOrigin = oPlaypen.CollisionComponent.Bounds.Origin;
            vExtent = oPlaypen.CollisionComponent.Bounds.BoxExtent;
            Outer.DrawDebugBox(vOrigin, vExtent, nRed, nGreen, 0, true);
            vStart = vOrigin;
            vStart.X -= 10;
            vEnd = vOrigin;
            vEnd.X += 10;
            Outer.DrawDebugLine(vStart, vEnd, nRed, nGreen, 0, true);
            vStart = vOrigin;
            vStart.Y -= 10;
            vEnd = vOrigin;
            vEnd.Y += 10;
            Outer.DrawDebugLine(vStart, vEnd, nRed, nGreen, 0, true);
            vStart = vOrigin;
            vStart.Z -= 10;
            vEnd = vOrigin;
            vEnd.Z += 10;
            Outer.DrawDebugLine(vStart, vEnd, nRed, nGreen, 0, true);
        }
    }
}

exec function Actors(class<Actor> ActorClass)
{
    local Actor TheActor;
    local Vector vOrigin;
    local Vector vExtent;
    local Vector vStart;
    local Vector vEnd;
    foreach Outer.AllActors(ActorClass, TheActor)
    {
        if (TheActor != None && TheActor.CollisionComponent != None)
        {
            vOrigin = TheActor.CollisionComponent.Bounds.Origin;
            vExtent = TheActor.CollisionComponent.Bounds.BoxExtent;
            Outer.DrawDebugBox(vOrigin, vExtent, 0, 255, 0, true);
            vStart = vOrigin;
            vStart.X -= 10;
            vEnd = vOrigin;
            vEnd.X += 10;
            Outer.DrawDebugLine(vStart, vEnd, 0, 255, 0, true);
            vStart = vOrigin;
            vStart.Y -= 10;
            vEnd = vOrigin;
            vEnd.Y += 10;
            Outer.DrawDebugLine(vStart, vEnd, 0, 255, 0, true);
            vStart = vOrigin;
            vStart.Z -= 10;
            vEnd = vOrigin;
            vEnd.Z += 10;
            Outer.DrawDebugLine(vStart, vEnd, 0, 255, 0, true);
        }
    }
}

function String PrettyFloat(float F, optional int decimals = 1)
{
    return Left(F, Len(F) - 4 + decimals);
}

function ProfileDamage()
{
    local BioPawn Pawn;
    local SFXModule_Damage Damage;
    Pawn = BioPawn(ProfileTarget);
    if (Pawn == None)
    {
        return;
    }
    Damage = Pawn.GetModule(Class'SFXModule_Damage');
    if (Damage != None)
    {
        DrawProfileHeaderText("Head: ");
        ShowBodyDamage(Pawn, 2);
        DrawProfileHeaderText("Right Arm: ");
        ShowBodyDamage(Pawn, 6);
        DrawProfileHeaderText("Left Arm: ");
        ShowBodyDamage(Pawn, 5);
        DrawProfileHeaderText("Right Leg: ");
        ShowBodyDamage(Pawn, 4);
        DrawProfileHeaderText("Left Leg: ");
        ShowBodyDamage(Pawn, 3);
        DrawProfileHeaderText("Torso: ");
        ShowBodyDamage(Pawn, 7);
    }
}

function ShowBodyDamage(BioPawn Pawn, EBioPartGroup Group)
{
    local RB_BodySetup Body;
    local SFXModule_Damage Damage;
    Damage = Pawn.GetModule(Class'SFXModule_Damage');
    foreach Pawn.Mesh.PhysicsAsset.BodySetup(Body)
    {
        if (Body.ePartGroup == Group)
        {
            if (Body.bPartBasedDamageEnabled)
            {
                DrawProfileHeaderText("Part based damage scale: ");
                DrawProfileText(Body.fDamageScale);
            }
            if (Body.bDetachable)
            {
                DrawProfileHeaderText("DamageApplied: ");
                DrawProfileText(Damage.DamageParts[Group].Damage);
                DrawProfileHeaderText("Detach Amount: ");
                DrawProfileText(1 - Body.fDetachHPPercent * Pawn.GetMaxHealth());
                DrawProfileHeaderText("Detach Threshold: ");
                DrawProfileText(1 - Body.fDetachHPThresholdPercent * Pawn.GetMaxHealth());
            }
        }
    }
}

exec function EmptyInventory()
{
    local BioPawn PlayerPawn;
    local SFXInventoryManager oInventory;
    PlayerPawn = SFXPawn_Player(Outer.Pawn);
    if (PlayerPawn == None)
    {
        return;
    }
    oInventory = SFXInventoryManager(Outer.Pawn.InvManager);
    oInventory.EmptyInventory();
}

function ProfileSysDesign()
{
    local SFXPRI PRI;
    local SFXGame Game;
    local int ChallengeLevel;
    local float ChallengeModifier;
    local BioGlobalVariableTable VariableTable;
    local BioWorldInfo oWorldInfo;
    local SFXPlotTreasure oTreasure;
    local SResourceBudget Budget;
    local TreasureBudget stLevelBudget;
    local SFXModule_DamagePlayer oDmg;
    local BioPawn BP;
    BP = BioPawn(Outer.Pawn);
    if (BP == None || BP.m_bInConversation)
    {
        return;
    }
    PRI = Class'SFXPRI'.FindLocalPRI();
    oWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    Game = SFXGame(oWorldInfo.Game);
    Game.GetChallengeLevel(0, ChallengeLevel);
    Game.GetChallengeModifier(0, ChallengeModifier);
    if (PRI.DifficultyScore > 0)
    {
        DrawProfileText("Difficulty: " $ SFXGame(oWorldInfo.Game).Difficulty $ " / " @ PRI.DifficultyScore);
        DrawProfileText(" ");
    }
    if (BP.GetMaxShields() > 0)
    {
        DrawProfileHeaderText("Shields: ");
        DrawProfileText(PrettyFloat(100 * BP.GetCurrentShields() / BP.GetMaxShields(), 1) $ "%");
    }
    if (BP.GetMaxHealth() > 0)
    {
        DrawProfileHeaderText("Health: ");
        DrawProfileText(PrettyFloat(100 * BP.GetCurrentHealth() / BP.GetMaxHealth(), 1) $ "%");
    }
    VariableTable = oWorldInfo.GetGlobalVariables();
    if (VariableTable == None)
    {
        return;
    }
    oTreasure = oWorldInfo.m_oTreasure;
    Budget = oTreasure.Budget();
    stLevelBudget = SFXGame(oWorldInfo.Game).gameconfig.GetLevelBudget();
    if (stLevelBudget.LevelId != Budget.nID)
    {
        SFXGame(oWorldInfo.Game).gameconfig.RecordTreasure(0, 0, Budget.nID);
    }
    DrawProfileHeaderText("  Level Id: ");
    DrawProfileText(Budget.nID);
    if (Budget.nCredits > 0 || stLevelBudget.Credits > 0)
    {
        DrawProfileHeaderText("  Level Credits: ");
        Budget.nCredits = Budget.nCredits > 0 ? Budget.nCredits : 1;
        DrawProfileText(stLevelBudget.Credits $ " of " $ Budget.nCredits $ " (" $ stLevelBudget.Credits * 100 / Budget.nCredits $ "%)");
    }
    if (Budget.nIridium > 0 || stLevelBudget.Iridium > 0)
    {
        DrawProfileHeaderText("  Level Iridium: ");
        Budget.nIridium = Budget.nIridium > 0 ? Budget.nIridium : 1;
        DrawProfileText(stLevelBudget.Iridium $ " of " $ Budget.nIridium $ " (" $ stLevelBudget.Iridium * 100 / Budget.nIridium $ "%)");
    }
    if (Budget.nPlatinum > 0 || stLevelBudget.Platinum > 0)
    {
        DrawProfileHeaderText("  Level Platinum: ");
        Budget.nPlatinum = Budget.nPlatinum > 0 ? Budget.nPlatinum : 1;
        DrawProfileText(stLevelBudget.Platinum $ " of " $ Budget.nPlatinum $ " (" $ stLevelBudget.Platinum * 100 / Budget.nPlatinum $ "%)");
    }
    if (Budget.nPalladium > 0 || stLevelBudget.Palladium > 0)
    {
        DrawProfileHeaderText("  Level Palladium: ");
        Budget.nPalladium = Budget.nPalladium > 0 ? Budget.nPalladium : 1;
        DrawProfileText(stLevelBudget.Palladium $ " of " $ Budget.nPalladium $ " (" $ stLevelBudget.Palladium * 100 / Budget.nPalladium $ "%)");
    }
    if (Budget.nEezo > 0 || stLevelBudget.Eezo > 0)
    {
        DrawProfileHeaderText("  Level Eezo: ");
        Budget.nEezo = Budget.nEezo > 0 ? Budget.nEezo : 1;
        DrawProfileText(stLevelBudget.Eezo $ " of " $ Budget.nEezo $ " (" $ stLevelBudget.Eezo * 100 / Budget.nEezo $ "%)");
    }
    if (SFXGame(oWorldInfo.Game).gameconfig.bLastDamageUpdate)
    {
        DrawProfileHeaderText("Last Bullet:");
        DrawProfileText(SFXGame(oWorldInfo.Game).gameconfig.nLastDamage);
    }
    oDmg = SFXModule_DamagePlayer(BP.GetModule(Class'SFXModule_Damage'));
    if (oDmg != None && BP.TimeSince(oDmg.BleedOutStartTime) < SFXGame(oWorldInfo.Game).Difficulty.Default.BleedOutInterval)
    {
        DrawProfileHeaderText("BLEEDOUT:");
        DrawProfileText("PROTECTED - " $ BP.TimeSince(oDmg.BleedOutStartTime));
    }
    if (BP.GetModule(Class'SFXModule_DamagePlayer').bDebugHealthBarEnabled == false)
    {
        DebugHealth();
    }
}

exec function LockDifficulty(float fDifficulty)
{
    local SFXPRI PRI;
    PRI = Class'SFXPRI'.FindLocalPRI();
    PRI.LockedScore = fDifficulty;
}

function ProfileCombat()
{
    local BioPawn Pawn;
    local Actor Target;
    local BioAttributesPawn Attributes;
    local SFXWeapon Weapon;
    local SFXWeapon TargetWeapon;
    local SFXArmor_Base Armor;
    local SFXShield_Base Shield;
    local SFXLoadoutData Loadout;
    local class<SFXWeapon> WeaponClass;
    local SFXInventoryManager InvManager;
    local SFXAI_Core AI;
    local BioAiController SquadAI;
    local EnemyInfo EnemyInfo;
    local int idx;
    local NavigationPoint Nav;
    local SFXModule_Damage DmgModule;
    local class<SFXDamageType> DamageType;
    local float F;
    Pawn = BioPawn(ProfileTarget);
    if (Pawn == None)
    {
        return;
    }
    Loadout = Pawn.GetPawnLoadout();
    if (BioAiController(Pawn.Controller) != None)
    {
        Target = BioAiController(Pawn.Controller).FireTarget;
    }
    if (BioPlayerController(Pawn.Controller) != None)
    {
        Target = BioPlayerController(Pawn.Controller).m_oPlayerSelection.m_oCurrentSelectionTarget;
    }
    DrawProfileHeaderText("Combat State: ");
    if (SFXGame(Outer.WorldInfo.Game).bInCombat || SFXGame(Outer.WorldInfo.Game).bForceCombat)
    {
        DrawProfileText("In Combat");
    }
    else
    {
        DrawProfileText("Out Of Combat ");
    }
    Attributes = BioAttributesPawn(Pawn.m_oBehavior.m_PawnAttributes);
    DrawProfileHeaderText("Actor Type: ");
    DrawProfileText(Pawn.PawnType.Name);
    DmgModule = Pawn.GetModule(Class'SFXModule_Damage');
    DrawProfileHeaderText("Level: ");
    DrawProfileText(Pawn.CharacterLevel);
    DrawProfileHeaderText("Controller: ");
    DrawProfileText(Pawn.Controller);
    DrawProfileHeaderText("In Combat: ");
    DrawProfileText(Pawn.InCombat());
    DrawProfileHeaderText("Anchor: ");
    DrawProfileText(Pawn.Anchor);
    DrawProfileHeaderText("Latent Action: ");
    DrawProfileText(GetLatentAction(Pawn.Controller));
    DrawProfileHeaderText("Physics: ");
    DrawProfileText(Pawn.Physics);
    DrawProfileHeaderText("Action State: ");
    DrawProfileText(Pawn.ActionState);
    DrawProfileHeaderText("Port Arms: ");
    DrawProfileText(Pawn.bInPortArms);
    DrawProfileText(" ");
    DrawProfileHeaderText("Focus: ");
    DrawProfileText(Pawn.Controller != None ? Pawn.Controller.Focus : None);
    DrawProfileHeaderText("FireTarget:");
    DrawProfileText(Target);
    DrawProfileHeaderText("Target Distance: ");
    DrawProfileText(Target != None ? VSize(Target.Location - Pawn.Location) : -1);
    AI = SFXAI_Core(Pawn.Controller);
    if (AI != None)
    {
        DrawProfileHeaderText("ForcedTarget:");
        DrawProfileText(AI.ForcedTarget);
        DrawProfileHeaderText("PreferredTarget:");
        DrawProfileText(AI.PreferredTarget);
        DrawProfileText(" ");
        DrawProfileHeaderText("Sight Radius: ");
        DrawProfileText(PrettyFloat(Pawn.SightRadius));
        DrawProfileHeaderText("Hearing Radius: ");
        DrawProfileText(PrettyFloat(Pawn.HearingThreshold));
        foreach AI.EnemyList(EnemyInfo)
        {
            DrawProfileHeaderText("EnemyList: ");
            DrawProfileText("Pawn:" @ EnemyInfo.Pawn @ "Visible:" @ EnemyInfo.bVisible);
        }
    }
    DrawProfileText(" ");
    DrawProfileHeaderText("Is Dead: ");
    DrawProfileText(Pawn.IsDead());
    DrawProfileHeaderText("Current Health: ");
    DrawProfileText(PrettyFloat(Pawn.GetCurrentHealth()) @ "/" @ PrettyFloat(Pawn.GetMaxHealth()));
    DrawProfileHeaderText("Health Regen: ");
    DrawProfileText(PrettyFloat(Pawn.GetRegenRate()));
    DrawProfileText(" ");
    DrawProfileHeaderText("Shields: ");
    DrawProfileText(PrettyFloat(Pawn.GetCurrentShields(), 4) @ "/" @ PrettyFloat(Pawn.GetMaxShields()));
    DrawProfileHeaderText("Shield Regen Delay: ");
    DrawProfileText(PrettyFloat(Pawn.GetShieldRegenDelay()));
    DrawProfileHeaderText("Shield Recharge Time: ");
    DrawProfileText(PrettyFloat(Pawn.GetShieldRechargeTime()));
    DrawProfileText(" ");
    DrawProfileHeaderText("Damage & Duration Multiplier: ");
    DrawProfileText(Attributes != None ? Attributes.GetCurrentFloatValue(Attributes.m_DamageDurationMult) : "-");
    DrawProfileHeaderText("Damage Taken Scaled: ");
    DrawProfileText(DmgModule != None ? DmgModule.DamageScale : "-");
    DrawProfileText(" ");
    DrawProfileHeaderText("Physics Level: ");
    DrawProfileText(Pawn.PawnType != None ? Pawn.PawnType.m_nPhysicsLevel : "-");
    SetProfileColumn(++CurrentColumn);
    Weapon = SFXWeapon(Pawn != None ? Pawn.Weapon : None);
    TargetWeapon = SFXWeapon(BioPawn(Target) != None ? BioPawn(Target).Weapon : None);
    DrawProfileHeaderText("Current Weapon: ");
    DrawProfileText(Weapon != None ? Weapon.Class : "None");
    DrawProfileHeaderText("Weapon Level: ");
    DrawProfileText(Weapon != None ? PrettyFloat(Weapon.WeaponLevel) : "-");
    DrawProfileHeaderText("Weapon Damage: ");
    DrawProfileText(Weapon != None ? PrettyFloat(Weapon.GetFireModeBaseDamage()) : "-");
    DrawProfileHeaderText("  Multiplier: ");
    if (Target != None && Weapon != None)
    {
        F = 1;
        Weapon.ModifyDamage(F, Target.Location, Target);
        DrawProfileText("x" $ PrettyFloat(F, 4));
    }
    else
    {
        DrawProfileText(" - ");
    }
    DamageType = Weapon != None ? Weapon.GetDamageType(0) : None;
    InvManager = BioPawn(Target) != None ? SFXInventoryManager(BioPawn(Target).InvManager) : None;
    if (InvManager != None)
    {
        foreach InvManager.InventoryActors(Class'SFXArmor_Base', Armor)
        {
            DrawProfileHeaderText("  Target Armor: ");
            DrawProfileText("" $ Armor.Class);
            DrawProfileHeaderText("  Penetration(Target Armor): ");
            DrawProfileText(PrettyFloat(Armor.GetDamageResistance(DamageType) * 100) $ "%");
        }
        foreach InvManager.InventoryActors(Class'SFXShield_Base', Shield)
        {
            DrawProfileHeaderText("  Target Shield: ");
            DrawProfileText("" $ Shield.Class);
            DrawProfileHeaderText("  Penetration(Target Shield): ");
            DrawProfileText(PrettyFloat(Shield.GetDamageResistance(DamageType) * 100) $ "%");
        }
    }
    else
    {
        DrawProfileHeaderText("  Penetration(vs Target): ");
        DrawProfileText(" - ");
    }
    DrawProfileHeaderText("Rate Of Fire: ");
    DrawProfileText(Weapon != None ? Weapon.GetRateOfFire() : "-");
    DrawProfileHeaderText("Magazine Size: ");
    DrawProfileText(Weapon != None ? Weapon.GetMagazineSize() : "-");
    DrawProfileHeaderText("Infinite Ammo: ");
    DrawProfileText(Weapon != None ? Weapon.bInfiniteAmmo : "-");
    DrawProfileHeaderText("Aim Error: ");
    if (Weapon != None)
    {
        DrawProfileText(PrettyFloat(Weapon.GetWeaponAimErrorRange().X) @ "-" @ PrettyFloat(Weapon.GetWeaponAimErrorRange().Y));
    }
    else
    {
        DrawProfileText("-");
    }
    DrawProfileText(" ");
    InvManager = SFXInventoryManager(Pawn.InvManager);
    if (InvManager != None)
    {
        DamageType = None;
        if (TargetWeapon != None)
        {
            DamageType = TargetWeapon.GetDamageType(0);
        }
        foreach InvManager.InventoryActors(Class'SFXArmor_Base', Armor)
        {
            DrawProfileHeaderText("Current Armor: ");
            DrawProfileText(Armor);
            DrawProfileHeaderText("  Fortified: ");
            DrawProfileText(Armor.bShowFortifyOnGUI);
            DrawProfileHeaderText("  Damage Resistance (vs Target): ");
            DrawProfileText(PrettyFloat(Armor.GetDamageResistance(DamageType) * 100) $ "%");
        }
        foreach InvManager.InventoryActors(Class'SFXShield_Base', Shield)
        {
            DrawProfileHeaderText("Current Shield: ");
            DrawProfileText(Shield);
            DrawProfileHeaderText("  Shield Strength: ");
            DrawProfileText(Shield.CurrentShields $ " / " $ Shield.MaxShields.Value);
            DrawProfileHeaderText("  Hardened: ");
            DrawProfileText(Shield.bHardened);
            DrawProfileHeaderText("  Damage Resistance (vs Target): ");
            DrawProfileText(PrettyFloat(Shield.GetDamageResistance(DamageType) * 100) $ "%");
        }
    }
    DrawProfileText(" ");
    DrawProfileHeaderText("Loadout: ");
    DrawProfileText(Loadout);
    if (Loadout != None)
    {
        foreach Loadout.Weapons(WeaponClass)
        {
            DrawProfileHeaderText("Loadout Weapon: ");
            DrawProfileText(WeaponClass);
        }
    }
    DrawProfileText(" ");
    if (InvManager != None)
    {
        foreach InvManager.InventoryActors(Class'SFXWeapon', Weapon)
        {
            if (Weapon == Pawn.Weapon)
            {
                DrawProfileHeaderText("Inventory Weapon: -> ");
            }
            else
            {
                DrawProfileHeaderText("Inventory Weapon: ");
            }
            DrawProfileText(Weapon.Class);
        }
        DrawProfileText("  ");
    }
    DrawProfileText(" ");
    DrawProfileHeaderText("Squad: ");
    if (Pawn.Squad != None)
    {
        DrawProfileText(Pawn.Squad);
        DrawProfileHeaderText("Has Playpen: ");
        DrawProfileText(Pawn.Squad.HasPlaypen());
        foreach Pawn.Squad.SquadMembers(SquadAI)
        {
            DrawProfileHeaderText("Squad Member: ");
            DrawProfileText(SquadAI.Pawn);
            DrawProfileHeaderText("    AI/State: ");
            DrawProfileText(SquadAI @ "/" @ SquadAI.GetStateName());
        }
    }
    else
    {
        DrawProfileText("Squad");
    }
    DrawProfileText(" ");
    SetProfileColumn(++CurrentColumn);
    if (Pawn.Controller != None)
    {
        DrawProfileHeaderText("Current State: ");
        DrawProfileText(Pawn.Controller.GetStateName());
    }
    idx = GetNumStates(Pawn.Controller) - 1;
    while (idx >= 0)
    {
        DrawProfileHeaderText("State: ");
        DrawProfileText(GetStateNameByIdx(Pawn.Controller, idx));
        idx--;
    }
    DrawProfileText(" ");
    if (Pawn.Controller != None)
    {
        if (SFXPlayerController(Pawn.Controller) != None)
        {
            DrawProfileHeaderText("Custom Action: ");
            DrawProfileText(SFXPlayerController(Pawn.Controller).CurrentCustomAction);
        }
        else
        {
            DrawProfileHeaderText("Custom Action: ");
            DrawProfileText(AI.CurrentCustomAction);
        }
    }
    DrawProfileText(" ");
    if (AI != None && AI.RouteCache.Length > 0)
    {
        foreach Pawn.Controller.RouteCache(Nav)
        {
            DrawProfileHeaderText("Route Cache: ");
            DrawProfileText(Nav);
        }
    }
}

exec function ClearCombatStats()
{
    if (Pawn(ProfileTarget) != None)
    {
        SFXPRI(Pawn(ProfileTarget).PlayerReplicationInfo).Clear();
    }
    else
    {
        Class'SFXPRI'.FindLocalPRI().Clear();
    }
}

function ProfileCombatStats()
{
    local SFXPRI Stats;
    local BioPawn Pawn;
    if (BioPawn(ProfileTarget) == None)
    {
        return;
    }
    Pawn = BioPawn(ProfileTarget);
    Stats = SFXPRI(Pawn.PlayerReplicationInfo);
    if (Stats == None)
    {
        return;
    }
    DrawProfileText(" ");
    DrawProfileText(" ");
    DrawProfileHeaderText("Shot Accuracy/Fired/Hit: ");
    DrawProfileText("" $ PrettyFloat(Stats.ShotsFired > 0 ? 100 * Stats.ShotsHit / Stats.ShotsFired : 0) $ "% / " $ Stats.ShotsFired $ " / " $ Stats.ShotsHit);
    DrawProfileHeaderText("Damage %/Net/Raw: ");
    DrawProfileText(PrettyFloat(Stats.RawDamage > 0 ? 100 * Stats.NetDamage / Stats.RawDamage : 0) $ "% / " $ PrettyFloat(Stats.NetDamage) $ " / " $ PrettyFloat(Stats.RawDamage));
    DrawProfileHeaderText("Headshot %/Net: ");
    DrawProfileText(PrettyFloat(Stats.ShotsFired > 0 ? 100 * Stats.HeadShots / Stats.ShotsFired : 0) $ "% / " $ Stats.HeadShots);
    DrawProfileText(" ");
    DrawProfileHeaderText("Current Target: ");
    DrawProfileText(Stats.CurrentTarget $ " (" $ Stats.CurrentTarget != None ? Stats.CurrentTarget.Tag : 'None' $ ")");
    DrawProfileHeaderText("Shot Accuracy/Fired/Hit: ");
    DrawProfileText("" $ PrettyFloat(Stats.ShotsFiredCurrentTarget > 0 ? 100 * Stats.ShotsHitCurrentTarget / Stats.ShotsFiredCurrentTarget : 0) $ "% / " $ Stats.ShotsFiredCurrentTarget $ " / " $ Stats.ShotsHitCurrentTarget);
    DrawProfileHeaderText("Damage %/Net/Raw: ");
    DrawProfileText(PrettyFloat(Stats.RawDamageCurrentTarget > 0 ? 100 * Stats.NetDamageCurrentTarget / Stats.RawDamageCurrentTarget : 0) $ "% / " $ PrettyFloat(Stats.NetDamageCurrentTarget) $ " / " $ PrettyFloat(Stats.RawDamageCurrentTarget));
    DrawProfileText(" ");
    DrawProfileHeaderText("Last Damage Calculation: ");
    DrawProfileText("Base Damage: " $ Stats.LastDamageCalculation.BaseDamage);
    DrawProfileText("Range: x" $ Stats.LastDamageCalculation.RangeMultiplier);
    DrawProfileText("Game Effects: x" $ Stats.LastDamageCalculation.GEMultiplier);
    DrawProfileText("Passive Game Effects: x" $ Stats.LastDamageCalculation.PassiveGEMultiplier);
    DrawProfileText("Stealth: x" $ Stats.LastDamageCalculation.StealthMultiplier);
    DrawProfileText("LastBullet: x" $ Stats.LastDamageCalculation.LastBulletMultiplier);
    DrawProfileText("Difficulty: x" $ Stats.LastDamageCalculation.DifficultyMultiplier);
    DrawProfileText("PartBasedDamage: x" $ Stats.LastDamageCalculation.PartBasedDamageMultiplier);
    DrawProfileText("DamageScale: x" $ Stats.LastDamageCalculation.DamageScale);
    DrawProfileText("Cover Bonus: x" $ Stats.LastDamageCalculation.CoverMultiplier);
    DrawProfileText("CoverLean Bonus: x" $ Stats.LastDamageCalculation.CoverLeanMultiplier);
    DrawProfileText("Storm Bonus: x" $ Stats.LastDamageCalculation.StormMultiplier);
    DrawProfileText("Ragdoll Bonus: x" $ Stats.LastDamageCalculation.RagdollMultiplier);
    DrawProfileText("AchievementBonus: x" $ Stats.LastDamageCalculation.AchievementBonusDamage);
    DrawProfileText("Total damage: " $ Stats.LastDamageCalculation.BaseDamage * Stats.LastDamageCalculation.RangeMultiplier * Stats.LastDamageCalculation.PassiveGEMultiplier * Stats.LastDamageCalculation.GEMultiplier * Stats.LastDamageCalculation.StealthMultiplier * Stats.LastDamageCalculation.LastBulletMultiplier * Stats.LastDamageCalculation.DifficultyMultiplier * Stats.LastDamageCalculation.PartBasedDamageMultiplier * Stats.LastDamageCalculation.DamageScale * Stats.LastDamageCalculation.CoverLeanMultiplier * Stats.LastDamageCalculation.CoverMultiplier * Stats.LastDamageCalculation.StormMultiplier * Stats.LastDamageCalculation.RagdollMultiplier * Stats.LastDamageCalculation.AchievementBonusDamage);
    DrawProfileText(" ");
}

function ProfilePawn()
{
    local BioPawn Pawn;
    local BioAttributesPawn Attributes;
    local Bio_Appr_Character Appearance;
    local BioTalentSpecification TalentSpec;
    local BioTalent Talent;
    local String Reason;
    local SFXGame Game;
    local BioEpicPawnBehavior oBehavior;
    local BioPlayerSquad oPlayerSquad;
    Pawn = BioPawn(ProfileTarget);
    if (Pawn == None)
    {
        return;
    }
    Game = SFXGame(Outer.WorldInfo.Game);
    oBehavior = BioEpicPawnBehavior(Pawn.oBioComponent);
    if (oBehavior == None)
    {
        return;
    }
    oPlayerSquad = BioPlayerSquad(oBehavior.Squad);
    Attributes = BioAttributesPawn(Pawn.m_oBehavior.m_PawnAttributes);
    if (Pawn.IsHumanControlled())
    {
        DrawProfileHeaderText("Name: ");
        DrawProfileText(Attributes != None ? Attributes.m_FirstName : "-" @ "Shepard");
    }
    DrawProfileHeaderText("Female: ");
    DrawProfileText(Pawn.bIsFemale);
    DrawProfileHeaderText("Race: ");
    DrawProfileText(Attributes != None ? Attributes.m_Race : "-");
    DrawProfileHeaderText("Faction: ");
    DrawProfileText(Pawn.FactionClass != None ? Pawn.FactionClass : Pawn.Squad.SquadFaction);
    DrawProfileHeaderText("Decryption / Electronics: ");
    DrawProfileText(Attributes != None ? Attributes.GetCurrentFloatValue(Attributes.m_SkillDecryption) @ "/" @ Attributes.GetCurrentFloatValue(Attributes.m_SkillElectronics) : "-");
    DrawProfileText(" ");
    DrawProfileHeaderText("In Stasis: ");
    DrawProfileText(Pawn.bStasis);
    DrawProfileHeaderText("Location: ");
    DrawProfileText(Pawn.Location);
    DrawProfileHeaderText("Rotation: ");
    DrawProfileText(Pawn.Rotation);
    DrawProfileHeaderText("Crouched: ");
    DrawProfileText(Pawn.bIsCrouched);
    DrawProfileHeaderText("Anchor: ");
    DrawProfileText(Pawn.Anchor);
    DrawProfileHeaderText("Velocity: ");
    DrawProfileText(Pawn.Velocity @ "Magnitude:" @ VSize(Pawn.Velocity));
    DrawProfileHeaderText("Acceleration: ");
    DrawProfileText(Pawn.Acceleration @ "Magnitude:" @ VSize(Pawn.Acceleration));
    DrawProfileHeaderText("Base: ");
    DrawProfileText(Pawn.Base);
    DrawProfileHeaderText("Floor: ");
    DrawProfileText(Pawn.Floor);
    DrawProfileHeaderText("Physics: ");
    DrawProfileText(Pawn.Physics);
    DrawProfileHeaderText("Can Crouch: ");
    DrawProfileText(Pawn.bCanCrouch);
    DrawProfileHeaderText("Can Strafe: ");
    DrawProfileText(Pawn.bCanStrafe);
    DrawProfileText(" ");
    DrawProfileHeaderText("Desired Speed: ");
    DrawProfileText(Pawn.DesiredSpeed);
    DrawProfileHeaderText("Max Desired Speed: ");
    DrawProfileText(Pawn.m_oBehavior.m_fDesiredMaxSpeed);
    DrawProfileHeaderText("Desired Rotation: ");
    DrawProfileText(Pawn.DesiredRotation);
    DrawProfileText(" ");
    if (oPlayerSquad.m_bCombatEnabled)
    {
        DrawProfileHeaderText("Max Storm Duration: ");
        DrawProfileText(Game.StormStamina);
        DrawProfileHeaderText("Storm Regen Rate: ");
        DrawProfileText(Game.StormRegen);
    }
    else
    {
        DrawProfileHeaderText("Max Storm Duration: ");
        DrawProfileText(Game.StormStaminaNonCombat);
        DrawProfileHeaderText("Storm Regen Rate: ");
        DrawProfileText(Game.StormRegen);
    }
    DrawProfileText(" ");
    Appearance = Pawn.PawnType.m_oAppearance;
    DrawProfileHeaderText("Movement Data: ");
    DrawProfileText(Appearance != None ? Appearance.MovementInfo : "-");
    DrawProfileHeaderText("Walk Speed: ");
    DrawProfileText(Pawn.WalkSpeed);
    DrawProfileHeaderText("Ground Speed: ");
    DrawProfileText(Pawn.GroundSpeed);
    DrawProfileHeaderText("Turn Speed: ");
    DrawProfileText(Pawn.TurnSpeed);
    DrawProfileHeaderText("Combat Walk Speed: ");
    DrawProfileText(Pawn.CombatWalkSpeed);
    DrawProfileHeaderText("Combat Ground Speed: ");
    DrawProfileText(Pawn.CombatGroundSpeed);
    DrawProfileHeaderText("Death Crawl Speed: ");
    DrawProfileText(Pawn.DeathCrawlSpeed);
    DrawProfileHeaderText("Cover Ground Speed: ");
    DrawProfileText(Pawn.CoverGroundSpeed);
    DrawProfileHeaderText("Low Cover Ground Speed: ");
    DrawProfileText(Pawn.CoverCrouchGroundSpeed);
    DrawProfileHeaderText("Crouch Ground Speed: ");
    DrawProfileText(Pawn.CrouchGroundSpeed);
    DrawProfileHeaderText("Storm Speed: ");
    DrawProfileText(Pawn.StormSpeed);
    DrawProfileHeaderText("Storm Turn Speed: ");
    DrawProfileText(Pawn.StormTurnSpeed);
    DrawProfileHeaderText("Air Speed: ");
    DrawProfileText(Pawn.AirSpeed);
    DrawProfileHeaderText("Accel Rate: ");
    DrawProfileText(Pawn.AccelRate);
    if (Pawn.IsHumanControlled())
    {
        DrawProfileText(" ");
        DrawProfileHeaderText("Left Stick: ");
        DrawProfileText(Outer.PlayerInput.RawJoyRight @ Outer.PlayerInput.RawJoyUp);
        DrawProfileHeaderText("Right Stick: ");
        DrawProfileText(Outer.PlayerInput.RawJoyLookRight @ Outer.PlayerInput.RawJoyLookUp);
    }
    SetProfileColumn(++CurrentColumn);
    DrawProfileHeaderText("Root Motion Reference Counts: ");
    DrawProfileText(Pawn.m_nRootMotionEnabledCountDeprecated);
    DrawProfileHeaderText("Root Motion Mode: ");
    DrawProfileText(Pawn.Mesh != None ? Pawn.Mesh.RootMotionMode : "-");
    DrawProfileHeaderText("Root Motion Start State: ");
    DrawProfileText(Pawn.m_oBehavior.m_eAnimStartState);
    DrawProfileHeaderText("Root Motion Skid State: ");
    DrawProfileText(Pawn.m_oBehavior.m_eAnimSkidState);
    DrawProfileHeaderText("Root Motion Stop State: ");
    DrawProfileText(Pawn.m_oBehavior.m_eAnimStopState);
    DrawProfileHeaderText("Root Motion Rotation Mode: ");
    DrawProfileText(Pawn.Mesh != None ? Pawn.Mesh.RootMotionRotationMode : "-");
    DrawProfileText(" ");
    DrawProfileHeaderText("Forced LOD: ");
    DrawProfileText(Pawn.Mesh != None ? Pawn.Mesh.ForcedLodModel : "-");
    DrawProfileHeaderText("Predicted LOD: ");
    DrawProfileText(Pawn.Mesh != None ? Pawn.Mesh.PredictedLODLevel : "-");
    DrawProfileHeaderText("Old Predicted LOD: ");
    DrawProfileText(Pawn.Mesh != None ? Pawn.Mesh.OldPredictedLODLevel : "-");
    DrawProfileText(" ");
    DrawProfileHeaderText("In Cover: ");
    DrawProfileText(Pawn.IsInCover());
    DrawProfileHeaderText("Cover Link: ");
    DrawProfileText(Pawn.CurrentLink);
    DrawProfileHeaderText("Current Slot Idx: ");
    DrawProfileText(Pawn.CurrentSlotIdx);
    DrawProfileHeaderText("Cover Type: ");
    DrawProfileText(Pawn.CoverType);
    DrawProfileHeaderText("Cover Action: ");
    DrawProfileText(Pawn.CoverAction);
    DrawProfileHeaderText("Cover Direction: ");
    DrawProfileText(Pawn.CoverDirection);
    DrawProfileText(" ");
    DrawProfileHeaderText("In Action Station: ");
    DrawProfileText(Pawn.m_oBehavior.m_bInActionStation);
    if (Pawn.IsHumanControlled())
    {
        DrawProfileText(" ");
        DrawProfileHeaderText("Can Currently Save: ");
        DrawProfileText(Outer.CanSave(Reason));
        DrawProfileHeaderText("Reason: ");
        DrawProfileText(Reason);
    }
    SetProfileColumn(++CurrentColumn);
    if (Pawn.m_oBehavior != None && Pawn.m_oBehavior.m_Talents != None)
    {
        foreach Pawn.m_oBehavior.m_Talents.m_TalentSpecArray(TalentSpec)
        {
            DrawProfileHeaderText("Talent: ");
            DrawProfileText("Name:" @ Class'SFXGame'.GetSimpleString(TalentSpec.m_LocalizedName) @ "Rank:" @ TalentSpec.m_Talent.GetRank());
        }
        foreach Pawn.m_oBehavior.m_Talents.m_SimpleTalentArray(Talent)
        {
            DrawProfileHeaderText("Hidden Talent: ");
            DrawProfileText("ID:" @ Talent.GetID() @ "Rank:" @ Talent.GetRank());
        }
    }
}

function ProfileVehicle()
{
    local SFXVehicleHover Vehicle;
    Vehicle = SFXVehicleHover(ProfileTarget);
    if (Vehicle == None)
    {
        return;
    }
    DrawProfileHeaderText("In Stasis: ");
    DrawProfileText(Vehicle.bStasis);
    DrawProfileHeaderText("Location: ");
    DrawProfileText(Vehicle.Location);
    DrawProfileHeaderText("Rotation: ");
    DrawProfileText(Vehicle.Rotation);
    DrawProfileHeaderText("Velocity: ");
    DrawProfileText(Vehicle.Velocity @ "Magnitude:" @ VSize(Vehicle.Velocity));
    DrawProfileHeaderText("Physics: ");
    DrawProfileText(Vehicle.Physics);
    if (Vehicle.GetModule(Class'SFXModule_Damage') != None)
    {
        DrawProfileHeaderText("Current Health: ");
        DrawProfileText(PrettyFloat(Vehicle.GetModule(Class'SFXModule_Damage').CurrentHealth) @ "/" @ PrettyFloat(Vehicle.GetModule(Class'SFXModule_Damage').MaxHealth.Value));
    }
    DrawProfileText(" ");
    if (BioPlayerController(Vehicle.Controller) != None)
    {
        DrawProfileHeaderText("Jumping: ");
        DrawProfileText(BioPlayerController(Vehicle.Controller).bJump);
        DrawProfileHeaderText("Boosting: ");
        DrawProfileText(BioPlayerController(Vehicle.Controller).bBoost);
    }
    DrawProfileHeaderText("Thrust Remaining: ");
    DrawProfileText(Vehicle.CurrentThrustJuice);
    DrawProfileHeaderText("Max Thrust Duration: ");
    DrawProfileText(Vehicle.MaxThrustJuice);
    DrawProfileHeaderText("Thrust Regeneration Factor: ");
    DrawProfileText(Vehicle.ThrustRegenerationFactor);
    DrawProfileText(" ");
    DrawProfileHeaderText("Vehicle on Ground: ");
    DrawProfileText(Vehicle.bVehicleOnGround);
}

function ProfileGameSettings()
{
    DrawProfileHeaderText("Controller Vibration:");
    DrawProfileText(Outer.ProfileSettings.GetControllerVibrationOption());
    DrawProfileHeaderText("Y Axis Inverted:");
    DrawProfileText(Outer.ProfileSettings.GetInvertYOption());
    DrawProfileHeaderText("Controller Sensitivity:");
    DrawProfileText(Outer.ProfileSettings.GetControllerSensitivityValue());
    DrawProfileHeaderText("Stick Config:");
    DrawProfileText(Outer.ProfileSettings.GetStickConfigOption());
    DrawProfileHeaderText("Trigger Config:");
    DrawProfileText(Outer.ProfileSettings.GetTriggerConfigOption());
    DrawProfileHeaderText("Subtitles:");
    DrawProfileText(Outer.ProfileSettings.GetSubtitleConfigOption());
    DrawProfileHeaderText("Aim Assist:");
    DrawProfileText(Outer.ProfileSettings.GetAimAssistValue());
    DrawProfileHeaderText("Difficulty:");
    DrawProfileText(Outer.ProfileSettings.GetDifficultyConfigOption());
    DrawProfileHeaderText("Auto Levelup:");
    DrawProfileText(Outer.ProfileSettings.GetAutoLevelConfigOption());
    DrawProfileHeaderText("Squad Power Use:");
    DrawProfileText(Outer.ProfileSettings.GetSquadPowerConfigOption());
    DrawProfileHeaderText("Auto Save:");
    DrawProfileText(Outer.ProfileSettings.GetAutoSaveConfigOption());
    DrawProfileHeaderText("Auto Login:");
    DrawProfileText(Outer.ProfileSettings.GetAutoLoginConfigOption());
    DrawProfileHeaderText("Music Volume:");
    DrawProfileText(Outer.ProfileSettings.GetMusicVolume());
    DrawProfileHeaderText("FX Volume:");
    DrawProfileText(Outer.ProfileSettings.GetFXVolume());
    DrawProfileHeaderText("Dialog Volume:");
    DrawProfileText(Outer.ProfileSettings.GetDialogVolume());
    DrawProfileHeaderText("Motion Blur:");
    DrawProfileText(Outer.ProfileSettings.GetMotionBlurConfigOption());
    DrawProfileHeaderText("Film Grain:");
    DrawProfileText(Outer.ProfileSettings.GetFilmgrainConfigOption());
    DrawProfileHeaderText("Save Device:");
    DrawProfileText(Outer.ProfileSettings.GetCurrentDeviceID());
    DrawProfileHeaderText("DisplayGamma:");
    DrawProfileText(Outer.ProfileSettings.GetDisplayGamma());
    SetProfileColumn(++CurrentColumn);
    DrawProfileHeaderText("Num Headshots:");
    DrawProfileText(Outer.ProfileSettings.GetNumHeadshots());
    DrawProfileHeaderText("Num Melee Combos:");
    DrawProfileText(Outer.ProfileSettings.GetNumMeleeCombos());
    DrawProfileHeaderText("Num Power Combos:");
    DrawProfileText(Outer.ProfileSettings.GetNumPowerCombos());
    DrawProfileHeaderText("Num Screams:");
    DrawProfileText(Outer.ProfileSettings.GetNumScreams());
    DrawProfileHeaderText("Num Shields Disrupted:");
    DrawProfileText(Outer.ProfileSettings.GetNumShieldsDisrupted());
    DrawProfileHeaderText("Num Barriers warped:");
    DrawProfileText(Outer.ProfileSettings.GetNumBarriersWarped());
    DrawProfileHeaderText("Num armour burned:");
    DrawProfileText(Outer.ProfileSettings.GetNumArmourIncinerated());
    DrawProfileHeaderText("Num n7 missions:");
    DrawProfileText(Outer.ProfileSettings.GetNumN7MissionsCompleted());
    DrawProfileHeaderText("Num codex entries:");
    DrawProfileText(Outer.ProfileSettings.GetNumCodex());
    DrawProfileHeaderText("Num tech upgrades:");
    DrawProfileText(Outer.ProfileSettings.GetNumTechUpgrades());
    DrawProfileHeaderText("Num game completions:");
    DrawProfileText(Outer.ProfileSettings.GetNumGameCompletions());
    DrawProfileHeaderText("Game completion career name:");
    DrawProfileText(Outer.ProfileSettings.GetFinishedGameCareerName());
    DrawProfileHeaderText("Chose Morinth not Samara:");
    DrawProfileText(Outer.ProfileSettings.GetChoseMorinthNotSamara());
    DrawProfileHeaderText("Current career name:");
    DrawProfileText(SFXEngine(Outer.Player.Outer).CurrentCareerName);
}

exec function SetProfileChoseMorinth(bool ChoseMorinth)
{
    local SFXProfileSettings Profile;
    Profile = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController().ProfileSettings;
    if (Profile != None)
    {
        Profile.SetChoseMorinthNotSamara(ChoseMorinth);
    }
}

function ProfilePower()
{
    local BioPawn Pawn;
    local BioPawnBehavior Behavior;
    local BioPower Power;
    local BioAttributesPawn Attributes;
    Pawn = BioPawn(ProfileTarget);
    if (Pawn == None)
    {
        return;
    }
    Behavior = Pawn.m_oBehavior;
    if (ProfileSubTarget == 'None')
    {
        DrawProfileHeaderText("Number of Powers: ");
        DrawProfileText(Behavior.m_powers.Length);
        DrawProfileHeaderText("Power Use Percent: ");
        DrawProfileText(Behavior.GetPowerUsePercent());
        DrawProfileText(" ");
        foreach Behavior.m_powers(Power)
        {
            DrawProfileHeaderText("Power: ");
            DrawProfileText(Power.m_nmPower);
            DrawProfileHeaderText("Enabled: ");
            DrawProfileText(Power.IsEnabled());
            DrawProfileHeaderText("Suppressed: ");
            DrawProfileText(Power.IsSuppressed());
            DrawProfileHeaderText("Cooldown: ");
            DrawProfileText(Power.GetCooldownRemaining());
            DrawProfileHeaderText("Can Move: ");
            DrawProfileText(Power.CanCasterMoveDuringPower());
            DrawProfileHeaderText("Active: ");
            DrawProfileText(Power.IsActive());
            if (Outer.myHUD.Canvas.CurY >= Outer.myHUD.Canvas.ClipY - 96)
            {
                SetProfileColumn(++CurrentColumn);
            }
            else
            {
                DrawProfileText(" ");
            }
        }
    }
    else
    {
        Power = Behavior.FindPower(ProfileSubTarget);
        DrawProfileHeaderText("Power: ");
        DrawProfileText(Power != None ? Power.m_nmPower : "Error");
        if (Power != None)
        {
            DrawProfileText(" ");
            DrawProfileHeaderText("Enabled: ");
            DrawProfileText(Power.IsEnabled());
            DrawProfileHeaderText("Suppressed: ");
            DrawProfileText(Power.IsSuppressed());
            DrawProfileHeaderText("Casting Time: ");
            DrawProfileText(Power.GetCastingTime());
            DrawProfileHeaderText("Cooldown Time: ");
            DrawProfileText(Power.GetCooldownTime());
            DrawProfileHeaderText("Projectile Speed: ");
            DrawProfileText(Power.GetProjectileSpeed());
            DrawProfileHeaderText("Projectile Radius: ");
            DrawProfileText(Power.GetProjectileRadius());
            DrawProfileHeaderText("Min Range: ");
            DrawProfileText(Power.GetMinimumRange());
            DrawProfileHeaderText("Max Range: ");
            DrawProfileText(Power.GetMaximumRange());
            DrawProfileHeaderText("Impact Radius: ");
            DrawProfileText(Power.GetImpactRadius());
            DrawProfileHeaderText("Effect Duration: ");
            DrawProfileText(Power.GetEffectDuration());
            DrawProfileHeaderText("VFX Intensity: ");
            DrawProfileText(Power.GetVFXIntensity());
            DrawProfileHeaderText("Cone Half Angle: ");
            DrawProfileText(Power.GetConeHalfAngle());
            DrawProfileHeaderText("Stop On First Hit: ");
            DrawProfileText(Power.GetStopOnFirstHit());
            DrawProfileHeaderText("Blocked By Objects: ");
            DrawProfileText(Power.GetBlockedByObjects());
            DrawProfileText(" ");
            Attributes = BioAttributesPawn(Behavior.m_PawnAttributes);
            DrawProfileHeaderText("Global Cooldown Scalar: ");
            DrawProfileText(Attributes != None ? Attributes.GetCurrentFloatValue(Attributes.m_PowerCooldownMultiplier) : "Error");
        }
    }
}

function DrawAIUtility()
{
    local BioPawn Pawn;
    local NavigationPoint Nav;
    local NavigationPoint LastNav;
    local BioAiController AI;
    Pawn = BioPawn(ProfileTarget);
    if (Pawn == None)
    {
        return;
    }
    if (Pawn.Controller != None)
    {
        if (Pawn.Controller.RouteCache.Length > 0)
        {
            LastNav = NavigationPoint(Pawn.Controller.MoveTarget);
            foreach Pawn.Controller.RouteCache(Nav)
            {
                if (LastNav != None)
                {
                    Outer.DrawDebugLine(LastNav.Location, Nav.Location, 0, 0, 255);
                }
                LastNav = Nav;
            }
        }
    }
    AI = BioAiController(Pawn.Controller);
    if (AI != None)
    {
        if (AI.MoveTarget != None)
        {
            Outer.DrawDebugLine(Pawn.Location, AI.MoveTarget.Location, 255, 255, 255);
        }
        if (AI.FireTarget != None)
        {
            Outer.DrawDebugLine(Pawn.GetPawnViewLocation(), AI.FireTarget.Location, 255, 0, 0);
        }
        if (AI.Focus != None)
        {
            Outer.DrawDebugLine(Pawn.GetPawnViewLocation(), AI.Focus.Location, 255, 255, 0);
        }
        else
        {
            if (IsZero(AI.FocalPoint) == false)
            {
                Outer.DrawDebugLine(Pawn.GetPawnViewLocation(), AI.FocalPoint, 255, 255, 0);
            }
        }
    }
}

function DrawTargetLineUtility()
{
    local BioPawn Pawn;
    local Vector EndPoint;
    Pawn = BioPawn(ProfileTarget);
    if (Pawn == None)
    {
        return;
    }
    EndPoint = Pawn.Location + vect(0,0,200);
    Outer.DrawDebugLine(Pawn.Location, EndPoint, 255, 255, 0);
}

function ProfileMultipliers()
{
    local SFXBonusList BL;
    BL = SFXGame(Outer.Pawn.WorldInfo.Game).BonusList;
    DrawProfileHeaderText("Damage: ");
    DrawProfileText("Assault Rifle" @ BL.RchAssaultRifleCache);
    DrawProfileHeaderText("Damage: ");
    DrawProfileText("Auto Pistol" @ BL.RchAutoPistolCache);
    DrawProfileHeaderText("Damage: ");
    DrawProfileText("Heavy Pistol" @ BL.RchHeavyPistolCache);
    DrawProfileHeaderText("Damage: ");
    DrawProfileText("Shotgun" @ BL.RchShotgunCache);
    DrawProfileHeaderText("Damage: ");
    DrawProfileText("Sniper Rifle" @ BL.RchSniperRifleCache);
    DrawProfileHeaderText("Damage: ");
    DrawProfileText("Melee" @ BL.RchMeleeUpgradeCache);
    DrawProfileHeaderText("Damage: ");
    DrawProfileText("Biotic" @ BL.RchBioDmgUpgradeCache);
    DrawProfileHeaderText("Damage: ");
    DrawProfileText("Tech" @ BL.RchTechDmgUpgradeCache);
    DrawProfileText(" ");
    DrawProfileHeaderText("Health: ");
    DrawProfileText("Player" @ BL.RchHealthUpgradeCache);
    DrawProfileHeaderText("Health: ");
    DrawProfileText("Grunt" @ BL.RchGruntUpgradeCache);
    DrawProfileHeaderText("Health: ");
    DrawProfileText("Legion" @ BL.RchLegionUpgradeCache);
    SetProfileColumn(++CurrentColumn);
    DrawProfileHeaderText("Ammo: ");
    DrawProfileText("HW" @ BL.RchHWAmmoCache);
    DrawProfileHeaderText("Capacity: ");
    DrawProfileText("Medigel" @ BL.RchMedigelCache);
    SetProfileColumn(++CurrentColumn);
    DrawProfileHeaderText("Cooldown: ");
    DrawProfileText("Biotic" @ BL.RchBioticCooldownCache);
    DrawProfileHeaderText("Cooldown: ");
    DrawProfileText("Tech" @ BL.RchTechCooldownCache);
}

function ProfileBonuses()
{
    local SFXBonusList BL;
    local BioGlobalVariableTable gv;
    BL = SFXGame(Outer.Pawn.WorldInfo.Game).BonusList;
    gv = BioWorldInfo(Outer.Pawn.WorldInfo).GetGlobalVariables();
    SetProfileColumn(++CurrentColumn);
    if (gv.GetInt(BL.RchAssaultRifleID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("Assault Rifle Bonus " @ gv.GetInt(BL.RchAssaultRifleID) * BL.RchAssaultRifleBonus);
    }
    if (gv.GetInt(BL.RchAutoPistolID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("AutoPistol Bonus " @ gv.GetInt(BL.RchAutoPistolID) * BL.RchAutoPistolBonus);
    }
    if (gv.GetInt(BL.RchHeavyPistolID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("Heavy Pistol Bonus " @ gv.GetInt(BL.RchHeavyPistolID) * BL.RchHeavyPistolBonus);
    }
    if (gv.GetInt(BL.RchShotgunID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("Shotgun Bonus " @ gv.GetInt(BL.RchShotgunID) * BL.RchShotgunBonus);
    }
    if (gv.GetInt(BL.RchSniperRifleID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("Sniper Rifle Bonus " @ gv.GetInt(BL.RchSniperRifleID) * BL.RchSniperRifleBonus);
    }
    if (gv.GetInt(BL.RchMedigelID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("Medigel Capacity Bonus " @ gv.GetInt(BL.RchMedigelID));
    }
    if (gv.GetInt(BL.RchBioticCooldownID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("Biotic Cooldown Bonus " @ gv.GetInt(BL.RchBioticCooldownID) * BL.RchBioticCooldownBonus);
    }
    if (gv.GetInt(BL.RchTechCooldownID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("Tech Cooldown Bonus " @ gv.GetInt(BL.RchTechCooldownID) * BL.RchTechCooldownBonus);
    }
    if (gv.GetInt(BL.RchMeleeUpgradeID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("Melee Upgrade Bonus " @ gv.GetInt(BL.RchMeleeUpgradeID) * BL.RchMeleeUpgradeBonus);
    }
    if (gv.GetInt(BL.RchShieldUpgradeID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("Shield Upgrade Bonus " @ gv.GetInt(BL.RchShieldUpgradeID) * BL.RchShieldUpgradeBonus);
    }
    if (gv.GetInt(BL.RchHealthUpgradeID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("Health Upgrade Bonus " @ gv.GetInt(BL.RchHealthUpgradeID) * BL.RchHealthUpgradeBonus);
    }
    if (gv.GetInt(BL.RchHWAmmoID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("HW Ammo Bonus " @ gv.GetInt(BL.RchHWAmmoID) * BL.RchHWAmmoBonus);
    }
    if (gv.GetInt(BL.RchGruntUpgradeID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("Grunt Health Bonus " @ gv.GetInt(BL.RchGruntUpgradeID) * BL.RchGruntUpgradeBonus);
    }
    if (gv.GetInt(BL.RchLegionUpgradeID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("Legion Health Bonus " @ gv.GetInt(BL.RchLegionUpgradeID) * BL.RchLegionUpgradeBonus);
    }
    if (gv.GetInt(BL.RchBioDmgUpgradeID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("Biotic Damage Bonus " @ gv.GetInt(BL.RchBioDmgUpgradeID) * BL.RchBioDmgUpgradeBonus);
    }
    if (gv.GetInt(BL.RchTechDmgUpgradeID) > 0)
    {
        DrawProfileHeaderText("Research: ");
        DrawProfileText("Tech Damage Bonus " @ gv.GetInt(BL.RchTechDmgUpgradeID) * BL.RchTechDmgUpgradeBonus);
    }
    SetProfileColumn(++CurrentColumn);
}

exec function UnlockAllTreasure(bool B)
{
    local BioWorldInfo oWorldInfo;
    local SFXPlotTreasure oTreasure;
    local int I;
    local int nTreasureId;
    oWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    oTreasure = oWorldInfo.m_oTreasure;
    I = 0;
    while (I != oTreasure.oPlotTreasureTreasure2DA.GetNumRows())
    {
        nTreasureId = oTreasure.oPlotTreasureTreasure2DA.GetRowNumber(I);
        oWorldInfo.GetGlobalVariables().SetBool(nTreasureId, B);
        ++I;
    }
}

exec function UnlockAllResearch(bool B)
{
    local BioWorldInfo oWorldInfo;
    local SFXPlotTreasure oTreasure;
    local int I;
    local int nTreasureId;
    local Name nmRequiredTech;
    local Name nmTech;
    oWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    oTreasure = oWorldInfo.m_oTreasure;
    I = 0;
    while (I != oTreasure.oPlotTreasureTreasure2DA.GetNumRows())
    {
        nTreasureId = oTreasure.oPlotTreasureTreasure2DA.GetRowNumber(I);
        oTreasure.oPlotTreasureTreasure2DA.GetNameEntryIN(I, 'nmRequiredTech', nmRequiredTech);
        oTreasure.oPlotTreasureTreasure2DA.GetNameEntryIN(I, 'nmTech', nmTech);
        if (nmRequiredTech != 'None')
        {
            oWorldInfo.GetGlobalVariables().SetBool(nTreasureId, B);
            oWorldInfo.GetGlobalVariables().SetIntByName(nmTech, B ? 1 : 0);
        }
        ++I;
    }
}

exec function UnlockAllTech(int I)
{
    local BioWorldInfo oWorldInfo;
    local SFXPlotTreasure oTreasure;
    local int N;
    local Name nmTech;
    oWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    oTreasure = oWorldInfo.m_oTreasure;
    N = 0;
    while (N != oTreasure.oPlotTreasureTech2DA.GetNumRows())
    {
        oTreasure.oPlotTreasureTech2DA.GetNameEntryIN(N, 'nmTech', nmTech);
        oWorldInfo.GetGlobalVariables().SetIntByName(nmTech, I);
        ++N;
    }
    oWorldInfo.GetGlobalVariables().SetIntByName('Dep_Tec_GruntShotgun', I);
    oWorldInfo.GetGlobalVariables().SetIntByName('Dep_Tec_JackUpgrade', I);
    oWorldInfo.GetGlobalVariables().SetIntByName('Dep_Tec_LegionSniper', I);
    oWorldInfo.GetGlobalVariables().SetIntByName('Dep_Tec_MordinUpgrade', I);
    oWorldInfo.GetGlobalVariables().SetIntByName('Dep_Tec_ShipArmor', I);
    oWorldInfo.GetGlobalVariables().SetIntByName('Dep_Tec_ShipFuel', I);
    oWorldInfo.GetGlobalVariables().SetIntByName('Dep_Tec_ShipGun', I);
    oWorldInfo.GetGlobalVariables().SetIntByName('Dep_Tec_ShipProbes', I);
    oWorldInfo.GetGlobalVariables().SetIntByName('Dep_Tec_ShipScanner', I);
    oWorldInfo.GetGlobalVariables().SetIntByName('Dep_Tec_ShipShield', I);
    oWorldInfo.GetGlobalVariables().SetIntByName('Tech_CounterForRetrain', I);
    oWorldInfo.GetGlobalVariables().SetIntByName('Tech_CounterForLoyaltyPower', I);
}

function ProfileTreasure()
{
    local BioGlobalVariableTable VariableTable;
    local SFXInventoryManager PlayerInventory;
    local int TreasureCount;
    local int TreasureCountMax;
    local BioWorldInfo oWorldInfo;
    local SFXPlotTreasure oTreasure;
    local SResourceBudget Budget;
    local int I;
    local int nTreasureId;
    local int Row;
    local Name nmTreasure;
    local bool bHasTreasure;
    local TreasureBudget stGlobalBudget;
    local TreasureBudget stLevelBudget;
    if (BioPawn(Outer.Pawn).m_bInConversation)
    {
        return;
    }
    VariableTable = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    if (VariableTable == None)
    {
        return;
    }
    PlayerInventory = SFXInventoryManager(Outer.Pawn.InvManager);
    oWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    oTreasure = oWorldInfo.m_oTreasure;
    Budget = oTreasure.Budget();
    stGlobalBudget = SFXGame(Outer.WorldInfo.Game).gameconfig.GetGlobalBudget();
    stLevelBudget = SFXGame(Outer.WorldInfo.Game).gameconfig.GetLevelBudget();
    if (stGlobalBudget.LevelId != Budget.nID)
    {
        SFXGame(Outer.WorldInfo.Game).gameconfig.RecordTreasure(0, 0, Budget.nID);
    }
    if (PlayerInventory != None)
    {
        DrawProfileHeaderText("Player Credits: ");
        DrawProfileText(PlayerInventory.GetResource(0) $ " / " $ stGlobalBudget.Credits + stLevelBudget.Credits);
        DrawProfileHeaderText("Player Eezo: ");
        DrawProfileText(PlayerInventory.GetResource(4) $ " / " $ stGlobalBudget.Eezo + stLevelBudget.Eezo);
        DrawProfileHeaderText("Player Iridium: ");
        DrawProfileText(PlayerInventory.GetResource(5) $ " / " $ stGlobalBudget.Iridium + stLevelBudget.Iridium);
        DrawProfileHeaderText("Player Palladium: ");
        DrawProfileText(PlayerInventory.GetResource(6) $ " / " $ stGlobalBudget.Palladium + stLevelBudget.Palladium);
        DrawProfileHeaderText("Player Platinum: ");
        DrawProfileText(PlayerInventory.GetResource(7) $ " / " $ stGlobalBudget.Platinum + stLevelBudget.Platinum);
        DrawProfileHeaderText("Player Probes: ");
        DrawProfileText(PlayerInventory.GetResource(8));
        DrawProfileHeaderText("Player Medigel: ");
        DrawProfileText(PlayerInventory.GetResource(1));
        DrawProfileText("  ");
        Row = 8;
    }
    DrawProfileHeaderText("  Level Id: ");
    DrawProfileText(Budget.nID);
    if (Budget.nCredits > 0 || stLevelBudget.Credits > 0)
    {
        DrawProfileHeaderText("  Level Credits: ");
        Budget.nCredits = Budget.nCredits > 0 ? Budget.nCredits : 1;
        DrawProfileText(stLevelBudget.Credits $ " of " $ Budget.nCredits $ " (" $ stLevelBudget.Credits * 100 / Budget.nCredits $ "%)");
        Row++;
    }
    if (Budget.nIridium > 0 || stLevelBudget.Iridium > 0)
    {
        DrawProfileHeaderText("  Level Iridium: ");
        Budget.nIridium = Budget.nIridium > 0 ? Budget.nIridium : 1;
        DrawProfileText(stLevelBudget.Iridium $ " of " $ Budget.nIridium $ " (" $ stLevelBudget.Iridium * 100 / Budget.nIridium $ "%)");
        Row++;
    }
    if (Budget.nPlatinum > 0 || stLevelBudget.Platinum > 0)
    {
        DrawProfileHeaderText("  Level Platinum: ");
        Budget.nPlatinum = Budget.nPlatinum > 0 ? Budget.nPlatinum : 1;
        DrawProfileText(stLevelBudget.Platinum $ " of " $ Budget.nPlatinum $ " (" $ stLevelBudget.Platinum * 100 / Budget.nPlatinum $ "%)");
        Row++;
    }
    if (Budget.nPalladium > 0 || stLevelBudget.Palladium > 0)
    {
        DrawProfileHeaderText("  Level Palladium: ");
        Budget.nPalladium = Budget.nPalladium > 0 ? Budget.nPalladium : 1;
        DrawProfileText(stLevelBudget.Palladium $ " of " $ Budget.nPalladium $ " (" $ stLevelBudget.Palladium * 100 / Budget.nPalladium $ "%)");
        Row++;
    }
    if (Budget.nEezo > 0 || stLevelBudget.Eezo > 0)
    {
        DrawProfileHeaderText("  Level Eezo: ");
        Budget.nEezo = Budget.nEezo > 0 ? Budget.nEezo : 1;
        DrawProfileText(stLevelBudget.Eezo $ " of " $ Budget.nEezo $ " (" $ stLevelBudget.Eezo * 100 / Budget.nEezo $ "%)");
        Row++;
    }
    DrawProfileText("  ");
    Row++;
    I = 0;
    while (I != oTreasure.oPlotTreasureTreasure2DA.GetNumRows())
    {
        nTreasureId = oTreasure.oPlotTreasureTreasure2DA.GetRowNumber(I);
        oTreasure.oPlotTreasureTreasure2DA.GetNameEntryIN(I, 'nmTreasure', nmTreasure);
        bHasTreasure = VariableTable.GetBool(nTreasureId);
        DrawProfileText("" $ nmTreasure $ ":", bHasTreasure);
        TreasureCount = bHasTreasure ? TreasureCount + 1 : TreasureCount;
        TreasureCountMax++;
        Row++;
        if (Row >= 38)
        {
            Row = 0;
            SetProfileColumn(++CurrentColumn);
        }
        ++I;
    }
    DrawProfileText("");
    DrawProfileHeaderText("Treasure Completion: ");
    DrawProfileText("" $ TreasureCount $ " / " $ TreasureCountMax, true);
}

function ProfileTech()
{
    local BioGlobalVariableTable VariableTable;
    local BioWorldInfo oWorldInfo;
    local SFXPlotTreasure oTreasure;
    local int I;
    local int Row;
    local int nTechLevel;
    local Name nmTech;
    VariableTable = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    if (VariableTable == None)
    {
        return;
    }
    oWorldInfo = BioWorldInfo(Outer.WorldInfo);
    oTreasure = oWorldInfo.m_oTreasure;
    I = 0;
    while (I != oTreasure.oPlotTreasureTech2DA.GetNumRows())
    {
        oTreasure.oPlotTreasureTech2DA.GetNameEntryIN(I, 'nmTech', nmTech);
        nTechLevel = oWorldInfo.GetGlobalVariables().GetIntByName(nmTech);
        DrawProfileHeaderText("" $ nmTech $ ":");
        DrawProfileText(nTechLevel, nTechLevel > 0);
        Row++;
        if (Row >= 38)
        {
            Row = 0;
            SetProfileColumn(++CurrentColumn);
        }
        ++I;
    }
}

exec function TestTreasure()
{
    local BioWorldInfo oWorldInfo;
    local SFXPlotTreasure oTreasure;
    oWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    oTreasure = oWorldInfo.m_oTreasure;
    Class'BioSeqAct_PlotTreasureTester'.Validate(oTreasure);
    Outer.ClientMessage("Treasure Tested (check your log file)");
}

function DisplayAngstWeaponRange(BioPawn oPawn, bool bShowIcons, bool bShowProfileData)
{
    local SFXAI_Core oCont;
    local SFXWeapon oWeapon;
    local float fDistanceToTarget;
    oCont = SFXAI_Core(oPawn.Controller);
    if (oCont == None || oCont.IsInFightingState())
    {
        return;
    }
    oWeapon = SFXWeapon(oPawn.Weapon);
    if (oCont.FireTarget != None && oWeapon != None)
    {
        if (bShowProfileData)
        {
            DrawProfileHeaderText("Weapon Range (Short: " @ oWeapon.Range_Short / 100 @ ",Mid: " @ oWeapon.Range_Medium / 100 @ ",Long: " @ oWeapon.Range_Long / 100 @ " )");
        }
        fDistanceToTarget = VSize(oCont.FireTarget.Location - oPawn.Location);
        if (fDistanceToTarget > oWeapon.Range_Long)
        {
            if (bShowIcons)
            {
                BioHUD(Outer.myHUD).DrawAngstIcon(oPawn);
            }
            if (bShowProfileData)
            {
                DrawProfileText("FireTarget " @ oCont.FireTarget @ " @ " @ fDistanceToTarget / 100 @ " m is too far away.");
            }
        }
        else
        {
            if (bShowProfileData)
            {
                DrawProfileText("FireTarget " @ oCont.FireTarget @ " @ " @ fDistanceToTarget / 100 @ " m is within range");
            }
        }
    }
    else
    {
        if (bShowProfileData)
        {
            DrawProfileHeaderText("Weapon Range :");
        }
        if (bShowProfileData)
        {
            DrawProfileText("Invalid Fire target or weapon!");
        }
    }
}

function DisplayAngstInfo(bool bShowIcons, bool bShowProfileData)
{
    local BioPawn oPawn;
    foreach Outer.WorldInfo.AllPawns(Class'BioPawn', oPawn)
    {
        if (oPawn != Outer.Pawn)
        {
            DisplayAngstWeaponRange(oPawn, bShowIcons, bShowProfileData && oPawn == ProfileTarget);
        }
    }
}

function ProfileAngst()
{
    DisplayAngstInfo(false, true);
}

exec function ToggleAngstIcons()
{
    m_bShowAngstIcons = !m_bShowAngstIcons;
}

function ProfileCooldown()
{
    local BioPawn Pawn;
    local BioPawnBehavior Behavior;
    local int nIndex;
    local BioPower Power;
    local float Cooldown;
    local String CooldownBar;
    Cooldown = 0;
    Pawn = BioPawn(ProfileTarget);
    if (Pawn == None)
    {
        return;
    }
    Behavior = Pawn.m_oBehavior;
    if (Behavior == None)
    {
        return;
    }
    nIndex = 0;
    while (nIndex < Behavior.m_powers.Length)
    {
        Power = Behavior.m_powers[nIndex];
        if (Power != None && Power.Script != None)
        {
            if (Power.Script.PowerSharesCooldown(Power))
            {
                Cooldown = Power.GetCooldownRemaining();
                break;
            }
        }
        nIndex++;
    }
    DrawProfileHeaderText("Power Cooldown: ");
    DrawProfileText(Cooldown @ " sec");
    CooldownBar = "     ";
    SetProfileColumn(++CurrentColumn);
    nIndex = 0;
    while (nIndex < Cooldown)
    {
        CooldownBar = CooldownBar @ "*";
        nIndex++;
    }
    nIndex = 0;
    while (nIndex < 32)
    {
        DrawProfileText(" ");
        nIndex++;
    }
    DrawProfileText(CooldownBar);
    DrawProfileText(CooldownBar);
}

function ProfileCover()
{
    local BioPawn Pawn;
    Pawn = BioPawn(ProfileTarget);
    if (Pawn == None)
    {
        return;
    }
    BioHUD(Outer.myHUD).ProfileCover(Pawn.m_oBehavior);
}

function ProfileDoor()
{
    local SFXDoor door;
    local BioVisualEffect Effect;
    door = SFXDoor(ProfileTarget);
    if (door == None)
    {
        return;
    }
    DrawProfileHeaderText("Role:");
    DrawProfileText(door.m_DoorType);
    DrawProfileHeaderText("Current State:");
    DrawProfileText(door.m_CurrentDoorState);
    DrawProfileHeaderText("Previous State:");
    DrawProfileText(door.m_PreviousDoorState);
    DrawProfileText(" ");
    DrawProfileHeaderText("Is Transitioning ?:");
    DrawProfileText(door.m_bIsTransitioning);
    DrawProfileText(" ");
    DrawProfileHeaderText("Door Icons:");
    foreach door.m_aCurrentIconVFX(Effect)
    {
        if (Effect != None)
        {
            DrawProfileText(Effect.Name);
        }
        else
        {
            DrawProfileText("None");
        }
    }
}

exec function EvalCover()
{
    local BioAiController oCont;
    local Pawn oPawn;
    oPawn = Pawn(ProfileTarget);
    if (oPawn != None)
    {
        oCont = BioAiController(oPawn.Controller);
        if (oCont != None)
        {
            oCont.bAcquireNewCover = true;
            return;
        }
    }
}

function ProfileConversation()
{
    BioHUD(Outer.myHUD).ProfileConversation(None);
}

function ProfileConversationBug()
{
    BioHUD(Outer.myHUD).ProfileConversationBug(None);
}

function ProfileGestures()
{
    local BioPawn Pawn;
    Pawn = BioPawn(ProfileTarget);
    if (Pawn == None)
    {
        return;
    }
    BioHUD(Outer.myHUD).ProfileGestures(Pawn.m_oBehavior);
}

function ProfileAnim()
{
    local BioPawn Pawn;
    Pawn = BioPawn(ProfileTarget);
    if (Pawn == None)
    {
        return;
    }
    BioHUD(Outer.myHUD).ProfileAnim(Pawn.m_oBehavior);
}

function ProfileCamera()
{
    local PlayerController Controller;
    local SFXPlayerCamera Camera;
    Controller = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController();
    Camera = SFXPlayerCamera(Controller.PlayerCamera);
    DrawProfileText("Camera:" @ Camera);
    DrawProfileText("Active:" @ Camera.CurrentCameraMode);
    DrawProfileText("Actual POV: (Rotation=" @ Camera.CameraCache.POV.Rotation @ ", Location=" @ Camera.CameraCache.POV.Location @ ", FOV=" @ Camera.CameraCache.POV.FOV);
    DrawProfileText("Target POV: (Rotation=" @ Camera.ViewTarget.POV.Rotation @ ", Location=" @ Camera.ViewTarget.POV.Location @ ", FOV=" @ Camera.ViewTarget.POV.FOV);
    Camera.CurrentCameraMode.DrawHUD(self);
}

function ProfileDesignBudget()
{
    BioHUD(Outer.myHUD).ProfileDesignBudgetGeneral();
}

function ProfileLookAt()
{
    local BioPawn Pawn;
    Pawn = BioPawn(ProfileTarget);
    if (Pawn == None)
    {
        return;
    }
    BioHUD(Outer.myHUD).ProfileLookAt(Pawn.m_oBehavior);
}

native final function ProfileLocomotion();

native final function ProfileAnimTree();

function ProfileTicket()
{
    local BioPawn PlayerPawn;
    local BioPawn HenchmanPawn;
    local BioAiController SquadAI;
    local BioPlayerSquad PlayerSquad;
    PlayerSquad = SFXGame(Outer.WorldInfo.Game).PlayerSquad;
    if (PlayerSquad != None)
    {
        PlayerPawn = PlayerSquad.m_playerPawn;
    }
    if (PlayerPawn != None)
    {
        DrawProfileHeaderText("Player target tickets: ");
        DrawProfileText(PlayerPawn.m_nTargetTickets);
        DrawProfileHeaderText("Player attack tickets: ");
        DrawProfileText(PlayerPawn.m_nAttackTickets);
        DrawProfileHeaderText("Player max target tickets: ");
        DrawProfileText(PlayerPawn.GetMaxTargetTickets());
        DrawProfileHeaderText("Player max attack tickets: ");
        DrawProfileText(PlayerPawn.GetMaxAttackTickets());
        DrawProfileText(" ");
        foreach PlayerPawn.Squad.SquadMembers(SquadAI)
        {
            HenchmanPawn = BioPawn(SquadAI.Pawn);
            if (HenchmanPawn != PlayerSquad.m_playerPawn)
            {
                DrawProfileHeaderText("Squadmate target tickets: ");
                DrawProfileText(HenchmanPawn.m_nTargetTickets);
                DrawProfileHeaderText("Squadmate attack tickets: ");
                DrawProfileText(HenchmanPawn.m_nAttackTickets);
                DrawProfileHeaderText("Squadmate max target tickets: ");
                DrawProfileText(HenchmanPawn.GetMaxTargetTickets());
                DrawProfileHeaderText("Squadmate max attack tickets: ");
                DrawProfileText(HenchmanPawn.GetMaxAttackTickets());
                DrawProfileText(" ");
            }
        }
    }
}

function ProfileHenchmen()
{
    local BioPlayerSquad PlayerSquad;
    local BioAiController AIController;
    local SFXAI_Henchman Controller;
    local SFXPawn_Henchman Pawn;
    local int nIndex;
    PlayerSquad = SFXGame(Outer.WorldInfo.Game).PlayerSquad;
    if (PlayerSquad == None)
    {
        return;
    }
    foreach PlayerSquad.SquadMembers(AIController)
    {
        Controller = SFXAI_Henchman(AIController);
        if (Controller != None)
        {
            Pawn = SFXPawn_Henchman(Controller.Pawn);
            if (Pawn != None || Pawn.m_oBehavior != None)
            {
                DrawProfileHeaderText("Henchman: ");
                DrawProfileText(Pawn.m_oBehavior.GetActorGameName(), true);
                DrawProfileText(" ");
                DrawProfileHeaderText("Controller: ");
                DrawProfileText(Controller.Name);
                DrawProfileText(" ");
                DrawProfileHeaderText("Current State: ");
                DrawProfileText(Controller.GetStateName());
                nIndex = GetNumStates(Controller) - 1;
                while (nIndex >= 0)
                {
                    DrawProfileHeaderText("State: ");
                    DrawProfileText(GetStateNameByIdx(Controller, nIndex));
                    nIndex--;
                }
                DrawProfileText(" ");
                DrawProfileHeaderText("Fire Target: ");
                DrawProfileText(Controller.FireTarget $ " (Melee = " $ Controller.m_bMeleeAttacker $ ")");
                if (Controller.FireTarget != None)
                {
                    DrawProfileHeaderText("Distance: ");
                    DrawProfileText(PrettyFloat(VSize(Controller.FireTarget.Location - Pawn.Location) / 100, 2) $ "m");
                }
                DrawProfileText(" ");
                DrawProfileHeaderText("Acquire New Cover: ");
                DrawProfileText(Controller.bAcquireNewCover);
                DrawProfileHeaderText("Too far to attack: ");
                DrawProfileText(Controller.m_bTooFarToAttack);
                DrawProfileText(" ");
                DrawProfileHeaderText("Follow Player: ");
                DrawProfileText(Controller.m_bFollowPlayer);
                DrawProfileHeaderText("Hold Position: ");
                DrawProfileText(Controller.m_bHoldingPosition);
                DrawProfileHeaderText("Hold Location: ");
                DrawProfileText(Controller.m_vHoldLocation);
                DrawProfileHeaderText("Forced Target: ");
                DrawProfileText(Controller.PreferredTarget);
                DrawProfileHeaderText("Preferred Target: ");
                DrawProfileText(Controller.ForcedTarget);
                DrawProfileHeaderText("Number of orders: ");
                DrawProfileText(Controller.m_Orders.Length);
                DrawProfileText(" ");
                DrawProfileHeaderText("In Cover: ");
                DrawProfileText(Pawn.IsInCover());
                DrawProfileHeaderText("In Combat: ");
                DrawProfileText(Pawn.InCombat());
                DrawProfileHeaderText("Move Goal: ");
                DrawProfileText(Controller.MoveGoal);
                DrawProfileText(" ");
                DrawProfileHeaderText("Current Weapon: ");
                DrawProfileText(Pawn.Weapon != None ? Pawn.Weapon.Class : "None");
                DrawProfileHeaderText("Current Power: ");
                if (Pawn.m_oBehavior.m_oCurrentModalActive != None)
                {
                    DrawProfileText(Controller.m_nmPowerToUse $ " (" $ Pawn.m_oBehavior.m_oCurrentModalActive.Name $ ")");
                }
                else
                {
                    DrawProfileText(Controller.m_nmPowerToUse);
                }
                DrawProfileText(" ");
                DrawProfileHeaderText("Current Health: ");
                DrawProfileText(PrettyFloat(Pawn.GetCurrentHealth()) $ " / " $ PrettyFloat(Pawn.GetMaxHealth()));
                DrawProfileHeaderText("Shields: ");
                DrawProfileText(PrettyFloat(Pawn.GetCurrentShields()) $ " / " $ PrettyFloat(Pawn.GetMaxShields()));
                DrawProfileText(" ");
                DrawProfileHeaderText("Time since last render: ");
                DrawProfileText(Pawn.GetTimeSinceLastRender());
                DrawProfileText(" ");
                SetProfileColumn(CurrentColumn + 2);
            }
        }
    }
}

function ProfileScaleform()
{
    local float fUpdateTime;
    local float fRenderTime;
    local float fAdvanceTime;
    local float fUpdateTotal;
    local float fRenderTotal;
    local float fAdvanceTotal;
    local BioSFManager pSFManager;
    local int nPanel;
    pSFManager = Outer.GetScaleFormManager();
    if (pSFManager == None)
    {
        return;
    }
    nPanel = 0;
    while (nPanel < pSFManager.m_lstPanels.Length)
    {
        pSFManager.m_lstPanels[nPanel].GetProfileTimes(fUpdateTime, fRenderTime, fAdvanceTime);
        DrawProfileHeaderText(pSFManager.m_lstPanels[nPanel].nmTag $ ":");
        DrawProfileText(" ");
        DrawProfileHeaderText("  Update:  ");
        DrawProfileText(fUpdateTime * 1000 $ "ms");
        DrawProfileHeaderText("  Render:  ");
        DrawProfileText(fRenderTime * 1000 $ "ms");
        DrawProfileHeaderText("  Advance: ");
        DrawProfileText(fAdvanceTime * 1000 $ "ms");
        DrawProfileHeaderText("  Total:   ");
        DrawProfileText(fUpdateTime + fRenderTime + fAdvanceTime * 1000 $ "ms");
        DrawProfileText(" ");
        fUpdateTotal += fUpdateTime;
        fRenderTotal += fRenderTime;
        fAdvanceTotal += fAdvanceTime;
        ++nPanel;
    }
    SetProfileColumn(++CurrentColumn);
    DrawProfileHeaderText("Total Update:   ");
    DrawProfileText(fUpdateTotal * 1000 $ "ms");
    DrawProfileHeaderText("Total Render:   ");
    DrawProfileText(fRenderTotal * 1000 $ "ms");
    DrawProfileHeaderText("Total Advance:  ");
    DrawProfileText(fAdvanceTotal * 1000 $ "ms");
    DrawProfileText(" ");
    DrawProfileHeaderText("Total Scaleform:");
    DrawProfileText(fAdvanceTotal + fRenderTotal + fUpdateTotal * 1000 $ "ms");
}

exec function CamLockCollision()
{
    Class'SFXCameraMode'.Default.Debugger.SetFlag(1, !Class'SFXCameraMode'.Default.Debugger.GetFlag(1));
}

exec function CamToggleDebug()
{
    Class'SFXCameraMode'.Default.Debugger.SetFlag(0, !Class'SFXCameraMode'.Default.Debugger.GetFlag(0));
}

exec function CamToggleHook()
{
    Class'SFXCameraMode'.Default.Debugger.SetFlag(2, !Class'SFXCameraMode'.Default.Debugger.GetFlag(2));
}

native final exec function PrintMuzzleLoc(String Target);

final exec function SetAimAssistValues(Vector2D Adhesion, float AimCorrection, Vector2D Friction)
{
    Outer.ConsoleCommand("set sfxweapon AdhesionRot (X=" $ Adhesion.X $ ", Y=" $ Adhesion.Y $ ")");
    Outer.ConsoleCommand("set sfxweapon AimCorrectionAmount " $ AimCorrection);
    Outer.ConsoleCommand("set sfxweapon FrictionMultiplierRange (X=" $ Friction.X $ ", Y=" $ Friction.Y $ ")");
}

final exec function AdjustCredits(int nCreditAdjust)
{
    local BioPawn oBioPawn;
    oBioPawn = BioPawn(Outer.Pawn);
    if (oBioPawn == None)
    {
        return;
    }
    oBioPawn.m_oBehavior.AdjustCredits(nCreditAdjust);
}

final exec function AdjustMediGel(int nMediGelAdjust)
{
    local BioPawn oBioPawn;
    oBioPawn = BioPawn(Outer.Pawn);
    if (oBioPawn == None)
    {
        return;
    }
    oBioPawn.m_oBehavior.AdjustMediGel(nMediGelAdjust);
}

exec function DebugCoverCheck()
{
    local BioHUD HUD;
    local BioPlayerController oController;
    oController = BioPlayerController(Outer.Pawn.Controller);
    HUD = BioHUD(Outer.myHUD);
    if (HUD != None)
    {
        HUD.SetDebugDraw(oController.DebugDraw_CoverCheck);
        Outer.ClientMessage("DebugCoverCheck" @ HUD.IsDrawing(oController.DebugDraw_CoverCheck));
    }
}

exec function ForceEndRagdoll()
{
    BioPawn(Outer.Pawn).ForceEndRagdoll();
}

native exec function BlazeTestDisconnect();

native exec function BlazeTestAcceptTOS(bool bAccept);

native exec function BlazeTestReconnect(SFXOnlineConnectMode connectMode);

native exec function BlazeTestSetNucleusRefused(bool bArg, optional bool bSave);

native exec function BlazeTestSetCerberusRefused(bool bArg, optional bool bSave);

native exec function BlazeTestSetNucleusSuccessful(bool bArg, optional bool bSave);

exec function BlazeTestSetAutoLoginOption(bool bAutoLogin)
{
    if (Outer.ProfileSettings != None)
    {
        Outer.ProfileSettings.SetProfileSettingValueId(44, bAutoLogin);
        Outer.bProfileSettingsUpdated = true;
    }
}

exec function BlazeTestJacobStory()
{
    local bool bStarted;
    local bool bFinished;
    bStarted = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.CheckEntitlement("MEJSOffers", "MEJacobStoryPlayed");
    bFinished = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.CheckEntitlement("MEJSOffers", "MEJacobStoryFinished");
    Outer.ClientMessage("Jacob's Story Played:" @ bStarted @ "Finished:" @ bFinished);
}

exec function BlazeTestConsumeCode(String sCode)
{
    local bool bMember;
    bMember = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentLogin().IsCerberusMember();
    Outer.ClientMessage("User is an entitled member of Cerberus Network:" @ bMember);
    if (!bMember)
    {
        Outer.ClientMessage("Consuming code" @ sCode);
        SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentLogin().SubmitRedeemCode(true, sCode);
    }
    else
    {
        Outer.ClientMessage("Already a member, not consuming code.");
    }
}

exec function BlazeTest_CreateGui()
{
    local MassEffectGuiManager oSFMgr;
    local SFXOnlineComponentUI oBlaze;
    oSFMgr = MassEffectGuiManager(Outer.GetScaleFormManager());
    oBlaze = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentUserInterface();
    if (oBlaze.m_oGUI == None)
    {
        if (oSFMgr != None)
        {
            oBlaze.SetGui(oSFMgr.CreateNetworkGUI(oBlaze.HandlerId));
        }
    }
}

exec function BlazeTest_DestroyGui()
{
    local MassEffectGuiManager oSFMgr;
    local SFXOnlineComponentUI oBlaze;
    oSFMgr = MassEffectGuiManager(Outer.GetScaleFormManager());
    oBlaze = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentUserInterface();
    if (oSFMgr != None && oBlaze != None)
    {
        oBlaze.ClearGui();
        oSFMgr.DestroyNetworkGUI(oBlaze.HandlerId);
    }
}

exec function BlazeTest_ShowIntroPage()
{
    local SFXOnlineComponentUI oBlaze;
    oBlaze = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentUserInterface();
    if (oBlaze != None)
    {
        if (oBlaze.m_oGUI != None)
        {
            oBlaze.ShowIntroPage();
        }
    }
}

exec function BlazeTest_ShowNucleusLogin(String Email, String Password, int eScreenState)
{
    local SFXOnlineComponentUI oBlaze;
    oBlaze = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentUserInterface();
    if (oBlaze != None)
    {
        if (oBlaze.m_oGUI != None)
        {
            oBlaze.ShowNucleusLogin(Email, Password, eScreenState);
        }
    }
}

exec function BlazeTest_ShowMessageBox(String sTitle, String sMessage, optional String sButton1Text, optional String sButton2Text, optional String sButton3Text)
{
    local SFXOnlineComponentUI oBlaze;
    oBlaze = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentUserInterface();
    if (oBlaze != None)
    {
        if (oBlaze.m_oGUI != None)
        {
            oBlaze.ShowMessageBox(sTitle, sMessage, sButton1Text, sButton2Text, sButton3Text);
        }
    }
}

exec function BlazeTest_ShowEmailPasswordMismatch(String Email, String Password)
{
    local SFXOnlineComponentUI oBlaze;
    oBlaze = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentUserInterface();
    if (oBlaze != None)
    {
        if (oBlaze.m_oGUI != None)
        {
            oBlaze.ShowEmailPasswordMismatch(Email, Password);
        }
    }
}

exec function BlazeTest_ShowCreateNucleusAccount(String sEmail, String sPassword, optional bool bEAProducts = false, optional bool bThirdParty = false, optional bool bRegisterProduct = false, optional bool bBioWareProducts = false, optional bool bUnderage = false)
{
    local SFXOnlineComponentUI oBlaze;
    oBlaze = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentUserInterface();
    if (oBlaze != None)
    {
        if (oBlaze.m_oGUI != None)
        {
            oBlaze.ShowCreateNucleusAccount(sEmail, sPassword, bEAProducts, bThirdParty, bRegisterProduct, bBioWareProducts, bUnderage);
        }
    }
}

exec function BlazeTest_ShowCreateNucleusAccountEx(String sEmail, String sPassword, bool bEAProducts, bool bThirdParty, bool bBioWareProducts, String i_sCountryCode, int BirthDay, int BirthMonth, int BirthYear)
{
    local SFXOnlineComponentUI oBlaze;
    local array<String> m_CountryCodeList;
    local array<String> m_CountryDisplayList;
    m_CountryCodeList.AddItem("CND");
    m_CountryDisplayList.AddItem("Canada");
    m_CountryCodeList.AddItem("US");
    m_CountryDisplayList.AddItem("United States");
    m_CountryCodeList.AddItem("JPN");
    m_CountryDisplayList.AddItem("Japan");
    oBlaze = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentUserInterface();
    if (oBlaze != None)
    {
        if (oBlaze.m_oGUI != None)
        {
            oBlaze.ShowCreateNucleusAccountEx(sEmail, sPassword, bEAProducts, bThirdParty, bBioWareProducts, i_sCountryCode, BirthDay, BirthMonth, BirthYear, "", m_CountryCodeList, m_CountryDisplayList);
        }
    }
}

exec function BlazeTest_ShowTermsOfService(String i_sTermsOfService, optional bool bTOSChanged = false)
{
    local SFXOnlineComponentUI oBlaze;
    oBlaze = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentUserInterface();
    if (oBlaze != None)
    {
        if (oBlaze.m_oGUI != None)
        {
            oBlaze.ShowTermsOfService(i_sTermsOfService, bTOSChanged);
        }
    }
}

exec function BlazeTest_ShowNucleusWelcomeMessage()
{
    local SFXOnlineComponentUI oBlaze;
    oBlaze = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentUserInterface();
    if (oBlaze != None)
    {
        if (oBlaze.m_oGUI != None)
        {
            oBlaze.ShowNucleusWelcomeMessage();
        }
    }
}

exec function BlazeTest_ShowCerberusIntro()
{
    local SFXOnlineComponentUI oBlaze;
    oBlaze = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentUserInterface();
    if (oBlaze != None)
    {
        if (oBlaze.m_oGUI != None)
        {
            oBlaze.ShowCerberusIntro();
        }
    }
}

exec function BlazeTest_ShowRedeemCode()
{
    local SFXOnlineComponentUI oBlaze;
    oBlaze = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentUserInterface();
    if (oBlaze != None)
    {
        if (oBlaze.m_oGUI != None)
        {
            oBlaze.ShowRedeemCode();
        }
    }
}

exec function BlazeTest_ShowAccountDemographics()
{
    local SFXOnlineComponentUI oBlaze;
    local array<String> m_CountryCodeList;
    local array<String> m_CountryDisplayList;
    m_CountryCodeList.AddItem("CND");
    m_CountryDisplayList.AddItem("Canada");
    m_CountryCodeList.AddItem("US");
    m_CountryDisplayList.AddItem("United States");
    m_CountryCodeList.AddItem("JPN");
    m_CountryDisplayList.AddItem("Japan");
    oBlaze = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentUserInterface();
    if (oBlaze != None)
    {
        if (oBlaze.m_oGUI != None)
        {
            oBlaze.ShowAccountDemographics(m_CountryCodeList, m_CountryDisplayList);
        }
    }
}

exec function BlazeTest_ShowParentEmail()
{
    local SFXOnlineComponentUI oBlaze;
    oBlaze = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentUserInterface();
    if (oBlaze != None)
    {
        if (oBlaze.m_oGUI != None)
        {
            oBlaze.ShowParentEmail();
        }
    }
}

exec function BlazeTest_ShowCerberusWelcomeMessage()
{
    local SFXOnlineComponentUI oBlaze;
    oBlaze = SFXGame(Outer.WorldInfo.Game).SFXOnlineSubsystem.GetComponentUserInterface();
    if (oBlaze != None)
    {
        if (oBlaze.m_oGUI != None)
        {
            oBlaze.ShowCerberusWelcomeMessage();
        }
    }
}

exec function MainMenu_AddCerberusItem(String i_sTitle, String i_sInfo)
{
    local MassEffectGuiManager oManager;
    local BioSFHandler_MainMenu oHandler;
    oManager = MassEffectGuiManager(Outer.GetScaleFormManager());
    if (oManager != None)
    {
        oHandler = oManager.GetMainMenuHandler();
        if (oHandler != None)
        {
            oHandler.AddCerberusItem(i_sTitle, i_sInfo, "");
        }
    }
}

exec function MainMenu_ClearNotifications()
{
    local MassEffectGuiManager oManager;
    local BioSFHandler_MainMenu oHandler;
    oManager = MassEffectGuiManager(Outer.GetScaleFormManager());
    if (oManager != None)
    {
        oHandler = oManager.GetMainMenuHandler();
        if (oHandler != None)
        {
            oHandler.ClearNotifications();
        }
    }
}

exec function DisplayNegotiate()
{
    local BioPawn MyPawn;
    MyPawn = BioPawn(Outer.Pawn);
    if (MyPawn != None && MyPawn.m_oBehavior != None)
    {
        Outer.ClientMessage("Negotiate skill is " $ MyPawn.m_oBehavior.GetIntimidateSkill());
    }
}

function String GetHenchCodename(String Henchman)
{
    switch (Henchman)
    {
        case "Miranda":
            return "Vixen";
        case "Jacob":
            return "Leading";
        case "Jack":
            return "Convict";
        case "Legion":
            return "Geth";
        case "Kasumi":
            return "Thief";
        case "Garrus":
            return "Garrus";
        case "Thane":
            return "Assassin";
        case "Tali":
            return "Tali";
        case "Mordin":
            return "Professor";
        case "Grunt":
            return "Grunt";
        case "Samara":
            return "Mystic";
        case "Zaeed":
            return "Veteran";
        default:
            return "";
    }
}

function int GetHenchIndex(String Henchman)
{
    switch (Henchman)
    {
        case "Miranda":
            return 0;
        case "Jacob":
            return 1;
        case "Jack":
            return 2;
        case "Legion":
            return 3;
        case "Kasumi":
            return 4;
        case "Garrus":
            return 5;
        case "Thane":
            return 6;
        case "Tali":
            return 7;
        case "Mordin":
            return 8;
        case "Grunt":
            return 9;
        case "Samara":
            return 10;
        case "Zaeed":
            return 11;
        default:
            return -1;
    }
}

function String GetHenchName(int Henchman)
{
    switch (Henchman)
    {
        case 0:
            return "Miranda";
        case 1:
            return "Jacob";
        case 2:
            return "Jack";
        case 3:
            return "Legion";
        case 4:
            return "Kasumi";
        case 5:
            return "Garrus";
        case 6:
            return "Thane";
        case 7:
            return "Tali";
        case 8:
            return "Mordin";
        case 9:
            return "Grunt";
        case 10:
            return "Samara";
        case 11:
            return "Zaeed";
        default:
            return "";
    }
}

event Actor GetHenchmanByName(String sName)
{
    local BioPawn Player;
    local BioPawn SquadMember;
    local int nIndex;
    Player = BioPawn(Outer.Pawn);
    if (Player != None && Player.Squad != None)
    {
        nIndex = 0;
        while (nIndex < Player.Squad.GetSquadSize())
        {
            SquadMember = BioPawn(Player.Squad.GetMember(nIndex));
            if (SquadMember != None && SquadMember.m_oBehavior != None)
            {
                if (InStr(Caps(SquadMember.m_oBehavior.GetActorGameName()), Caps(sName)) != -1)
                {
                    return SquadMember;
                }
            }
            nIndex++;
        }
    }
    return None;
}

function int GetHenchIsDeadGVarIndex(String Henchman)
{
    switch (Henchman)
    {
        case "Miranda":
            return 195;
        case "Jacob":
            return 196;
        case "Jack":
            return 197;
        case "Legion":
            return 198;
        case "Kasumi":
            return 199;
        case "Garrus":
            return 200;
        case "Thane":
            return 201;
        case "Tali":
            return 202;
        case "Mordin":
            return 203;
        case "Grunt":
            return 204;
        case "Samara":
            return 205;
        case "Zaeed":
            return 206;
        default:
            return -1;
    }
}

exec function SetHenchmanLoyalty(String Henchman, bool Value)
{
    local String HenchmanCodename;
    local Name nmLabel;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    HenchmanCodename = GetHenchCodename(Henchman);
    if (HenchmanCodename != "")
    {
        nmLabel = "IsLoyal" $ HenchmanCodename;
        GVars.SetBoolByName(nmLabel, Value);
        Outer.ClientMessage(Henchman @ "has been set" @ Value ? "" : "not" @ "loyal");
    }
    else
    {
        Outer.ClientMessage(Henchman @ "is not a Henchman name!");
    }
}

exec function GetHenchmanLoyalty(String Henchman)
{
    local String HenchmanCodename;
    local bool loyalty;
    local Name nmLabel;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    HenchmanCodename = GetHenchCodename(Henchman);
    if (HenchmanCodename != "")
    {
        nmLabel = "IsLoyal" $ HenchmanCodename;
        loyalty = GVars.GetBoolByName(nmLabel);
        Outer.ClientMessage(Henchman @ "is" @ loyalty ? "" : "not" @ "loyal.");
    }
    else
    {
        Outer.ClientMessage(Henchman @ "is not a Henchman name!");
    }
}

exec function SetHenchmanSpecialization(String Henchman, bool Value)
{
    local String HenchmanCodename;
    local Name nmLabel;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    HenchmanCodename = GetHenchCodename(Henchman);
    if (HenchmanCodename != "")
    {
        nmLabel = "IsSpecialized" $ HenchmanCodename;
        GVars.SetBoolByName(nmLabel, Value);
        Outer.ClientMessage(Henchman @ "has been set" @ Value ? "" : "not" @ "specialized");
    }
    else
    {
        Outer.ClientMessage(Henchman @ "is not a Henchman name!");
    }
}

exec function GetHenchmanSpecialization(String Henchman)
{
    local String HenchmanCodename;
    local bool specialized;
    local Name nmLabel;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    HenchmanCodename = GetHenchCodename(Henchman);
    if (HenchmanCodename != "")
    {
        nmLabel = "IsSpecialized" $ HenchmanCodename;
        specialized = GVars.GetBoolByName(nmLabel);
        Outer.ClientMessage(Henchman @ "is" @ specialized ? "" : "not" @ "specialized.");
    }
    else
    {
        Outer.ClientMessage(Henchman @ "is not a Henchman name!");
    }
}

exec function SetHenchmanDead(String Henchman, bool Value)
{
    local int GVarIndex;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    GVarIndex = GetHenchIsDeadGVarIndex(Henchman);
    if (GVarIndex != -1)
    {
        GVars.SetBool(GVarIndex, Value);
        Outer.ClientMessage(Henchman @ "has been set" @ Value ? "" : "not" @ "dead");
    }
    else
    {
        Outer.ClientMessage(Henchman @ "is not a Henchman name!");
    }
}

exec function GetHenchmanDead(String Henchman)
{
    local int GVarIndex;
    local bool Dead;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    GVarIndex = GetHenchIsDeadGVarIndex(Henchman);
    if (GVarIndex != -1)
    {
        Dead = GVars.GetBool(GVarIndex);
        Outer.ClientMessage(Henchman @ "is" @ Dead ? "" : "not" @ "dead.");
    }
    else
    {
        Outer.ClientMessage(Henchman @ "is not a Henchman name!");
    }
}

exec function SetHenchmanKnown(String Henchman, bool Value)
{
    local String HenchmanCodename;
    local Name nmLabel;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    HenchmanCodename = GetHenchCodename(Henchman);
    if (HenchmanCodename != "")
    {
        nmLabel = "KnowExist" $ HenchmanCodename;
        GVars.SetBoolByName(nmLabel, Value);
        Outer.ClientMessage(Henchman @ "has been set" @ Value ? "" : "not" @ "known");
    }
    else
    {
        Outer.ClientMessage(Henchman @ "is not a Henchman name!");
    }
}

exec function GetHenchmanKnown(String Henchman)
{
    local String HenchmanCodename;
    local bool known;
    local Name nmLabel;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    HenchmanCodename = GetHenchCodename(Henchman);
    if (HenchmanCodename != "")
    {
        nmLabel = "KnowExist" $ HenchmanCodename;
        known = GVars.GetBoolByName(nmLabel);
        Outer.ClientMessage(Henchman @ "is" @ known ? "" : "not" @ "known.");
    }
    else
    {
        Outer.ClientMessage(Henchman @ "is not a Henchman name!");
    }
}

exec function SetHenchmanAvailable(String Henchman, bool Value)
{
    local String HenchmanCodename;
    local Name nmLabel;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    HenchmanCodename = GetHenchCodename(Henchman);
    if (HenchmanCodename != "")
    {
        nmLabel = "IsSelectable" $ HenchmanCodename;
        GVars.SetBoolByName(nmLabel, Value);
        Outer.ClientMessage(Henchman @ "has been set" @ Value ? "" : "not" @ "available");
    }
    else
    {
        Outer.ClientMessage(Henchman @ "is not a Henchman name!");
    }
}

exec function GetHenchmanAvailable(String Henchman)
{
    local String HenchmanCodename;
    local bool available;
    local Name nmLabel;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    HenchmanCodename = GetHenchCodename(Henchman);
    if (HenchmanCodename != "")
    {
        nmLabel = "IsSelectable" $ HenchmanCodename;
        available = GVars.GetBoolByName(nmLabel);
        Outer.ClientMessage(Henchman @ "is" @ available ? "" : "not" @ "available.");
    }
    else
    {
        Outer.ClientMessage(Henchman @ "is not a Henchman name!");
    }
}

exec function SetHenchmanAppearance(String Henchman, int Value)
{
    local String HenchmanCodename;
    local Name nmLabel;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    HenchmanCodename = GetHenchCodename(Henchman);
    if (HenchmanCodename != "")
    {
        nmLabel = "Appearance" $ HenchmanCodename;
        GVars.SetIntByName(nmLabel, Value);
        Outer.ClientMessage(Henchman @ "appearance has been set to" @ Value);
    }
    else
    {
        Outer.ClientMessage(Henchman @ "is not a Henchman name!");
    }
}

exec function GetHenchmanAppearance(String Henchman)
{
    local String HenchmanCodename;
    local int Appearance;
    local Name nmLabel;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    HenchmanCodename = GetHenchCodename(Henchman);
    if (HenchmanCodename != "")
    {
        nmLabel = "Appearance" $ HenchmanCodename;
        Appearance = GVars.GetIntByName(nmLabel);
        Outer.ClientMessage(Henchman @ "appearance is" @ Appearance);
    }
    else
    {
        Outer.ClientMessage(Henchman @ "is not a Henchman name!");
    }
}

exec function SetFireTeam(String Henchman)
{
    local int HenchmanIndex;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    HenchmanIndex = GetHenchIndex(Henchman);
    if (HenchmanIndex == -1)
    {
        GVars.SetIntByName('FireTeamLeader', 0);
        Outer.ClientMessage("Fire team leader has been cleared");
    }
    else
    {
        GVars.SetIntByName('FireTeamLeader', HenchmanIndex + 1);
        Outer.ClientMessage("Fire team leader has been set to" @ GetHenchName(HenchmanIndex));
    }
}

exec function GetFireTeam()
{
    local String Henchman;
    local int HenchmanIndex;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    HenchmanIndex = GVars.GetIntByName('FireTeamLeader') - 1;
    if (HenchmanIndex == -1)
    {
        Outer.ClientMessage("Fire team leader is not set");
    }
    else
    {
        Henchman = GetHenchName(HenchmanIndex);
        Outer.ClientMessage("Fire team leader is" @ Henchman);
    }
}

exec function SetSpecialist(String Henchman)
{
    local int HenchmanIndex;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    HenchmanIndex = GetHenchIndex(Henchman);
    if (HenchmanIndex == -1)
    {
        GVars.SetIntByName('Specialist', 0);
        Outer.ClientMessage("Specialist has been cleared");
    }
    else
    {
        GVars.SetIntByName('Specialist', HenchmanIndex + 1);
        Outer.ClientMessage("Specialist has been set to" @ GetHenchName(HenchmanIndex));
    }
}

exec function GetSpecialist()
{
    local String Henchman;
    local int HenchmanIndex;
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    HenchmanIndex = GVars.GetIntByName('Specialist') - 1;
    Henchman = GetHenchName(HenchmanIndex);
    if (HenchmanIndex == -1)
    {
        Outer.ClientMessage("Specialist is not set");
    }
    else
    {
        Henchman = GetHenchName(HenchmanIndex);
        Outer.ClientMessage("Specialist is" @ Henchman);
    }
}

exec function SetMorinth(bool Value)
{
    local BioGlobalVariableTable GVars;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    GVars.SetBoolByName('Morinth_not_Samara', Value);
    Outer.ClientMessage(Value ? "Morinth" : "Samara");
}

exec function GetMorinth()
{
    local BioGlobalVariableTable GVars;
    local bool Value;
    GVars = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    Value = GVars.GetBoolByName('Morinth_not_Samara');
    Outer.ClientMessage(Value ? "Morinth" : "Samara");
}

exec function MapPower(Name PowerName)
{
    if (BioPlayerInput(Outer.PlayerInput).m_nmMappedPower != PowerName)
    {
        BioPlayerInput(Outer.PlayerInput).m_nmMappedPower2 = BioPlayerInput(Outer.PlayerInput).m_nmMappedPower;
        BioPlayerInput(Outer.PlayerInput).m_nmMappedPower = PowerName;
    }
    Outer.ClientMessage("Mapped powers set to" @ PowerName @ " and " @ BioPlayerInput(Outer.PlayerInput).m_nmMappedPower2);
}

final private function class<SFXPower> FindPowerClass(String PowerClassName)
{
    local class<SFXPower> FoundClass;
    FoundClass = Class<SFXPower>(FindObject(PowerClassName, Class'Class'));
    if (FoundClass == None && Class'SFXEngine'.IsSeekFreeObjectSupported(PowerClassName))
    {
        FoundClass = Class<SFXPower>(Class'SFXEngine'.LoadSeekFreeObject(PowerClassName, Class'Class'));
    }
    if (FoundClass == None)
    {
        FoundClass = Class<SFXPower>(DynamicLoadObject(PowerClassName, Class'Class'));
    }
    return FoundClass;
}

exec function GivePower(String nmPawn, String PowerClassName)
{
    local class<SFXPower> PowerClass;
    local BioPawn oPawn;
    local SFXPower oPower;
    PowerClass = FindPowerClass(PowerClassName);
    if (PowerClass == None)
    {
        PowerClass = FindPowerClass("SFXGameContent_Powers." $ PowerClassName);
    }
    if (PowerClass == None)
    {
        Outer.ClientMessage("Unable to give power - Failed to load power " $ PowerClassName);
        return;
    }
    oPawn = BioPawn(GetActorFromString(nmPawn));
    if (oPawn == None || oPawn.PowerManager == None)
    {
        Outer.ClientMessage("Unable to give power - " $ nmPawn $ " is not a valid pawn name");
        return;
    }
    oPawn.PowerManager.AddPower(PowerClass);
    oPower = oPawn.PowerManager.GetPowerByClass(PowerClass);
    if (oPower != None)
    {
        Outer.ClientMessage("The power has been given to the pawn");
        oPower.Rank = 1;
    }
    else
    {
        Outer.ClientMessage("Unable to give the power to the pawn");
    }
}

exec function SetRank(String nmPawn, class<Object> PowerClass, float fRank)
{
    local BioPawn oPawn;
    local SFXPower oPower;
    oPawn = BioPawn(GetActorFromString(nmPawn));
    if (oPawn == None || oPawn.PowerManager == None)
    {
        Outer.ClientMessage("Unable to set the rank - " $ nmPawn $ " is not a valid pawn name");
        return;
    }
    oPower = oPawn.PowerManager.GetPowerByClass(PowerClass);
    if (oPower != None)
    {
        if (fRank == oPawn.PowerManager.EvolveRank && oPower.EvolvedPowerClass1 != None || oPower.EvolvedPowerClass2 != None)
        {
            Outer.ClientMessage("Unable to set the rank - it would cause the power to evolve");
            return;
        }
        oPower.Rank = fRank;
        Outer.ClientMessage("The rank has been set");
    }
    else
    {
        Outer.ClientMessage("Unable to set the rank - the pawn does not have the " $ PowerClass $ " power");
    }
}

exec function RemovePower(String nmPawn, class<Object> PowerClass)
{
    local BioPawn oPawn;
    oPawn = BioPawn(GetActorFromString(nmPawn));
    if (oPawn == None || oPawn.PowerManager == None)
    {
        Outer.ClientMessage("Unable to remove power - " $ nmPawn $ " is not a valid pawn name");
        return;
    }
    if (oPawn.PowerManager.RemovePower(PowerClass))
    {
        Outer.ClientMessage("The power has been removed from the pawn");
    }
    else
    {
        Outer.ClientMessage("Unable to remove the power from the pawn");
    }
}

exec function GiveEffect(String nmPawn, String EffectClassName)
{
    local BioPawn oPawn;
    local SFXGameEffect Effect;
    local class<SFXGameEffect> EffectClass;
    local SFXModule_GameEffectManager Manager;
    EffectClass = FindSFXGameEffectClass(EffectClassName);
    if (EffectClass == None)
    {
        EffectClass = FindSFXGameEffectClass("SFXGame." $ EffectClassName);
        if (EffectClass == None)
        {
            EffectClass = FindSFXGameEffectClass("SFXGameContent_Powers." $ EffectClassName);
            if (EffectClass == None)
            {
                Outer.ClientMessage("Unable to give effect - Failed to load effect " $ EffectClassName);
                return;
            }
        }
    }
    oPawn = BioPawn(GetActorFromString(nmPawn));
    if (oPawn == None)
    {
        Outer.ClientMessage("Unable to give effect - " $ nmPawn $ " is not a valid pawn name");
        return;
    }
    Manager = oPawn.GetModule(Class'SFXModule_GameEffectManager');
    if (Manager == None)
    {
        Outer.ClientMessage("Unable to give effect - " $ nmPawn $ " does not have an SFXModule_GameEffectManager");
        return;
    }
    Effect = Manager.CreateEffect(EffectClass, EffectClass.Default.Category, 0, 2, EffectClass.Default.EffectValue);
    if (Effect != None)
    {
        Outer.ClientMessage("The effect has been given to the pawn");
        Effect.OnApplied();
    }
    else
    {
        Outer.ClientMessage("Failed to create the effect");
    }
}

exec function GiveEffectWithValue(String nmPawn, String EffectClassName, float Value)
{
    local BioPawn oPawn;
    local SFXGameEffect Effect;
    local class<SFXGameEffect> EffectClass;
    local SFXModule_GameEffectManager Manager;
    EffectClass = FindSFXGameEffectClass(EffectClassName);
    if (EffectClass == None)
    {
        EffectClass = FindSFXGameEffectClass("SFXGame." $ EffectClassName);
        if (EffectClass == None)
        {
            EffectClass = FindSFXGameEffectClass("SFXGameContent_Powers." $ EffectClassName);
            if (EffectClass == None)
            {
                Outer.ClientMessage("Unable to give effect - Failed to load effect " $ EffectClassName);
                return;
            }
        }
    }
    oPawn = BioPawn(GetActorFromString(nmPawn));
    if (oPawn == None)
    {
        Outer.ClientMessage("Unable to give effect - " $ nmPawn $ " is not a valid pawn name");
        return;
    }
    Manager = oPawn.GetModule(Class'SFXModule_GameEffectManager');
    if (Manager == None)
    {
        Outer.ClientMessage("Unable to give effect - " $ nmPawn $ " does not have an SFXModule_GameEffectManager");
        return;
    }
    Effect = Manager.CreateEffect(EffectClass, EffectClass.Default.Category, 0, 2, Value);
    if (Effect != None)
    {
        Outer.ClientMessage("The effect has been given to the pawn");
        Effect.OnApplied();
    }
    else
    {
        Outer.ClientMessage("Failed to create the effect");
    }
}

exec function RemoveEffect(String nmPawn, String EffectClassName)
{
    local BioPawn oPawn;
    local class<SFXGameEffect> EffectClass;
    local SFXModule_GameEffectManager Manager;
    EffectClass = FindSFXGameEffectClass(EffectClassName);
    if (EffectClass == None)
    {
        EffectClass = FindSFXGameEffectClass("SFXGame." $ EffectClassName);
        if (EffectClass == None)
        {
            EffectClass = FindSFXGameEffectClass("SFXGameContent_Powers." $ EffectClassName);
            if (EffectClass == None)
            {
                Outer.ClientMessage("Unable to find effect - Failed to load effect " $ EffectClassName);
                return;
            }
        }
    }
    oPawn = BioPawn(GetActorFromString(nmPawn));
    if (oPawn == None || oPawn.PowerManager == None)
    {
        Outer.ClientMessage("Unable to get effect - " $ nmPawn $ " is not a valid pawn name");
        return;
    }
    Manager = oPawn.GetModule(Class'SFXModule_GameEffectManager');
    if (Manager == None)
    {
        Outer.ClientMessage("Unable to get effect - " $ nmPawn $ " does not have an SFXModule_GameEffectManager");
        return;
    }
    Manager.RemoveEffectsByType(EffectClass);
    Outer.ClientMessage("All effects of type " $ EffectClassName $ " have been removed");
}

exec function ProfileEffects()
{
    local SFXModule_GameEffectManager Manager;
    local SFXGameEffect Effect;
    local String output;
    if (ProfileTarget == None)
    {
        return;
    }
    Manager = ProfileTarget.GetModule(Class'SFXModule_GameEffectManager');
    if (Manager == None)
    {
        return;
    }
    foreach Manager.GameEffects(Effect)
    {
        DrawProfileHeaderText("Class: ");
        output = Effect.Class $ "    Category: " $ Effect.Category $ "    Value: " $ Effect.EffectValue;
        if (Effect.DurationType == 1)
        {
            output = output $ "    Duration: " $ Effect.Duration - Effect.CurrentTime;
        }
        DrawProfileText(output);
    }
}

exec function ResetTalents(String nmPawn)
{
    local BioPawn oPawn;
    oPawn = BioPawn(GetActorFromString(nmPawn));
    if (oPawn == None)
    {
        Outer.ClientMessage("Unable to get pawn - " $ nmPawn $ " is not a valid pawn name");
        return;
    }
    if (oPawn.PowerManager == None)
    {
        Outer.ClientMessage("Unable to get the pawn's power manager - " $ nmPawn);
        return;
    }
    oPawn.PowerManager.RefundAllTalentPoints();
    Outer.ClientMessage("All talent points reset for pawn - " $ nmPawn);
}

exec function GUICMD(Name nmPanelTag, String sCmd, optional String sArg1, optional String sArg2)
{
    local MassEffectGuiManager oSFMgr;
    local String sResultMessage;
    oSFMgr = MassEffectGuiManager(Outer.GetScaleFormManager());
    if (oSFMgr != None)
    {
        oSFMgr.GUICMD(sResultMessage, nmPanelTag, sCmd, sArg1, sArg2);
        Outer.ClientMessage(sResultMessage);
    }
    else
    {
        Outer.ClientMessage("GUICMD: Cannot find gui manager to whom to issue command.");
    }
}

exec function ShowDebugText(bool bShow)
{
    if (Outer.WorldInfo != None)
    {
        Outer.WorldInfo.bShowDebugText = bShow;
    }
}

exec function TeleportHenchman(String nmPawn)
{
    local BioPawn oPawn;
    local SFXAI_Henchman oController;
    local Pawn oPlayer;
    oPawn = BioPawn(GetActorFromString(nmPawn));
    if (oPawn == None || oPawn.Squad == None)
    {
        return;
    }
    oController = SFXAI_Henchman(oPawn.Controller);
    if (oController == None)
    {
        return;
    }
    oPlayer = oPawn.Squad.GetSquadLeader();
    if (oPlayer != None)
    {
        return;
    }
    oController.TeleportToActor(oPlayer);
}

native exec function Unreaper();

native function UpdateSplitscreenPlayers(bool bCreatePlayers);

exec function Splitscreen()
{
    local SFXPlayerController PC;
    UpdateSplitscreenPlayers(true);
    foreach SFXGame(Outer.WorldInfo.Game).LocalPlayerControllers(Class'SFXPlayerController', PC)
    {
        if (PC.Pawn == None)
        {
            PC.ServerBecomeActivePlayer();
        }
    }
}

exec function GetParagon()
{
    local SFXGame Game;
    Game = Outer.WorldInfo != None ? SFXGame(Outer.WorldInfo.Game) : None;
    if (Game != None)
    {
        Outer.ClientMessage("The player has " $ Game.GetParagonPoints() $ " paragon points");
    }
}

exec function SetParagon(int nPoints)
{
    local SFXGame Game;
    Game = Outer.WorldInfo != None ? SFXGame(Outer.WorldInfo.Game) : None;
    if (Game != None)
    {
        Game.SetParagonPoints(nPoints);
        Outer.ClientMessage("The player's paragon points have been set to " $ nPoints);
    }
}

exec function GetRenegade()
{
    local SFXGame Game;
    Game = Outer.WorldInfo != None ? SFXGame(Outer.WorldInfo.Game) : None;
    if (Game != None)
    {
        Outer.ClientMessage("The player has " $ Game.GetRenegadePoints() $ " renegade points");
    }
}

exec function SetRenegade(int nPoints)
{
    local SFXGame Game;
    Game = Outer.WorldInfo != None ? SFXGame(Outer.WorldInfo.Game) : None;
    if (Game != None)
    {
        Game.SetRenegadePoints(nPoints);
        Outer.ClientMessage("The player's renegade points have been set to " $ nPoints);
    }
}

exec function GetCharm()
{
    local BioPawnBehavior Behavior;
    Behavior = Outer.Pawn != None ? BioPawnBehavior(Outer.Pawn.oBioComponent) : None;
    if (Behavior != None)
    {
        Outer.ClientMessage("The player's charm skill is " $ Behavior.GetCharmSkill());
    }
}

exec function GetIntimidate()
{
    local BioPawnBehavior Behavior;
    Behavior = Outer.Pawn != None ? BioPawnBehavior(Outer.Pawn.oBioComponent) : None;
    if (Behavior != None)
    {
        Outer.ClientMessage("The player's intimidate skill is " $ Behavior.GetIntimidateSkill());
    }
}

exec function SetUgly(int Ugliness)
{
    local SFXPawn_Player Player;
    if (Ugliness < 0 || Ugliness > 4)
    {
        Outer.ClientMessage("Valid range is from [0,4]");
        return;
    }
    Player = SFXPawn_Player(Outer.Pawn);
    if (Player != None)
    {
        switch (Ugliness)
        {
            case 0:
                Outer.ConsoleCommand("SetRenegade 0");
                Outer.ConsoleCommand("SetParagon 2000");
                break;
            case 1:
                Outer.ConsoleCommand("SetRenegade 0");
                Outer.ConsoleCommand("SetParagon 350");
                break;
            case 2:
                Outer.ConsoleCommand("SetRenegade 0");
                Outer.ConsoleCommand("SetParagon 0");
                break;
            case 3:
                Outer.ConsoleCommand("SetRenegade 350");
                Outer.ConsoleCommand("SetParagon 0");
                break;
            case 4:
                Outer.ConsoleCommand("SetRenegade 2000");
                Outer.ConsoleCommand("SetParagon 0");
                break;
        }
        Player.UpdateAppearance();
    }
}

final exec function ImportAllCareers()
{
    local BioPlayerController PC;
    local OnlineSubsystem OnlineSub;
    local int ControllerId;
    if (!Class'WorldInfo'.IsConsoleBuild(1))
    {
        return;
    }
    PC = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController();
    OnlineSub = Class'GameEngine'.GetOnlineSubsystem();
    if (PC != None && OnlineSub != None)
    {
        PlayerIntEx = OnlineSub.PlayerInterfaceEx;
        ControllerId = LocalPlayer(PC.Player).ControllerId;
        if (NotEqual_InterfaceInterface(PlayerIntEx, OnlinePlayerInterfaceEx(None)))
        {
            PlayerIntEx.AddDeviceSelectionDoneDelegate(ControllerId, ImportAllCareers_OnDeviceSelectionComplete);
            if (PlayerIntEx.ShowDeviceSelectionUI(ControllerId, 1048576, true))
            {
                return;
            }
            else
            {
                PlayerIntEx.ClearDeviceSelectionDoneDelegate(ControllerId, ImportAllCareers_OnDeviceSelectionComplete);
            }
        }
    }
}

final function ImportAllCareers_OnDeviceSelectionComplete(bool bWasSuccessful)
{
    local BioPlayerController PC;
    local OnlineSubsystem OnlineSub;
    local int ControllerId;
    local int DeviceID;
    local String DeviceName;
    PC = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController();
    OnlineSub = Class'GameEngine'.GetOnlineSubsystem();
    if (PC != None && OnlineSub != None)
    {
        PlayerIntEx = OnlineSub.PlayerInterfaceEx;
        ControllerId = LocalPlayer(PC.Player).ControllerId;
        if (NotEqual_InterfaceInterface(PlayerIntEx, OnlinePlayerInterfaceEx(None)))
        {
            PlayerIntEx.ClearDeviceSelectionDoneDelegate(ControllerId, ImportAllCareers_OnDeviceSelectionComplete);
            if (bWasSuccessful)
            {
                DeviceID = PlayerIntEx.GetDeviceSelectionResults(ControllerId, DeviceName);
                ImportAllCareersXenon(DeviceID);
                return;
            }
        }
    }
}

native final function bool ImportAllCareersXenon(int TargetDeviceId);

native final exec function SaveGame(String SaveName);

native final exec function LoadGame(String LoadName);

native final exec function LoadExternalSave(String SavePath);

final event int GetCharacterClassName(class<SFXPawn_Player> PlayerClass)
{
    local SFXEngine Engine;
    local class<SFXCharacterClass> CharacterClass;
    Engine = SFXEngine(Outer.Player.Outer);
    CharacterClass = Class<SFXCharacterClass>(Engine.LoadSeekFreeObject(PlayerClass.Default.PlayerClassName, Class'Class'));
    Class'SFXEngine'.ReleaseSeekFreeObject(PlayerClass.Default.PlayerClassName);
    return CharacterClass.Default.srClassName;
}

function ProfileDifficulty()
{
    local SFXPRI PRI;
    local SFXGame Game;
    local int ChallengeLevel;
    local float ChallengeModifier;
    local float LevelModifier;
    local float TechModifier;
    PRI = Class'SFXPRI'.FindLocalPRI();
    Game = SFXGame(Outer.WorldInfo.Game);
    Game.GetChallengeLevel(0, ChallengeLevel);
    Game.GetChallengeModifier(0, ChallengeModifier, LevelModifier, TechModifier);
    DrawProfileText("PRI: " @ PRI);
    DrawProfileText("Difficulty: " @ SFXGame(Outer.WorldInfo.Game).Difficulty);
    DrawProfileText("DifficultyScore: " @ PRI.DifficultyScore);
    DrawProfileText("Health Multiplier: " @ PRI.CurrentHealthMultiplier);
    DrawProfileText("Max Attack Tickets: " @ PRI.CurrentMaxAttackTickets);
    DrawProfileText("Max Target Tickets: " @ PRI.CurrentMaxTargetTickets);
    DrawProfileText("Shield Recharge Modifier:" @ PRI.CurrentShieldModifier);
    DrawProfileText("----------------------------------------");
    DrawProfileText("Challenge Level: " @ ChallengeLevel);
    DrawProfileText("Challenge Modifier: " @ ChallengeModifier);
}

exec function DisplayTime()
{
}

exec function OpenGui(Name HandlerId)
{
    local MassEffectGuiManager oManager;
    oManager = MassEffectGuiManager(Outer.GetScaleFormManager());
    oManager.CreatePanel(HandlerId, true);
}

exec function CloseGui(Name HandlerId)
{
    local MassEffectGuiManager oManager;
    oManager = MassEffectGuiManager(Outer.GetScaleFormManager());
    oManager.RemovePanel(oManager.GetPanelByTag(HandlerId));
}

function class<SFXGameEffect> FindSFXGameEffectClass(String ClassName)
{
    local class<SFXGameEffect> FoundClass;
    FoundClass = Class<SFXGameEffect>(FindObject(ClassName, Class'Class'));
    if (FoundClass == None && Class'SFXEngine'.IsSeekFreeObjectSupported(ClassName))
    {
        FoundClass = Class<SFXGameEffect>(Class'SFXEngine'.LoadSeekFreeObject(ClassName, Class'Class'));
    }
    if (FoundClass == None)
    {
        FoundClass = Class<SFXGameEffect>(DynamicLoadObject(ClassName, Class'Class'));
    }
    return FoundClass;
}

exec function ApplyAchPartyHealthBonus()
{
    local BioPawn Player;
    local SFXModule_GameEffectManager Manager;
    local class<SFXGameEffect> EffectClass;
    local SFXGameEffect Effect;
    Player = BioPawn(Outer.Pawn);
    if (Player != None)
    {
        Manager = Player.GetModule(Class'SFXModule_GameEffectManager');
        if (Manager != None)
        {
            EffectClass = FindSFXGameEffectClass("SFXGameContent_Bonuses.SFXGameEffect_AchievementPartyHealthBonus");
            Effect = Manager.CreateEffect(EffectClass, EffectClass.Default.Category, 0, 2, EffectClass.Default.EffectValue);
            if (Effect != None)
            {
                Effect.OnApplied();
            }
        }
    }
}

exec function ApplyAchPartyDamageBonus()
{
    local BioPawn Player;
    local SFXModule_GameEffectManager Manager;
    local class<SFXGameEffect> EffectClass;
    local SFXGameEffect Effect;
    Player = BioPawn(Outer.Pawn);
    if (Player != None)
    {
        Manager = Player.GetModule(Class'SFXModule_GameEffectManager');
        if (Manager != None)
        {
            EffectClass = FindSFXGameEffectClass("SFXGameContent_Bonuses.SFXGameEffect_AchievementPartyDamageBonus");
            Effect = Manager.CreateEffect(EffectClass, EffectClass.Default.Category, 0, 2, EffectClass.Default.EffectValue);
            if (Effect != None)
            {
                Effect.OnApplied();
            }
        }
    }
}

exec function UpdateBonusListCache()
{
    SFXGame(Outer.Pawn.WorldInfo.Game).BonusList.UpdateCache(BioWorldInfo(Outer.Pawn.WorldInfo));
}

exec function UpdateShields()
{
    BioPawn(Outer.Pawn).ScaleEquipment(BioPawn(Outer.Pawn).CharacterLevel);
}

exec function UpdateHealth()
{
    BioPawn(Outer.Pawn).GetModule(Class'SFXModule_Damage').ScaleHealth();
}

exec function CerberusDLC(bool bAvail)
{
    local MassEffectGuiManager oManager;
    local BioSFHandler_MainMenu oHandler;
    oManager = MassEffectGuiManager(Outer.GetScaleFormManager());
    if (oManager != None)
    {
        oHandler = oManager.GetMainMenuHandler();
        if (oHandler != None)
        {
            oHandler.HasCerberusDLC(bAvail);
        }
    }
}

exec function ShowWwiseVolumeLocations(String nmVolume)
{
    local WwiseAudioVolume AudioVolume;
    AudioVolume = WwiseAudioVolume(GetActorFromString(nmVolume));
    if (AudioVolume != None)
    {
        AudioVolume.DrawSoundLocations = true;
        AudioVolume.m_oTrackTimer.SetTimer(0,01, true);
        Outer.ClientMessage("Drawing sound locations for" @ nmVolume);
    }
}

exec function HideWwiseVolumeLocations(String nmVolume)
{
    local WwiseAudioVolume AudioVolume;
    AudioVolume = WwiseAudioVolume(GetActorFromString(nmVolume));
    if (AudioVolume != None)
    {
        AudioVolume.DrawSoundLocations = false;
        Outer.ClientMessage("Hiding sound locations for" @ nmVolume);
    }
}

exec function GetPlotVariableID(Name plotVarName)
{
    local BioGlobalVariableTable oGV;
    local int Index;
    oGV = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    oGV.oNameLookupTable.GetIntEntryNN(plotVarName, 'Index', Index);
    Outer.ClientMessage("" $ Index);
}

exec function DebugSetPlotState(Name Type, Name plotVarName, float Value)
{
    local BioGlobalVariableTable oGV;
    oGV = BioWorldInfo(Outer.WorldInfo).GetGlobalVariables();
    if (Type == 'Float')
    {
        oGV.SetFloatByName(plotVarName, Value);
    }
    else
    {
        if (Type == 'Int')
        {
            oGV.SetIntByName(plotVarName, Value);
        }
        else
        {
            if (Type == 'Bool')
            {
                oGV.SetIntByName(plotVarName, Value);
                jump 196;
            }
        }
    }
    return;
}

exec function TogglePlayersOnly()
{
    BioWorldInfo(Outer.WorldInfo).bPlayersOnly = !BioWorldInfo(Outer.WorldInfo).bPlayersOnly;
}

exec function UnlockAchievement(EAchievementID AchievementId)
{
    Outer.UnlockAchievement(AchievementId);
}

exec function UnlockAllAchievements()
{
    local int idx;
    idx = 1;
    while (idx <= 50)
    {
        if (Outer.OnlineSub != None && NotEqual_InterfaceInterface(Outer.OnlineSub.PlayerInterfaceEx, OnlinePlayerInterfaceEx(None)))
        {
            Outer.OnlineSub.PlayerInterfaceEx.UnlockAchievement(0, idx);
        }
        idx++;
    }
}

exec function UnlockAchievementRange(int Start, int End)
{
    local int idx;
    local int Lower;
    local int Upper;
    if (Start < End)
    {
        Lower = Start;
        Upper = End;
    }
    else
    {
        Lower = End;
        Upper = Start;
    }
    idx = Lower;
    while (idx <= Upper)
    {
        if (Outer.OnlineSub != None && NotEqual_InterfaceInterface(Outer.OnlineSub.PlayerInterfaceEx, OnlinePlayerInterfaceEx(None)))
        {
            Outer.OnlineSub.PlayerInterfaceEx.UnlockAchievement(0, idx);
        }
        idx++;
    }
}

exec function UnlockAllGamerpics()
{
    Outer.OnlineSub.PlayerInterfaceEx.UnlockGamerPicture(0, 1);
    Outer.OnlineSub.PlayerInterfaceEx.UnlockGamerPicture(0, 2);
}

exec function ReadAchievements(optional int TitleId)
{
    local int Index;
    local array<AchievementDetails> Details;
    if (Outer.OnlineSub != None && NotEqual_InterfaceInterface(Outer.OnlineSub.PlayerInterfaceEx, OnlinePlayerInterfaceEx(None)))
    {
        Outer.OnlineSub.PlayerInterfaceEx.GetAchievements(0, Details, TitleId);
        Index = 0;
        while (Index < Details.Length)
        {
            Index++;
        }
    }
}

exec function EnableFaceFX(bool bEnable)
{
    local Pawn oPawn;
    foreach Outer.AllActors(Class'Pawn', oPawn)
    {
        if (oPawn.Mesh != None)
        {
            oPawn.Mesh.BioEnableFaceFX(bEnable);
        }
    }
}

native final exec function LogAnimSequences();

native final exec function LogActorPawns();

native final exec function LogActorSkelMeshes();

native final exec function LogActorEmitters();

native final exec function LogActorPrefabInstances();

native final exec function LogActorArtPlaceables();

native final exec function LogActorUnclassified();

native final exec function LogActorClasses();

exec function ShowOnlineMarketplace()
{
    local SFXOfferDescriptor oOffer;
    local array<SFXOfferDescriptor> aOffers;
    local MassEffectGuiManager oManager;
    local BioSFHandler_MainMenu oHandler;
    oManager = MassEffectGuiManager(Outer.GetScaleFormManager());
    if (oManager != None)
    {
        oHandler = oManager.GetMainMenuHandler();
        if (oHandler != None)
        {
            oOffer.Description = "Quick Brown fox jumped over the lazy dog.";
            oOffer.Name = "Lazy Dog";
            oOffer.internalId = 0;
            oOffer.Price = 800;
            oOffer.UserHasIt = false;
            aOffers.AddItem(oOffer);
            oOffer.Description = "Some cool new armors.";
            oOffer.Name = "Armor Pack 1";
            oOffer.internalId = 1;
            oOffer.Price = 10000;
            oOffer.UserHasIt = true;
            aOffers.AddItem(oOffer);
            oOffer.Description = "Some cool Level";
            oOffer.Name = "LEvel Pack 1";
            oOffer.internalId = 2;
            oOffer.Price = 1200;
            oOffer.UserHasIt = false;
            aOffers.AddItem(oOffer);
            oHandler.OpenMarketplaceGui(aOffers);
        }
    }
}

exec function Speak(byte EventID, String sInstigator, optional String sRecipient)
{
    local BioPawn TheInstigator;
    local BioPawn TheRecipient;
    TheInstigator = BioPawn(GetActorFromString(sInstigator));
    if (TheInstigator == None)
    {
        return;
    }
    if (Len(sRecipient) > 0)
    {
        TheRecipient = BioPawn(GetActorFromString(sRecipient));
        if (TheRecipient == None)
        {
            return;
        }
    }
    SFXGame(Outer.WorldInfo.Game).VocManager.bAlwaysPlay = true;
    SFXGame(Outer.WorldInfo.Game).TriggerVocalizationEvent(EventID, TheInstigator, TheRecipient, 0, 100);
    Outer.SetTimer(0,1, false, 'ResetAlwaysPlay', self);
}

function ResetAlwaysPlay()
{
    SFXGame(Outer.WorldInfo.Game).VocManager.bAlwaysPlay = false;
}

exec function HolsterWeapon(String nmPawn)
{
    local BioPawn oPawn;
    local SFXAI_Core oController;
    oPawn = BioPawn(GetActorFromString(nmPawn));
    if (oPawn == None)
    {
        return;
    }
    oController = SFXAI_Core(oPawn.Controller);
    if (oController == None)
    {
        return;
    }
    if (oPawn.IsReloading(true))
    {
        return;
    }
    if (oPawn.InCombat())
    {
        return;
    }
    if (oPawn.IsInCover())
    {
        oPawn.LeaveCover();
    }
    if (oPawn.m_oBehavior != None)
    {
        oPawn.m_oBehavior.SetActionState(0);
    }
}

exec function PowersWhileVisible()
{
    local BioPlayerSquad PlayerSquad;
    local BioAiController Controller;
    local SFXAI_Henchman Henchman;
    local bool bEnabled;
    PlayerSquad = SFXGame(Outer.WorldInfo.Game).PlayerSquad;
    if (PlayerSquad != None)
    {
        foreach PlayerSquad.SquadMembers(Controller)
        {
            Henchman = SFXAI_Henchman(Controller);
            if (Henchman != None)
            {
                Henchman.m_bAllowInstantPowerWhileVisible = !Henchman.m_bAllowInstantPowerWhileVisible;
                if (Henchman.m_bAllowInstantPowerWhileVisible)
                {
                    bEnabled = true;
                }
            }
        }
    }
    if (bEnabled)
    {
        Outer.ClientMessage("The henchmen can use powers instantly");
    }
    else
    {
        Outer.ClientMessage("The henchmen can not use powers instantly");
    }
}

exec function Test_Notifications()
{
    local BioHintSystem oHintSystem;
    local BioPlayerController oController;
    oController = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController();
    oHintSystem = BioHintSystem(oController.HintSystem);
    if (oHintSystem != None)
    {
        oHintSystem.AddNotification_LevelUp(2);
        oHintSystem.AddNotification_JournalChange();
        oHintSystem.AddNotification_CodexChange();
        oHintSystem.AddNotification_CreditRecovery(99);
        oHintSystem.AddNotification_MedigelRecovery(98);
        oHintSystem.AddNotification_ParagonChange(97);
        oHintSystem.AddNotification_RenegadeChange(96);
        oHintSystem.AddNotification_IridiumRecovery(95);
        oHintSystem.AddNotification_PalladiumRecovery(94);
        oHintSystem.AddNotification_PlatinumRecovery(93);
        oHintSystem.AddNotification_ElementZeroRecovery(92);
        oHintSystem.AddNotification_HeavyWeaponAmmoRecovery(91);
        oHintSystem.AddNotification_AmmoRecovery(90);
        oHintSystem.AddNotification_AchievementChange("Ach Title", "Ach SubTitle", "Ach Body", 5, 10);
        oHintSystem.AddNotification_HeavyAmmoFull();
        oHintSystem.AddNotification_MediGelFull();
        oHintSystem.AddNotification_SalvageRecovery(89);
        oHintSystem.AddNotification_ProbeRecovery(88);
        oHintSystem.AddNotification_LevelUp(87);
        oHintSystem.AddNotification_XP(86);
        oHintSystem.AddNotification_TechUnlocked("TechUnlocked Title", "TechUnlocked Name", "TechUnlocked Message");
        oHintSystem.AddNotification_Tech("Tech Title", "Tech Name", "Tech Message");
    }
}

exec function PauseNotifications(bool bPause)
{
    local BioHintSystem oHintSystem;
    local BioPlayerController oController;
    oController = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController();
    oHintSystem = BioHintSystem(oController.HintSystem);
    if (oHintSystem != None)
    {
        oHintSystem.SetNotificationPaused(bPause);
    }
}

exec function DumpSFXObjectPool()
{
    SFXGame(Outer.WorldInfo.Game).ObjectPool.DumpPools();
}

exec function TestPulseFullAmmo()
{
    MassEffectGuiManager(Outer.GetScaleFormManager()).m_oSFHudHandler.PulseFullAmmoMessage();
}

exec function TestSavingMessage(float fSaveTime)
{
    local SFXSFHandler_SavingDisplay oHandler;
    oHandler = MassEffectGuiManager(Outer.GetScaleFormManager()).m_oSavingDisplayHandler;
    oHandler.ShowSavingMessage(true);
    SFXSFPanel_SaveWidget(oHandler.oPanel).m_fElapsedSaveTime = fSaveTime;
}

exec function ToggleThreatRadius(optional String PawnName)
{
    local BioPawn oPawn;
    local SFXAI_Core oController;
    if (Len(PawnName) != 0)
    {
        oPawn = BioPawn(GetActorFromString(PawnName));
        if (oPawn == None)
        {
            return;
        }
        oController = SFXAI_Core(oPawn.Controller);
        if (oController == None)
        {
            return;
        }
        oController.bDebug_ThreatRadius = !oController.bDebug_ThreatRadius;
    }
    else
    {
        foreach Outer.AllActors(Class'BioPawn', oPawn)
        {
            if (oPawn != None)
            {
                oController = SFXAI_Core(oPawn.Controller);
                if (oController != None)
                {
                    oController.bDebug_ThreatRadius = !oController.bDebug_ThreatRadius;
                }
            }
        }
    }
    DebugAI();
}

exec function IgnorePlayer(bool bIgnore)
{
    local BioPawn Player;
    local BioAiController AI;
    Player = SFXGame(Outer.WorldInfo.Game).PlayerSquad.m_playerPawn;
    foreach Outer.WorldInfo.AllControllers(Class'BioAiController', AI)
    {
        if (SFXAI_Henchman(AI) == None)
        {
            if (bIgnore)
            {
                AI.AddIgnoredTarget(Player);
            }
            else
            {
                AI.RemoveIgnoredTarget(Player);
            }
        }
    }
    if (bIgnore)
    {
        Outer.ClientMessage("The player has been ignored");
    }
    else
    {
        Outer.ClientMessage("The player has not been ignored");
    }
}

defaultproperties
{
    AllProfiles=System.Byte[]
    ColumnWidth=350
    ProfileTitleColor=(B=51,G=153,R=255,A=255)
    ProfileHeaderColor=(B=205,G=204,R=204,A=255)
    ProfileTextColor=(B=255,G=255,R=255,A=255)
    ProfileHighlightColor=(B=255,G=255,R=0,A=255)
    m_bShowAngstIcons=true
    m_bEnablePowerCooldown=true
}
