﻿class BioCustomAction extends Object
    config(Game)
    abstract
    native;

var ESFXVocalizationEventID VocalizationEvent;

var const Rotator PreciseRotation;

var const float PreciseRotationInterpolationTime;

var const Vector PreciseDestination;

var transient float LastCanDoCustomActionTime;

var bool bForcePrecisePosition;

var const bool bReachedPreciseRotation;

var const bool bReachPreciseRotation;

var const bool bReachedPreciseDestination;

var const bool bReachPreciseDestination;

var bool bIgnoreInputForCustomAction;

var bool bLastCanDoCustomAction;

var const bool bCameraFocusOnPawn;

var const bool bNotifyKnockedOutOfCover;

var const bool bIgnoreDamage;

var const bool bDisableLeftHandIK;

var const bool bDisablePhysics;

var const bool bDisableCollision;

var const bool bDisableLook;

var const bool bDisableMovement;

var const bool bBreakFromCover;

var const bool bLockPawnRotation;

var() const config float SpeedModifier;

var BioPawn oPawn;

final function bool CanDoCustomAction()
{
    if (oPawn != None)
    {
        if (oPawn.WorldInfo.TimeSeconds != LastCanDoCustomActionTime)
        {
            bLastCanDoCustomAction = InternalCanDoCustomAction();
            LastCanDoCustomActionTime = oPawn.WorldInfo.TimeSeconds;
        }
        return bLastCanDoCustomAction;
    }
    return false;
}

protected function bool InternalCanDoCustomAction()
{
    return true;
}

function StartCustomAction()
{
    local BioPlayerController PlayerController;
    local SFXAI_Core AIController;
    local bool bForceRotation;
    local bool bWasInCover;
    if (oPawn == None)
    {
        return;
    }
    PlayerController = BioPlayerController(oPawn.Controller);
    AIController = SFXAI_Core(oPawn.Controller);
    if (PlayerController != None)
    {
        if (bDisableMovement)
        {
            PlayerController.IgnoreMoveInput(true);
        }
        if (bDisableLook)
        {
            PlayerController.IgnoreLookInput(true);
        }
    }
    if (bBreakFromCover)
    {
        if (AIController != None)
        {
            bWasInCover = oPawn.IsInCover();
            bForceRotation = AIController.bForceDesiredRotation;
            AIController.InvalidateCover();
            AIController.bForceDesiredRotation = bForceRotation;
            if (bWasInCover && bNotifyKnockedOutOfCover)
            {
                AIController.NotifyKnockedOutOfCover();
            }
        }
        else
        {
            if (PlayerController != None)
            {
                PlayerController.LeaveCover();
            }
        }
        oPawn.SetCrouchStateInstantly(false);
    }
    if (bDisableMovement)
    {
        oPawn.Acceleration = vect(0,0,0);
    }
    if (bDisableCollision)
    {
        TogglePawnCollision(false);
    }
    if (bDisablePhysics)
    {
        oPawn.SetPhysics(0);
        oPawn.LastPhysicsSetter = self;
    }
    if (bIgnoreDamage)
    {
        oPawn.bCanBeDamaged = false;
    }
    SFXGame(oPawn.WorldInfo.Game).TriggerVocalizationEvent(VocalizationEvent, oPawn, None, 0);
}

function StopCustomAction()
{
    local BioPlayerController PlayerController;
    if (oPawn == None)
    {
        return;
    }
    PlayerController = BioPlayerController(oPawn.Controller);
    if (PlayerController != None)
    {
        if (bDisableMovement)
        {
            PlayerController.IgnoreMoveInput(false);
        }
        if (bDisableLook)
        {
            PlayerController.IgnoreLookInput(false);
        }
    }
    if (bDisableCollision)
    {
        TogglePawnCollision(true);
    }
    if (bDisablePhysics && oPawn.LastPhysicsSetter == self)
    {
        oPawn.SetPhysics(2);
    }
    if (bIgnoreDamage)
    {
        oPawn.bCanBeDamaged = true;
    }
}

function PauseCustomAction();

function ContinueCustomAction();

function PawnLeftCover();

function bool NotifyBump(Actor Other, Vector HitNormal);

function bool NotifyHitWall(Vector HitNormal, Actor Wall);

function AnimNotify(AnimNodeSequence SeqNode, BioAnimNotify_CustomAction NotifyObject);

function BodyStanceAnimEndNotification(AnimNodeSequence SeqNode, float PlayedTime, float ExcessTime)
{
    if (BioPlayerController(oPawn.Controller) != None)
    {
        SFXPlayerController(oPawn.Controller).EndCustomAction();
    }
    else
    {
        if (SFXAI_Core(oPawn.Controller) != None)
        {
            SFXAI_Core(oPawn.Controller).EndCustomAction();
        }
    }
}

native final function SetReachPreciseDestination(Vector DestinationToReach);

native final function SetFacePreciseRotation(Rotator RotationToFace, float InterpolationTime);

event ReachedPrecisePosition();

final function TogglePawnCollision(bool bToggleOn)
{
    if (bToggleOn)
    {
        if (oPawn.bCollideWorld != oPawn.Default.bCollideWorld || oPawn.bBlockActors != oPawn.Default.bBlockActors)
        {
            oPawn.SetCollision(oPawn.bCollideActors, oPawn.Default.bBlockActors);
            oPawn.bCollideWorld = oPawn.Default.bCollideWorld;
            oPawn.FitCollision();
        }
    }
    else
    {
        oPawn.SetCollision(oPawn.bCollideActors, false);
        oPawn.bCollideWorld = false;
    }
}

function RootMotionModeChanged(SkeletalMeshComponent SkelComp);

function RootMotionExtracted(SkeletalMeshComponent SkelComp, out BoneAtom ExtractedRootMotionDelta);

final function PlayScreenShake(ScreenShakeStruct Shake)
{
    local SFXPlayerController PC;
    PC = SFXPlayerController(oPawn.Controller);
    if (PC != None && PC.PlayerCamera != None)
    {
        SFXPlayerCamera(PC.PlayerCamera).AddScreenShake(Shake);
    }
}

final function PlayForceFeedback(ForceFeedbackWaveform FF)
{
    local SFXPlayerController PC;
    PC = SFXPlayerController(oPawn.Controller);
    if (PC != None)
    {
        PC.ClientPlayForceFeedbackWaveform(FF);
    }
}

event TickCustomAction(float fDeltaTime);

defaultproperties
{
}
