﻿class BioCustomAction_CoverClimbMantleBase extends BioCustomAction
    config(Game);

var transient float m_fCollisionOffTime;

var float m_fCollisionOffPerioud;

var transient CovPosInfo m_CurrentCoverInfo;

var transient bool m_bCoverCacheValid;

var ECoverBodyStanceID m_CurrentBodyStance;

var() BodyStance m_BS_StanceFromCover;

var() BodyStance m_BS_StanceFromCombat;

var() BodyStance m_BS_StanceFromExplore;

enum ECoverBodyStanceID
{
    ECS_None,
    ECS_FromExplore,
    ECS_FromCombat,
    ECS_FromCover,
};

function StartCustomAction()
{
    local BioEpicPawnBehavior PawnBehavior;
    local bool bPlaySuccess;
    local bool bWasInCover;
    PawnBehavior = BioEpicPawnBehavior(oPawn.oBioComponent);
    if (PawnBehavior.m_eCurrentActionState != 1 && PawnBehavior.m_eCurrentActionState != 0)
    {
        return;
    }
    oPawn.SetCollision(oPawn.bCollideActors, false);
    m_fCollisionOffTime = 0;
    bWasInCover = oPawn.IsInCover();
    StartCustomAction();
    if (m_CurrentBodyStance != 0)
    {
        StopCustomAction();
    }
    if (bWasInCover == true)
    {
        bPlaySuccess = StartPlayBodyStance(3);
    }
    else
    {
        if (PawnBehavior.m_eCurrentActionState == 0)
        {
            bPlaySuccess = StartPlayBodyStance(1);
        }
        else
        {
            if (PawnBehavior.m_eCurrentActionState == 1)
            {
                bPlaySuccess = StartPlayBodyStance(2);
            }
        }
    }
    if (bPlaySuccess)
    {
        oPawn.Mesh.RootMotionMode = 0;
        oPawn.Mesh.bRootMotionModeChangeNotify = true;
        oPawn.SetPhysics(0);
        oPawn.LastPhysicsSetter = self;
        oPawn.Velocity = vect(0,0,0);
    }
}

function StopCustomAction()
{
    oPawn.SetCollision(oPawn.bCollideActors, true);
    m_bCoverCacheValid = false;
    EndPlayBodyStance(m_CurrentBodyStance);
    StopPlayBodyStance(m_CurrentBodyStance);
    oPawn.Mesh.RootMotionMode = oPawn.Mesh.Default.RootMotionMode;
    if (oPawn.LastPhysicsSetter == self)
    {
        oPawn.SetPhysics(1);
    }
    StopCustomAction();
}

event TickCustomAction(float fDeltaTime)
{
    TickCustomAction(fDeltaTime);
    m_fCollisionOffTime += fDeltaTime;
    if (m_fCollisionOffTime > m_fCollisionOffPerioud)
    {
        oPawn.SetCollision(oPawn.bCollideActors, true);
    }
}

function BodyStanceAnimEndNotification(AnimNodeSequence SeqNode, float PlayedTime, float ExcessTime)
{
    EndPlayBodyStance(m_CurrentBodyStance);
    if (SFXPlayerController(oPawn.Controller) != None)
    {
        SFXPlayerController(oPawn.Controller).EndCustomAction();
    }
    else
    {
        if (SFXAI_Core(oPawn.Controller) != None)
        {
            SFXAI_Core(oPawn.Controller).EndCustomAction();
        }
    }
    oPawn.Mesh.RootMotionMode = oPawn.Mesh.Default.RootMotionMode;
    if (oPawn.LastPhysicsSetter == self)
    {
        oPawn.SetPhysics(1);
    }
}

final private function bool GetBodyStance(ECoverBodyStanceID StanceID, out BodyStance out_BodyStance)
{
    switch (StanceID)
    {
        case 1:
            out_BodyStance = m_BS_StanceFromExplore;
            return true;
        case 2:
            out_BodyStance = m_BS_StanceFromCombat;
            return true;
        case 3:
            out_BodyStance = m_BS_StanceFromCover;
            return true;
        default:
            return false;
    }
}

final private function bool StartPlayBodyStance(ECoverBodyStanceID StanceID)
{
    local BodyStance Stance;
    local float fBlendOutTime;
    if (GetBodyStance(StanceID, Stance))
    {
        if (oPawn.IsPlayerPawn())
        {
            fBlendOutTime = 1;
        }
        m_CurrentBodyStance = StanceID;
        if (oPawn.PlayBodyStance(Stance, 1, 0,2, fBlendOutTime, false, true, true) != 0)
        {
            oPawn.SetBodyStanceAnimEndNotification(Stance, true);
            oPawn.SetBodyStanceRootBoneAxisOption(Stance, 2, 2, 2);
            return true;
        }
    }
    return false;
}

final private function StopPlayBodyStance(ECoverBodyStanceID StanceID)
{
    local BodyStance Stance;
    if (GetBodyStance(StanceID, Stance))
    {
        oPawn.StopBodyStance(Stance, 0,1);
        oPawn.SetBodyStanceAnimEndNotification(Stance, false);
        m_CurrentBodyStance = 0;
    }
}

final private function EndPlayBodyStance(ECoverBodyStanceID StanceID)
{
    local BodyStance Stance;
    if (GetBodyStance(StanceID, Stance))
    {
        oPawn.SetBodyStanceRootBoneAxisOption(Stance, 1, 1, 1);
    }
}

function CacheCoverInfo(CovPosInfo CoverInfo)
{
    m_bCoverCacheValid = true;
    m_CurrentCoverInfo = CoverInfo;
}

defaultproperties
{
    m_fCollisionOffPerioud=0,4
    bBreakFromCover=true
    bDisableMovement=true
}
