﻿class BioGameProperty extends Object
    config(Game)
    native;

enum EBioGamePropertyType
{
    GPT_Static,
    GPT_Timed,
    GPT_Instant,
    GPT_Frame,
    GPT_StaticFrame,
};

var int m_level;

var Name m_nmGamePropertyName;

var /* 00000008 */ BioGamePropertyEventSet m_oGamePropertyEvents;

var float m_fPower;

var int m_nGamePropertyGroup;

var float m_fTickInterval;

var bool m_bSendEffectDurationComplete;

var bool m_bApplyOnCreation;

var bool m_bFireAndForget;

var bool m_bTicked;

var float m_fDuration;

var native Map_Mirror m_mapApplicationArray;

var EBioGamePropertyType m_ePropertyType;

var array<BioGameEffect> m_aGameEffects;

var array<BioGamePropertyConditional> m_aPropConditionals;

var Object m_oOwner;

struct native BioGamePropertyApplication
{
    var Object m_pActualTarget;

    var BioGameEffect m_pGE;

}

const GAME_PROPERTY_GROUP_NONE = -1;
native function AddGameEffectToProperty(BioGameEffect oEffect);

event AddConditionalToProperty(BioGamePropertyConditional oConditional)
{
    if (oConditional != None)
    {
        if (m_aPropConditionals.Find(oConditional) != -1)
        {
        }
        else
        {
            m_aPropConditionals[m_aPropConditionals.Length] = oConditional;
            oConditional.Initialize();
        }
    }
}

native function bool TestConditionals(Object oTarget);

native function TickEffects(float fDeltaTime);

native function ApplyEffects(Object oTarget, optional bool bIsInstant);

native function RemoveEffects(Object oTarget);

native function ScaleToLevel(int nPowerLevel);

native static function RunTests(BioPawn oPawn);

static event BioGameProperty CreateGameProperty(BioGamePropertyContainer oGPContainer, Object oOwner, EBioGamePropertyType ePropertyType, float fTimedDuration, bool bTicked, float fTickInterval, int nGamePropertyGroup, float fPower, bool bFireAndForget)
{
    local BioGameProperty gp;
    if (ePropertyType == 2 || ePropertyType == 1 && fTimedDuration <= 0)
    {
        return CreateInstant();
    }
    gp = new(oGPContainer,Class'BioGameProperty');
    oGPContainer.m_aGameProperties.AddItem(gp);
    gp.m_oOwner = oOwner;
    gp.m_ePropertyType = ePropertyType;
    gp.m_fDuration = fTimedDuration;
    gp.m_bTicked = bTicked;
    gp.m_fTickInterval = fTickInterval;
    gp.m_nGamePropertyGroup = nGamePropertyGroup;
    gp.m_fPower = fPower;
    gp.m_bFireAndForget = bFireAndForget;
    return gp;
}

static event BioGameProperty CreateInstant()
{
    local BioGameProperty gp;
    gp = new(Class'BioGameProperty');
    gp.m_ePropertyType = 2;
    return gp;
}

event String ToString(optional String sIndent)
{
    local String sOutput;
    if (m_level == 0)
    {
        return sOutput;
    }
    sOutput $= sIndent $ "Label," $ m_nmGamePropertyName $ ",";
    sOutput $= sIndent $ "Type," $ m_ePropertyType $ ",";
    sOutput $= sIndent $ "Group," $ m_nGamePropertyGroup $ ",";
    sOutput $= sIndent $ "Duration," $ m_fDuration $ ",";
    sOutput $= sIndent $ "Is Ticked," $ m_bTicked $ ",";
    sOutput $= sIndent $ "Tick Interval," $ m_fTickInterval $ ",";
    sOutput $= sIndent $ "Power," $ m_fPower $ ",";
    sOutput $= sIndent $ "Level," $ m_level $ ",";
    return sOutput;
}

event String ToLine()
{
    local String sOutput;
    local array<String> asVariableValuePairs;
    local int I;
    Class'BioObject'.ParseIntoArray(asVariableValuePairs, ToString(), ",", true);
    I = 0;
    while (I != asVariableValuePairs.Length)
    {
        sOutput $= "(" $ asVariableValuePairs[I] $ "=";
        ++I;
        if (I == asVariableValuePairs.Length)
        {
            break;
        }
        sOutput $= asVariableValuePairs[I] $ ")";
        ++I;
    }
    return sOutput;
}

native final function Apply(Object pTargetObject);

native final function Unapply(Object pTargetObject);

native final function bool HasEffect(class<Object> GameEffectClass);

defaultproperties
{
    m_bApplyOnCreation=true
    m_nGamePropertyGroup=-1
    m_oGamePropertyEvents=BioGamePropertyEventSet'eventsnew'
    // child object eventsnew of type BioGamePropertyEventSet
}
