﻿class BioHintSystem extends BioHintSystemBase
    config(Game);

const UUToM = 0.01;
var transient Name m_nmTutorialHighlightHint;

var transient Name m_nmTutorialSelectHint;

var transient SFXPowerTutorialType m_ePowerTutorialType;

var transient float m_fRealTimeSeconds;

var transient int m_nMostUnspentTalentPoints;

var transient int m_nMedigel;

var transient Name m_nmNearestEnemy;

var transient Name m_nmClassMappedPower;

var transient Name m_nmRightMappedPower;

var transient Name m_nmLeftMappedPower;

var transient Name m_nmMappedPower;

var transient Name m_nmLastUsedPower;

var transient Name m_nmCurrentWeapon;

var transient float m_fNearestEnemyDistance;

var transient float m_fHealthPercent;

var transient float m_fAmmoPercent;

var transient bool m_bShowingSelectHint;

var transient bool m_bWeaponWheelTutorialEnabled;

var transient bool m_bPowerWheelTutorialEnabled;

var transient bool m_bAnyHenchmenDead;

var transient bool m_bHasAnyAmmoPowers;

var transient bool m_bUsingAmmoPower;

var transient bool m_bUsingShotgun;

var transient bool m_bUsingSniperRifle;

var transient bool m_bUsingHeavyWeapon;

var transient bool m_bInCombat;

enum SFXPowerTutorialType
{
    PowerTutorial_Pull,
    PowerTutorial_Unity,
    PowerTutorial_Overload,
};

event CacheCurrentState()
{
    local BioWorldInfo oWorldInfo;
    local BioPlayerController oController;
    local BioPawn oPawn;
    local BioBaseSquad oSquad;
    local BioPlayerInput oInput;
    local SFXInventoryManager oInvManager;
    local SFXWeapon oWeapon;
    local LocalEnemy oEnemy;
    local float fDist;
    local int nSquadIndex;
    local SFXPawn_Henchman oHenchman;
    m_bInCombat = false;
    m_fAmmoPercent = 100;
    m_fHealthPercent = 100;
    m_fNearestEnemyDistance = -1;
    m_nmCurrentWeapon = 'None';
    m_nmMappedPower = 'None';
    m_nmLeftMappedPower = 'None';
    m_nmRightMappedPower = 'None';
    m_nmClassMappedPower = 'None';
    m_nmNearestEnemy = 'None';
    m_nMedigel = 0;
    m_bAnyHenchmenDead = false;
    oWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    m_fRealTimeSeconds = oWorldInfo.RealTimeSeconds;
    oController = oWorldInfo.GetLocalPlayerController();
    if (oController != None)
    {
        oPawn = BioPawn(oController.Pawn);
        if (oPawn != None)
        {
            m_bInCombat = SFXGame(oWorldInfo.Game).InCombat();
            m_fHealthPercent = 100 * oPawn.GetHealthPct();
            m_nMostUnspentTalentPoints = oPawn.TalentPoints;
            m_bHasAnyAmmoPowers = false;
            if (oPawn.IsA('SFXPawn_PlayerSoldier') || oPawn.IsA('SFXPawn_PlayerVanguard') || oPawn.IsA('SFXPawn_PlayerInfiltrator'))
            {
                m_bHasAnyAmmoPowers = true;
            }
            foreach oController.EnemyList(oEnemy)
            {
                if (oEnemy.Enemy != None)
                {
                    fDist = 0,01 * VSize(oEnemy.Enemy.Location - oPawn.Location);
                    if (m_fNearestEnemyDistance < 0 || fDist < m_fNearestEnemyDistance)
                    {
                        m_fNearestEnemyDistance = fDist;
                        m_nmNearestEnemy = oEnemy.Enemy.Class.Name;
                    }
                }
            }
            oWeapon = SFXWeapon(oPawn.Weapon);
            if (oWeapon != None)
            {
                m_nmCurrentWeapon = oWeapon.Class.Name;
                m_fAmmoPercent = 100 * oWeapon.GetAmmoCountInMagazine() / oWeapon.GetMagazineSize();
            }
            m_bUsingAmmoPower = false;
            if (oWeapon != None && Len(oWeapon.AmmoPowerName) > 0)
            {
                m_bUsingAmmoPower = true;
            }
            m_bUsingSniperRifle = false;
            if (SFXWeapon_SniperRifle_Base(oWeapon) != None)
            {
                m_bUsingSniperRifle = true;
            }
            m_bUsingShotgun = false;
            if (SFXWeapon_Shotgun_Base(oWeapon) != None)
            {
                m_bUsingShotgun = true;
            }
            if (SFXHeavyWeapon(oWeapon) != None)
            {
                m_bUsingHeavyWeapon = true;
            }
            else
            {
                m_bUsingHeavyWeapon = false;
            }
            oInput = BioPlayerInput(oController.PlayerInput);
            if (oInput != None)
            {
                m_nmMappedPower = oInput.m_nmMappedPower;
                m_nmLeftMappedPower = oInput.m_nmMappedPower;
                m_nmRightMappedPower = oInput.m_nmMappedPower2;
                m_nmClassMappedPower = oInput.m_nmMappedPower3;
            }
            oInvManager = SFXInventoryManager(oPawn.InvManager);
            if (oInvManager != None)
            {
                m_nMedigel = oInvManager.GetResource(1);
            }
            oSquad = oPawn.Squad;
            if (oSquad != None)
            {
                nSquadIndex = 0;
                while (nSquadIndex < oSquad.GetSquadSize())
                {
                    oHenchman = SFXPawn_Henchman(oSquad.GetMember(nSquadIndex));
                    if (oHenchman != None)
                    {
                        if (oHenchman.bIsDead)
                        {
                            m_bAnyHenchmenDead = true;
                        }
                        if (oHenchman.TalentPoints > m_nMostUnspentTalentPoints)
                        {
                            m_nMostUnspentTalentPoints = oHenchman.TalentPoints;
                        }
                    }
                    nSquadIndex++;
                }
            }
        }
    }
}

function bool ManualReloadHint()
{
    local HintTrackingData oManual;
    local HintTrackingData oFast;
    local HintTrackingData oReload;
    if (m_fAmmoPercent <= 25 && m_fAmmoPercent > 0 && m_bUsingHeavyWeapon == false)
    {
        oManual = GetTrackingData('ManualReload');
        oFast = GetTrackingData('FastReload');
        oReload = GetTrackingData('Reload');
        if (oReload.Num >= 3 && oManual.Num == 0 || oManual.LastTime < GetPreviousTime(oReload, 2) && oFast.Num == 0 || oFast.LastTime < GetPreviousTime(oReload, 2))
        {
            return true;
        }
    }
    return false;
}

function bool TakeCoverHint()
{
    local HintTrackingData oTakeCover;
    local HintTrackingData oLeaveCover;
    if (m_fHealthPercent > 0 && m_fHealthPercent < 50 && m_fNearestEnemyDistance > 10)
    {
        oTakeCover = GetTrackingData('EnterCover');
        oLeaveCover = GetTrackingData('LeaveCover');
        if (oTakeCover.Num == 0 || oLeaveCover.Num > 0 && oLeaveCover.LastTime < oTakeCover.LastTime && TimeSince(oLeaveCover.LastTime) > 60)
        {
            return true;
        }
    }
    return false;
}

function bool ZoomHint()
{
    local HintTrackingData oFired;
    local HintTrackingData oEnteredCombat;
    local HintTrackingData oZoomed;
    local HintTrackingData oSwitchWeaponData;
    local float TimeSince10thLastShot;
    if (m_bInCombat && !m_bUsingShotgun)
    {
        oFired = GetTrackingData('Fire');
        oEnteredCombat = GetTrackingData('EnterCombat');
        oZoomed = GetTrackingData('Zoom');
        oSwitchWeaponData = GetTrackingData('ChangeWeapon');
        TimeSince10thLastShot = TimeSince(GetPreviousTime(oFired, 9));
        if (!m_bUsingHeavyWeapon && oFired.Num > 10 && TimeSince(oFired.LastTime) < 5 && oEnteredCombat.Num > 0 && TimeSince(oEnteredCombat.LastTime) >= TimeSince10thLastShot && TimeSince(oSwitchWeaponData.LastTime) > TimeSince10thLastShot && oZoomed.Num == 0 || TimeSince(oZoomed.LastTime) > 300)
        {
            return true;
        }
    }
    return false;
}

function bool PCMappedPowerHint()
{
    local HintTrackingData oCast;
    local HintTrackingData oUseMapped;
    if (m_nmMappedPower != 'None' && m_nmLastUsedPower == m_nmMappedPower && !Class'WorldInfo'.IsConsoleBuild(0))
    {
        oCast = GetTrackingData('PowerCast', m_nmLastUsedPower);
        oUseMapped = GetTrackingData('UseMappedPower');
        if (TimeSince(oCast.LastTime) < TimeSince(oUseMapped.LastTime))
        {
            return true;
        }
    }
    return false;
}

function bool LeftMappedPowerHint()
{
    local HintTrackingData oCast;
    local HintTrackingData oUseMapped;
    local float fLastCastTime;
    local float fDelta;
    if (m_nmLeftMappedPower != 'None' && m_nmLastUsedPower == m_nmLeftMappedPower)
    {
        oCast = GetTrackingData('PowerCast', m_nmLastUsedPower);
        oUseMapped = GetTrackingData('UseMappedPower', m_nmLeftMappedPower);
        fLastCastTime = TimeSince(oCast.LastTime);
        fDelta = Abs(fLastCastTime - TimeSince(oUseMapped.LastTime));
        if (fLastCastTime < 1 && fDelta > 0,25)
        {
            return true;
        }
    }
    return false;
}

function bool RightMappedPowerHint()
{
    local HintTrackingData oCast;
    local HintTrackingData oUseMapped;
    local float fLastCastTime;
    local float fDelta;
    if (m_nmRightMappedPower != 'None' && m_nmLastUsedPower == m_nmRightMappedPower)
    {
        oCast = GetTrackingData('PowerCast', m_nmLastUsedPower);
        oUseMapped = GetTrackingData('UseMappedPower', m_nmRightMappedPower);
        fLastCastTime = TimeSince(oCast.LastTime);
        fDelta = Abs(fLastCastTime - TimeSince(oUseMapped.LastTime));
        if (fLastCastTime < 1 && fDelta > 0,25)
        {
            return true;
        }
    }
    return false;
}

function bool ClassMappedPowerHint()
{
    local HintTrackingData oCast;
    local HintTrackingData oUseMapped;
    local float fLastCastTime;
    local float fDelta;
    if (m_nmClassMappedPower != 'None' && m_nmLastUsedPower == m_nmClassMappedPower)
    {
        oCast = GetTrackingData('PowerCast', m_nmLastUsedPower);
        oUseMapped = GetTrackingData('UseMappedPower', m_nmClassMappedPower);
        fLastCastTime = TimeSince(oCast.LastTime);
        fDelta = Abs(fLastCastTime - TimeSince(oUseMapped.LastTime));
        if (fLastCastTime < 1 && fDelta > 0,25)
        {
            return true;
        }
    }
    return false;
}

function bool MeleeHint()
{
    local HintTrackingData oMelee;
    local HintTrackingData oMeleeDamaged;
    local HintTrackingData oHuskMeleeDamaged;
    if (m_bInCombat && m_fNearestEnemyDistance < 3)
    {
        oMelee = GetTrackingData('MELEE');
        oMeleeDamaged = GetTrackingData('Damaged', 'SFXDamageType_Melee');
        oHuskMeleeDamaged = GetTrackingData('Damaged', 'SFXDamageType_HuskMelee');
        if (oMelee.Num == 0 || TimeSince(oMelee.LastTime) > 300 && oMeleeDamaged.Num >= 2 && TimeSince(GetPreviousTime(oMeleeDamaged, 2)) < 10 || oHuskMeleeDamaged.Num >= 2 && TimeSince(GetPreviousTime(oHuskMeleeDamaged, 2)) < 10)
        {
            return true;
        }
    }
    return false;
}

function bool ResurrectHint()
{
    if (m_bInCombat && m_bAnyHenchmenDead && m_nMedigel > 2)
    {
        return true;
    }
    return false;
}

function bool BypassGameHint()
{
    local HintTrackingData oStartedBypass;
    local HintTrackingData oClearEvent;
    local float TimeSinceStart;
    local float TimeSinceClear;
    oStartedBypass = GetTrackingData('StartBypassGame');
    oClearEvent = GetTrackingData('ClearMinigameHint');
    TimeSinceStart = TimeSince(oStartedBypass.LastTime);
    TimeSinceClear = TimeSince(oClearEvent.LastTime);
    if (oStartedBypass.Num >= 1 && oStartedBypass.Num <= 2 && TimeSinceStart < 1 && TimeSinceClear > 10)
    {
        return true;
    }
    return false;
}

function bool HackGameHint()
{
    local HintTrackingData oStartedHack;
    local HintTrackingData oClearEvent;
    local float TimeSinceStart;
    local float TimeSinceClear;
    oStartedHack = GetTrackingData('StartHackGame');
    oClearEvent = GetTrackingData('ClearMinigameHint');
    TimeSinceStart = TimeSince(oStartedHack.LastTime);
    TimeSinceClear = TimeSince(oClearEvent.LastTime);
    if (oStartedHack.Num >= 1 && oStartedHack.Num <= 2 && TimeSinceStart < 1 && TimeSinceClear > 10)
    {
        return true;
    }
    return false;
}

function bool LevelUpHint()
{
    if (m_nMostUnspentTalentPoints >= 4 && m_fRealTimeSeconds > 5 && m_fRealTimeSeconds < 10)
    {
        return true;
    }
    return false;
}

function bool CooldownHint()
{
    local HintTrackingData oHintData;
    oHintData = GetTrackingData('PowerOnCooldown');
    if (oHintData.Num >= 1 && TimeSince(oHintData.LastTime) < 1)
    {
        return true;
    }
    return false;
}

function bool PlanetScanHint()
{
    local HintTrackingData oOpenScanner;
    local HintTrackingData oCloseScanner;
    local HintTrackingData oStartScanning;
    local float fOpenScanner;
    local float fCloseScanner;
    local float fStartScanning;
    oOpenScanner = GetTrackingData('StartPlanetScanner');
    fOpenScanner = TimeSince(oOpenScanner.LastTime);
    oCloseScanner = GetTrackingData('ClosePlanetScanner');
    fCloseScanner = TimeSince(oCloseScanner.LastTime);
    oStartScanning = GetTrackingData('StartScanning');
    fStartScanning = TimeSince(oStartScanning.LastTime);
    if (fOpenScanner < fCloseScanner && fOpenScanner < fStartScanning && fOpenScanner >= 8)
    {
        return true;
    }
    return false;
}

function bool LaunchProbeHint()
{
    local HintTrackingData oOpenScanner;
    local HintTrackingData oCloseScanner;
    local HintTrackingData oStartScanning;
    local HintTrackingData oFireProbe;
    local HintTrackingData oAnomaly;
    local float fOpenScanner;
    local float fCloseScanner;
    local float fStartScanning;
    local float fFireProbe;
    local float fAnomaly;
    oOpenScanner = GetTrackingData('StartPlanetScanner');
    fOpenScanner = TimeSince(oOpenScanner.LastTime);
    oCloseScanner = GetTrackingData('ClosePlanetScanner');
    fCloseScanner = TimeSince(oCloseScanner.LastTime);
    oStartScanning = GetTrackingData('StartScanning');
    fStartScanning = TimeSince(oStartScanning.LastTime);
    oFireProbe = GetTrackingData('LaunchProbe');
    fFireProbe = TimeSince(oFireProbe.LastTime);
    oAnomaly = GetTrackingData('PlanetScanAnomaly');
    fAnomaly = TimeSince(oAnomaly.LastTime);
    if (fOpenScanner < fCloseScanner && fStartScanning < fOpenScanner && fFireProbe > fOpenScanner && fAnomaly > fStartScanning && fOpenScanner > 15)
    {
        return true;
    }
    return false;
}

function bool FindLandingSite()
{
    local HintTrackingData oOpenScanner;
    local HintTrackingData oCloseScanner;
    local HintTrackingData oStartScanning;
    local HintTrackingData oAnomaly;
    local float fOpenScanner;
    local float fCloseScanner;
    local float fStartScanning;
    local float fAnomaly;
    oOpenScanner = GetTrackingData('StartPlanetScanner');
    fOpenScanner = TimeSince(oOpenScanner.LastTime);
    oCloseScanner = GetTrackingData('ClosePlanetScanner');
    fCloseScanner = TimeSince(oCloseScanner.LastTime);
    oStartScanning = GetTrackingData('StartScanning');
    fStartScanning = TimeSince(oStartScanning.LastTime);
    oAnomaly = GetTrackingData('PlanetScanAnomaly');
    fAnomaly = TimeSince(oAnomaly.LastTime);
    if (fOpenScanner < fCloseScanner && fStartScanning < fOpenScanner && fAnomaly <= fStartScanning && fOpenScanner > 15)
    {
        return true;
    }
    return false;
}

function bool UseAmmoPowerHint()
{
    local int nReloadLimit;
    local HintTrackingData oHintData;
    local HintTrackingData oCombatData;
    local HintTrackingData oReloadData;
    local HintTrackingData oSwitchWeaponData;
    local float fReloadTime;
    local BioPlayerController PC;
    oHintData = GetTrackingData('Hint', 'UseAmmoPowerHint');
    oCombatData = GetTrackingData('EnterCombat');
    oReloadData = GetTrackingData('Reload');
    oSwitchWeaponData = GetTrackingData('ChangeWeapon');
    nReloadLimit = 2;
    if (m_bUsingSniperRifle)
    {
        nReloadLimit = 5;
    }
    fReloadTime = TimeSince(GetPreviousTime(oReloadData, nReloadLimit));
    PC = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController();
    if (!PC.GameModeManager2.IsActive(5) && !PC.GameModeManager2.IsActive(6) && m_bInCombat && !m_bUsingHeavyWeapon && !m_bUsingAmmoPower && m_bHasAnyAmmoPowers && oHintData.Num == 0 && oCombatData.Num > 0 && oReloadData.Num >= nReloadLimit && fReloadTime < TimeSince(oCombatData.LastTime) && oSwitchWeaponData.Num == 0 || fReloadTime < TimeSince(oSwitchWeaponData.LastTime))
    {
        return true;
    }
    return false;
}

function EnablePowerTutorialHint(bool bOn, SFXPowerTutorialType eTutType)
{
    m_bPowerWheelTutorialEnabled = bOn;
    m_ePowerTutorialType = eTutType;
}

function EnableWeaponTutorialHint(bool bOn)
{
    m_bWeaponWheelTutorialEnabled = bOn;
}

function bool FindHintDefinition(Name nmHint, out HintDefinition oHint)
{
    local bool bFound;
    local int I;
    bFound = false;
    I = 0;
    while (I < m_aHints.Length)
    {
        if (m_aHints[I].HintName == nmHint)
        {
            oHint = m_aHints[I];
            bFound = true;
            break;
        }
        I++;
    }
    return bFound;
}

function GeneratePowerWheelTutorialHint(bool bPowerWheel, bool bWeaponWheel)
{
    local BioWorldInfo oWorldInfo;
    local MassEffectGuiManager Manager;
    local HintDefinition oHint;
    local bool bShowHint;
    oWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    Manager = MassEffectGuiManager(oWorldInfo.GetLocalPlayerController().GetScaleFormManager());
    bShowHint = false;
    if (m_bPowerWheelTutorialEnabled && bPowerWheel)
    {
        switch (m_ePowerTutorialType)
        {
            case 0:
                m_nmTutorialSelectHint = 'TutorialPullSelectHint';
                m_nmTutorialHighlightHint = 'TutorialPullHighlightHint';
                break;
            case 1:
                m_nmTutorialSelectHint = 'TutorialUnitySelectHint';
                m_nmTutorialHighlightHint = 'TutorialUnityHighlightHint';
                break;
            case 2:
                m_nmTutorialSelectHint = 'TutorialOverloadSelectHint';
                m_nmTutorialHighlightHint = 'TutorialOverloadHighlightHint';
                break;
        }
        m_bShowingSelectHint = true;
        bShowHint = true;
    }
    else
    {
        if (m_bWeaponWheelTutorialEnabled && bWeaponWheel)
        {
            m_nmTutorialSelectHint = 'TutorialWeaponWheelHint';
            m_nmTutorialHighlightHint = 'TutorialWeaponEquipHint';
            m_bShowingSelectHint = true;
            bShowHint = true;
        }
    }
    if (bShowHint && m_nmTutorialSelectHint != 'None')
    {
        if (FindHintDefinition(m_nmTutorialSelectHint, oHint))
        {
            Manager.ShowPlatformSpecificHint(oHint.DefaultText, oHint.PS3Text, oHint.PCText, oHint.DisplayDuration, oHint.HintPosition);
        }
    }
}

function UpdatePowerWheelTutorialHint(Name nmPower)
{
    local BioWorldInfo oWorldInfo;
    local MassEffectGuiManager Manager;
    local Name nmHint;
    local Name nmMatchingPowerName;
    local HintDefinition oHint;
    if (m_bPowerWheelTutorialEnabled)
    {
        switch (m_ePowerTutorialType)
        {
            case 0:
                nmMatchingPowerName = 'Pull';
                break;
            case 1:
                nmMatchingPowerName = 'FirstAid';
                break;
            case 2:
                nmMatchingPowerName = 'Overload';
                break;
        }
        if (nmPower == nmMatchingPowerName)
        {
            if (m_bShowingSelectHint)
            {
                nmHint = m_nmTutorialHighlightHint;
                m_bShowingSelectHint = false;
            }
        }
        else
        {
            if (!m_bShowingSelectHint)
            {
                nmHint = m_nmTutorialSelectHint;
                m_bShowingSelectHint = true;
            }
        }
        if (nmHint != 'None')
        {
            oWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
            Manager = MassEffectGuiManager(oWorldInfo.GetLocalPlayerController().GetScaleFormManager());
            if (FindHintDefinition(nmHint, oHint))
            {
                Manager.ShowPlatformSpecificHint(oHint.DefaultText, oHint.PS3Text, oHint.PCText, oHint.DisplayDuration, oHint.HintPosition);
            }
        }
    }
}

function UpdateWeaponWheelTutorialHint(int nWeaponType)
{
    local BioWorldInfo oWorldInfo;
    local MassEffectGuiManager Manager;
    local HintDefinition oHint;
    local Name nmHint;
    if (m_bWeaponWheelTutorialEnabled)
    {
        if (nWeaponType == 15)
        {
            if (m_bShowingSelectHint)
            {
                nmHint = m_nmTutorialHighlightHint;
                m_bShowingSelectHint = false;
            }
        }
        else
        {
            if (!m_bShowingSelectHint)
            {
                nmHint = m_nmTutorialSelectHint;
                m_bShowingSelectHint = true;
            }
        }
        if (nmHint != 'None')
        {
            oWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
            Manager = MassEffectGuiManager(oWorldInfo.GetLocalPlayerController().GetScaleFormManager());
            if (FindHintDefinition(nmHint, oHint))
            {
                Manager.ShowPlatformSpecificHint(oHint.DefaultText, oHint.PS3Text, oHint.PCText, oHint.DisplayDuration, oHint.HintPosition);
            }
        }
    }
}

function bool PowerCast(out Name nmContext)
{
    m_nmLastUsedPower = nmContext;
    return true;
}

function AddNotification_ResearchRecovered(String sName, String sMessage, optional Texture oImage = None)
{
    local SFXNotificationData oNotificationData;
    local String sTitle;
    oNotificationData = GetNotificationData('ResearchRecovered');
    sTitle = oNotificationData.srTitle;
    AddNotification_Custom(oNotificationData.DisplayTime, sTitle, sName, sMessage, oImage, 'None', oNotificationData.nmType, oNotificationData.nmSound, oNotificationData.Priority, oNotificationData.nFlourishID);
}

function AddNotification_Weapon(String sTitle, String sName, String sMessage, optional Texture oImage = None)
{
    local SFXNotificationData oNotificationData;
    oNotificationData = GetNotificationData('WeaponRecovered');
    AddNotification_Custom(oNotificationData.DisplayTime, sName, sTitle, sMessage, oImage, 'None', oNotificationData.nmType, oNotificationData.nmSound, oNotificationData.Priority, oNotificationData.nFlourishID);
}

function AddNotification_Tech(String sTitle, String sName, String sMessage, optional Texture oImage = None)
{
    local SFXNotificationData oNotificationData;
    oNotificationData = GetNotificationData('TechRecovered');
    AddNotification_Custom(oNotificationData.DisplayTime, sName, sTitle, sMessage, oImage, 'None', oNotificationData.nmType, oNotificationData.nmSound, oNotificationData.Priority, oNotificationData.nFlourishID);
}

function AddNotification_TechUnlocked(String sTitle, String sName, String sMessage, optional Texture oImage = None)
{
    local SFXNotificationData oNotificationData;
    oNotificationData = GetNotificationData('TechUnlocked');
    AddNotification_Custom(oNotificationData.DisplayTime, sName, sTitle, sMessage, oImage, 'None', oNotificationData.nmType, oNotificationData.nmSound, oNotificationData.Priority, oNotificationData.nFlourishID);
}

function AddNotification_XP(int Amount)
{
    AddNotificationOneParameter(GetNotificationData('XpReceived'), Amount);
}

function AddNotification_LevelUp(int Level)
{
    AddNotificationOneParameter(GetNotificationData('LevelUp'), Level);
}

function AddNotification_ProbeRecovery(int Amount)
{
    AddNotificationOneParameter(GetNotificationData('ProbesRecovery'), Amount);
}

function AddNotification_SalvageRecovery(int Amount)
{
    AddNotificationOneParameter(GetNotificationData('SalvageRecovery'), Amount);
}

function AddNotification_MediGelFull()
{
    AddNotificationOneParameter(GetNotificationData('MediGelFull'), 0);
}

function AddNotification_HeavyAmmoFull()
{
    AddNotificationOneParameter(GetNotificationData('HeavyAmmoFull'), 0);
}

event CodexUpdate()
{
    local SFXNotificationData oNoteData;
    local BioPlayerController oController;
    oController = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController();
    oController.ProfileSettings.GrinderAchievementIncrement(44, oController);
    oNoteData = GetNotificationData('CodexChange');
    if (Len(srTitle) > 0 && Len(srDescription) > 0)
    {
        oNoteData.srSubtitle = srTitle;
        oNoteData.srBody = srDescription;
        AddNotificationOneParameter(oNoteData, 0);
    }
}

event JournalUpdate()
{
    local SFXNotificationData oNoteData;
    oNoteData = GetNotificationData('JournalChange');
    if (Len(srTitle) > 0 && Len(srDescription) > 0)
    {
        oNoteData.srSubtitle = srTitle;
        oNoteData.srBody = srDescription;
        AddNotificationOneParameter(oNoteData, 0);
    }
}

protected function AddNotificationOneParameter(SFXNotificationData oNoteData, int nParam1)
{
    local SFXNotification oNotification;
    PopulateNotificationOneParameter(oNoteData, oNotification, nParam1);
    AddNotification(oNotification);
}

defaultproperties
{
}
