﻿class BioMessageBox extends BioGUIBox
    native;

var() float StartDrawTime;

var int m_nMaxScrollPosition;

var int m_nLineCount;

var int m_nScrollPosition;

var array<String> m_aWrappedText;

var array<String> m_aRawText;

protected function ComputeDrawParameters(Canvas oCanvas)
{
    local float fTextHeight;
    local float fTextWidth;
    local float fMaxWidth;
    local float fMaxHeight;
    local int I;
    if (!m_bComputeDrawParameters)
    {
        return;
    }
    m_bComputeDrawParameters = false;
    StartDrawTime = Class'Engine'.GetCurrentWorldInfo().TimeSeconds;
    m_nCanvasWidth = oCanvas.SizeX;
    m_nCanvasHeight = oCanvas.SizeY;
    fMaxWidth = 0,75 * oCanvas.SizeX;
    m_aWrappedText.Remove(0, m_aWrappedText.Length);
    I = 0;
    while (I < m_aRawText.Length)
    {
        if (Len(m_aRawText[I]) == 0)
        {
            m_aWrappedText[m_aWrappedText.Length] = Chr(10);
        }
        else
        {
            oCanvas.WrapStringToArray(m_aRawText[I], m_aWrappedText, fMaxWidth, "\n");
        }
        I++;
    }
    m_fWidth = 0;
    I = 0;
    while (I != m_aWrappedText.Length)
    {
        oCanvas.StrLen(m_aWrappedText[I], fTextWidth, fTextHeight);
        if (fTextWidth > m_fWidth)
        {
            m_fWidth = fTextWidth;
        }
        ++I;
    }
    m_fHeight = fTextHeight * m_aWrappedText.Length;
    fMaxHeight = 0,75 * oCanvas.SizeY;
    m_nScrollPosition = 0;
    m_nLineCount = m_aWrappedText.Length;
    if (m_fHeight > fMaxHeight)
    {
        m_nLineCount = m_aWrappedText.Length - m_fHeight - fMaxHeight / fTextHeight;
    }
    m_nMaxScrollPosition = m_aWrappedText.Length - m_nLineCount;
    if (m_fHeight > fMaxHeight)
    {
        m_fHeight = fMaxHeight;
    }
    m_fX = oCanvas.SizeX - m_fWidth / 2;
    m_fY = oCanvas.SizeY - m_fHeight / 2;
}

protected function bool ShouldRender()
{
    return HasText();
}

function SetText(coerce String S)
{
    m_aRawText.Length = 0;
    m_aRawText[0] = S;
    m_bComputeDrawParameters = Len(S) != 0;
}

native function SetTextArray(const out array<String> aText);

event bool HasText()
{
    local int I;
    I = 0;
    while (I < m_aRawText.Length)
    {
        if (Len(m_aRawText[I]) != 0)
        {
            return true;
        }
        I++;
    }
    return false;
}

protected function RenderContent(Canvas oCanvas)
{
    local int I;
    oCanvas.CurY = m_fY;
    oCanvas.CurYL = 0;
    oCanvas.DrawColor = m_colorForeground;
    I = 0;
    while (I != m_nLineCount)
    {
        oCanvas.CurX = m_fX;
        if (m_aWrappedText[m_nScrollPosition + I] == Chr(10))
        {
            oCanvas.DrawText(" ");
        }
        else
        {
            oCanvas.DrawText(m_aWrappedText[m_nScrollPosition + I]);
        }
        ++I;
    }
    if (Class'Engine'.GetCurrentWorldInfo().TimeSince(StartDrawTime) > 5)
    {
        m_aRawText.Length = 0;
        StartDrawTime = 0;
    }
}

function ScrollDown()
{
    if (m_nScrollPosition < m_nMaxScrollPosition)
    {
        ++m_nScrollPosition;
    }
}

function ScrollUp()
{
    if (m_nScrollPosition > 0)
    {
        --m_nScrollPosition;
    }
}

function PageDown()
{
    m_nScrollPosition += m_nLineCount;
    if (m_nScrollPosition > m_nMaxScrollPosition)
    {
        m_nScrollPosition = m_nMaxScrollPosition;
    }
}

function PageUp()
{
    m_nScrollPosition -= m_nLineCount;
    if (m_nScrollPosition < 0)
    {
        m_nScrollPosition = 0;
    }
}

defaultproperties
{
}
