﻿class BioPlayerInput extends PlayerInput
    within BioPlayerController
    hidecategories(Object,UIRoot)
    config(Input)
    transient
    native;

struct native DebugMenuEntry
{
    var String Name;

    var String Command;

}

enum EAxisBuffer
{
    AxisBuffer_LX,
    AxisBuffer_LY,
    AxisBuffer_RX,
    AxisBuffer_RY,
    AxisBuffer_MouseX,
    AxisBuffer_MouseY,
};

var transient int CoverRemapThreshold;

var transient int CoverRemapThresholdPC;

var transient InterpCurveFloat MouseDampeningCurve;

var config bool bUseMouseDampening;

var Vector AccumulatedAimVector;

var float AccumulatedRotationSpeed;

var InterpCurveFloat LookStickResponseCurve;

var array<DebugMenuEntry> DebugSubMenu;

var config array<DebugMenuEntry> DebugMenu;

var transient Name m_nmMappedPower3;

var transient Name m_nmMappedPower2;

var transient Name m_nmMappedPower;

var transient config float m_fQuickOrderTime;

var transient float m_fConversationReplyUpDown;

var transient float m_fConversationReplyLeftRight;

var transient int m_nCurConvSelection;

var() array<StaticKeyBind> StaticPCBinds;

var() array<StaticKeyBind> StaticConsoleBinds;

var transient int m_KeyInputDisabled;

var native transient Map_Mirror InputTimers;

var config float GuiDeadzone;

var float AxisBuffer[6];

var /* 00000004 */ float aGuiMouseY;

var /* 00000004 */ float aGuiMouseX;

var /* 00000004 */ float aGuiLookUp;

var /* 00000004 */ float aGuiTurn;

var /* 00000004 */ float aGuiBaseY;

var /* 00000004 */ float aGuiStrafe;

var /* 00000004 */ byte bWantsToZoom;

struct native StaticKeyBind
{
    var Name Name;

    var String Command;

    var bool Control;

    var bool Shift;

    var bool Alt;

    var bool bDebug;

}

native final function bool ActivatePower(Name nmPower, optional Actor oTarget, optional Vector vTargetLocation, optional Vector vOriginalCameraLocation, optional Rotator rOriginalCameraRotation);

native final function bool IsCombatEnabled();

native function String GetBind(Name Key, bool Control, bool Shift, bool Alt);

native final function DebugExecInputCommands(String Cmd, bool bButtonPressed);

event bool GetInputDisabled()
{
    return Outer.IsLookInputIgnored() && Outer.IsMoveInputIgnored() && IsKeyInputIgnored();
}

event SetInputDisabled(bool bValue)
{
    Outer.IgnoreLookInput(bValue);
    Outer.IgnoreMoveInput(bValue);
    IgnoreKeyInput(bValue);
}

native function IgnoreKeyInput(bool bValue);

native function bool IsKeyInputIgnored();

function float ComputeStickResponse(float RawStickValue, SFXCameraInput Input, float DeltaTime)
{
    local float DesiredSpeed;
    local float AngleDelta;
    local Vector NewAimVector;
    NewAimVector.X = RawJoyLookRight;
    NewAimVector.Y = RawJoyLookUp;
    NewAimVector.Z = 0;
    AngleDelta = NewAimVector Dot AccumulatedAimVector;
    AccumulatedRotationSpeed *= FClamp(AngleDelta, 0, 1);
    AccumulatedAimVector = NewAimVector;
    if (RawStickValue > 0,95)
    {
        DesiredSpeed = Input.MaxCameraRotationSpeed - 1;
        if (DesiredSpeed < AccumulatedRotationSpeed)
        {
            AccumulatedRotationSpeed = DesiredSpeed;
        }
        else
        {
            AccumulatedRotationSpeed += DesiredSpeed / Input.TimeToReachFullSpeed * DeltaTime;
        }
    }
    else
    {
        DesiredSpeed = RawStickValue;
        AccumulatedRotationSpeed = 0;
    }
    Class'BioInterpolator'.InterpolateFloatCurve(RawStickValue, LookStickResponseCurve, 0, 1, RawStickValue);
    return AccumulatedRotationSpeed + RawStickValue;
}

function PreProcessInput(float DeltaTime)
{
    local SFXWeapon Weapon;
    local SFXCameraInput Input;
    local float RawStickFactor;
    local float StickFactor;
    local float fRemappedX;
    local float fRemappedY;
    local float StickMag;
    local float X;
    local float Y;
    local int idx;
    if (bUseMouseDampening)
    {
        Class'BioInterpolator'.InterpolateFloatCurve(fRemappedX, MouseDampeningCurve, 0, 1, Abs(aMouseX));
        Class'BioInterpolator'.InterpolateFloatCurve(fRemappedY, MouseDampeningCurve, 0, 1, Abs(aMouseY));
        aMouseX = aMouseX > 0 ? 1 : -1 * fRemappedX;
        aMouseY = aMouseY > 0 ? 1 : -1 * fRemappedY;
    }
    RawStickFactor = Sqrt(aTurn * aTurn + aLookUp * aLookUp);
    if (SFXPlayerCamera(Outer.PlayerCamera).CurrentCameraMode != None)
    {
        Input = SFXPlayerCamera(Outer.PlayerCamera).CurrentCameraMode.Input;
        if (Input != None && RawStickFactor > Input.StickDeadZone)
        {
            StickFactor = ComputeStickResponse(RawStickFactor, Input, DeltaTime) / RawStickFactor;
            aTurn = aTurn * Input.CameraSensitivity.X * StickFactor * Outer.GetSensitivityScaling() / 360 * 2;
            aLookUp = aLookUp * Input.CameraSensitivity.Y * StickFactor * Outer.GetSensitivityScaling() / 360 * 2;
        }
        else
        {
            AccumulatedRotationSpeed = 0;
        }
    }
    Weapon = Outer.Pawn != None ? SFXWeapon(Outer.Pawn.Weapon) : None;
    if (Weapon != None && Outer.AimAssist != None)
    {
        Outer.AimAssist.CacheCurrentAimAssistTarget();
        if (Weapon.IsZoomed())
        {
            Outer.AimAssist.ViewFriction(DeltaTime, Weapon);
        }
        else
        {
            Outer.AimAssist.LastFrictionTarget = None;
        }
    }
    if (AxisBuffer[0] != 0 || AxisBuffer[1] != 0)
    {
        X = AxisBuffer[0];
        Y = AxisBuffer[1];
        StickMag = Sqrt(X * X + Y * Y);
        if (StickMag <= GuiDeadzone)
        {
            X = 0;
            Y = 0;
        }
        Outer.GetScaleFormManager().HandleInputEvent(2, 0, X, X);
        Outer.GetScaleFormManager().HandleInputEvent(3, 0, Y, Y);
    }
    if (AxisBuffer[2] != 0 || AxisBuffer[3] != 0)
    {
        X = AxisBuffer[2];
        Y = AxisBuffer[3];
        StickMag = Sqrt(X * X + Y * Y);
        if (StickMag <= GuiDeadzone)
        {
            X = 0;
            Y = 0;
        }
        Outer.GetScaleFormManager().HandleInputEvent(4, 0, X, X);
        Outer.GetScaleFormManager().HandleInputEvent(5, 0, Y, Y);
    }
    if (AxisBuffer[4] != 0 || AxisBuffer[5] != 0)
    {
        X = AxisBuffer[4];
        Y = AxisBuffer[5];
        Outer.GetScaleFormManager().HandleInputEvent(6, 0, X, X);
        Outer.GetScaleFormManager().HandleInputEvent(7, 0, Y, Y);
    }
    idx = 0;
    while (idx < 6)
    {
        AxisBuffer[idx] = 0;
        idx++;
    }
}

function AdjustMouseSensitivity(float FOVScale)
{
    local SFXCameraInput Input;
    AdjustMouseSensitivity(FOVScale);
    if (SFXPlayerCamera(Outer.PlayerCamera).CurrentCameraMode != None)
    {
        Input = SFXPlayerCamera(Outer.PlayerCamera).CurrentCameraMode.Input;
        if (Input != None)
        {
            aMouseX *= Input.CameraSensitivity.X;
            aMouseY *= Input.CameraSensitivity.Y;
        }
    }
}

function PostProcessInput(float DeltaTime)
{
    local BioPawn BP;
    local bool bForceZoom;
    local bool bForceNoZoom;
    if (Outer.Pawn != None && Outer.Pawn.Controller.IsInState('PlayerDriving') == false && Outer.Pawn.Physics == 10)
    {
        bWantsToZoom = 0;
        Outer.bFire = 0;
    }
    BP = BioPawn(Outer.Pawn);
    if (BP != None && BP.Weapon != None)
    {
        if (Outer.IsReloading() || Outer.bWantsToStorm == 1 || BP.Weapon.IsInState('WeaponEquipping') || BP.Weapon.IsInState('WeaponPuttingDown'))
        {
            bForceNoZoom = true;
        }
        else
        {
            if (BP.IsInCover())
            {
                if (BP.CoverAction == 3 || BP.CoverAction == 4 || BP.CoverAction == 7)
                {
                    bForceZoom = true;
                }
                else
                {
                    if (BP.CoverAction == 0)
                    {
                        bForceNoZoom = true;
                    }
                }
            }
            else
            {
                if (BP.IsPerformingCustomAction())
                {
                    bForceNoZoom = true;
                }
            }
        }
        if (Outer.bFire == 1 && SFXWeapon(BP.Weapon).OutOfAmmo())
        {
            bWantsToZoom = 0;
        }
        if (!bForceZoom && bWantsToZoom == 0 || bForceNoZoom)
        {
            if (Outer.IsZoomed())
            {
                Outer.SetZoomed(false);
            }
        }
        else
        {
            if (!bForceNoZoom && bWantsToZoom != 0 || bForceZoom)
            {
                if (Outer.IsZoomed() == false)
                {
                    Outer.SetZoomed(true);
                }
            }
        }
    }
}

event PlayerInput(float DeltaTime)
{
    local Vector CamLoc;
    local Rotator CamRot;
    PlayerInput(DeltaTime);
    UpdateViewRotation(DeltaTime);
    Outer.RemappedJoyRight = RawJoyRight;
    Outer.RemappedJoyUp = RawJoyUp;
    if (Outer.IsInCoverState() && BioPawn(Outer.Pawn) != None)
    {
        Outer.GetPlayerViewPoint(CamLoc, CamRot);
        RemapControlsByRotation(Normalize(Outer.Pawn.Rotation - CamRot), Outer.RemappedJoyRight, Outer.RemappedJoyUp);
    }
}

function UpdateViewRotation(float DeltaTime)
{
    local Rotator Kickback;
    local Rotator KickbackFade;
    local SFXPlayerCamera CamManager;
    CamManager = SFXPlayerCamera(Outer.PlayerCamera);
    if (Outer.IsLookInputIgnored() == false)
    {
        CamManager.CameraStick = vect2d(RawJoyLookRight, -RawJoyLookUp);
        CamManager.MovementStick = vect2d(RawJoyRight, RawJoyUp);
    }
    else
    {
        CamManager.CameraStick = vect2d(0, 0);
        CamManager.MovementStick = vect2d(0, 0);
    }
    if (Outer.GameModeManager2.AllowsCameraMods())
    {
        if (Outer.Pawn != None && SFXInventoryManager(Outer.Pawn.InvManager) != None)
        {
            SFXInventoryManager(Outer.Pawn.InvManager).UpdateKickback(DeltaTime, Kickback);
            Kickback -= SFXInventoryManager(Outer.Pawn.InvManager).Drift;
            SFXInventoryManager(Outer.Pawn.InvManager).ApplySniperDrift(DeltaTime);
            Kickback += SFXInventoryManager(Outer.Pawn.InvManager).Drift;
            SFXInventoryManager(Outer.Pawn.InvManager).UpdateKickbackFade(DeltaTime, KickbackFade);
            Kickback -= KickbackFade;
            Outer.SetRotation(Outer.Rotation + Kickback);
        }
        if (Outer.AimAssist != None)
        {
            Outer.AimAssist.UpdateViewRotationForAdhesion(DeltaTime, Outer.Rotation);
        }
    }
}

function RemapControlsByRotation(Rotator DeltaRot, out float NewRight, out float NewUp)
{
    local Vector Controls;
    local Vector NewControls;
    Controls.X = RawJoyRight;
    Controls.Y = RawJoyUp;
    if (IsZero(Controls))
    {
        NewRight = 0;
        NewUp = 0;
        return;
    }
    if (Abs(DeltaRot.Yaw) > Class'WorldInfo'.IsConsoleBuild() ? CoverRemapThreshold : CoverRemapThresholdPC)
    {
        DeltaRot.Pitch = 0;
        NewControls = Normal(Controls >> DeltaRot);
        NewRight = NewControls.X;
        NewUp = NewControls.Y;
    }
}

event CancelReload()
{
    if (SFXWeapon(Outer.Pawn.Weapon) != None)
    {
        SFXWeapon(Outer.Pawn.Weapon).CancelReload();
    }
}

exec function GhostMoveUp(bool bState)
{
    if (Outer.Pawn.Controller.IsInState('PlayerFlying'))
    {
        Outer.m_bDEBUGFlyUpPressed = bState;
    }
}

exec function GhostMoveDown(bool bState)
{
    if (Outer.Pawn.Controller.IsInState('PlayerFlying'))
    {
        Outer.m_bDEBUGFlyDownPressed = bState;
    }
}

defaultproperties
{
    GuiDeadzone=0,4
    StaticConsoleBinds=System.Byte[]
    StaticPCBinds=System.Byte[]
    m_fQuickOrderTime=0,2
    DebugMenu=System.Byte[]
    LookStickResponseCurve=(Points=?,InterpMethod=?)
    bUseMouseDampening=true
    MouseDampeningCurve=(Points=?,InterpMethod=?)
    CoverRemapThresholdPC=20000
    MouseSamples=20
    MouseSamplingTotal=0,4
}
