﻿class BioPlayerSquad extends BioBaseSquad
    hidecategories(Navigation,Actor,Collision,Display,Movement,Physics,RigidBody)
    config(Party)
    native
    placeable;

var ForceFeedbackWaveform m_CoverWaveForm;

var const config float m_fPercentHealthOnEndOfCombat;

var const config float m_fPercentHealthOnResurrection;

var config EBioGameOverCondition m_eGameOverCondition;

var transient SquadTargetData m_aSimpleSquadTargets[3];

var transient SquadTargetData m_aSquadTargets[3];

var transient bool m_bCombatEnabled;

var config float m_fRevivalRange;

var Bio2DA m_oCreditsTable;

var config float m_fPerceptionTolerance;

var bool m_bSquadHasVehicle;

var transient Pawn m_oSquadLeader;

var transient BioPawn m_InitialPlayerPawn;

var transient BioPawn m_playerPawn;

struct native SquadTargetData
{
    var Actor oTarget;

    var Vector vLocation;

    var int nActionIcon;

    var int nSquadIcon;

    var bool bHidden;

    var bool bActive;

    var float fTimeOut;

}

enum EExperienceSourceType
{
    EXPSourceType_SimpleDeath,
    EXPSourceType_SkillUse,
    EXPSourceType_QuestCompletion,
    EXPSourceType_Generic,
};

enum EBioGameOverCondition
{
    BIO_GAME_OVER_CONDITION_PLAYER_PAWN_DIES,
    BIO_GAME_OVER_CONDITION_PLAYER_SQUAD_DIES,
    BIO_GAME_OVER_CONDITION_PLAYER_SQUAD_MAX,
};

event bool OnPlayerSwitchStance(byte FromState, byte ToState)
{
    if (FromState == 0 && ToState == 1)
    {
        SquadEnterCombatMode();
    }
    else
    {
        if (FromState == 1 && ToState == 0)
        {
            SquadExitCombatMode();
        }
    }
    return true;
}

event onPlayerEnterCover()
{
    local BioPlayerController pController;
    pController = BioWorldInfo(WorldInfo).GetLocalPlayerController();
    if (pController != None && pController.Pawn != None && IsZero(pController.Pawn.Velocity))
    {
        pController.ClientPlayForceFeedbackWaveform(m_CoverWaveForm);
    }
}

event NotifyEnemyPerceived()
{
    SquadEnterCombatMode();
    SFXGame(WorldInfo.Game).CombatStarted();
}

event NotifyNoEnemiesPerceived()
{
    local BioPawn Member;
    local MemberData MD;
    foreach Members(MD)
    {
        Member = BioPawn(MD.SquadMember);
        if (BioAiController(Member.Controller) != None && BioAiController(Member.Controller).EnemyList.Length != 0)
        {
            return;
        }
        if (BioPlayerController(Member.Controller) != None && BioPlayerController(Member.Controller).EnemyList.Length != 0)
        {
            return;
        }
    }
    SquadExitCombatMode();
    SFXGame(WorldInfo.Game).CombatEnded();
}

event bool LookupCredits(int nPlayerSquadLevel, Name nmCREDClass, out int nCredits)
{
    local bool bWhetherSucceeded;
    if (m_oCreditsTable == None)
    {
        m_oCreditsTable = Bio2DA(DynamicLoadObject("BIOG_2DA_LevelUp_X.LevelUp_Credits", Class'Bio2DA'));
    }
    if (m_oCreditsTable != None)
    {
        bWhetherSucceeded = m_oCreditsTable.GetIntEntryNumN(nPlayerSquadLevel, nmCREDClass, nCredits);
    }
    else
    {
        bWhetherSucceeded = false;
        nCredits = 0;
    }
    return bWhetherSucceeded;
}

function bool Died(Pawn pPawn, Controller Killer)
{
    Died(pPawn, Killer);
    SFXGame(WorldInfo.Game).OnPlayerSquadDeath();
    return true;
}

function RemoveDyingMember(Pawn oPawn)
{
    if (oPawn != GetMember(0))
    {
        RemoveMember(oPawn);
    }
}

native final function int GetSquadScanAbility();

static function class<BioBaseSaveObject> GetSaveObjectClass()
{
    return Class'BioSquadSaveObject';
}

function SquadEnterCombatMode(optional bool bEngageEnemy = true)
{
    local BioAiController oMember;
    foreach SquadMembers(oMember)
    {
        oMember.ReadyMode(FRand());
    }
    if (bEngageEnemy == true)
    {
        m_bEngagedHostileAction = bEngageEnemy;
    }
}

function SquadExitCombatMode()
{
    local BioAiController oMember;
    local BioPawn oMemberPawn;
    local SFXGame GameInfo;
    GameInfo = SFXGame(WorldInfo.Game);
    if (GameInfo != None && GameInfo.bInCombat || GameInfo.bForceCombat)
    {
        return;
    }
    foreach SquadMembers(oMember)
    {
        oMemberPawn = BioPawn(oMember.Pawn);
        if (oMemberPawn != None)
        {
            oMemberPawn.ShouldCrouch(false);
            SFXGame(WorldInfo.Game).TriggerVocalizationEvent(30, oMemberPawn, None);
        }
    }
    m_bEngagedHostileAction = false;
}

native function bool SetSquadLeader(Pawn pPawn);

native function Pawn GetSquadLeader();

native final function float GetPercentHealthOnResurrection();

native final function bool IsFieldingInitialPlayerPawn();

event int AddMember(Pawn Pawn, optional bool bCheckPlaypens = true)
{
    local int nMemberIndex;
    local SFXAI_Henchman oController;
    local BioPawn oSquadMember;
    nMemberIndex = AddMember(Pawn, bCheckPlaypens);
    if (Members.Length >= 3)
    {
        oController = Pawn != None ? SFXAI_Henchman(Pawn.Controller) : None;
        if (oController != None)
        {
            oController.NotifyHenchmenLoaded();
        }
    }
    nMemberIndex = 0;
    while (nMemberIndex < GetSquadSize())
    {
        oSquadMember = BioPawn(GetMember(nMemberIndex));
        if (oSquadMember != None && oSquadMember != Pawn)
        {
            oSquadMember.OnSquadMemberAdded(Pawn);
        }
        nMemberIndex++;
    }
    return nMemberIndex;
}

native function int GetSquadSize();

event int GetSquadSizeFromScript()
{
    local int nIndex;
    local int nCount;
    local Pawn ThePawn;
    nIndex = 0;
    while (nIndex < Members.Length)
    {
        ThePawn = Members[nIndex].SquadMember;
        if (ThePawn != None)
        {
            if (BioPlayerController(ThePawn.Controller) != None || SFXAI_Henchman(ThePawn.Controller) != None)
            {
                nCount++;
            }
        }
        nIndex++;
    }
    return nCount;
}

event MemberRemoved(Pawn oPawn)
{
    local BioPawn Pawn;
    local BioWorldInfo oWorldInfo;
    local MassEffectGuiManager oGuiManager;
    Pawn = BioPawn(oPawn);
    oWorldInfo = BioWorldInfo(WorldInfo);
    if (Pawn != None && oWorldInfo != None)
    {
        oGuiManager = MassEffectGuiManager(oWorldInfo.GetLocalPlayerController().GetScaleFormManager());
        if (oGuiManager != None && oGuiManager.m_oSFPowerWheelHandler != None)
        {
            oGuiManager.m_oSFPowerWheelHandler.RemoveHenchman(Pawn);
        }
    }
    MemberRemoved(oPawn);
}

defaultproperties
{
    m_fPerceptionTolerance=5
    m_fRevivalRange=2500
    m_bCombatEnabled=true
    m_fPercentHealthOnResurrection=0,2
    m_fPercentHealthOnEndOfCombat=0,5
    m_CoverWaveForm=ForceFeedbackWaveform'ForceFeedbackWaveformBase'
    SquadFaction=class'BioFaction_Player'
    Components=None
    // child object SquadLines of type BioSquadLinesComponent
    // child object CollisionCylinder of type CylinderComponent
    // child object ForceFeedbackWaveformBase of type ForceFeedbackWaveform
}
