﻿class BioPowerScriptDesign extends BioPowerScript
    within BioPower
    abstract;

const GFX_GROUP_ALWAYS_STACKS = -1;
const GFX_GROUP_Marksman = 0;
const GFX_GROUP_Critical_Dmg = 1;
const GFX_GROUP_Critical_Stun = 2;
const GFX_GROUP_Suppression_Caster = 3;
const GFX_GROUP_Suppression_Target = 4;
const GFX_GROUP_Shield_Boost = 5;
const GFX_GROUP_Adrenaline = 6;
const GFX_GROUP_Take_Down = 7;
const GFX_GROUP_Lift = 8;
const GFX_GROUP_Warp_DoT = 9;
const GFX_GROUP_Warp_WeakenArmor = 10;
const GFX_GROUP_Stasis = 11;
const GFX_GROUP_Sabotage_WeaponPowers = 12;
const GFX_GROUP_AI_Hacking = 13;
const GFX_GROUP_Damping_Field = 14;
const GFX_GROUP_Neural_Shock = 15;
const GFX_GROUP_Immunity = 16;
const GFX_GROUP_Resin_Slow = 18;
const GFX_GROUP_Acid_DR_Lowered = 19;
const GFX_GROUP_Singularity = 20;
const GFX_GROUP_Sabotage_Pistol = 21;
const GFX_GROUP_Sabotage_Shotgun = 22;
const GFX_GROUP_Sabotage_Assault = 23;
const GFX_GROUP_Sabotage_Sniper = 24;
const GFX_GROUP_Entrench = 25;
const GFX_GROUP_Carnage = 26;
const GFX_GROUP_Sabotage_BurnDamage = 27;
const GFX_GROUP_Concussive_Stun = 28;
const GFX_GROUP_Concussive_AccDebuff = 29;
const GFX_GROUP_TraumaRegen = 30;
const GFX_GROUP_TraumaDoT = 31;
const GFX_Group_DisruptBeam = 32;
const GFX_Group_BioticImmunity = 33;
const GFX_Group_TechImmunity = 34;
const GFX_GROUP_Overload_Dmg = 35;
const GFX_GROUP_HealthRegen = 36;
const GFX_GROUP_HealthRegenDebuff = 37;
const GFX_GROUP_RateOfFireDebuff = 38;
const GFX_GROUP_SpeedDebuff = 39;
const GFX_GROUP_NeuralShockDoT = 40;
const GFX_GROUP_Crush_DoT = 41;
enum ECombatReaction
{
    CombatReaction_None,
    CombatReaction_Twitch,
    CombatReaction_Standard,
    CombatReaction_Stagger,
    CombatReaction_Ragdoll,
    CombatReaction_GreatPain,
    CombatReaction_OnFire,
};

enum EModalPower
{
    ModalPower_Carnage,
    ModalPower_Overkill,
    ModalPower_Marksman,
    ModalPower_Assassinate,
};

static function bool IsPlaceable(Actor oImpacted)
{
    if (BioPawn(oImpacted) != None)
    {
        return false;
    }
    if (BioArtPlaceable(oImpacted) != None)
    {
        return true;
    }
    if (KActor(oImpacted) != None)
    {
        return true;
    }
    return false;
}

function bool IsDeadBody(Actor oImpacted)
{
    local BioPawn oImpactPawn;
    oImpactPawn = BioPawn(oImpacted);
    if (oImpactPawn == None)
    {
        return false;
    }
    return oImpactPawn.IsDead();
}

static function bool IsOfRace(Actor oImpacted, ECharacterType eRace)
{
    local BioPawn oImpactedPawn;
    oImpactedPawn = BioPawn(oImpacted);
    if (oImpactedPawn == None || oImpactedPawn.m_oBehavior == None)
    {
        return false;
    }
    return oImpactedPawn.PawnType.CharacterType == eRace;
}

function bool IsHumanoidRace(Actor oImpacted)
{
    local BioPawn BP;
    if (BioPawn(oImpacted) != None && BioPawn(oImpacted).PawnType != None)
    {
        BP = BioPawn(oImpacted);
        if (BP.PawnType.RaceType == 1)
        {
            return true;
        }
    }
    return false;
}

function bool IsAnimalRace(Actor oImpacted)
{
    local BioPawn BP;
    if (BioPawn(oImpacted) != None && BioPawn(oImpacted).PawnType != None)
    {
        BP = BioPawn(oImpacted);
        if (BP.PawnType.RaceType == 3)
        {
            return true;
        }
    }
    return false;
}

static function bool IsMachineRace(Actor oImpacted)
{
    local BioPawn BP;
    if (BioPawn(oImpacted) != None && BioPawn(oImpacted).PawnType != None)
    {
        BP = BioPawn(oImpacted);
        if (BP.PawnType.RaceType == 2)
        {
            return true;
        }
    }
    return false;
}

static function int GetPhysicsLevel(Actor oImpacted, optional bool bIgnoreResistance = false)
{
    local SFXGameConfig oConfig;
    local BioPawn oPawn;
    oPawn = BioPawn(oImpacted);
    if (oPawn != None && oPawn.ActorType != None)
    {
        if (oPawn.IsHumanControlled() || SFXPawn_Henchman(oPawn) != None)
        {
            return 5;
        }
        oConfig = SFXGame(oPawn.WorldInfo.Game).gameconfig;
        if (oConfig != None && bIgnoreResistance == false && oConfig.bShieldsBlockPowers && oPawn.GetCurrentShields() > 0)
        {
            return 5;
        }
        return BioActorType(oImpacted.ActorType).m_nPhysicsLevel;
    }
    return 0;
}

function float GetDistanceModifier(Actor oTarget)
{
    local float fDistanceFromImpact;
    local float fPowerMaxRange;
    local float fDistanceModifier;
    local Vector vInvalid;
    if (oTarget == None)
    {
        return 1;
    }
    vInvalid.X = 0;
    vInvalid.Y = 0;
    vInvalid.Z = 0;
    if (m_vWorldImpactLocation == vInvalid || oTarget.Location == vInvalid)
    {
        return 1;
    }
    fDistanceFromImpact = VSize(m_vWorldImpactLocation - oTarget.Location);
    fPowerMaxRange = Outer.GetImpactRadius();
    if (fPowerMaxRange <= 0)
    {
        return 1;
    }
    fDistanceModifier = fPowerMaxRange - fDistanceFromImpact / fPowerMaxRange;
    fDistanceModifier = FMin(1, FMax(0, fDistanceModifier));
    fDistanceModifier *= fDistanceModifier;
    return fDistanceModifier;
}

function EBioFactionRelationship GetFactionRelationship(Actor oCaster, Actor oTarget)
{
    local BioPawn oCasterPawn;
    local BioPawn oTargetPawn;
    oCasterPawn = BioPawn(oCaster);
    oTargetPawn = BioPawn(oTarget);
    if (oCasterPawn == None || oTargetPawn == None)
    {
        return 1;
    }
    if (oCasterPawn.IsHostile(oTargetPawn))
    {
        return 2;
    }
    return 0;
}

function bool SetBioticResistance(Actor oTarget, float fResistance)
{
    local BioPawn Pawn;
    Pawn = BioPawn(oTarget);
    if (Pawn == None)
    {
        return false;
    }
    Pawn.BioticResistance = fResistance;
    Pawn.BioticResistance = FClamp(Pawn.BioticResistance, 0, 1);
    return true;
}

function bool SetTechResistance(Actor oTarget, float fResistance)
{
    local BioPawn Pawn;
    Pawn = BioPawn(oTarget);
    if (Pawn == None)
    {
        return false;
    }
    Pawn.TechResistance = fResistance;
    Pawn.TechResistance = FClamp(Pawn.TechResistance, 0, 1);
    return true;
}

function bool ResetBioticResistance(Actor oTarget)
{
    local BioPawn ChkPawn;
    local SFXLoadoutData Loadout;
    ChkPawn = BioPawn(oTarget);
    if (ChkPawn == None)
    {
        return false;
    }
    Loadout = ChkPawn.GetPawnLoadout();
    if (Loadout != None)
    {
        ChkPawn.BioticResistance = Loadout.BioticResistance;
    }
    return true;
}

function bool ResetTechResistance(Actor oTarget)
{
    local BioPawn ChkPawn;
    local SFXLoadoutData Loadout;
    ChkPawn = BioPawn(oTarget);
    if (ChkPawn == None)
    {
        return false;
    }
    Loadout = ChkPawn.GetPawnLoadout();
    if (Loadout != None)
    {
        ChkPawn.TechResistance = Loadout.TechResistance;
    }
    return true;
}

function bool ModifyArmor(Actor oTarget, float fArmor, optional bool bSetHeavyArmor = false)
{
    local Pawn TargetPawn;
    local SFXArmor_Base Armor;
    TargetPawn = Pawn(oTarget);
    if (TargetPawn == None)
    {
        return false;
    }
    Armor = SFXArmor_Base(TargetPawn.InvManager.FindInventoryType(Class'SFXArmor_Base', true));
    if (Armor != None)
    {
        Armor.DamageResistance.X = FMax(0, Armor.DamageResistance.X + fArmor);
        Armor.DamageResistance.Y = FMax(0, Armor.DamageResistance.Y + fArmor);
        Armor.bShowFortifyOnGUI = bSetHeavyArmor;
    }
    return true;
}

function bool ClearAllPowerCooldowns(Actor oActor)
{
    local BioEpicPawnBehavior oBehavior;
    local int nIndex;
    local BioPower oPower;
    if (Outer.m_bDisableDamageAndEffects)
    {
        return true;
    }
    if (oActor == None)
    {
        return false;
    }
    oBehavior = BioEpicPawnBehavior(oActor.oBioComponent);
    if (oBehavior == None)
    {
        return false;
    }
    nIndex = 0;
    while (nIndex < oBehavior.m_powers.Length)
    {
        oPower = oBehavior.m_powers[nIndex];
        if (oPower != None)
        {
            oPower.SetCooldownRemaining(0);
        }
        nIndex++;
    }
    return true;
}

function bool IncreaseAllPowerCooldowns(Actor oActor, float fDuration, optional bool bIncreaseTech = true, optional bool bIncreaseBiotic = true, optional bool bIncreaseWeapon = false, optional bool bIncreaseCombat = false)
{
    local BioEpicPawnBehavior oBehavior;
    local int nIndex;
    local BioPower oPower;
    local float fCooldownRemaining;
    local EBioCapMode eDiscipline;
    if (Outer.m_bDisableDamageAndEffects)
    {
        return true;
    }
    if (oActor == None)
    {
        return false;
    }
    oBehavior = BioEpicPawnBehavior(oActor.oBioComponent);
    if (oBehavior == None)
    {
        return false;
    }
    nIndex = 0;
    while (nIndex < oBehavior.m_powers.Length)
    {
        oPower = oBehavior.m_powers[nIndex];
        if (oPower != None)
        {
            eDiscipline = oPower.GetDiscipline();
            if (eDiscipline == 2 && bIncreaseTech || eDiscipline == 1 && bIncreaseBiotic || eDiscipline == 0 && bIncreaseWeapon || eDiscipline == 3 && bIncreaseCombat)
            {
                fCooldownRemaining = oPower.GetCooldownRemaining();
                if (fDuration > fCooldownRemaining)
                {
                    oPower.SetCooldownRemaining(fDuration);
                }
            }
        }
        nIndex++;
    }
    return true;
}

function bool AdjustInventoryResource(Actor oActor, EInventoryResourceTypes eResource, int nAmount)
{
    local BioEpicPawnBehavior oBehavior;
    local BioPlayerSquad oPlayerSquad;
    local SFXInventoryManager oInventory;
    local int nCurrentAmount;
    if (Outer.m_bDisableDamageAndEffects)
    {
        return true;
    }
    if (nAmount == 0)
    {
        return true;
    }
    oBehavior = BioEpicPawnBehavior(oActor.oBioComponent);
    if (oBehavior == None)
    {
        return false;
    }
    oPlayerSquad = BioPlayerSquad(oBehavior.Squad);
    if (oPlayerSquad == None)
    {
        return false;
    }
    oInventory = SFXInventoryManager(oPlayerSquad.m_playerPawn.InvManager);
    if (oInventory == None)
    {
        return false;
    }
    nCurrentAmount = oInventory.GetResource(eResource);
    if (nCurrentAmount < nAmount)
    {
        return false;
    }
    oInventory.AdjustResource(eResource, nAmount, true);
    return true;
}

function bool RemovePowerEffects(Actor oTarget, Actor oCaster, Name sRemovePower, optional bool bRemoveFromSquad = false)
{
    local BioPawnBehavior oTargetInfo;
    local BioBaseSquad oTargetSquad;
    local int nSquadIndex;
    local Pawn oSquadMember;
    local array<BioGameProperty> aGP;
    local int nGPIndex;
    local BioGameProperty oCurrGP;
    local bool bReturn;
    local SFXGamePropertiesModule oGameProperties;
    oGameProperties = oTarget.GetModule(Class'SFXGamePropertiesModule');
    oTargetInfo = BioPawnBehavior(oTarget.oBioComponent);
    if (oTargetInfo == None || oGameProperties.m_oAppliedGameProperties == None)
    {
        return false;
    }
    bReturn = false;
    if (bRemoveFromSquad != true)
    {
        if (oGameProperties != None && oGameProperties.m_oAppliedGameProperties.FindGamePropertyByName(sRemovePower, -1, aGP) == true)
        {
            nGPIndex = 0;
            while (nGPIndex < aGP.Length)
            {
                oCurrGP = aGP[nGPIndex];
                if (oCurrGP.m_oOwner == oCaster)
                {
                    oCurrGP.Unapply(oTarget);
                    bReturn = true;
                }
                nGPIndex++;
            }
        }
    }
    else
    {
        oTargetSquad = oTargetInfo.GetSquad();
        nSquadIndex = 0;
        while (nSquadIndex < oTargetSquad.GetSquadSize())
        {
            oSquadMember = oTargetSquad.GetMember(nSquadIndex);
            if (oSquadMember != None)
            {
                oGameProperties = oSquadMember.GetModule(Class'SFXGamePropertiesModule');
                if (oGameProperties != None)
                {
                    if (oGameProperties.m_oAppliedGameProperties.FindGamePropertyByName(sRemovePower, -1, aGP) == true)
                    {
                        nGPIndex = 0;
                        while (nGPIndex < aGP.Length)
                        {
                            oCurrGP = aGP[nGPIndex];
                            if (oCurrGP.m_oOwner == oCaster)
                            {
                                oCurrGP.Unapply(oSquadMember);
                                bReturn = true;
                            }
                            nGPIndex++;
                        }
                    }
                }
            }
            nSquadIndex++;
        }
    }
    return bReturn;
}

static function SpawnWeaponImpactVFX(ImpactInfo Impact, ParticleSystem ImpactParticleSystem, optional bool UseImpactNormal = true, optional bool UseGodBone = false, optional float DrawScale = 1)
{
    local Actor oHitActor;
    local BioPawn oPawn;
    local SFXGame oGame;
    local SFXObjectPool oPool;
    local SkeletalMeshComponent oSkeletalMesh;
    local Vector DirectionToSprayParticles;
    local ParticleSystemComponent PSCO_ImpactVFX;
    local Name BoneName;
    oHitActor = Impact.HitActor;
    if (oHitActor == None)
    {
        return;
    }
    oPawn = BioPawn(oHitActor);
    if (oPawn == None)
    {
        return;
    }
    oGame = SFXGame(oPawn.WorldInfo.Game);
    if (oGame == None)
    {
        return;
    }
    oPool = oGame.ObjectPool;
    if (oPool == None)
    {
        return;
    }
    oSkeletalMesh = oPawn.Mesh;
    if (oSkeletalMesh == None)
    {
        return;
    }
    if (UseImpactNormal)
    {
        DirectionToSprayParticles = Impact.HitNormal;
    }
    else
    {
        DirectionToSprayParticles = Impact.RayDir * -1;
    }
    if (UseGodBone)
    {
        BoneName = 'God';
    }
    else
    {
        BoneName = Impact.HitInfo.BoneName;
    }
    PSCO_ImpactVFX = oPool.GetImpactParticleSystemComponent(ImpactParticleSystem);
    if (PSCO_ImpactVFX == None)
    {
        return;
    }
    oPool.AttachParticleSystemComponent(PSCO_ImpactVFX, oPawn, oPawn.Mesh, BoneName, Impact.HitLocation, DirectionToSprayParticles, false);
    oPool.ApplyLODLevel(PSCO_ImpactVFX, oPawn.Location);
    PSCO_ImpactVFX.SetScale(DrawScale);
    PSCO_ImpactVFX.SetActive(true);
}

static function Name GetRandomImpactBone(BioPawn Pawn)
{
    local int BodyPartIndex;
    local ReactionPart BodyPart;
    local int BoneIndex;
    local Name BoneName;
    if (Pawn != None && Pawn.ReactionBones.Length > 0)
    {
        BodyPartIndex = Rand(Pawn.ReactionBones.Length);
        BodyPart = Pawn.ReactionBones[BodyPartIndex];
        if (BodyPart.BoneNames.Length > 0)
        {
            BoneIndex = Rand(BodyPart.BoneNames.Length);
            BoneName = BodyPart.BoneNames[BoneIndex];
        }
    }
    return BoneName;
}

static function SpawnRandomImpactVFX(BioPawn Pawn, ParticleSystem ImpactParticleSystem)
{
    local SFXGame oGame;
    local SFXObjectPool oPool;
    local SkeletalMeshComponent oSkeletalMesh;
    local Name BoneName;
    local ParticleSystemComponent PSCO_ImpactVFX;
    local Vector Location;
    if (Pawn == None || ImpactParticleSystem == None)
    {
        return;
    }
    oGame = SFXGame(Pawn.WorldInfo.Game);
    if (oGame == None)
    {
        return;
    }
    oPool = oGame.ObjectPool;
    if (oPool == None)
    {
        return;
    }
    BoneName = GetRandomImpactBone(Pawn);
    if (BoneName == 'None')
    {
        return;
    }
    oSkeletalMesh = Pawn.Mesh;
    if (oSkeletalMesh == None)
    {
        return;
    }
    PSCO_ImpactVFX = oPool.GetImpactParticleSystemComponent(ImpactParticleSystem);
    if (PSCO_ImpactVFX == None)
    {
        return;
    }
    oSkeletalMesh.AttachComponent(PSCO_ImpactVFX, BoneName, vect(0,0,0), rot(0,0,0));
    oPool.ApplyLODLevel(PSCO_ImpactVFX, Location);
    PSCO_ImpactVFX.SetActive(true);
}

function bool SetWeaponTracerVFX(Actor oCaster, StaticMesh Mesh)
{
    local BioPawn oPawn;
    local InventoryManager oInventoryManager;
    local SFXWeapon oWeapon;
    oPawn = BioPawn(oCaster);
    if (oPawn == None)
    {
        return false;
    }
    if (Mesh == None)
    {
        return false;
    }
    oInventoryManager = oPawn.InvManager;
    if (oInventoryManager == None)
    {
        return false;
    }
    foreach oInventoryManager.InventoryActors(Class'SFXWeapon', oWeapon)
    {
        if (oWeapon != None)
        {
            oWeapon.TracerInfo.StaticMesh = Mesh;
        }
    }
    return true;
}

function BioPawn GetPawnByName(WorldInfo oWorldInfo, Name PawnName)
{
    local BioPawn Pawn;
    if (oWorldInfo == None)
    {
        return None;
    }
    foreach oWorldInfo.AllPawns(Class'BioPawn', Pawn)
    {
        if (Pawn.Name == PawnName)
        {
            return Pawn;
        }
    }
    return None;
}

function bool PurgeWeaponHeat(SFXWeapon Weapon, optional bool PurgeHeavyWeapons = false)
{
    local SFXHeavyWeapon HeavyWeapon;
    if (Weapon == None)
    {
        return false;
    }
    HeavyWeapon = SFXHeavyWeapon(Weapon);
    if (HeavyWeapon != None)
    {
        if (!PurgeHeavyWeapons)
        {
            return false;
        }
    }
    Weapon.AmmoUsedCount = 0;
    return true;
}

function bool HideActorMesh(Actor oActor)
{
    local SkeletalMeshComponent oComponent;
    if (oActor != None)
    {
        oComponent = Pawn(oActor).Mesh;
        if (oComponent != None)
        {
            oComponent.SetHidden(true);
            return true;
        }
    }
    return false;
}

static function bool ApplyTemporaryGameEffect(Actor oTarget, class<SFXGameEffect> ClassName, float fDuration, float fEffectValue, Name nmCategory, Controller Instigator)
{
    local SFXModule_GameEffectManager Manager;
    local SFXGameEffect Effect;
    if (oTarget != None)
    {
        Manager = oTarget.GetModule(Class'SFXModule_GameEffectManager');
        if (Manager != None)
        {
            Effect = Manager.CreateEffect(ClassName, nmCategory, fDuration, 1, fEffectValue, Instigator);
            if (Effect != None)
            {
                Effect.OnApplied();
            }
            return true;
        }
    }
    return false;
}

static function bool ApplyPermanentGameEffect(Actor oTarget, class<SFXGameEffect> ClassName, float fEffectValue, Name nmCategory, Controller Instigator)
{
    local SFXModule_GameEffectManager Manager;
    local SFXGameEffect Effect;
    if (oTarget != None)
    {
        Manager = oTarget.GetModule(Class'SFXModule_GameEffectManager');
        if (Manager != None)
        {
            Effect = Manager.CreateEffect(ClassName, nmCategory, 0, 2, fEffectValue, Instigator);
            if (Effect != None)
            {
                Effect.OnApplied();
            }
            return true;
        }
    }
    return false;
}

function bool CauseDamageReaction(Actor oActor, Controller InstigatedBy, ECombatReaction Reaction, optional bool OverrideCurrentReaction = false)
{
    local SFXAI_Core AICore;
    local BioPawn oPawn;
    local EAICustomAction CustomAction;
    local BioCustomAction ActionObject;
    if (oActor == None || Reaction == 0)
    {
        return false;
    }
    oPawn = BioPawn(oActor);
    if (oPawn == None)
    {
        return false;
    }
    AICore = SFXAI_Core(oPawn.Controller);
    if (AICore == None)
    {
        return false;
    }
    if (IsInRagdoll(oActor))
    {
        return false;
    }
    CustomAction = GetCustomActionMapping(Reaction);
    if (IsPlayingCombatReaction(oActor))
    {
        if (OverrideCurrentReaction)
        {
            if (Reaction == 2 && AICore.CurrentCustomAction == 17)
            {
                return false;
            }
            AICore.PopState();
            if (AICore.GetCurrentCustomAction(ActionObject) && ActionObject != None)
            {
                ActionObject.StopCustomAction();
            }
        }
        else
        {
            return false;
        }
    }
    oPawn.PlayAnimatedReaction(CustomAction, InstigatedBy);
    return true;
}

function EAICustomAction GetCustomActionMapping(ECombatReaction Reaction)
{
    switch (Reaction)
    {
        case 2:
            return 31;
        case 3:
            return 30;
        case 5:
            return 16;
        case 6:
            return 28;
        default:
            return 0;
    }
}

static function bool IsPlayingCombatReaction(Actor oTarget)
{
    local SFXAI_Core AICore;
    local Pawn oPawn;
    local BioCustomAction oCustomAction;
    local SFXCustomAction_DamageReaction oDamageReaction;
    if (oTarget == None)
    {
        return false;
    }
    oPawn = Pawn(oTarget);
    if (oPawn != None)
    {
        AICore = SFXAI_Core(oPawn.Controller);
        if (AICore != None)
        {
            if (AICore.GetCurrentCustomAction(oCustomAction))
            {
                oDamageReaction = SFXCustomAction_DamageReaction(oCustomAction);
                if (oDamageReaction != None)
                {
                    return true;
                }
            }
        }
    }
    return false;
}

static function int HasAnimatedReactions(BioPawn oPawn, array<EAICustomAction> Actions)
{
    local SFXAI_Core AI;
    local EAICustomAction CustomAction;
    local int nCount;
    if (oPawn == None)
    {
        return 0;
    }
    AI = SFXAI_Core(oPawn.Controller);
    if (AI == None)
    {
        return 0;
    }
    foreach Actions(CustomAction)
    {
        if (AI.CustomActionClasses[CustomAction] != None)
        {
            nCount++;
        }
    }
    return nCount;
}

static function bool PlayRandomReaction(BioPawn oPawn, Controller InstigatedBy, array<EAICustomAction> Actions)
{
    local SFXAI_Core AI;
    local SFXPlayerController PC;
    local EAICustomAction CustomAction;
    local array<EAICustomAction> AvailableActions;
    local BioCustomAction CurrentCustomAction;
    if (oPawn == None)
    {
        return false;
    }
    AI = SFXAI_Core(oPawn.Controller);
    if (AI != None)
    {
        foreach Actions(CustomAction)
        {
            if (CustomAction < AI.CustomActionClasses.Length && AI.CustomActionClasses[CustomAction] != None)
            {
                AvailableActions.AddItem(CustomAction);
            }
        }
    }
    PC = SFXPlayerController(oPawn.Controller);
    if (PC != None)
    {
        foreach Actions(CustomAction)
        {
            if (CustomAction < PC.CustomActionClasses.Length && PC.CustomActionClasses[CustomAction] != None)
            {
                AvailableActions.AddItem(CustomAction);
            }
        }
    }
    if (AvailableActions.Length > 0)
    {
        CustomAction = AvailableActions[Rand(AvailableActions.Length)];
        oPawn.PlayAnimatedReaction(CustomAction, InstigatedBy);
        if (AI != None && AI.GetCurrentCustomAction(CurrentCustomAction) && CurrentCustomAction != None && CurrentCustomAction.Class == AI.CustomActionClasses[CustomAction])
        {
            return true;
        }
        else
        {
            if (PC != None && PC.GetCurrentCustomAction(CurrentCustomAction) && CurrentCustomAction != None && CurrentCustomAction.Class == PC.CustomActionClasses[CustomAction])
            {
                return true;
            }
        }
    }
    return false;
}

function bool IsInRagdoll(Actor oActor)
{
    local BioPawn oPawn;
    oPawn = BioPawn(oActor);
    if (oPawn == None)
    {
        return false;
    }
    return oPawn.Physics == 10;
}

function bool CanBeRagdolledByPowers(BioPawn oPawn)
{
    local SFXAI_Core Controller;
    if (oPawn == None)
    {
        return false;
    }
    Controller = SFXAI_Core(oPawn.Controller);
    if (Controller == None)
    {
        return false;
    }
    if (Controller.CustomActionClasses[5] == None)
    {
        return false;
    }
    return true;
}

function bool GetPowerRank(Actor oActor, int TalentID, out int PowerRank)
{
    local BioTalentContainer oTalents;
    local BioPawnBehavior oBehavior;
    oBehavior = BioPawnBehavior(oActor.oBioComponent);
    if (oBehavior == None)
    {
        return false;
    }
    oTalents = oBehavior.m_Talents;
    if (oTalents == None)
    {
        return false;
    }
    return oTalents.GetTalentRank(TalentID, PowerRank);
}

function float LerpPowerParameter(Vector Parameter, int PowerRank)
{
    local float Rank;
    Rank = PowerRank - 1 / 10;
    return Parameter.Y - Parameter.X * Rank + Parameter.X;
}

function float LerpPowerParameter2D(Vector2D Parameter, int PowerRank)
{
    local float Rank;
    Rank = PowerRank - 1 / 10;
    return Parameter.Y - Parameter.X * Rank + Parameter.X;
}

static function float GetBulletsPerSecond(SFXWeapon Weapon)
{
    local float RateOfFire;
    local SFXWeapon_Shotgun_Base oShotgun;
    if (Weapon == None)
    {
        return 0;
    }
    RateOfFire = Weapon.GetRateOfFire();
    if (Weapon.BurstRefireTime > 0)
    {
        RateOfFire = Weapon.BurstRounds / RateOfFire / 60;
        RateOfFire += Weapon.BurstRefireTime / 2;
        RateOfFire = RateOfFire / Weapon.BurstRounds;
        RateOfFire = 60 / RateOfFire;
    }
    oShotgun = SFXWeapon_Shotgun_Base(Weapon);
    if (oShotgun != None)
    {
        return RateOfFire / 60 * 8;
    }
    return RateOfFire / 60;
}

static function float GetWeaponDamage(SFXWeapon Weapon, Vector HitLocation, Actor HitActor)
{
    local float RangeMultiplier;
    local float Range;
    local float BaseDamage;
    if (Weapon != None)
    {
        RangeMultiplier = 1;
        Weapon.GetRangeMultiplier(RangeMultiplier, Range, HitLocation, HitActor);
        RangeMultiplier = FClamp(RangeMultiplier, 0, 1);
        BaseDamage = Weapon.GetFireModeBaseDamage();
        return BaseDamage * RangeMultiplier;
    }
    return 0;
}

static function float GetBioticMultiplier(Actor oTarget)
{
    local SFXModule_GameEffectManager Manager;
    local float fMultiplier;
    fMultiplier = 1;
    if (oTarget != None)
    {
        Manager = oTarget.GetModule(Class'SFXModule_GameEffectManager');
        if (Manager != None)
        {
            fMultiplier = Manager.GetEffectValue(Class'SFXGameEffect_BioticWeakness', 1);
        }
    }
    return fMultiplier;
}

static function float GetPhysicsLevelResistance(Actor oTarget)
{
    local int PhysicsLevel;
    local float Resistance;
    Resistance = 0;
    if (oTarget != None)
    {
        PhysicsLevel = GetPhysicsLevel(oTarget, true);
        if (PhysicsLevel > 0)
        {
            Resistance = PhysicsLevel + 1 * 0,1;
        }
    }
    return Resistance;
}

function bool HasEffectType(Actor oTarget, class<Object> EffectClass)
{
    local SFXModule_GameEffectManager Manager;
    local SFXGameEffect oEffect;
    if (oTarget == None)
    {
        return false;
    }
    Manager = oTarget.GetModule(Class'SFXModule_GameEffectManager');
    if (Manager != None)
    {
        foreach Manager.GameEffects(oEffect)
        {
            if (oEffect.Class == EffectClass)
            {
                return true;
            }
        }
    }
    return false;
}

static function RemoveBioticEffects(BioPawn oPawn, optional bool bRemoveStasis)
{
    local SFXModule_GameEffectManager Manager;
    if (oPawn == None)
    {
        return;
    }
    Manager = oPawn.GetModule(Class'SFXModule_GameEffectManager');
    if (Manager == None)
    {
        return;
    }
    Manager.RemoveEffectsByType(Class'SFXGameEffect_Singularity');
    Manager.RemoveEffectsByType(Class'SFXGameEffect_Crush');
    Manager.RemoveEffectsByType(Class'SFXGameEffect_Pull');
    Manager.RemoveEffectsByType(Class'SFXGameEffect_AntiGravity');
    Manager.RemoveEffectsByType(Class'SFXGameEffect_Ragdoll');
    Manager.RemoveEffectsByType(Class'SFXGameEffect_BioticWeakness');
    if (bRemoveStasis)
    {
        Manager.RemoveEffectsByType(Class'SFXGameEffect_Stasis');
    }
}

static function CheckBioticComboAchievement(BioPawn oCaster, BioPawn oTarget, bool bOrderedByPlayer)
{
    local BioPlayerController oPC;
    local BioWorldInfo pWorldInfo;
    pWorldInfo = BioWorldInfo(oCaster.WorldInfo);
    if (pWorldInfo == None)
    {
        return;
    }
    oPC = pWorldInfo.GetLocalPlayerController();
    if (oPC == None)
    {
        return;
    }
    if (oCaster.IsHumanControlled() || bOrderedByPlayer && oPC != None && oTarget.HasGameEffect('SFXGameEffect_BioticWeakness'))
    {
        oPC.UpdateAchievementProgression(33, oTarget);
    }
}

function RagdollPhysicsImpact(Pawn oPawn, Actor oImpactActor, Vector vImpactDir, optional float fVelocityToDamageMult = 0,05)
{
    local BioPawn oBioPawn;
    local SFXModule_Damage DmgModule;
    local SFXPower oPower;
    local float fDamage;
    local Vector HitLocation;
    local Vector Momentum;
    local BioPawn oCaster;
    local float fVelocity;
    local SFXModule_GameEffectManager Manager;
    local array<EBioPartGroup> Parts;
    local Vector2D OldCastTime;
    local bool bWasDeathPowerOn;
    oBioPawn = BioPawn(oPawn);
    if (oBioPawn == None || oBioPawn.m_oBehavior == None || oBioPawn.PowerManager == None)
    {
        return;
    }
    oCaster = BioPawn(Outer.Owner);
    if (oCaster == None)
    {
        return;
    }
    DmgModule = oPawn.GetModule(Class'SFXModule_Damage');
    if (DmgModule == None)
    {
        return;
    }
    Manager = oPawn.GetModule(Class'SFXModule_GameEffectManager');
    if (Manager == None)
    {
        return;
    }
    fVelocity = VSize(oPawn.Velocity);
    if (fVelocity <= 0)
    {
        fVelocity = VSize(vImpactDir);
    }
    fDamage = fVelocity * fVelocityToDamageMult;
    fDamage *= Manager.GetEffectValue(Class'SFXGameEffect_PhysicsDamageMultiplier', 1);
    if (fVelocity > 350)
    {
        if (GetPhysicsLevel(oPawn) <= 0)
        {
            Parts = oPawn.GetModule(Class'SFXModule_Damage').GetConstraints();
            if (Parts.Length > 0)
            {
                oPawn.GetModule(Class'SFXModule_Damage').DestroyConstraint(Parts[Rand(Parts.Length)]);
            }
        }
    }
    if (IsMachineRace(oPawn))
    {
        foreach oBioPawn.PowerManager.Powers(oPower)
        {
            if (oPower.CapabilityType == 9)
            {
                OldCastTime = oPower.CastingTime;
                oPower.CastingTime.X = 0;
                oPower.CastingTime.Y = 0;
                break;
            }
        }
        bWasDeathPowerOn = oBioPawn.m_oBehavior.m_bCanUseDeathCapabilities;
        oBioPawn.m_oBehavior.m_bCanUseDeathCapabilities = true;
    }
    oBioPawn.TakeDamage(fDamage, oCaster.Controller, HitLocation, Momentum, Class'SFXDamageType_PowerPhysics');
    if (oBioPawn.GetCurrentHealth() <= 0)
    {
        if (IsMachineRace(oPawn))
        {
            if (Manager.HasEffectOfType(Class'SFXGameEffect_CryoFreeze') || Manager.HasEffectOfType(Class'SFXGameEffect_DelayedCryoFreeze'))
            {
                Manager.RemoveEffectsByType(Class'SFXGameEffect_CryoFreeze');
                Manager.RemoveEffectsByType(Class'SFXGameEffect_DelayedCryoFreeze');
            }
        }
    }
    else
    {
        if (IsMachineRace(oPawn))
        {
            foreach oBioPawn.PowerManager.Powers(oPower)
            {
                if (oPower.CapabilityType == 9)
                {
                    oPower.CastingTime.X = OldCastTime.X;
                    oPower.CastingTime.Y = OldCastTime.Y;
                    break;
                }
            }
            oBioPawn.m_oBehavior.m_bCanUseDeathCapabilities = bWasDeathPowerOn;
        }
    }
}

static function bool ImpactPlaceable(Actor oCaster, Actor oImpacted, Vector Momentum, float PlotPlaceableDamage, optional bool bForceDamage)
{
    local KActor oKActor;
    local SFXKActor oSFXKActor;
    local BioPawn oCasterPawn;
    local BioPhysicsActor PhysActor;
    local float fDamage;
    oCasterPawn = BioPawn(oCaster);
    if (oCasterPawn == None)
    {
        return false;
    }
    oKActor = KActor(oImpacted);
    if (oKActor == None)
    {
        return false;
    }
    oImpacted.ExceededPhysicsThreshold(oCaster);
    oSFXKActor = SFXKActor(oImpacted);
    if (!bForceDamage && oSFXKActor != None && !oSFXKActor.bImmovable)
    {
        oKActor.CollisionComponent.AddForce(Momentum, oKActor.Location, 'None');
    }
    else
    {
        PhysActor = BioPhysicsActor(oImpacted);
        if (PhysActor == None || PhysActor.m_bToughPlaceable)
        {
            fDamage = PlotPlaceableDamage;
        }
        else
        {
            fDamage = 10000;
        }
        oKActor.TakeDamage(fDamage, oCasterPawn.Controller, vect(0,0,0), Momentum, Class'SFXDamageType_Default');
    }
    return true;
}

function SpawnVFXAtLocation(Vector Location, ParticleSystem PS)
{
    local SFXGame oGame;
    local SFXObjectPool oPool;
    local SFXEmitter oEmitter;
    local Rotator Rotation;
    if (Outer.Owner == None)
    {
        return;
    }
    oGame = SFXGame(Outer.Owner.WorldInfo.Game);
    if (oGame == None)
    {
        return;
    }
    oPool = oGame.ObjectPool;
    if (oPool == None)
    {
        return;
    }
    oEmitter = SFXEmitter(oPool.GetImpactEmitter(PS, Location, Rotation));
    if (oEmitter != None && oEmitter.ParticleSystemComponent != None)
    {
        oEmitter.ParticleSystemComponent.SetActive(true);
    }
}

function bool HasRemainingShieldOfType(BioPawn oPawn, class<SFXShield_Base> ShieldClass)
{
    local SFXShield_Base Shield;
    if (oPawn != None && oPawn.InvManager != None)
    {
        foreach oPawn.InvManager.InventoryActors(Class'SFXShield_Base', Shield)
        {
            if (ClassIsChildOf(Shield.Class, ShieldClass) && Shield.CurrentShields > 0)
            {
                return true;
            }
        }
    }
    return false;
}

defaultproperties
{
}
