﻿class BioSFHandler_Options extends BioSFHandler
    transient
    native;

const Option_Close = 1;
const Option_ResetDefaults = 2;
const Option_Initialize = 3;
const Option_SetOption = 4;
const Option_SaveAndExit = 5;
const Option_ConfirmCancelExit = 6;
enum EOptionsGuiMode
{
    GuiMode_BrowserWheel,
    GuiMode_MainMenu,
    GuiMode_NewGame,
};

// srNo: StringRefProperty[44]
// srYes: StringRefProperty[44]
// srSkipConfirm: StringRefProperty[44]
var /* 00000008 */ SFXProfileSettings NewSettings;

// CancelExitFromMenuText: StringRefProperty[44]
// ConfirmExitFromMenuText: StringRefProperty[44]
// ExitConfirmText: StringRefProperty[44]
// CancelResetToDefaultsText: StringRefProperty[44]
// ConfirmResetToDefaultsText: StringRefProperty[44]
// ResetToDefaultsText: StringRefProperty[44]
var array<int> m_lstOptsToReset;

var() bool bWaitingOnMsgBox;

var() bool m_bSelfPanelClose;

var() transient EOptionsGuiMode GuiMode;

delegate OnCloseCallback();

function SetOnCloseCallback(delegate<OnCloseCallback> fn_OnCloseDelegate)
{
    __OnCloseCallback__Delegate = fn_OnCloseDelegate;
}

function HandleEvent(byte Command, const out array<String> lstArguments)
{
    local MassEffectGuiManager GuiMan;
    local BioPlayerController PC;
    if (bWaitingOnMsgBox)
    {
        return;
    }
    GuiMan = MassEffectGuiManager(oPanel.oParentManager);
    if (GuiMan == None)
    {
        return;
    }
    PC = BioPlayerController(GuiMan.Player);
    if (PC == None)
    {
        return;
    }
    switch (Command)
    {
        case 1:
            CloseGui();
            break;
        case 2:
            Helper_CreateResetConfirmPopup(GuiMan, Callback_ResetToDefaults);
            m_lstOptsToReset.Length = 0;
            break;
        case 3:
            if (GuiMode == 2)
            {
                PlayGuiSound('NewGameOptions');
            }
            NewSettings.SetToDefaults();
            CopyProfileSettings(PC.ProfileSettings, NewSettings);
            SetOptionsOnGUI(NewSettings);
            break;
        case 4:
            SetOption(lstArguments[0], lstArguments[1]);
            break;
        case 5:
        case 6:
            CopyProfileSettings(NewSettings, PC.ProfileSettings);
            PC.SaveProfile();
            UpdateAudioVolumeSettings(PC.ProfileSettings);
            CloseGui();
            break;
    }
    if (Command == 1 || Command == 5 || Command == 6)
    {
        if (PC.ProfileSettings.GetDifficultyConfigOption() < 4)
        {
            BioWorldInfo(PC.WorldInfo).GetGlobalVariables().SetBoolByName('ChangedDifficulty', true);
        }
    }
}

function Helper_CreateResetConfirmPopup(MassEffectGuiManager GuiMan, delegate<InputCallback> pInputDelegate)
{
    local BioSFHandler_MessageBox MessageBox;
    local BioMessageBoxOptionalParams Params;
    MessageBox = GuiMan.CreateMessageBox();
    MessageBox.SetInputDelegate(pInputDelegate);
    Params.srAText = ConfirmResetToDefaultsText;
    Params.srBText = CancelResetToDefaultsText;
    Params.bNoFade = true;
    MessageBox.DisplayMessageBox(ResetToDefaultsText, Params);
    bWaitingOnMsgBox = true;
}

function Helper_ExtractPackedOptIDs(out array<int> lstOptIDs, String sPackedIDs)
{
    local int I;
    local array<String> lstStringParts;
    if (Len(sPackedIDs) > 0)
    {
        ParseStringIntoArray(sPackedIDs, lstStringParts, ",", true);
        lstOptIDs.Length = lstStringParts.Length;
        I = 0;
        while (I < lstStringParts.Length)
        {
            lstOptIDs[I] = lstStringParts[I];
            ++I;
        }
    }
}

function onExIntResetToDefaults(String sPackedIDs)
{
    local MassEffectGuiManager GuiMan;
    GuiMan = MassEffectGuiManager(oPanel.oParentManager);
    if (GuiMan != None)
    {
        Helper_CreateResetConfirmPopup(GuiMan, Callback_ResetToDefaults);
    }
    Helper_ExtractPackedOptIDs(m_lstOptsToReset, sPackedIDs);
}

function Callback_ResetToDefaults(bool bAPressed, int Context)
{
    local BioPlayerController PC;
    local array<EProfileSetting> lstProfileSettings;
    if (bAPressed)
    {
        PC = BioPlayerController(GetPlayerController());
        if (PC != None)
        {
            GetProfileSettingsForOptions(lstProfileSettings, m_lstOptsToReset);
            NewSettings.SetToDefaultsEx(lstProfileSettings);
            SetOptionsOnGUI(NewSettings);
            UpdateDisplayGamma(NewSettings);
        }
    }
    bWaitingOnMsgBox = false;
}

function Callback_ExitFromMenu(bool bAPressed, int Context);

native function CopyProfileSettings(SFXProfileSettings Source, SFXProfileSettings Dest);

native function SetOptionsOnGUI(SFXProfileSettings Settings);

native function SetOption(int Type, int Value);

native function EProfileSetting GetProfileSettingForOption(int Type);

native function GetProfileSettingsForOptions(out array<EProfileSetting> lstProfSettings, const out array<int> lstTypes);

native function UpdateDisplayGamma(SFXProfileSettings ReadFrom);

native function UpdateAudioVolumeSettings(SFXProfileSettings ReadFrom);

final function CloseGui()
{
    local MassEffectGuiManager GuiMan;
    if (__OnCloseCallback__Delegate != None)
    {
        OnCloseCallback();
    }
    GuiMan = MassEffectGuiManager(oPanel.oParentManager);
    if (GuiMan != None)
    {
        switch (GuiMode);
        case 1;
        GuiMan.ShowMainMenu(9);
        if (!m_bSelfPanelClose) jump 125;
        GuiMan.RemovePanel(oPanel, true);
        jump 249;
        case 0;
        PlayGuiSound('ReturnToBrowser');
        GuiMan.ReturnToBrowserWheel(oPanel);
        jump 249;
        case 2;
        GuiMan.StopSound('NewGameOptions');
        if (!m_bSelfPanelClose) jump 243;
        GuiMan.RemovePanel(oPanel, true);
        jump 249;
        default;
    }
    bWaitingOnMsgBox = false;
    __OnCloseCallback__Delegate = None;
    return;
}

function OnPanelAdded()
{
    OnPanelAdded();
    oPanel.SetExternalInterface(self);
}

defaultproperties
{
    m_bSelfPanelClose=true
    ResetToDefaultsText=System.Byte[]
    ConfirmResetToDefaultsText=System.Byte[]
    CancelResetToDefaultsText=System.Byte[]
    ExitConfirmText=System.Byte[]
    ConfirmExitFromMenuText=System.Byte[]
    CancelExitFromMenuText=System.Byte[]
    NewSettings=SFXProfileSettings'Settings0'
    srSkipConfirm=System.Byte[]
    srYes=System.Byte[]
    srNo=System.Byte[]
    nHandlerID=28
    // child object Settings0 of type SFXProfileSettings
}
