﻿class BioVehicleBase extends SVehicle
    hidecategories(Navigation,Sounds)
    config(Game)
    abstract
    native
    placeable;

var /* 00000008 */ AudioComponent m_pLowShieldSound;

var /* 00000008 */ LightEnvironmentComponent m_pLightEnvComponent;

var transient config String m_sAudioTiresVariableValuePavement;

var transient config String m_sAudioTiresVariableValueWater;

var transient config String m_sAudioTiresVariableValueAir;

var transient config String m_sAudioTiresVariableName;

var(BioSound) /* 00000008 */ AudioComponent m_OperationalSounds;

var bool m_bDebugVehicleEnterExitSequence;

var(BioInit) bool m_bAwakeOnLevelStart;

var() config bool m_bShowVehicleDebugHUD;

var(Behavior) BioVehicleBehaviorBase m_oBehavior;

var transient GFxMovieInfo m_oCurrentAreaMap;

// m_sVehicleName: StringRefProperty[44]
var(AreaMap) GFxMovieInfo m_oAreaMap;

native function BioVISSimple GetVisualImpactSet();

native function bool BioCanEnterVehicle();

native function bool BioCanExitVehicle();

native function bool PerformForceExitCleanup();

native final private function InitializeForPlay();

native function bool IsExitLocationValid(Vector vExitLocation);

native function KillAllMomentum();

function SetPartyCollisionState(bool bCollideState)
{
    local Pawn oPawn;
    local int J;
    J = 0;
    while (J < m_oBehavior.m_oDriverSquad.GetSquadSize())
    {
        oPawn = m_oBehavior.m_oDriverSquad.GetMember(J);
        if (oPawn == self)
        {
            if (m_bDebugVehicleEnterExitSequence == true)
            {
            }
        }
        else
        {
            if (m_bDebugVehicleEnterExitSequence == true)
            {
            }
            oPawn.SetHardAttach(!bCollideState);
            oPawn.bCollideWorld = bCollideState;
            oPawn.SetCollision(bCollideState, bCollideState);
        }
        J++;
    }
}

event bool DriverLeave(bool bForceLeave)
{
    local bool bTryNormalExit;
    local bool bTryForceExit;
    local Pawn oTempDriver;
    local Rotator ExitRotation;
    local Controller C;
    local PlayerController PC;
    if (m_bDebugVehicleEnterExitSequence == true)
    {
    }
    if (Driver.IsDead())
    {
        if (m_bDebugVehicleEnterExitSequence == true)
        {
        }
        return false;
    }
    oTempDriver = Driver;
    oTempDriver.SetWasInVehicleTransitionStatus(true);
    if (Driver != None)
    {
        SetPartyCollisionState(true);
        if (bForceLeave == false)
        {
            bTryNormalExit = PlaceExitingDriver();
        }
        if (m_bDebugVehicleEnterExitSequence == true)
        {
        }
        if (bTryNormalExit == false && bForceLeave == false)
        {
            if (m_bDebugVehicleEnterExitSequence == true)
            {
            }
            SetPartyCollisionState(false);
        }
        if (bTryNormalExit == true || bForceLeave == true)
        {
            if (bTryNormalExit == false && bForceLeave == true)
            {
                if (m_bDebugVehicleEnterExitSequence == true)
                {
                }
                bTryForceExit = SetExitingSquadLocation(m_oBehavior.m_vForceExitLocation, 4 * Driver.GetCollisionRadius());
                if (bTryForceExit == false)
                {
                    if (m_bDebugVehicleEnterExitSequence == true)
                    {
                    }
                    SetPartyCollisionState(false);
                }
            }
            if (bTryNormalExit == true || bTryForceExit == true)
            {
                if (m_bDebugVehicleEnterExitSequence == true)
                {
                }
                ExitRotation.Yaw = Controller.Rotation.Yaw;
                SetDriving(false);
                C = Controller;
                if (C.RouteGoal == self)
                {
                    C.RouteGoal = None;
                }
                if (C.MoveTarget == self)
                {
                    C.MoveTarget = None;
                }
                Controller.UnPossess();
                if (Driver != None && Driver.Health > 0)
                {
                    Driver.SetRotation(ExitRotation);
                    Driver.SetOwner(C);
                    C.Possess(Driver, true);
                    PC = PlayerController(C);
                    if (PC != None)
                    {
                        PC.ClientSetViewTarget(Driver);
                    }
                    Driver.StopDriving(self);
                }
                if (C == Controller)
                {
                    Controller = None;
                }
                WorldInfo.Game.DriverLeftVehicle(self, Driver);
                DriverLeft();
                m_oBehavior.OnVehicleExit();
                if (m_oBehavior.m_bIncludeDriverSquad)
                {
                    if (m_bDebugVehicleEnterExitSequence == true)
                    {
                    }
                    m_oBehavior.SquadExit();
                }
            }
        }
        oTempDriver.SetWasInVehicleTransitionStatus(false);
    }
    return bTryNormalExit == true || bTryForceExit == true;
}

function bool DriverEnter(Pawn P)
{
    local bool bRetval;
    bRetval = false;
    P.SetWasInVehicleTransitionStatus(true);
    if (DriverEnter(P))
    {
        m_oBehavior.OnVehicleEnter();
        if (m_oBehavior.m_bIncludeDriverSquad)
        {
            m_oBehavior.SquadEnter(BioEpicPawnBehavior(P.oBioComponent).Squad);
        }
        bRetval = true;
    }
    P.SetWasInVehicleTransitionStatus(false);
    return bRetval;
}

function bool PlaceExitingDriver(optional Pawn ExitingDriver)
{
    local int I;
    local Vector tryPlace;
    local Vector Extent;
    local Vector ZOffset;
    local float fExitRadius;
    local float fPartyDistanceApart;
    if (m_bDebugVehicleEnterExitSequence == true)
    {
    }
    if (ExitingDriver == None)
    {
        ExitingDriver = Driver;
    }
    if (ExitingDriver == None)
    {
        return false;
    }
    fExitRadius = ExitingDriver.GetCollisionRadius();
    if (m_bDebugVehicleEnterExitSequence == true)
    {
    }
    if (m_oBehavior.m_bIncludeDriverSquad == true && m_oBehavior.m_oDriverSquad != None)
    {
        if (m_oBehavior.m_oDriverSquad.GetNumberOfLiveMembers() > 0)
        {
            fExitRadius *= 3;
            if (m_bDebugVehicleEnterExitSequence == true)
            {
            }
        }
    }
    Extent = fExitRadius * vect(1,1,0);
    Extent.Z = ExitingDriver.GetCollisionHeight();
    ZOffset = Extent.Z * vect(0,0,1);
    fPartyDistanceApart = 4 * ExitingDriver.GetCollisionRadius();
    if (ExitPositions.Length > 0)
    {
        I = 0;
        while (I < ExitPositions.Length)
        {
            if (ExitPositions[0].Z != 0)
            {
                ZOffset = vect(0,0,1) * ExitPositions[0].Z;
            }
            else
            {
                ZOffset = ExitingDriver.CylinderComponent.Default.CollisionHeight * vect(0,0,2);
            }
            tryPlace = Location + ExitPositions[I] - ZOffset >> Rotation + ZOffset;
            if (m_bDebugVehicleEnterExitSequence == true)
            {
                DrawDebugLine(Location, tryPlace, 0, 255, 255, true);
            }
            if (IsExitLocationValid(tryPlace) == false)
            {
                if (m_bDebugVehicleEnterExitSequence == true)
                {
                }
            }
            else
            {
                if (SetExitingSquadLocation(tryPlace, fPartyDistanceApart) == false)
                {
                    if (m_bDebugVehicleEnterExitSequence == true)
                    {
                    }
                    jump 643;
                }
                else
                {
                    if (m_bDebugVehicleEnterExitSequence == true)
                    {
                    }
                    return true;
                }
            }
            I++;
        }
        if (m_bDebugVehicleEnterExitSequence == true)
        {
        }
        return false;
    }
    else
    {
        if (m_bDebugVehicleEnterExitSequence == true)
        {
        }
        return FindAutoExit(ExitingDriver);
    }
}

function bool SetExitingSquadLocation(Vector vExitCenterLocation, float fPlacementOffset)
{
    local Vector vPlacementOffset;
    local Rotator rPartyFormationAngle;
    local Rotator rVehicleYawAligned;
    local int J;
    local Pawn oPawn;
    if (m_bDebugVehicleEnterExitSequence == true)
    {
    }
    if (m_oBehavior.m_bIncludeDriverSquad == true && m_oBehavior.m_oDriverSquad != None)
    {
        if (m_bDebugVehicleEnterExitSequence == true)
        {
        }
        vPlacementOffset = vect(0,0,0);
        rVehicleYawAligned.Yaw = Rotation.Yaw;
        rPartyFormationAngle.Yaw = 60 * 182,0444;
        J = 0;
        while (J < m_oBehavior.m_oDriverSquad.GetSquadSize())
        {
            oPawn = m_oBehavior.m_oDriverSquad.GetMember(J);
            if (oPawn == self)
            {
                if (m_bDebugVehicleEnterExitSequence == true)
                {
                }
            }
            else
            {
                if (m_bDebugVehicleEnterExitSequence == true)
                {
                    DrawDebugLine(vExitCenterLocation, vExitCenterLocation + vPlacementOffset, 0, 255, 0, true);
                }
                if (!oPawn.SetLocation(vExitCenterLocation + vPlacementOffset))
                {
                    if (m_bDebugVehicleEnterExitSequence == true)
                    {
                    }
                    break;
                }
                if (J == 0)
                {
                    vPlacementOffset = vect(1,0,0) >> rVehicleYawAligned;
                    vPlacementOffset *= -fPlacementOffset;
                }
                vPlacementOffset = vPlacementOffset >> rPartyFormationAngle;
            }
            J++;
        }
        if (m_bDebugVehicleEnterExitSequence == true)
        {
        }
        return J == m_oBehavior.m_oDriverSquad.GetSquadSize();
    }
    if (m_bDebugVehicleEnterExitSequence == true)
    {
    }
    return true;
}

simulated function AttachDriver(Pawn P)
{
    if (!bAttachDriver)
    {
        return;
    }
    SetPhysics(0);
    if (m_bDebugVehicleEnterExitSequence == true)
    {
    }
    P.SetWasInVehicleTransitionStatus(true);
    P.bCollideWorld = false;
    P.SetBase(None);
    P.SetHardAttach(true);
    P.SetPhysics(0);
    P.SetCollision(false, false);
    if (bDriverIsVisible == true && P.Mesh != None && Mesh != None)
    {
        P.Mesh.SetShadowParent(Mesh);
    }
    if (!bDriverIsVisible)
    {
        if (BioPawn(P) != None)
        {
            BioPawn(P).RequestHidden(true);
        }
        else
        {
            P.SetHidden(true);
        }
        P.SetLocation(Location);
    }
    P.SetPhysics(0);
    P.SetBase(self);
    if (P.oBioComponent != None)
    {
        BioActorBehavior(P.oBioComponent).m_bTargetableOverride = true;
        BioActorBehavior(P.oBioComponent).m_bTargetable = false;
    }
    P.SetWasInVehicleTransitionStatus(false);
    SetPhysics(10);
}

event BioProxyAttachDriver(Pawn P)
{
    AttachDriver(P);
}

simulated function DetachDriver(Pawn P)
{
    if (!bAttachDriver)
    {
        return;
    }
    if (m_bDebugVehicleEnterExitSequence == true)
    {
    }
    P.SetWasInVehicleTransitionStatus(true);
    P.SetCollision(true, true);
    P.SetHardAttach(false);
    P.bCollideWorld = true;
    P.SetBase(None);
    P.SetPhysics(1);
    P.SetBase(None);
    if (P.Mesh != None && Mesh != None)
    {
        P.Mesh.SetShadowParent(None);
    }
    if (!bDriverIsVisible)
    {
        if (BioPawn(P) != None)
        {
            BioPawn(P).RequestHidden(false);
        }
        else
        {
            P.SetHidden(false);
        }
    }
    P.SetPhysics(1);
    if (P.oBioComponent != None)
    {
        BioActorBehavior(P.oBioComponent).m_bTargetableOverride = false;
        BioActorBehavior(P.oBioComponent).m_bTargetable = false;
    }
    P.SetWasInVehicleTransitionStatus(false);
}

event BioProxyDetachDriver(Pawn P)
{
    DetachDriver(P);
}

simulated function DrawHUD(HUD H)
{
    local float fX;
    local float fY;
    local float fInc;
    local float fCarKMPerHour;
    if (m_bShowVehicleDebugHUD)
    {
        H.Canvas.SetDrawColor(255, 255, 255, 255);
        fX = 0;
        fY = 120;
        fInc = 12;
        fCarKMPerHour = Abs(ForwardVel * 3600 / 140800) * 1,609;
        H.Canvas.Font = Class'Engine'.GetSmallFont();
        H.Canvas.SetPos(fX, fY);
        fY += fInc;
        H.Canvas.DrawText("Inputs: Throttle: " $ Throttle $ " Steer: " $ Steering $ " Rise: " $ Rise);
        H.Canvas.SetPos(fX, fY);
        fY += fInc;
        H.Canvas.DrawText("Outputs: Gas: " $ OutputGas $ " OutSteer: " $ OutputSteering);
        H.Canvas.SetPos(fX, fY);
        fY += fInc;
        H.Canvas.DrawText("Speed UU/S: " $ ForwardVel $ " Speed (KM/H): " $ fCarKMPerHour);
        fY += fInc;
        H.Canvas.SetPos(fX, fY);
        fY += fInc;
        H.Canvas.DrawText("Thrusters   : " $ Class'BioAttributes'.GetCurrentFloatValue(BioAttributesPawnVehicle(m_oBehavior.m_PawnAttributes).m_ThrusterAmount));
        H.Canvas.SetPos(fX, fY);
        fY += fInc;
        H.Canvas.DrawText("FL Scalar: " $ m_oBehavior.m_fThrusterScalarFL $ " FR Scalar: " $ m_oBehavior.m_fThrusterScalarFR);
        H.Canvas.SetPos(fX, fY);
        fY += fInc;
        H.Canvas.DrawText("BL Scalar: " $ m_oBehavior.m_fThrusterScalarBL $ " BR Scalar: " $ m_oBehavior.m_fThrusterScalarBR);
    }
}

event bool ProxyTryToDrive(Pawn P)
{
    return TryToDrive(P);
}

simulated function SetInputs(float InForward, float InStrafe, float InUp)
{
    if (m_oBehavior.m_bPowertrainEnabled == true && m_oBehavior.m_bIsPaused == false)
    {
        SetInputs(InForward, InStrafe, InUp);
    }
    else
    {
        Throttle = 0;
        Steering = 0;
        Rise = 0;
    }
}

static function class<BioBaseSaveObject> GetSaveObjectClass()
{
    return Class'BioVehicleSaveObject';
}

simulated function OnTeleport(SeqAct_Teleport Action)
{
    local BioPlayerController oPlayerController;
    oPlayerController = BioPlayerController(Controller);
    if (oPlayerController != None && m_oBehavior.m_nCurrentZoomTurret < 0)
    {
        oPlayerController.OnTeleportCameraSync(Action);
    }
    OnTeleport(Action);
    SetPhysics(0);
    Mesh.SetRBPosition(Location);
    SetPhysics(10);
    if (Action.bUpdateRotation == true)
    {
        Mesh.SetRBRotation(Rotation);
    }
    if (Action.m_bPreserveVelocity == false)
    {
        KillAllMomentum();
    }
}

defaultproperties
{
    m_bAwakeOnLevelStart=true
    m_sAudioTiresVariableName="tires"
    m_sAudioTiresVariableValueAir="air"
    m_sAudioTiresVariableValueWater="water"
    m_sAudioTiresVariableValuePavement="pavement"
    m_pLightEnvComponent=DynamicLightEnvironmentComponent'LightEnvComponent0'
    StayUprightConstraintSetup=None
    StayUprightConstraintInstance=None
    UprightLiftStrength=1500
    UprightTorqueStrength=1000
    ExitPositions=None
    // child object CollisionCylinder of type CylinderComponent
    // child object SVehicleMesh of type SkeletalMeshComponent
    // child object LightEnvComponent0 of type DynamicLightEnvironmentComponent
    // child object MyStayUprightSetup of type RB_StayUprightSetup
    // child object MyStayUprightConstraintInstance of type RB_ConstraintInstance
    // child object RadarModule of type SFXModule_Radar
    // child object GPModule of type SFXGamePropertiesModule
    // child object oBehavior of type BioVehicleBehaviorBase
}
