﻿class BioVehicleBehaviorBase extends BioEpicPawnBehavior
    config(Game)
    native
    editinlinenew;

struct native BioVehicleWeaponInfo
{
    var int nTurretIndex;

    var bool bClampToTurret;

    var float fTurretYawHalfAngleInDeg;

    var float fTurretPitchHalfAngleInDeg;

}

var transient Vector m_vPauseVelocity;

var transient int m_nCurrentZoomTurret;

var Vector m_vForceExitLocation;

var transient int m_nLastDriverXPLevel;

var transient float m_fPreviousStopThreshold;

var transient bool m_bIsPaused;

var transient bool m_bBrakesEnabled;

var transient BioVisualEffect m_oDamageVisualEffect;

var transient array<BioVisualEffect> m_aThrusterVisualEffects;

var float m_fThrusterScalarBL;

var float m_fThrusterScalarBR;

var float m_fThrusterScalarFL;

var float m_fThrusterScalarFR;

var transient BioBaseSquad m_oVehicleDefaultSquad;

var transient BioBaseSquad m_oDriverSquad;

var Vector m_vRecoilImpulseAccumulator;

var(VehicleWeapons) array<BioVehicleWeaponInfo> m_aVehicleWeaponInfo;

var(Designer) bool m_bInheritsPlayerStats;

var config bool m_bVehicleAudioEnable;

var bool m_bVehicleFunctionEnabled;

var bool m_bPowertrainEnabled;

var bool m_bIsThrustersOn;

var(Uprighting) bool m_bUpRightEnabled;

var bool m_bIncludeDriverSquad;

var(VehicleWeapons) bool m_bWeaponsEnabled;

native final private function InitializeActionFramework();

native final private function InitializePawnBehaviorForPlay();

native function bool ReleaseActive();

native function bool SetSuperMode(EBioSuperModes nSuperMode, bool bActivate, bool bReturnToSuperMode);

native function SetPowertrainState(bool bStatus);

native function SetVehicleFunctionState(bool bStatus);

native function SetBrakeState(bool bStatus);

native function bool OnVehicleEnter();

native function bool OnVehicleExit();

native function bool InitializeVehicle();

native function bool InitializeVehicleWeapons();

native function Rotator SetDesiredTurretAimPoint(int nTurretIndex, Vector vTargetLocation);

native function bool IsWorldLocationWithinTurretCone(int nTurretIndex, Vector vLocation);

native function bool EnterZoomMode(int nZoomTurretIndex);

native function bool ExitZoomMode();

native function bool UpdateCurrentZoomTurret(float fDeltaYaw, float fDeltaPitch);

native function OnDriverPartyDeath();

native function SquadEnter(BioBaseSquad oEnterSquad);

native function SquadExit();

native function InitializeVehicleSquad();

native function bool InitializeThrusters();

native function UpdateThrusters(float fDeltaTime);

native function ActivateThruster(int nIndex, Vector vMaxForce, float fScale);

native function DeactivateThruster(int nIndex);

native function UpdateThrusterScalars(float fXaxis, float fYaxis);

native function bool InitializeSounds();

native function bool UpdateDamageModel();

native function float GetCameraFollowHeight();

native function float GetCameraFollowDistance();

native function Name GetCameraHookName();

native function int GetRequiredEquipmentSophisticationLevel(int nCurrentPlayerLevel);

native function bool ScaleVehicleEquipment();

native function bool OnVehiclePause(bool bPaused);

native function SetMaterialParameterState(bool bState);

event bool StartUprighting()
{
    local SVehicle oVehicle;
    oVehicle = SVehicle(Owner);
    oVehicle.bFlipRight = !oVehicle.bFlipRight;
    if (oVehicle.bIsInverted == true && oVehicle.bVehicleOnGround == false && VSize(oVehicle.Velocity) <= 5)
    {
        oVehicle.UprightStartTime = oVehicle.WorldInfo.TimeSeconds;
        if (m_bUpRightEnabled == true && oVehicle.bIsUprighting == false)
        {
            oVehicle.bIsUprighting = true;
            return true;
        }
    }
    return oVehicle.bIsUprighting;
}

static function class<BioBaseSaveObject> GetSaveObjectClass()
{
    return Class'BioVehicleBehaviorSaveObject';
}

defaultproperties
{
    m_bWeaponsEnabled=true
    m_bUpRightEnabled=true
    m_bPowertrainEnabled=true
    m_bVehicleFunctionEnabled=true
    m_bVehicleAudioEnable=true
    m_bInheritsPlayerStats=true
    m_fThrusterScalarFR=1
    m_fThrusterScalarFL=1
    m_fThrusterScalarBR=1
    m_fThrusterScalarBL=1
    m_nLastDriverXPLevel=1
    m_nCurrentZoomTurret=-1
    m_attributesClass=class'BioAttributesPawnVehicle'
    m_PawnAttributes=BioAttributesPawnVehicle'AttributesPawn01'
    ActorGameName="BioVehicleBase"
    m_oAppearanceType=BioInterfaceAppearanceVehicle'VehicleApperInterface01'
    m_aTargetTipDefaults=System.Byte[]
    // child object AttributesPawn01 of type BioAttributesPawnVehicle
    // child object VehicleApperInterface01 of type BioInterfaceAppearanceVehicle
}
