﻿class SFXAI_Humanoid extends SFXAI_Core
    hidecategories(Navigation)
    config(AI)
    placeable;

enum EHumanoidBehaviours
{
    HB_Defensive,
    HB_Aggressive,
};

var transient int m_nCoverMoveCount;

var transient int m_nInvalidatedCoverRetries;

var transient EHumanoidBehaviours m_eCurrentBehaviour;

function bool SetDefaultBehaviour(Name NewBehaviour)
{
    local int nBehaviourIndex;
    nBehaviourIndex = BehaviourList.Find(NewBehaviour);
    if (nBehaviourIndex != -1 && NewBehaviour != m_nmCurrentBehavior)
    {
        m_nmCurrentBehavior = NewBehaviour;
        m_eCurrentBehaviour = nBehaviourIndex;
        return true;
    }
    return false;
}

function NotifyUnderAttack(bool bHit)
{
    if (MyBP == None)
    {
        return;
    }
    if (IsMovingToCover())
    {
        return;
    }
    if (MyBP.IsInCover())
    {
        if (IsLeaning())
        {
            CancelAction();
            m_bWaitBeforeNextAttack = true;
        }
        else
        {
            if (bHit)
            {
                bAcquireNewCover = true;
            }
        }
        if (bHit)
        {
            m_nCoverMoveCount = 0;
        }
    }
    else
    {
        if (m_eCurrentBehaviour == 0)
        {
            bAcquireNewCover = true;
            if (IsAttacking() && IsInState('MovingToGoal') == false)
            {
                CancelAction();
            }
        }
        else
        {
            if (bHit)
            {
                bAcquireNewCover = true;
            }
        }
    }
}

state Combat_Aggressive
{
    function ReEvaluateCover()
    {
        bAcquireNewCover = true;
    }

Begin:
    bAcquireNewCover = true;
    while (true)
    {
        if (FireTarget == None)
        {
            while (SelectTarget() == false)
            {
                Sleep(1);
            }
        }
        if (bAcquireNewCover && WorldInfo.TimeSeconds > m_fLastCoverEnterTime + 3)
        {
            if (FindClosestApproachDistance(FireTarget) > MyBP.GetWeaponRange(4))
            {
                if (m_nCoverMoveCount < 3 || !MyBP.IsInCover())
                {
                    m_nCoverMoveCount++;
                }
                else
                {
                    bAcquireNewCover = false;
                }
            }
            else
            {
                m_nCoverMoveCount = 0;
            }
            if (bAcquireNewCover)
            {
                bAcquireNewCover = false;
                m_bWaitBeforeNextAttack = false;
                MoveToCover(AtCover_WeaponRange, FireTarget);
                if (!bReachedMoveGoal)
                {
                }
                m_nInvalidatedCoverRetries = 0;
                while (m_bInvalidatedCover && m_nInvalidatedCoverRetries < 3)
                {
                    MoveToCover(AtCover_WeaponRange, FireTarget);
                    ++m_nInvalidatedCoverRetries;
                    Sleep(0,25);
                }
            }
        }
        else
        {
            if (RespondToPush())
            {
                bAcquireNewCover = true;
            }
        }
        if (m_bWaitBeforeNextAttack)
        {
            Sleep(1);
            m_bWaitBeforeNextAttack = false;
        }
        else
        {
            Sleep(0,1);
        }
        if (MyBP.IsInCover() && IsFlankedByTarget(FireTarget))
        {
            InvalidateCover();
            Focus = FireTarget;
            FinishRotation();
        }
        ATTACK();
        if (m_AttackResult == 3)
        {
            bAcquireNewCover = true;
        }
        if (MyBP.Weapon != None && MyBP.IsWithinWeaponRange(FireTarget, 4) == false)
        {
            bAcquireNewCover = true;
        }
    }
}

state Combat_Defensive
{
    event PoppedState()
    {
        PoppedState();
        ClearTimer('FindNewCover');
    }

    while (SelectTarget() == false)
    {
        Sleep(1);
    }
    while (true)
    {
        MoveToCover(AtCover_Defensive, MyBP);
        bAcquireNewCover = false;
        m_nInvalidatedCoverRetries = 0;
        while (m_bInvalidatedCover && m_nInvalidatedCoverRetries < 3)
        {
            MoveToCover(AtCover_Defensive, MyBP);
            ++m_nInvalidatedCoverRetries;
            Sleep(0,25);
        }
        if (MyBP != None && MyBP.IsInCover() == false)
        {
            SetTimer(1,5 + FRand() * 0,5, false, 'FindNewCover');
        }
        m_bWaitBeforeNextAttack = false;
        while (bAcquireNewCover == false)
        {
            if (MyBP != None && MyBP.IsInCover() == false)
            {
                Focus = FireTarget;
            }
            ATTACK();
            if (bAcquireNewCover == false)
            {
                if (MyBP.IsInCover() && IsFlankedByTarget(FireTarget))
                {
                    InvalidateCover();
                    if (MyBP.bIsCrouched)
                    {
                        MyBP.ShouldCrouch(false);
                    }
                    Focus = FireTarget;
                }
                if (m_bWaitBeforeNextAttack)
                {
                    m_bWaitBeforeNextAttack = false;
                    Sleep(1);
                }
                else
                {
                    Sleep(0,1);
                }
            }
            if (RespondToPush())
            {
                bAcquireNewCover = true;
            }
        }
        Sleep(0,5);
    }
    jump 0;
    // unresolved reference 0 ;
}

defaultproperties
{
    m_bAIGoHereCanUseCover=true
    m_nmCurrentBehavior=Combat_Defensive
    m_bAvoidDangerLinks=true
    m_fSpecialMoveTimeDelay=5
    BehaviourList(0)=Combat_Defensive
    BehaviourList(1)=Combat_Aggressive
    AtCover_WeaponRange=Goal_AtCover'AtCov_EvalWeaponRange'
    AtCover_Defensive=Goal_AtCover'AtCov_EvalDefensive'
    AtCover_Aggressive=Goal_AtCover'AtCov_EvalAggressive'
    AtCover_NearMoveGoal=Goal_AtCover'AtCov_EvalNearGoal'
    AtCover_AIGoHere=Goal_AtGoHereCover'AtCov_EvalGoHere'
    CustomActionClasses(0)=None
    CustomActionClasses(1)=None
    CustomActionClasses(2)=None
    CustomActionClasses(3)=None
    CustomActionClasses(4)=class'SFXCustomAction_Frozen'
    CustomActionClasses(5)=class'SFXCustomAction_Ragdoll'
    CustomActionClasses(6)=None
    CustomActionClasses(7)=None
    CustomActionClasses(8)=class'BioCustomAction_CombatActivate'
    CustomActionClasses(9)=class'BioCustomAction_CoverClimb'
    CustomActionClasses(10)=class'BioCustomAction_CoverMantle'
    CustomActionClasses(11)=class'BioCustomAction_WaypointAnimation'
    CustomActionClasses(12)=class'SFXCustomAction_Freezing'
    CustomActionClasses(13)=None
    CustomActionClasses(14)=class'SFXCustomAction_Knockback'
    CustomActionClasses(15)=class'SFXCustomAction_Headshot'
    CustomActionClasses(16)=class'SFXCustomAction_GreatPain'
    CustomActionClasses(17)=class'SFXCustomAction_Staggering'
    CustomActionClasses(18)=class'SFXCustomAction_Reave'
    CustomActionClasses(19)=None
    CustomActionClasses(20)=class'SFXCustomAction_HitLeftLeg'
    CustomActionClasses(21)=class'SFXCustomAction_HitRightLeg'
    CustomActionClasses(22)=class'SFXCustomAction_CorkscrewDeath'
    CustomActionClasses(23)=class'SFXCustomAction_ExplosionDeath'
    CustomActionClasses(24)=class'SFXCustomAction_LeftArmDeath'
    CustomActionClasses(25)=class'SFXCustomAction_RightArmDeath'
    CustomActionClasses(26)=class'SFXCustomAction_StomachDeath'
    CustomActionClasses(27)=class'SFXCustomAction_DeathKnockback'
    CustomActionClasses(28)=class'SFXCustomAction_OnFire'
    CustomActionClasses(29)=class'SFXCustomAction_TakeAKnee'
    CustomActionClasses(30)=class'SFXCustomAction_StaggerImpact'
    CustomActionClasses(31)=class'SFXCustomAction_StandardImpact'
    CustomActionClasses(32)=None
    CustomActionClasses(33)=None
    CustomActionClasses(34)=None
    CustomActionClasses(35)=class'SFXCustomAction_ShieldKnockback'
    CustomActionClasses(36)=class'SFXCustomAction_ShieldStaggerImpact'
    CustomActionClasses(37)=class'SFXCustomAction_ShieldStandardImpact'
    CustomActionClasses(38)=None
    CustomActionClasses(39)=None
    CustomActionClasses(40)=None
    CustomActionClasses(41)=class'SFXCustomAction_NeuralShock'
    CustomActionClasses(42)=None
    CustomActionClasses(43)=None
    CustomActionClasses(44)=None
    CustomActionClasses(45)=None
    CustomActionClasses(46)=None
    CustomActionClasses(47)=None
    CustomActionClasses(48)=None
    CustomActionClasses(49)=None
    CustomActionClasses(50)=None
    CustomActionClasses(51)=None
    CustomActionClasses(52)=None
    CustomActionClasses(53)=None
    CustomActionClasses(54)=None
    CustomActionClasses(55)=None
    CustomActionClasses(56)=None
    CustomActionClasses(57)=None
    CustomActionClasses(58)=class'SFXCustomAction_Meleed'
    CustomActionClasses(59)=None
    CustomActionClasses(60)=None
    CustomActionClasses(61)=None
    CustomActionClasses(62)=None
    CustomActionClasses(63)=class'SFXCustomAction_BloodyPlayerStandardImpact'
    CustomActionClasses(64)=class'SFXCustomAction_PlayerOnFire'
    CustomActionClasses(65)=None
    CustomActionClasses(66)=None
    CustomActionClasses(67)=class'SFXCustomAction_PlayerShortStagger'
    CustomActionClasses(68)=class'SFXCustomAction_StumbleForward'
    CustomActionClasses(69)=None
    CustomActionClasses(70)=None
    CustomActionClasses(71)=class'SFXCustomAction_ElectricStandardImpact'
    CustomActionClasses(72)=None
    CustomActionClasses(73)=class'SFXCustomAction_GreatPainII'
    CustomActionClasses(74)=class'SFXCustomAction_KnockbackII'
    CustomActionClasses(75)=class'SFXCustomAction_MeleedII'
    CustomActionClasses(76)=None
    CustomActionClasses(77)=None
    CustomActionClasses(78)=class'SFXCustomAction_PunchDeathKnockback'
    CustomActionClasses(79)=None
    CustomActionClasses(80)=None
    CustomActionClasses(81)=None
    CustomActionClasses(82)=None
    CustomActionClasses(83)=None
    CustomActionClasses(84)=None
    CustomActionClasses(85)=class'SFXCustomAction_ExplosiveKnockback'
    CustomActionClasses(86)=class'SFXCustomAction_StandardImpact'
    CustomActionClasses(87)=class'SFXCustomAction_PlayerShortStagger'
    CustomActionClasses(88)=class'SFXCustomAction_StaggerImpact'
    CustomActionClasses(89)=class'SFXCustomAction_MeleedII'
    CustomActionClasses(90)=class'SFXCustomAction_Meleed'
    CustomActionClasses(91)=class'SFXCustomAction_KnockbackII'
    CustomActionClasses(92)=class'SFXCustomAction_StumbleForward'
    CustomActionClasses(93)=class'SFXCustomAction_StumbleForward'
    CustomActionClasses(94)=class'SFXCustomAction_OnFireShort'
    CustomActionClasses(95)=class'SFXCustomAction_OnFire'
    CustomActionClasses(96)=class'SFXCustomAction_PlayerOnFire'
    CustomActionClasses(97)=class'SFXCustomAction_NeuralShock'
    CustomActionClasses(98)=class'SFXCustomAction_GreatPainII'
    // child object AtCov_EvalAggressive of type Goal_AtCover
    // child object AtCov_EvalDefensive of type Goal_AtCover
    // child object AtCov_EvalNearGoal of type Goal_AtCover
    // child object AtCov_EvalWeaponRange of type Goal_AtCover
    // child object AtCov_EvalGoHere of type Goal_AtGoHereCover
}
