﻿class SFXCustomAction_HeavyPunchLunge extends BioCustomAction
    config(Game);

const LUNGE_HEIGHT_THRESHOLD = 200.f;
const LUNGE_VISIBLE_THRESHOLD = 0.5f;
var transient VisualEffect UtilizeCrust;

var float TimeDilationAmount;

var float TimeDilationDuration;

var() ForceFeedbackWaveform HitForceFeedback;

var() ScreenShakeStruct HitShake;

var() float HitPoseDuration;

var() float SwingDuration;

var() float ChargePoseDuration;

var() float WindUpDuration;

var() BodyStance BS_Punch;

var() transient int MaxRagdollPhysLevel;

var() transient float ConeAngle;

var() transient bool bRequireTarget;

var() transient array<BioPawn> PawnTargets;

var() float MaxLungeDistance;

function SetLeftArmIKEnabled(bool bEnabled)
{
    local SkelControlLimb IKNode;
    if (oPawn.Mesh != None)
    {
        IKNode = SkelControlLimb(oPawn.Mesh.FindSkelControl('LeftArm'));
        if (IKNode != None)
        {
            IKNode.SetSkelControlActive(bEnabled);
            IKNode.bSetStrengthFromAnimNode = bEnabled;
        }
    }
}

function StartCustomAction()
{
    StartCustomAction();
    SetLeftArmIKEnabled(false);
    if (oPawn.PlayBodyStance(BS_Punch, 1, 0,3, 0,3, false, true, true, 0) != 0)
    {
        oPawn.SetBodyStanceAnimEndNotification(BS_Punch, true);
        oPawn.SetTimer(WindUpDuration + ChargePoseDuration, false, 'Lunge', self);
        Class'BioPowerScriptDesign'.ApplyTemporaryGameEffect(oPawn, Class'SFXGameEffect_TimeDilation', TimeDilationDuration, TimeDilationAmount, 'HeavyPunchLunge', oPawn.Controller);
    }
}

function Lunge()
{
    local Vector Velocity;
    if (PawnTargets.Length > 0)
    {
        oPawn.SuggestJumpVelocity(Velocity, PawnTargets[0].Location, oPawn.Location);
        oPawn.Velocity = Velocity;
    }
    else
    {
        oPawn.Velocity = oPawn.Rotation * 4000;
    }
    oPawn.SetTimer(SwingDuration, false, 'PlayImpact', self);
}

function PlayImpact()
{
    local BioPlayerController PC;
    local BioPawn Impacted;
    local array<EBioPartGroup> Parts;
    local Vector HitLocation;
    local float DamageWithBonus;
    local float fDamage;
    local TraceHitInfo HitInfo;
    local float fImpactRadius;
    local float nMaxPhysicsLevel;
    local SFXModule_GameEffectManager GEModule;
    local SFXModule_GameEffectManager TargetGEModule;
    local SFXPower HeavyPunchPower;
    GEModule = oPawn.GetModule(Class'SFXModule_GameEffectManager');
    if (GEModule == None)
    {
        return;
    }
    HeavyPunchPower = oPawn.PowerManager.GetPower('HeavyPunch');
    if (HeavyPunchPower == None)
    {
        return;
    }
    PC = BioPlayerController(oPawn.Controller);
    if (PC != None && PC.PlayerCamera != None && PawnTargets.Length > 0)
    {
        SFXPlayerCamera(PC.PlayerCamera).AddScreenShake(HitShake);
        PC.ClientPlayForceFeedbackWaveform(HitForceFeedback);
    }
    if (PawnTargets.Length > 0 && oPawn.IsInvisible())
    {
        oPawn.BreakStealth();
    }
    DamageWithBonus = HeavyPunchPower.GetDamage();
    DamageWithBonus *= SFXGame(oPawn.WorldInfo.Game).BonusList.RchMeleeUpgradeCache;
    DamageWithBonus *= GEModule.GetEffectValue(Class'SFXGameEffect_MeleeDamageBonus', 1);
    nMaxPhysicsLevel = MaxRagdollPhysLevel + SFXGame(oPawn.WorldInfo.Game).BonusList.RchMeleePhysLevelCache;
    fImpactRadius = HeavyPunchPower.GetArrayValue(HeavyPunchPower.ImpactRadius);
    foreach oPawn.CollidingActors(Class'BioPawn', Impacted, fImpactRadius)
    {
        if (Impacted.IsFriendly(oPawn))
        {
            continue;
        }
        if (oPawn.Rotation Dot Normal(Impacted.Location - oPawn.Location) >= ConeAngle)
        {
            fDamage = DamageWithBonus;
            TargetGEModule = Impacted.GetModule(Class'SFXModule_GameEffectManager');
            if (TargetGEModule != None)
            {
                fDamage *= TargetGEModule.GetEffectValue(Class'SFXGameEffect_PhysicsDamageMultiplier', 1);
            }
            Impacted.TakeDamage(fDamage, PC, HitLocation, Normal(Impacted.Location - oPawn.Location), Class'SFXDamageType_HeavyMelee');
            if (Class'BioPowerScriptDesign'.GetPhysicsLevel(Impacted, true) <= nMaxPhysicsLevel)
            {
                HitInfo.BoneName = Impacted.AimNodes[4];
                Impacted.PlayAnimatedReaction(78, PC, true, Impacted.Location, Normal(Impacted.Location - oPawn.Location), HitInfo);
                if (Impacted.PawnType.ChallengeType == 1)
                {
                    Parts = Impacted.GetModule(Class'SFXModule_Damage').GetConstraints();
                    if (Parts.Length > 0)
                    {
                        Impacted.GetModule(Class'SFXModule_Damage').DestroyConstraint(Parts[Rand(Parts.Length)]);
                        Impacted.Died(PC, Class'SFXDamageType_HeavyMelee', Impacted.Location);
                    }
                }
            }
            else
            {
                HitInfo.BoneName = Impacted.AimNodes[4];
                Impacted.PlayAnimatedReaction(76, PC, true, Impacted.Location, Normal(Impacted.Location - oPawn.Location), HitInfo);
            }
        }
    }
}

function StopCustomAction()
{
    StopCustomAction();
    oPawn.StopBodyStance(BS_Punch, 0,2);
    oPawn.SetBodyStanceAnimEndNotification(BS_Punch, false);
    SetLeftArmIKEnabled(true);
}

function BodyStanceAnimEndNotification(AnimNodeSequence SeqNode, float PlayedTime, float ExcessTime)
{
    if (SFXPlayerController(oPawn.Controller) != None)
    {
        SFXPlayerController(oPawn.Controller).EndCustomAction();
    }
    else
    {
        if (SFXAI_Core(oPawn.Controller) != None)
        {
            SFXAI_Core(oPawn.Controller).EndCustomAction();
        }
    }
}

defaultproperties
{
    MaxLungeDistance=100
    bRequireTarget=true
    ConeAngle=0,5
    MaxRagdollPhysLevel=1
    WindUpDuration=0,27
    ChargePoseDuration=0,31
    SwingDuration=0,24
    HitPoseDuration=0,25
    HitForceFeedback=ForceFeedbackWaveform'HitShakeFF0'
    TimeDilationDuration=1
    TimeDilationAmount=0,8
    bBreakFromCover=true
    bDisableMovement=true
    bDisableLook=true
    bNotifyKnockedOutOfCover=true
    // child object HitShakeFF0 of type ForceFeedbackWaveform
}
