﻿class SFXDamageType extends DamageType
    config(Weapon)
    abstract
    native;

enum EWeaponRange
{
    WeaponRange_Invalid,
    WeaponRange_Melee,
    WeaponRange_Short,
    WeaponRange_Medium,
    WeaponRange_Long,
};

enum ESFXDamageFalloffType
{
    DamageFalloffType_Constant,
    DamageFalloffType_Linear,
};

var const int HeavyWeaponId;

var BioVFXTemplate PlayerFrameBufferEffect;

var BioVFXTemplate DeathEffect;

var() config float fPhysicsReactionTimeScale;

var() config array<HitReaction> HitReactions;

var() config float fChanceOfRagdoll;

var() float ArmorBonus;

var() float ShieldBonus;

var() ResistanceInfo Resistance;

var() ForceFeedbackWaveform ShieldHitFFWaveform;

var float DamageRadius;

var() bool bIgnoresCoverDirection;

var() bool bImmediateDeath;

var bool bResearchBonus_SniperHeadShotBonus;

var bool bResearchBonus_ShotgunResistanceDamage;

var bool bResearchBonus_AutoPistolResistanceDamage;

var bool bResearchBonus_AssaultRifleResistanceDamage;

var bool bResearchBonus_HeavyPistolArmorDamage;

var bool bResearchBonus_SniperArmorDamage;

var() config bool bCausesRagdoll;

var() config bool bCausesPhysicsImpact;

var() bool bIgnoresBleedout;

var() bool bDamagesFriends;

var() bool bHealthDamage;

var() bool bIgnoreArmor;

var() bool bPartBasedDamageDisabled;

var() bool bIgnoreShieldHitLimit;

var() bool bIgnoreShields;

var bool bNoShieldBreakVFX;

var bool bSpawnWeaponImpacts;

var bool bRadiusDamage;

var ESFXDamageFalloffType FalloffType;

struct native HitReaction
{
    var() Name BodyPart;

    var() EAICustomAction AnimatedReaction;

    var() float ReactionChance;

    var() bool bDeathReaction;

    var() bool bTriggerOnMove;

    var() bool bTriggerOnIdle;

    var() bool bFrontArc;

    var() bool bRearArc;

    var() bool bFromCrouch;

    var() bool bIgnoreShields;

    var() EWeaponRange MaxRange;

}

struct native ResistanceInfo
{
    var float Shield;

    var float Armour;

    var float Biotic;

}

static event float GetWeaponRange(class<SFXWeapon> Weapon, EWeaponRange Range)
{
    switch (Range)
    {
        case 1:
            return Weapon.Default.Range_Melee;
        case 2:
            return Weapon.Default.Range_Short;
        case 3:
            return Weapon.Default.Range_Medium;
        case 4:
            return Weapon.Default.Range_Long;
        default:
            return 10000;
    }
}

defaultproperties
{
    FalloffType=DamageFalloffType_Linear
    bHealthDamage=true
    ShieldHitFFWaveform=ForceFeedbackWaveform'DamagedFFWave'
    HitReactions=System.Byte[]
    HeavyWeaponId=-1
    KDamageImpulse=1
    DamagedFFWaveform=None
    KilledFFWaveform=None
    // child object DamagedFFWave of type ForceFeedbackWaveform
}
