﻿class SFXDroppedAmmo extends DroppedPickup
    hidecategories(Navigation)
    config(Weapon);

var transient Name PlayerNameCache;

var transient bool bPooled;

var transient bool bDecays;

var transient bool bIsDisabled;

var transient MaterialInterface Clip_MM_Disabled;

var transient MaterialInterface Clip_MM;

var() WwiseEvent AmmoPickupSound;

var() /* 00000008 */ StaticMeshComponent AmmoMesh;

static function float CalculateAmmoStarvation(InventoryManager InvManager)
{
    local SFXWeapon Weapon;
    local float StarvedRating;
    local int NumWeapons;
    local float AmmoUsed;
    local float MaxAmmo;
    local float MostStarved;
    local float NextMostStarved;
    if (InvManager == None)
    {
        return 0;
    }
    MostStarved = 1;
    NextMostStarved = 1;
    foreach InvManager.InventoryActors(Class'SFXWeapon', Weapon)
    {
        if (SFXHeavyWeapon(Weapon) != None)
        {
            continue;
        }
        AmmoUsed = Weapon.GetCurrentTotalAmmo();
        MaxAmmo = Weapon.GetMaxSpareAmmo() + Weapon.GetMagazineSize();
        if (AmmoUsed / MaxAmmo <= MostStarved)
        {
            NextMostStarved = MostStarved;
            MostStarved = AmmoUsed / MaxAmmo;
            NumWeapons++;
        }
        else
        {
            if (AmmoUsed / MaxAmmo <= NextMostStarved)
            {
                NextMostStarved = AmmoUsed / MaxAmmo;
                NumWeapons++;
            }
        }
    }
    if (NumWeapons <= 1)
    {
        StarvedRating = MostStarved;
    }
    else
    {
        StarvedRating = 2 * MostStarved + NextMostStarved / 3;
    }
    if (Max == 0)
    {
        return 0;
    }
    return StarvedRating;
}

static function bool CanPickUpAmmo(InventoryManager InvManager)
{
    local SFXWeapon oWeapon;
    foreach InvManager.InventoryActors(Class'SFXWeapon', oWeapon)
    {
        if (SFXHeavyWeapon(oWeapon) != None)
        {
            continue;
        }
        if (oWeapon.GetCurrentSpareAmmo() < oWeapon.GetMaxSpareAmmo())
        {
            return true;
        }
    }
    return false;
}

static function float GetUsedAmmoPct(InventoryManager InvManager, bool bCountAmmoInMagazine)
{
    local SFXWeapon Weapon;
    local float UsedPct;
    local float WpnUsedPct;
    local int NumWeapons;
    local int AmmoUsed;
    local int MaxAmmo;
    if (InvManager == None)
    {
        return 0;
    }
    foreach InvManager.InventoryActors(Class'SFXWeapon', Weapon)
    {
        if (SFXHeavyWeapon(Weapon) != None)
        {
            continue;
        }
        if (bCountAmmoInMagazine)
        {
            AmmoUsed = Weapon.GetCurrentTotalAmmo();
            MaxAmmo = Weapon.GetMaxSpareAmmo() + Weapon.GetMagazineSize();
        }
        else
        {
            AmmoUsed = Weapon.GetCurrentSpareAmmo();
            MaxAmmo = Weapon.GetMaxSpareAmmo();
        }
        WpnUsedPct = AmmoUsed / MaxAmmo;
        WpnUsedPct = FClamp(WpnUsedPct, 0, 1);
        UsedPct += WpnUsedPct;
        NumWeapons++;
    }
    UsedPct = UsedPct / NumWeapons;
    if (Max == 0)
    {
        return 0;
    }
    return UsedPct;
}

static function bool ShouldDropAmmo(SFXWeapon Weapon, PlayerController PC)
{
    local float StarvRtg;
    local float ChanceToDrop;
    local Pawn OwnerPawn;
    local NavigationPoint Nav;
    local NavigationPoint OwnerAnchor;
    local float DistToAnchor;
    local float fRoll;
    if (PC == None || PC.Pawn == None)
    {
        return false;
    }
    Nav = PC.Pawn.GetBestAnchor(PC.Pawn, PC.Pawn.Location, true, true, DistToAnchor);
    if (Weapon != None)
    {
        OwnerPawn = Weapon.Instigator;
        if (OwnerPawn != None)
        {
            OwnerAnchor = OwnerPawn.Anchor;
            if (OwnerAnchor == None)
            {
                OwnerAnchor = OwnerPawn.GetBestAnchor(OwnerPawn, OwnerPawn.Location, true, true, DistToAnchor);
            }
        }
    }
    if (Nav != None && OwnerAnchor != None && Nav.NetworkID != OwnerAnchor.NetworkID)
    {
        return false;
    }
    StarvRtg = CalculateAmmoStarvation(PC.Pawn.InvManager);
    if (StarvRtg <= 0,05)
    {
        ChanceToDrop = 0,9;
    }
    else
    {
        if (StarvRtg <= 0,3)
        {
            ChanceToDrop = 0,75;
        }
        else
        {
            if (StarvRtg <= 0,4)
            {
                ChanceToDrop = 0,7;
            }
            else
            {
                if (StarvRtg <= 0,5)
                {
                    ChanceToDrop = 0,55;
                }
                else
                {
                    if (StarvRtg <= 0,65)
                    {
                        ChanceToDrop = 0,4;
                    }
                    else
                    {
                        if (StarvRtg <= 0,8)
                        {
                            ChanceToDrop = 0,3;
                        }
                        else
                        {
                            ChanceToDrop = 0,15;
                        }
                    }
                }
            }
        }
    }
    fRoll = FRand();
    return fRoll < ChanceToDrop;
}

static function bool GiveAmmo(InventoryManager InvManager, optional float fBonusAmmo = 0)
{
    local SFXWeapon Weapon;
    local float StarvRtg;
    local float AmmoPctToGive;
    local int AmmoToGive;
    local bool bAcquiredAmmo;
    StarvRtg = CalculateAmmoStarvation(SFXInventoryManager(InvManager));
    if (StarvRtg == 1)
    {
        return false;
    }
    if (StarvRtg <= 0,05)
    {
        AmmoPctToGive = 0,4;
    }
    else
    {
        if (StarvRtg <= 0,25)
        {
            AmmoPctToGive = 0,35;
        }
        else
        {
            if (StarvRtg <= 0,5)
            {
                AmmoPctToGive = 0,25;
            }
            else
            {
                AmmoPctToGive = 0,175;
            }
        }
    }
    AmmoPctToGive += fBonusAmmo;
    foreach InvManager.InventoryActors(Class'SFXWeapon', Weapon)
    {
        AmmoToGive = Weapon.GetMaxSpareAmmo() * AmmoPctToGive + 0,5;
        AmmoToGive = AmmoToGive <= 0 ? 1 : AmmoToGive;
        AmmoToGive *= Weapon.fAmmoMultiplier;
        if (Weapon.AddAmmo(AmmoToGive) > 0)
        {
            bAcquiredAmmo = true;
        }
    }
    if (!SFXGame(InvManager.WorldInfo.Game).gameconfig.bIsFullAmmo)
    {
        SFXGame(InvManager.WorldInfo.Game).gameconfig.bIsFullAmmo = !CanPickUpAmmo(InvManager);
        if (SFXGame(InvManager.WorldInfo.Game).gameconfig.bIsFullAmmo)
        {
            SFXGame(InvManager.WorldInfo.Game).TriggerVocalizationEvent(108, BioPawn(InvManager.Instigator));
        }
        else
        {
            if (bAcquiredAmmo)
            {
                SFXGame(InvManager.WorldInfo.Game).TriggerVocalizationEvent(107, BioPawn(InvManager.Instigator));
            }
        }
    }
    return true;
}

function GiveTo(Pawn P)
{
    Inventory = None;
    GiveAmmo(P.InvManager);
    P.PlaySound(AmmoPickupSound);
    PickedUpBy(P);
    ClearTimer('CheckPlayerFull');
    ClearTimer('RecheckValidTouch');
}

state Pickup
{
    function BeginState(Name PreviousStateName)
    {
        SetTimer(1, false, 'CheckPlayerFull');
    }

    function CheckPlayerFull()
    {
        local MaterialInterface Instance;
        local StaticMeshComponent oStaticMesh;
        local ActorComponent Comp;
        SetTimer(1, false, 'CheckPlayerFull');
        if (SFXGame(WorldInfo.Game).gameconfig.bIsFullAmmo)
        {
            Instance = Clip_MM_Disabled;
            if (bIsDisabled)
            {
                return;
            }
            bIsDisabled = true;
        }
        else
        {
            Instance = Clip_MM;
            if (!bIsDisabled)
            {
                return;
            }
            bIsDisabled = false;
        }
        foreach AllOwnedComponents(Class'ActorComponent', Comp)
        {
            oStaticMesh = StaticMeshComponent(Comp);
            if (oStaticMesh != None)
            {
                oStaticMesh.SetMaterial(0, Instance);
            }
        }
    }

    simulated function bool ValidTouch(Pawn Other)
    {
        if (bStasis == true)
        {
            return false;
        }
        if (Other == None || !Other.bCanPickupInventory || Other.DrivenVehicle == None && Other.Controller == None)
        {
            return false;
        }
        if (GetUsedAmmoPct(Other.InvManager, false) >= 1)
        {
            SetTimer(0,5, false, 'RecheckValidTouch');
            if (BioPlayerController(Other.Controller) != None && PlayerNameCache == 'None')
            {
                PlayerNameCache = Other.Name;
                SFXGame(BioWorldInfo(Other.WorldInfo).Game).PulseFullAmmoMessage();
            }
            return false;
        }
        return true;
    }

    function RecheckValidTouch()
    {
        CheckTouching();
    }

    function CheckTouching()
    {
        local Pawn P;
        foreach TouchingActors(Class'Pawn', P)
        {
            Touch(P, None, Location, Normal(Location - P.Location));
        }
    }

}

function SetCollisionCylinderSize(float CollisionRadius, float CollisionHeight)
{
    local CylinderComponent Comp;
    foreach AllOwnedComponents(Class'CylinderComponent', Comp)
    {
        Comp.SetCylinderSize(CollisionRadius, CollisionHeight);
    }
}

event Landed(Vector HitNormal, Actor FloorActor)
{
    Landed(HitNormal, FloorActor);
    SetCollisionCylinderSize(80, 10);
}

function Reset()
{
    if (bPooled)
    {
        Recycle();
    }
    else
    {
        Destroy();
    }
}

event EncroachedBy(Actor Other)
{
    if (bPooled)
    {
        Recycle();
    }
    else
    {
        Destroy();
    }
}

function PickedUpBy(Pawn P)
{
    if (bPooled)
    {
        Recycle();
    }
    else
    {
        Destroy();
    }
}

event Tick(float DeltaTime)
{
    Tick(DeltaTime);
    if (bPooled && LifeSpan != 0 && LifeSpan <= Max(DeltaTime, 0,0001))
    {
        Recycle();
    }
}

function Recycle()
{
    Clip_MM = Default.Clip_MM;
    Clip_MM_Disabled = Default.Clip_MM_Disabled;
    bIsDisabled = Default.bIsDisabled;
    bDecays = Default.bDecays;
    LifeSpan = Default.LifeSpan;
    Inventory = Default.Inventory;
    InventoryClass = Default.InventoryClass;
    PickupCache = Default.PickupCache;
    bFadeOut = Default.bFadeOut;
    SetHidden(true);
    bStasis = true;
}

function Reuse()
{
    SetHidden(false);
    bStasis = false;
    if (!IsTimerActive('CheckPlayerFull'))
    {
        SetTimer(1, false, 'CheckPlayerFull');
    }
}

event UnTouch(Actor Other)
{
    UnTouch(Other);
    if (Other.Name == PlayerNameCache)
    {
        PlayerNameCache = 'None';
    }
}

defaultproperties
{
    AmmoMesh=StaticMeshComponent'DroppedAmmoMesh0'
    AmmoPickupSound=WwiseEvent'Play_gui_ammo_pu'
    Clip_MM=Material'Clip_MM'
    Clip_MM_Disabled=MaterialInstanceConstant'Clip_MM_Disabled'
    bDecays=true
    Components=None
    07_PowerArmor=System.Byte[]
    // child object CollisionCylinder of type CylinderComponent
    // child object DroppedAmmoMesh0 of type StaticMeshComponent
}
