﻿class SFXGame extends GameInfo
    hidecategories(Navigation,Movement,Collision)
    config(Game)
    abstract
    native;

var config array<Name> NGPlusPlotVariables;

var class<SFXDifficulty> Difficulty;

var WwiseEvent StopSlowEvent;

var WwiseEvent StartSlowEvent;

var array<TimeDilationStruct> TimeDilationCurves;

var float TimeDilationOverride;

var() config array<HenchPackageMap> HenchmanPackageMap;

var config float fAutoUnlitDownsizeThreshold;

var config float HackCrouchCoverOffset;

var config float StormRegenNonCombat;

var config float StormStaminaNonCombat;

var config float StormRegen;

var config float StormStamina;

var transient bool bShowSquadScreenMessageBoxes;

var transient bool bGenerateTutorialEvents;

var transient bool bForceCombat;

var transient bool bInCombat;

var String FemaleCustomHead;

var String MaleCustomHead;

var String FemaleActorType;

var String MaleActorType;

var BioPlayerSquad PlayerSquad;

var /* 00000008 */ SFXBonusList BonusList;

var transient SFXOnlineSubsystem SFXOnlineSubsystem;

var SFXVocalizationManager VocManager;

var SFXObjectPool ObjectPool;

var config float PhysicsDamageMultiplier;

var SFXGameConfig gameconfig;

var config bool bAutoUnlitDownsizeInCombat;

var bool bMin1Health;

var bool EnableDamage;

enum ECharacterClass
{
    ClassType_Invalid,
    ClassType_Soldier,
    ClassType_Adept,
    ClassType_Infiltrator,
    ClassType_Engineer,
    ClassType_Vanguard,
    ClassType_Sentinel,
};

struct native HenchPackageMap
{
    var() String HenchTag;

    var() String PackageName;

}

struct native TimeDilationStruct
{
    var Name Identifier;

    var InterpCurveFloat Curve;

    var float TotalTime;

    var float Time;

}

struct ScaledFloat
{
    var() float X;

    var() float Y;

    var() int MaxLevel;

    var() int Level;

    var float Value;

    var array<SFXGameEffect> Bonuses;

    var float StaticBonus;

}

static function ReCalculate(out ScaledFloat F)
{
    local SFXGameEffect GE;
    local float DynamicBonus;
    DynamicBonus = 1;
    foreach F.Bonuses(GE)
    {
        GE.ComputeEffectValue(DynamicBonus);
    }
    F.Value = Lerp(F.X, F.Y, F.Level / F.MaxLevel) * F.StaticBonus + DynamicBonus - 1;
}

simulated event PostBeginPlay()
{
    PostBeginPlay();
    ObjectPool = Spawn(Class'SFXObjectPool');
    VocManager = Spawn(Class'SFXVocalizationManager');
    PlayerSquad = Spawn(Class'BioPlayerSquad');
    gameconfig.bIsFullAmmo = !Class'SFXDroppedAmmo'.CanPickUpAmmo(BioWorldInfo(WorldInfo).GetLocalPlayerController().Pawn.InvManager);
}

native final static function String GetSimpleString(optional bool bParse);

native final static function SortActorsByAngle(out array<Actor> List, Vector RefLocation, Rotator RefRotation, bool bPreferBioPawns);

final function UpdateDamage(int Damage)
{
    if (WorldInfo.TimeSeconds == gameconfig.fLastDamageUpdate)
    {
        Damage += gameconfig.fLastDamageUpdate;
    }
    if (Damage > 5)
    {
        gameconfig.fLastDamageUpdate = WorldInfo.TimeSeconds;
        gameconfig.nLastDamage = Damage;
    }
}

final function SFXPawn_Player GetPlayer(int ControllerId)
{
    local BioPlayerController PC;
    local LocalPlayer LP;
    foreach LocalPlayerControllers(Class'BioPlayerController', PC)
    {
        LP = LocalPlayer(PC.Player);
        if (LP != None && LP.ControllerId == ControllerId)
        {
            return SFXPawn_Player(PC.Pawn);
        }
    }
    return None;
}

final event bool GetPlayerLevel(int ControllerId, out int PlayerLevel)
{
    local BioPlayerController PC;
    local LocalPlayer LP;
    foreach LocalPlayerControllers(Class'BioPlayerController', PC)
    {
        LP = LocalPlayer(PC.Player);
        if (LP != None && LP.ControllerId == ControllerId)
        {
            PlayerLevel = BioPawn(PC.Pawn).CharacterLevel;
            return true;
        }
    }
    return false;
}

function GetChallengeModifier(int idx, out float ChallengeModifier, optional out float LevelModifier, optional out float TechModifier)
{
    local int ChallengeLevel;
    GetChallengeLevel(idx, ChallengeLevel);
    if (ChallengeLevel <= 300)
    {
        LevelModifier = 1,1;
    }
    else
    {
        if (ChallengeLevel <= 1200)
        {
            LevelModifier = 1 - 0,2 * ChallengeLevel - 400 / 1100;
        }
        else
        {
            if (ChallengeLevel <= 2000)
            {
                LevelModifier = 0,8 - 0,12 * ChallengeLevel - 1200 / 800;
            }
            else
            {
                if (ChallengeLevel <= 3000)
                {
                    LevelModifier = 0,68 - 0,08 * ChallengeLevel - 2000 / 1000;
                }
                else
                {
                    LevelModifier = 0,6;
                }
            }
        }
    }
    ChallengeModifier = LevelModifier;
}

function bool GetChallengeLevel(int idx, out int ChallengeLevel)
{
    local BioPawn PlayerPawn;
    local int PlayerLevel;
    PlayerPawn = BioPawn(BioWorldInfo(WorldInfo).GetLocalPlayerController().Pawn);
    GetPlayerLevel(idx, PlayerLevel);
    if (PlayerPawn == None)
    {
        return false;
    }
    ChallengeLevel = PlayerLevel * 100;
    ChallengeLevel = FClamp(ChallengeLevel, 0, 4000);
    return true;
}

function SetPlayerDefaults(Pawn PlayerPawn)
{
    local int PlayerLevel;
    local BioPawn Pawn;
    if (BioPawn(PlayerPawn) != None)
    {
        PlayerLevel = BioPawn(PlayerPawn).CharacterLevel;
        foreach WorldInfo.AllPawns(Class'BioPawn', Pawn)
        {
            if (SFXPawn_Player(Pawn) == None && SFXPawn_Henchman(Pawn) == None)
            {
                Pawn.ScaleEquipment(PlayerLevel);
            }
        }
    }
}

function NavigationPoint FindPlayerStart(Controller Player, optional byte InTeam, optional String IncomingName)
{
    local Actor chkActor;
    local Name ATStart;
    ATStart = BioWorldInfo(WorldInfo).CurrentGame.m_DesiredStartPoint;
    if (ATStart != 'None')
    {
        foreach WorldInfo.AllActors(Class'Actor', chkActor)
        {
            if (chkActor.Tag == ATStart)
            {
                if (BioStartPoint(chkActor) != None)
                {
                    return NavigationPoint(chkActor);
                }
                else
                {
                    if (BioStartLocation(chkActor) != None)
                    {
                        return Spawn(Class'DynamicAnchor', ATStart, chkActor.Location, chkActor.Rotation);
                    }
                }
            }
        }
    }
    return FindPlayerStart(Player, InTeam, IncomingName);
}

final function ECharacterClass GetIDByClass(class<Pawn> PlayerClass)
{
    if (Class<SFXPawn_PlayerAdept>(PlayerClass) != None)
    {
        return 2;
    }
    else
    {
        if (Class<SFXPawn_PlayerInfiltrator>(PlayerClass) != None)
        {
            return 3;
        }
        else
        {
            if (Class<SFXPawn_PlayerEngineer>(PlayerClass) != None)
            {
                return 4;
            }
            else
            {
                if (Class<SFXPawn_PlayerVanguard>(PlayerClass) != None)
                {
                    return 5;
                }
                else
                {
                    if (Class<SFXPawn_PlayerSentinel>(PlayerClass) != None)
                    {
                        return 6;
                    }
                }
            }
        }
    }
    return 1;
}

final function class<SFXPawn_Player> GetCharacterClassByName(coerce String ClassName)
{
    if (ClassName == "Adept")
    {
        return Class'SFXPawn_PlayerAdept';
    }
    else
    {
        if (ClassName == "Infiltrator")
        {
            return Class'SFXPawn_PlayerInfiltrator';
        }
        else
        {
            if (ClassName == "Engineer")
            {
                return Class'SFXPawn_PlayerEngineer';
            }
            else
            {
                if (ClassName == "Vanguard")
                {
                    return Class'SFXPawn_PlayerVanguard';
                }
                else
                {
                    if (ClassName == "Sentinel")
                    {
                        return Class'SFXPawn_PlayerSentinel';
                    }
                }
            }
        }
    }
    return Class'SFXPawn_PlayerSoldier';
}

function Pawn SpawnDefaultPawnFor(Controller NewPlayer, NavigationPoint StartSpot)
{
    local Vector StartLocation;
    local Rotator StartRotation;
    local SFXEngine Engine;
    local BioWorldInfo Info;
    local bool bIsFemale;
    local bool bStartedGame;
    local int IsFemale;
    local class<SFXPawn_Player> CharacterClass;
    local String FirstName;
    local EOriginType Origin;
    local ENotorietyType Notoriety;
    local SFXPower Power;
    local float Rank;
    local int Refund;
    local float Rank1Cost;
    local int Index;
    local String BonusTalent;
    local String FaceCode;
    local BioMorphFace NewMorphHead;
    local String PackageName;
    local SFXPawn_Player ResultPawn;
    local BioPawnType PawnType;
    local SkeletalMesh CustomMesh;
    local bool bPendingDefaultMorphHead;
    local bool bLastBonusHadRanks;
    local bool bHadBonusPower;
    StartLocation = StartSpot.Location;
    StartRotation.Yaw = StartSpot.Rotation.Yaw;
    Info = BioWorldInfo(WorldInfo);
    Engine = SFXEngine(PlayerController(NewPlayer).Player.Outer);
    bStartedGame = Info.CurrentGame.bStartedGame && !Class'WorldInfo'.IsPlayInEditor();
    bPendingDefaultMorphHead = false;
    if (bStartedGame == false)
    {
        if (Info.CurrentGame.bNewPlayer)
        {
            bIsFemale = Info.CurrentGame.NewPlayer.bIsFemale;
            CharacterClass = Info.CurrentGame.NewPlayer.CharacterClass;
            FirstName = Info.CurrentGame.NewPlayer.FirstName;
            Origin = Info.CurrentGame.NewPlayer.Origin;
            Notoriety = Info.CurrentGame.NewPlayer.Notoriety;
            BonusTalent = Info.CurrentGame.NewPlayer.BonusTalentClass;
            FaceCode = Info.CurrentGame.NewPlayer.FaceCode;
            NewMorphHead = Info.CurrentGame.NewPlayer.MorphHead;
        }
        else
        {
            bIsFemale = Info.CurrentGame.DefaultPlayer.bIsFemale;
            CharacterClass = Info.CurrentGame.DefaultPlayer.CharacterClass;
            FirstName = Info.CurrentGame.DefaultPlayer.FirstName;
            Origin = Info.CurrentGame.DefaultPlayer.Origin;
            Notoriety = Info.CurrentGame.DefaultPlayer.Notoriety;
            BonusTalent = Info.CurrentGame.DefaultPlayer.BonusTalentClass;
            FaceCode = Info.CurrentGame.DefaultPlayer.FaceCode;
            bPendingDefaultMorphHead = true;
        }
    }
    else
    {
        if (Engine != None)
        {
            if (Engine != None && Engine.ME2ImportSaveGame != None)
            {
                Engine.ME2ImportSaveGame.GetSpawnData(IsFemale, CharacterClass, FirstName, Origin, Notoriety);
            }
            else
            {
                Engine.CurrentSaveGame.GetSpawnData(IsFemale, CharacterClass, FirstName, Origin, Notoriety);
            }
            bIsFemale = IsFemale == 0 ? false : true;
        }
    }
    PackageName = Left(bIsFemale ? FemaleActorType : MaleActorType, InStr(bIsFemale ? FemaleActorType : MaleActorType, ".", false));
    LoadPackage(PackageName);
    PawnType = BioPawnType(FindObject(bIsFemale ? FemaleActorType : MaleActorType, Class'BioPawnType'));
    if (PawnType == None)
    {
        return None;
    }
    if (bPendingDefaultMorphHead)
    {
        NewMorphHead = PawnType.m_oMorphFace;
    }
    ResultPawn = Spawn(CharacterClass, 'Player', StartLocation, StartRotation, PawnType, true, true);
    if (BonusTalent != "")
    {
        bHadBonusPower = false;
        Index = ResultPawn.PowerManager.Powers.Length - 1;
        while (Index >= 0)
        {
            Power = ResultPawn.PowerManager.Powers[Index];
            if (Power != None)
            {
                if (Power.IsHenchmenUnique)
                {
                    bHadBonusPower = true;
                    if (Power.Rank > Rank)
                    {
                        bLastBonusHadRanks = true;
                        Rank = Power.Rank;
                        Refund = Class'SFXPowerManager'.GetRefundAmount2(Power.Class, Power.Rank);
                        if (Power.RankCosts.Length > 0)
                        {
                            Rank1Cost = Power.RankCosts[0];
                        }
                        else
                        {
                            Rank1Cost = 1;
                        }
                        Refund -= Rank1Cost;
                    }
                    ResultPawn.PowerManager.RemovePower(Power.Class);
                }
            }
            Index--;
        }
        Power = ResultPawn.PowerManager.AddPowerByClassName(BonusTalent);
        if (Power != None)
        {
            if (Refund > 0)
            {
                ResultPawn.AddTalentPoints(Refund);
            }
            if (bHadBonusPower)
            {
                if (bLastBonusHadRanks)
                {
                    Power.Rank = 1;
                }
            }
            else
            {
                Power.Rank = 1;
            }
        }
    }
    ResultPawn.FirstName = FirstName;
    ResultPawn.Origin = Origin;
    ResultPawn.Notoriety = Notoriety;
    ResultPawn.FaceCode = FaceCode;
    NewPlayer.Pawn = ResultPawn;
    ResultPawn.Controller = NewPlayer;
    if (NewMorphHead != None)
    {
        ResultPawn.MorphHead = NewMorphHead;
        if (NewMorphHead.m_oBaseHead == None)
        {
            CustomMesh = SkeletalMesh(DynamicLoadObject(bIsFemale ? FemaleCustomHead : MaleCustomHead, Class'SkeletalMesh'));
            if (CustomMesh != None)
            {
                ResultPawn.MorphHead.m_oBaseHead = CustomMesh;
            }
        }
        ResultPawn.m_oBehavior.m_oAppearanceType.UpdateAppearance(true);
    }
    ResultPawn.AddPlayerInventory();
    UpdatePlotVariables(ResultPawn);
    if (Engine != None)
    {
        if (Info.CurrentGame.bStartedGame == true || Engine.ME2ImportSaveGame != None)
        {
            Engine.LoadPlayer();
        }
    }
    Info.CurrentGame.bStartedGame = true;
    return ResultPawn;
}

final function UpdatePlotVariables(SFXPawn_Player Player)
{
    local BioGlobalVariableTable VarTable;
    VarTable = BioWorldInfo(WorldInfo).GetGlobalVariables();
    if (VarTable != None && Player != None)
    {
        VarTable.SetBool(66, Player.bIsFemale);
        VarTable.SetInt(230, GetIDByClass(Player.Class));
        VarTable.SetInt(7, Player.Origin);
        VarTable.SetBool(1533, Player.Origin == 1);
        VarTable.SetBool(1535, Player.Origin == 2);
        VarTable.SetBool(1534, Player.Origin == 3);
        VarTable.SetInt(8, Player.Notoriety);
        VarTable.SetBool(1537, Player.Notoriety == 1);
        VarTable.SetBool(1538, Player.Notoriety == 2);
        VarTable.SetBool(1539, Player.Notoriety == 3);
    }
    BioWorldInfo(WorldInfo).SetGlobalTlk(!Player.bIsFemale);
    UpdatePlotStateRTPCs(VarTable, Player);
}

final function UpdatePlotStateRTPCs(BioGlobalVariableTable VarTable, SFXPawn_Player Player)
{
    local WwiseAudioComponent WorldWwiseComponent;
    local int CurrentMusicVal;
    local int CurrentParagonVal;
    local int CurrentRenegadeVal;
    if (Player != None)
    {
        Class'WwiseAudioComponent'.SetGlobalRTPCFromScript("Player_Gender", Player.bIsFemale ? 2 : 1);
    }
    CurrentParagonVal = GetParagonPoints();
    CurrentRenegadeVal = GetRenegadePoints();
    Class'WwiseAudioComponent'.SetGlobalRTPCFromScript("Para_Rene_State", CurrentParagonVal - CurrentRenegadeVal);
    if (VarTable != None)
    {
        CurrentMusicVal = VarTable.GetInt(9);
        WorldWwiseComponent = Class'WwiseAudioComponent'.CreateComponentFromScript(WorldInfo);
        if (WorldWwiseComponent != None)
        {
            WorldWwiseComponent.SetWwiseRTPC("Current_Music_Plot_State", CurrentMusicVal);
        }
    }
}

native final static function LoadPackage(String PackageName);

final function String FindHenchPackage(String HenchTag)
{
    local HenchPackageMap Hench;
    foreach HenchmanPackageMap(Hench)
    {
        if (Hench.HenchTag == HenchTag)
        {
            return Hench.PackageName;
        }
    }
    return "";
}

function RequestTimeDilation(InterpCurveFloat Curve, float TotalTime, optional Name Identifier)
{
    local TimeDilationStruct Dilation;
    Dilation.Identifier = Identifier;
    Dilation.Curve = Curve;
    Dilation.TotalTime = TotalTime;
    Dilation.Time = 0;
    TimeDilationCurves.AddItem(Dilation);
}

function CancelTimeDilation(Name Identifier)
{
    local int idx;
    idx = TimeDilationCurves.Length - 1;
    while (idx >= 0)
    {
        if (TimeDilationCurves[idx].Identifier == Identifier)
        {
            TimeDilationCurves.Remove(idx, 1);
        }
        idx--;
    }
}

native final function float UpdateTimeDilationArray(float TimeDelta);

function Tick(float TimeDelta)
{
    local float SlowestDilation;
    SlowestDilation = UpdateTimeDilationArray(TimeDelta);
    if (gameconfig.fAmmoPulseTimer > 0)
    {
        gameconfig.fAmmoPulseTimer -= TimeDelta;
    }
    if (TimeDilationOverride != 1)
    {
        SlowestDilation = TimeDilationOverride;
    }
    if (WorldInfo.TimeDilation != SlowestDilation)
    {
        OnTimeDilationChange(SlowestDilation);
    }
}

function OnTimeDilationChange(float NewTimeDilation)
{
    if (NewTimeDilation < 1)
    {
        PlayTransientSound(StartSlowEvent, vect(0,0,0));
    }
    else
    {
        PlayTransientSound(StopSlowEvent, vect(0,0,0));
    }
    Class'WwiseAudioComponent'.SetGlobalRTPCFromScript("time_scale", NewTimeDilation);
    WorldInfo.TimeDilation = NewTimeDilation;
}

event TriggerVocalizationEvent(byte VocalizationID, BioPawn InstigatedBy, optional BioPawn Recipient, optional float delay, optional float fChanceToPlayModifier = 1)
{
    VocManager.TriggerVocalizationEvent(VocalizationID, InstigatedBy, Recipient, None, delay, fChanceToPlayModifier);
}

function ModifyDamage(out float Damage, out TraceHitInfo HitInfo, out Vector HitLocation, out Vector Momentum, class<SFXDamageType> DamageType, Actor injured, Controller InstigatedBy, optional Actor DamageCauser)
{
    local BioWorldInfo Info;
    local BioPlayerController PC;
    local Pawn TargetPawn;
    local BioPawn TargetBP;
    local SFXPRI InstigatorPRI;
    local SFXModule_GameEffectManager Manager;
    local RB_BodySetup HitBody;
    local SFXModule_Damage DamageMod;
    local float Multiplier;
    Multiplier = 1;
    if (SFXWeapon(DamageCauser) != None)
    {
        SFXWeapon(DamageCauser).ModifyDamage(Multiplier, HitLocation, injured);
    }
    if (InstigatedBy != None)
    {
        InstigatorPRI = SFXPRI(InstigatedBy.PlayerReplicationInfo);
        jump 106;
    }
    if (EnableDamage == false)
    {
        Damage = 0;
        Momentum = vect(0,0,0);
        return;
    }
    foreach LocalPlayerControllers(Class'BioPlayerController', PC)
    {
        Info = BioWorldInfo(WorldInfo);
        if (Info != None && PC.GameModeManager2.IsActive(6) && Info.m_bForceCinematicDamage == false)
        {
            Damage = 0;
            Momentum = vect(0,0,0);
            return;
        }
        if (PC.GameModeManager2.IsActive(5))
        {
            Damage = 0;
            Momentum = vect(0,0,0);
            return;
        }
    }
    if (injured != None)
    {
        if (injured.bCanBeDamaged == false || injured.PhysicsVolume != None && injured.PhysicsVolume.bNeutralZone)
        {
            Damage = 0;
            Momentum = vect(0,0,0);
            return;
        }
        TargetPawn = Pawn(injured);
        if (TargetPawn != None)
        {
            TargetBP = BioPawn(TargetPawn);
            if (TargetPawn.InGodMode() || TargetBP != None && TargetBP.m_oBehavior.IsPlotProtected())
            {
                Damage = 0;
                Momentum = vect(0,0,0);
                return;
            }
            if (InstigatedBy != None && DamageType.Default.bDamagesFriends == false && InstigatedBy.Pawn == injured || TargetBP != None && TargetBP.IsHostile(InstigatedBy.Pawn) == false)
            {
                Damage = 0;
                Momentum = vect(0,0,0);
                return;
            }
            if (TargetBP != None && TargetBP.GetCurrentHealth() <= 0)
            {
                Damage = 1;
                return;
            }
            DamageMod = TargetBP.GetModule(Class'SFXModule_Damage');
            if (DamageMod.bPartBasedDamageEnabled && DamageType.Default.bPartBasedDamageDisabled == false)
            {
                HitBody = DamageMod.GetBodyFromHit(HitInfo);
                if (HitBody != None && HitBody.bPartBasedDamageEnabled && InstigatedBy != None)
                {
                    Multiplier += HitBody.fDamageScale - 1;
                    SFXPRI(InstigatedBy.PlayerReplicationInfo).LastDamageCalculation.PartBasedDamageMultiplier = HitBody.fDamageScale;
                    if (HitBody.fDamageScale >= 1,5)
                    {
                        if (InstigatedBy != None)
                        {
                            if (DamageType.Default.bResearchBonus_SniperHeadShotBonus)
                            {
                                Multiplier += SFXGame(TargetBP.WorldInfo.Game).BonusList.RchSniperHeadShotDmgCache - 1;
                            }
                            if (InstigatedBy.Pawn != None)
                            {
                                Manager = InstigatedBy.Pawn.GetModule(Class'SFXModule_GameEffectManager');
                                if (Manager != None)
                                {
                                    Multiplier += Manager.GetEffectValue(Class'SFXGameEffect_PassiveHeadShotBonus');
                                }
                            }
                            self.ScoreHeadshot(InstigatedBy, DamageType);
                        }
                    }
                }
            }
            if (SFXPawn_Player(TargetBP) != None || SFXPawn_Henchman(TargetBP) != None)
            {
                if (TargetBP.IsInCoverLeaning())
                {
                    Multiplier *= gameconfig.PlayerCoverLeanDamageMultiplier;
                    InstigatorPRI.LastDamageCalculation.CoverLeanMultiplier = gameconfig.PlayerCoverLeanDamageMultiplier;
                }
                else
                {
                    if (TargetBP.IsInCover())
                    {
                        if (!DamageType.Default.bIgnoresCoverDirection && Normal(HitLocation - InstigatedBy.Pawn.Location) Dot TargetBP.CurrentLink.GetSlotRotation(TargetBP.CurrentSlotIdx) < -0,3)
                        {
                            if (VSize(HitLocation - InstigatedBy.Pawn.Location) > 300)
                            {
                                Damage = 0;
                                Momentum = vect(0,0,0);
                                return;
                            }
                        }
                        Multiplier *= gameconfig.PlayerCoverDamageMultiplier;
                        InstigatorPRI.LastDamageCalculation.CoverMultiplier = gameconfig.PlayerCoverDamageMultiplier;
                    }
                    else
                    {
                        Manager = TargetBP.GetModule(Class'SFXModule_GameEffectManager');
                        if (Manager != None)
                        {
                            Multiplier *= Manager.GetEffectValue(Class'SFXGameEffect_NonCoverDamageResistBonus', 1);
                        }
                    }
                }
                if (SFXPawn_Player(TargetBP) != None && BioPlayerController(TargetBP.Controller).bStorming)
                {
                    Multiplier *= gameconfig.PlayerStormDamageMultiplier;
                    InstigatorPRI.LastDamageCalculation.StormMultiplier = gameconfig.PlayerStormDamageMultiplier;
                }
            }
            if (TargetPawn != None && SFXPawn_Player(TargetBP) == None && TargetPawn.Physics == 10 && SFXWeapon(DamageCauser) != None)
            {
                Multiplier += gameconfig.PawnInRagdollDamageMultiplier;
                InstigatorPRI.LastDamageCalculation.RagdollMultiplier = gameconfig.PawnInRagdollDamageMultiplier;
            }
        }
    }
    Damage *= Multiplier;
    if (Damage > 0 && TargetPawn != None)
    {
        if (Damage > 0 && SFXInventoryManager(TargetPawn.InvManager) != None)
        {
            SFXInventoryManager(TargetPawn.InvManager).ProcessDamage(Damage, HitInfo, HitLocation, Momentum, DamageType, InstigatedBy, DamageCauser);
        }
    }
    return;
}

function Killed(Controller Killer, Controller KilledPlayer, Pawn KilledPawn, class<DamageType> DamageType)
{
    local BioPlayerController PC;
    local SFXEngine Engine;
    Killed(Killer, KilledPlayer, KilledPawn, DamageType);
    foreach LocalPlayerControllers(Class'BioPlayerController', PC)
    {
        Engine = SFXEngine(PC.Player.Outer);
        if (Engine != None && KilledPawn.GetNetIndex() != -1 && SFXPawn_Henchman(KilledPawn) == None)
        {
            Engine.DeadPawnList.AddItem(BioPawn(KilledPawn).MyGuid);
            break;
        }
    }
    if (Killer != None && Killer.Pawn.IsHumanControlled())
    {
        foreach LocalPlayerControllers(Class'BioPlayerController', PC)
        {
            PC.HintSystem.HintEvent('PlayerKill');
        }
    }
}

function bool PreventDeath(Pawn KilledPawn, Controller Killer, class<DamageType> DamageType, Vector HitLocation)
{
    local BioPawn Pawn;
    Pawn = BioPawn(KilledPawn);
    if (Pawn != None && Pawn.m_oBehavior != None && Pawn.m_oBehavior.IsMin1Health())
    {
        return true;
    }
    if (SFXPawn_Player(KilledPawn) != None && BonusList.RchBovineFortitudeCache > 0)
    {
        if (FRand() < BonusList.RchBovineFortitudeChance)
        {
            BioPawn(KilledPawn).PlayAnimatedReaction(6, KilledPawn.Controller);
            Class'BioPowerScriptDesign'.ApplyTemporaryGameEffect(KilledPawn, Class'SFXGameEffect_DamageImmunity', 4, 1, 'BovineFortitude', KilledPawn.Controller);
            return true;
        }
    }
    return PreventDeath(KilledPawn, Killer, DamageType, HitLocation);
}

function bool PreventPermanentDeath(BioPawn KilledPawn)
{
    if (BioPlayerSquad(KilledPawn.Squad) != None)
    {
        return !CheckGameOver();
    }
    return false;
}

final event bool InCombat()
{
    return bInCombat || bForceCombat;
}

final private function OnCombatStart()
{
    local BioPlayerController PC;
    local BioPawnBehavior PawnBehavior;
    local array<SequenceObject> SeqObjs;
    local SequenceObject Sequence;
    local BioPawn oPlayerPawn;
    local BioAiController AI;
    local SFXAI_Henchman oHenchAI;
    foreach LocalPlayerControllers(Class'BioPlayerController', PC)
    {
        if (PC.Pawn == None)
        {
            SetTimer(0,25, false, 'OnCombatStart');
            return;
        }
    }
    ProbeOnCombatBegin();
    WorldInfo.GetGameSequence().FindSeqObjectsByClass(Class'SFXSeqEvt_CombatStarted', true, SeqObjs);
    foreach SeqObjs(Sequence)
    {
        if (SFXSeqEvt_CombatStarted(Sequence) != None)
        {
            SFXSeqEvt_CombatStarted(Sequence).CheckActivate(self, self);
        }
    }
    foreach LocalPlayerControllers(Class'BioPlayerController', PC)
    {
        if (PC.Pawn != None)
        {
            PawnBehavior = BioPawnBehavior(PC.Pawn.oBioComponent);
            if (PawnBehavior != None)
            {
                PawnBehavior.SetActionState(1);
                PC.PlayerSquadSetActionState(1);
            }
            oPlayerPawn = BioPawn(PC.Pawn);
            if (oPlayerPawn != None && oPlayerPawn.Squad != None)
            {
                foreach oPlayerPawn.Squad.SquadMembers(AI)
                {
                    oHenchAI = SFXAI_Henchman(AI);
                    if (oHenchAI != None)
                    {
                        oHenchAI.OnCombatStart();
                    }
                }
            }
        }
        PC.HintSystem.HintEvent('EnterCombat');
    }
    StartFirstUsePowerDelay();
    if (IsTimerActive('OnCombatEnd'))
    {
        ClearTimer('OnCombatEnd');
    }
}

final private function OnCombatEnd()
{
    local BioPlayerController PC;
    local array<SequenceObject> SeqObjs;
    local SequenceObject Sequence;
    local BioPawn Pawn;
    local SFXModule_GameEffectManager EffectManager;
    ProbeOnCombatEnd();
    foreach LocalPlayerControllers(Class'BioPlayerController', PC)
    {
        PC.HintSystem.HintEvent('LeaveCombat');
    }
    WorldInfo.GetGameSequence().FindSeqObjectsByClass(Class'SFXSeqEvt_CombatEnded', true, SeqObjs);
    foreach SeqObjs(Sequence)
    {
        if (SFXSeqEvt_CombatEnded(Sequence) != None)
        {
            SFXSeqEvt_CombatEnded(Sequence).CheckActivate(self, self);
        }
    }
    foreach WorldInfo.AllPawns(Class'BioPawn', Pawn)
    {
        EffectManager = Pawn.GetModule(Class'SFXModule_GameEffectManager');
        if (EffectManager != None)
        {
            EffectManager.OnCombatEnd();
        }
    }
}

native final function ProbeOnCombatBegin();

native final function ProbeOnCombatEnd();

final function CombatStarted()
{
    if (bInCombat == false)
    {
        OnCombatStart();
    }
    bInCombat = true;
}

final function CombatEnded()
{
    if (bInCombat && bForceCombat == false)
    {
        SetTimer(2, false, 'OnCombatEnd');
    }
    bInCombat = false;
}

final function ToggleCombatOverride(bool bCombatOverride)
{
    local bool bWasInCombat;
    local bool bNowInCombat;
    bWasInCombat = InCombat();
    bForceCombat = bCombatOverride;
    bNowInCombat = InCombat();
    if (bNowInCombat && !bWasInCombat)
    {
        OnCombatStart();
    }
    else
    {
        if (bWasInCombat && !bNowInCombat)
        {
            SetTimer(2, false, 'OnCombatEnd');
        }
    }
}

final function StartFirstUsePowerDelay()
{
    local BioAiController AI;
    local BioPawn Pawn;
    if (WorldInfo != None)
    {
        foreach WorldInfo.AllControllers(Class'BioAiController', AI)
        {
            if (AI != None)
            {
                Pawn = BioPawn(AI.Pawn);
                if (Pawn != None && Pawn.PowerManager != None)
                {
                    Pawn.PowerManager.StartFirstTimeDelay();
                }
            }
        }
    }
}

final function SFXInventoryManager GetPlayerResourcesManager()
{
    local BioWorldInfo oBWI;
    oBWI = BioWorldInfo(WorldInfo);
    if (oBWI != None && oBWI.m_playerSquad != None && oBWI.m_playerSquad.m_playerPawn != None)
    {
        return SFXInventoryManager(oBWI.m_playerSquad.m_playerPawn.InvManager);
    }
    return None;
}

final simulated function PlayTransientSound(WwiseBaseSoundObject InWwiseEvent, Vector SoundLocation, optional array<String> RTPCName, optional array<float> RTPCValue)
{
    local WwiseAudioComponent Component;
    local int I;
    Component = ObjectPool.GetWwiseAudioComponent();
    if (Component != None)
    {
        Component.SetLocation(SoundLocation);
        if (RTPCName.Length > 0)
        {
            if (RTPCValue.Length == RTPCName.Length)
            {
                I = 0;
                while (I < RTPCName.Length)
                {
                    Component.SetWwiseRTPC(RTPCName[I], RTPCValue[I]);
                    I++;
                }
                jump 156;
            }
        }
        Component.Play(InWwiseEvent);
    }
    return;
}

final event bool GrantScaledCredits(int nSize, optional bool bTickResource = true)
{
    local SFXInventoryManager oPlayerResMgr;
    oPlayerResMgr = GetPlayerResourcesManager();
    if (oPlayerResMgr != None)
    {
        return oPlayerResMgr.GrantScaledCredits(nSize, bTickResource);
    }
    return false;
}

event InitGame(String Options, out String ErrorMessage)
{
    if (WorldInfo.IsConsoleBuild() || gameconfig.bUseConsoleControls)
    {
        WorldInfo.bUseConsoleInput = true;
    }
    InitGame(Options, ErrorMessage);
}

event PreExit()
{
    DestroySFXOnlineSubsystem();
    PreExit();
}

native final function DestroySFXOnlineSubsystem();

function ScoreFlamethrowerTankKill(Controller Killer, Controller Other)
{
    local SFXPRI InstigatorPRI;
    InstigatorPRI = SFXPRI(Killer.PlayerReplicationInfo);
    if (InstigatorPRI != None)
    {
        InstigatorPRI.AddScore(Class'SFXPRI'.Default.PointsFor_FlamethrowerTankKill, "Flamethrower Tank Kill");
    }
}

function ScoreKill(Controller Killer, Controller Other)
{
    local SFXPRI InstigatorPRI;
    InstigatorPRI = SFXPRI(Killer.PlayerReplicationInfo);
    if (InstigatorPRI != None)
    {
        InstigatorPRI.Kills += 1;
    }
    InstigatorPRI.DivyPoints(Other);
}

final function ScoreDamage(Controller InstigatorController, Controller Damaged, float RawDamageDealt, float ActualDamageDealt, class<SFXDamageType> DamageType)
{
    local SFXPRI InstigatorPRI;
    local BioPawn DamagedPawn;
    if (InstigatorController != None)
    {
        InstigatorPRI = SFXPRI(InstigatorController.PlayerReplicationInfo);
    }
    if (InstigatorPRI != None)
    {
        InstigatorPRI.ShotsHitCurrentTarget++;
        InstigatorPRI.RawDamageCurrentTarget += RawDamageDealt;
        InstigatorPRI.NetDamageCurrentTarget += ActualDamageDealt;
        InstigatorPRI.ShotsHit++;
        InstigatorPRI.RawDamage += RawDamageDealt;
        InstigatorPRI.NetDamage += ActualDamageDealt * Class'SFXPRI'.Default.PointsFor_DamageReceived;
    }
    if (Damaged != None)
    {
        InstigatorPRI = SFXPRI(Damaged.PlayerReplicationInfo);
        if (InstigatorPRI != None)
        {
            DamagedPawn = BioPawn(Damaged.Pawn);
            if (DamagedPawn.GetCurrentHealth() + ActualDamageDealt >= DamagedPawn.GetMaxHealth() - 1)
            {
                InstigatorPRI.ResetDamageDealt();
            }
            InstigatorPRI.RecordDamageDone(InstigatorController, ActualDamageDealt);
            InstigatorPRI.DifficultyScore -= ActualDamageDealt * Class'SFXPRI'.Default.PointsFor_DamageReceived;
        }
    }
}

final function ScoreHeadshot(Controller InstigatorController, class<SFXDamageType> DamageType)
{
    local SFXPRI InstigatorPRI;
    if (InstigatorController != None)
    {
        InstigatorPRI = SFXPRI(InstigatorController.PlayerReplicationInfo);
    }
    if (InstigatorPRI != None)
    {
        InstigatorPRI.AddScore(Class'SFXPRI'.Default.PointsFor_HeadShot, "Headshot");
        InstigatorPRI.HeadShots++;
    }
}

simulated function bool IsCurrentTarget(Controller InstigatorController, Actor Target)
{
    local SFXPRI InstigatorPRI;
    InstigatorPRI = SFXPRI(InstigatorController.PlayerReplicationInfo);
    if (InstigatorPRI != None)
    {
        InstigatorPRI.ShotsFiredCurrentTarget = 0;
        return InstigatorPRI.CurrentTarget == Target;
    }
}

simulated event SwitchTarget(Controller InstigatorController, Actor Target)
{
    local SFXPRI InstigatorPRI;
    InstigatorPRI = SFXPRI(InstigatorController.PlayerReplicationInfo);
    if (InstigatorPRI != None)
    {
        InstigatorPRI.SwitchTarget(Target);
    }
}

function SignalEndGame()
{
    local BioWorldInfo BWI;
    local LocalPlayer Player;
    local BioPlayerController PC;
    local Name nmStreamingChunk;
    BWI = BioWorldInfo(WorldInfo);
    Player = LocalPlayer(BWI.GetLocalPlayerController().Player);
    MassEffectGuiManager(Player.Outer.GameViewport.m_pScaleFormManager).OnPlayerDeath();
    WorldInfo.bPlayersOnly = false;
    BioSPGame(self).srGameOverString = GameOverString;
    SetTimer(BWI.m_fGameOverPauseTime, false, 'SpawnGameOverGUI');
    foreach WorldInfo.AllControllers(Class'BioPlayerController', PC)
    {
        PC.GameModeManager2.EnableMode(7);
    }
    BWI.PlayEndGameMusic();
    BWI.GetCurrentStreamingChunkName(nmStreamingChunk);
    Class'BioTelemetry'.SendEventName('PLAYER_DEFEATED', nmStreamingChunk);
}

function bool CheckGameOver()
{
    local BioPlayerController C;
    foreach WorldInfo.AllControllers(Class'BioPlayerController', C)
    {
        if (!C.IsDead() && !C.Pawn.IsInState('Downed'))
        {
            return false;
        }
    }
    return true;
}

function OnPlayerSquadDeath()
{
    SignalEndGame();
}

native static function ImpactInfo CalcWeaponFire(Weapon W, out Vector StartTrace, out Vector EndTrace, optional out array<ImpactInfo> ImpactList);

function int GetParagonPoints()
{
    local BioWorldInfo oWorldInfo;
    local BioGlobalVariableTable oVariables;
    oWorldInfo = BioWorldInfo(WorldInfo);
    if (oWorldInfo == None)
    {
        return 0;
    }
    oVariables = oWorldInfo.GetGlobalVariables();
    if (oVariables == None)
    {
        return 0;
    }
    return oVariables.GetInt(2);
}

function SetParagonPoints(int nPoints)
{
    local BioWorldInfo oWorldInfo;
    local BioGlobalVariableTable oVariables;
    oWorldInfo = BioWorldInfo(WorldInfo);
    if (oWorldInfo == None)
    {
        return;
    }
    oVariables = oWorldInfo.GetGlobalVariables();
    if (oVariables == None)
    {
        return;
    }
    oVariables.SetInt(2, nPoints);
}

function int GetRenegadePoints()
{
    local BioWorldInfo oWorldInfo;
    local BioGlobalVariableTable oVariables;
    oWorldInfo = BioWorldInfo(WorldInfo);
    if (oWorldInfo == None)
    {
        return 0;
    }
    oVariables = oWorldInfo.GetGlobalVariables();
    if (oVariables == None)
    {
        return 0;
    }
    return oVariables.GetInt(3);
}

function SetRenegadePoints(int nPoints)
{
    local BioWorldInfo oWorldInfo;
    local BioGlobalVariableTable oVariables;
    oWorldInfo = BioWorldInfo(WorldInfo);
    if (oWorldInfo == None)
    {
        return;
    }
    oVariables = oWorldInfo.GetGlobalVariables();
    if (oVariables == None)
    {
        return;
    }
    oVariables.SetInt(3, nPoints);
}

function OnGameCompleted(int nEndID)
{
    local String SParam;
    local float fTimePlayed;
    local SFXEngine Engine;
    local LocalPlayer Player;
    Player = LocalPlayer(BioWorldInfo(WorldInfo).GetLocalPlayerController().Player);
    Engine = SFXEngine(Player.Outer);
    if (Engine != None)
    {
        fTimePlayed = Engine.GetPlayTimeSeconds();
    }
    SParam = nEndID $ "," $ fTimePlayed;
    Class'BioTelemetry'.SendEventString('GAME_COMPLETED', SParam);
}

function AddCustomElementListPlotVars(out array<CustomizableElement> Elements, out array<int> Vars)
{
    local int idx;
    idx = 0;
    while (idx < Elements.Length)
    {
        if (Elements[idx].PlotFlag != -1 && Elements[idx].PlotFlag != 0)
        {
            Vars.AddItem(Elements[idx].PlotFlag);
        }
        idx++;
    }
}

function EnumeratePersonalizationVars(out array<int> Vars)
{
    AddCustomElementListPlotVars(Class'SFXPawn_Player'.Default.CasualAppearances, Vars);
    AddCustomElementListPlotVars(Class'SFXPawn_Player'.Default.FullBodyAppearances, Vars);
    AddCustomElementListPlotVars(Class'SFXPawn_Player'.Default.TorsoAppearances, Vars);
    AddCustomElementListPlotVars(Class'SFXPawn_Player'.Default.ShoulderAppearances, Vars);
    AddCustomElementListPlotVars(Class'SFXPawn_Player'.Default.ArmAppearances, Vars);
    AddCustomElementListPlotVars(Class'SFXPawn_Player'.Default.LegAppearances, Vars);
    AddCustomElementListPlotVars(Class'SFXPawn_Player'.Default.HelmetAppearances, Vars);
    AddCustomElementListPlotVars(Class'SFXPawn_Player'.Default.SpecAppearances, Vars);
    AddCustomElementListPlotVars(Class'SFXPawn_Player'.Default.Tint1Appearances, Vars);
    AddCustomElementListPlotVars(Class'SFXPawn_Player'.Default.Tint2Appearances, Vars);
    AddCustomElementListPlotVars(Class'SFXPawn_Player'.Default.PatternAppearances, Vars);
    AddCustomElementListPlotVars(Class'SFXPawn_Player'.Default.PatternColorAppearances, Vars);
}

function SetNGPlusPlotStates()
{
    local BioWorldInfo bioWorld;
    local BioGlobalVariableTable PlotVars;
    local array<int> boolVars;
    local array<int> BoolVals;
    local array<int> IntVals;
    local int idx;
    local int nIndex;
    bioWorld = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    if (bioWorld == None || bioWorld.CurrentGame == None)
    {
        return;
    }
    PlotVars = bioWorld.GetGlobalVariables();
    IntVals.Length = NGPlusPlotVariables.Length;
    idx = 0;
    while (idx < NGPlusPlotVariables.Length)
    {
        if (PlotVars.oNameLookupTable.GetIntEntryNN(NGPlusPlotVariables[idx], 'Index', nIndex) == true)
        {
            IntVals[idx] = PlotVars.GetInt(nIndex);
        }
        idx++;
    }
    EnumeratePersonalizationVars(boolVars);
    BoolVals.Length = boolVars.Length;
    idx = 0;
    while (idx < boolVars.Length)
    {
        BoolVals[idx] = PlotVars.GetBool(boolVars[idx]) ? 1 : 0;
        idx++;
    }
    bioWorld.GetGlobalVariables().ClearAllVariables();
    bioWorld.m_oQuestProgress.Clear();
    bioWorld.m_oDiscoveredCodex.Clear();
    idx = 0;
    while (idx < NGPlusPlotVariables.Length)
    {
        if (PlotVars.oNameLookupTable.GetIntEntryNN(NGPlusPlotVariables[idx], 'Index', nIndex) == true)
        {
            if (IntVals[idx] != 0)
            {
                PlotVars.SetInt(nIndex, IntVals[idx]);
            }
        }
        idx++;
    }
    idx = 0;
    while (idx < boolVars.Length)
    {
        if (BoolVals[idx] != 0)
        {
            PlotVars.SetBool(boolVars[idx], true);
        }
        idx++;
    }
}

function StartMatch()
{
    BonusList.UpdateCache(BioWorldInfo(WorldInfo));
    StartMatch();
}

function PulseFullAmmoMessage()
{
    if (gameconfig.fAmmoPulseTimer > 0)
    {
        return;
    }
    gameconfig.fAmmoPulseTimer = gameconfig.Default.fAmmoPulseTimer;
    MassEffectGuiManager(BioWorldInfo(WorldInfo).GetLocalPlayerController().GetScaleFormManager()).m_oSFHudHandler.PulseFullAmmoMessage();
}

defaultproperties
{
    EnableDamage=true
    gameconfig=SFXGameConfig'GameConfigBase'
    PhysicsDamageMultiplier=0,25
    BonusList=SFXBonusList'Bonus'
    MaleActorType="BIOG_Male_Player_C.Human.human_male"
    FemaleActorType="BIOG_Female_Player_C.Human.human_female"
    MaleCustomHead="BIOG_HMM_HED_PROMorph.Custom.HMM_HED_PROCustom_MDL"
    FemaleCustomHead="BIOG_HMF_HED_PROMorph_R.Custom.HMF_HED_PROCustom_MDL"
    StormStamina=4
    StormRegen=0,6
    StormStaminaNonCombat=2,5
    StormRegenNonCombat=0,375
    fAutoUnlitDownsizeThreshold=1,5
    HenchmanPackageMap=System.Byte[]
    TimeDilationOverride=1
    Difficulty=class'SFXDifficulty_Level2'
    NGPlusPlotVariables(0)=Wpn_MissileLauncher
    NGPlusPlotVariables(1)=Wpn_CryoBlaster
    NGPlusPlotVariables(2)=Wpn_NukeLauncher
    NGPlusPlotVariables(3)=Wpn_AssaultRifle2
    NGPlusPlotVariables(4)=Wpn_AutoPistol2
    NGPlusPlotVariables(5)=Wpn_BonusAssaultRifle
    NGPlusPlotVariables(6)=Wpn_BonusShotgun
    NGPlusPlotVariables(7)=Wpn_BonusSniperRifle
    NGPlusPlotVariables(8)=Wpn_GethPulseRifle
    NGPlusPlotVariables(9)=Wpn_GrenadeLauncher
    NGPlusPlotVariables(10)=Wpn_HeavyPistol2
    NGPlusPlotVariables(11)=Wpn_ParticleBeam
    NGPlusPlotVariables(12)=Wpn_Shotgun2
    NGPlusPlotVariables(13)=Wpn_SniperRifle2
    NGPlusPlotVariables(14)=Wpn_SuperAssaultRifle
    NGPlusPlotVariables(15)=Wpn_SuperShotgun
    NGPlusPlotVariables(16)=Wpn_SuperSniperRifle
    bRestartLevel=false
    DefaultPawnClass=None
    HUDType=None
    PlayerControllerClass=None
    PlayerReplicationInfoClass=None
    GameReplicationInfoClass=None
    // child object Bonus of type SFXBonusList
    // child object GameConfigBase of type SFXGameConfig
}
