﻿class SFXGameEffect_Singularity extends SFXGameEffect
    hidecategories(Object);

var float minHeightForBodyFallSound;

var Name BoneName;

var float Force;

var float SingularityRadius;

var Vector SingularityLocation;

var float ForceInterval;

var BioPawn Caster;

var float InnerRadius;

var float ForceTimer;

var bool bApplied;

function OnApplied()
{
    OnApplied();
    if (bApplied)
    {
        return;
    }
    if (Owner == None)
    {
        return;
    }
    InnerRadius = SingularityRadius * 0,5;
    InitialMovement();
    bApplied = true;
}

function OnUpdate(float DeltaSeconds)
{
    local BioPawn oBioPawn;
    ForceTimer += DeltaSeconds;
    if (ForceTimer > ForceInterval)
    {
        MoveActor();
        ForceTimer = 0;
    }
    oBioPawn = BioPawn(Owner);
    if (oBioPawn != None && oBioPawn.Location.Z - oBioPawn.m_fInitialZVal > minHeightForBodyFallSound)
    {
        oBioPawn.m_bRagdollEnteredPendingBodyFallSound = true;
    }
}

function InitialMovement()
{
    local Vector Direction;
    if (Owner == None || Owner.CollisionComponent == None)
    {
        return;
    }
    Direction = SingularityLocation - Owner.Location;
    if (VSize(Direction) < 1)
    {
        Direction = VRand();
    }
    Direction = Normal(Direction);
    Owner.CollisionComponent.AddForce(Direction * 50, Owner.Location, BoneName);
}

function MoveActor()
{
    local Vector Direction;
    local float Distance;
    local Vector ForceVector;
    local float fVelocity;
    local float fTimeLeft;
    if (Owner == None || Owner.CollisionComponent == None)
    {
        return;
    }
    Direction = SingularityLocation - Owner.Location;
    Distance = VSize(Direction);
    Direction = Normal(Direction);
    fVelocity = VSize(Owner.Velocity);
    if (OutOfRangeCheck(Distance))
    {
        return;
    }
    fTimeLeft = Duration - CurrentTime;
    if (fTimeLeft < 3)
    {
        ForceVector = Normal(Owner.Velocity) * 20;
        Owner.CollisionComponent.AddForce(ForceVector, Owner.Location, BoneName);
        return;
    }
    if (Distance > InnerRadius)
    {
        ForceVector = Direction * 10;
        Owner.CollisionComponent.AddForce(ForceVector, Owner.Location, BoneName);
    }
    else
    {
        if (fVelocity < 100)
        {
            ForceVector = Normal(Owner.Velocity) * 10;
            Owner.CollisionComponent.AddForce(ForceVector, Owner.Location, BoneName);
        }
    }
}

function OnRemoved()
{
    local BioPawn oBioPawn;
    OnRemoved();
    if (!bApplied)
    {
        return;
    }
    bApplied = false;
    oBioPawn = BioPawn(Owner);
    if (oBioPawn != None)
    {
        oBioPawn.m_bRagdollEnteredPendingBodyFallSound = true;
    }
}

function bool IsStillInSingularity()
{
    local SFXModule_GameEffectManager Manager;
    local SFXGameEffect oEffect;
    if (Owner == None)
    {
        return false;
    }
    Manager = Owner.GetModule(Class'SFXModule_GameEffectManager');
    if (Manager != None)
    {
        foreach Manager.GameEffects(oEffect)
        {
            if (oEffect.Class == Class'SFXGameEffect_Singularity' && oEffect != self)
            {
                return true;
            }
        }
    }
    return false;
}

function bool OutOfRangeCheck(float Distance)
{
    local SFXModule_GameEffectManager Manager;
    if (Distance < SingularityRadius * 4)
    {
        return false;
    }
    CurrentTime = Duration + 1;
    if (Owner != None)
    {
        Manager = Owner.GetModule(Class'SFXModule_GameEffectManager');
        if (Manager != None)
        {
            Manager.RemoveEffectsByCategory(Category);
        }
    }
    return true;
}

defaultproperties
{
    minHeightForBodyFallSound=5
    bSendCompletionEvent=true
}
