﻿class SFXModule_DamagePlayer extends SFXModule_DamageParty
    hidecategories(Object)
    config(Game)
    editinlinenew;

var transient bool bDebugHealthBarEnabled;

var transient float fDebugCooldownIndicator;

var transient float fDebugHealthIndicator;

var transient float fDebugShieldIndicator;

var transient float fBleedOutInterpolatedValue;

var config Name BleedOutVFXParameter;

var BioVFXTemplate BleedOutVFXTemplate;

var String BleedOutRTPC;

var WwiseEventPairObject BleedOutEventPair;

var /* 00000008 */ transient WwiseAudioComponent WwiseComponent;

var config float BleedOutSoundVolumeEnd;

var config float BleedOutSoundVolumeStart;

var const config float BleedOutSoundDurationEnd;

var const config float BleedOutSoundDurationStart;

var transient float BleedOutStartTime;

var const config float BleedOutRecoveryThreshold;

var const config float BleedOutEntryThreshold;

var BioVisualEffect DamageFrameBuffer;

var BioVisualEffect BleedOutVFX;

var float BleedOutSoundDuration;

var float BleedOutSoundTime;

var bool bIsBleedingOut;

function SFXTakeDamage(float Damage, out TraceHitInfo HitInfo, out Vector HitLocation, out Vector Momentum, class<DamageType> DamageType, Controller InstigatedBy, optional Actor DamageCauser)
{
    local class<SFXDamageType> DamageClass;
    local MassEffectGuiManager oSFManager;
    DamageClass = Class<SFXDamageType>(DamageType);
    oSFManager = MassEffectGuiManager(BioPlayerController(Pawn(Owner).Controller).GetScaleFormManager());
    if (oSFManager != None)
    {
        oSFManager.InterruptActiveSkillGame();
    }
    if (DamageClass.Default.PlayerFrameBufferEffect != None)
    {
        if (DamageFrameBuffer != None)
        {
            DamageFrameBuffer.SetState(2);
        }
        DamageFrameBuffer = Class'BioVisualEffect'.CreateVFXOnPosition(DamageClass.Default.PlayerFrameBufferEffect, vect(0,0,0), rot(0,0,0));
        if (DamageFrameBuffer != None)
        {
            DamageFrameBuffer.SetState(0, true);
        }
    }
    SFXTakeDamage(Damage, HitInfo, HitLocation, Momentum, DamageType, InstigatedBy, DamageCauser);
}

simulated function ApplyDamageToHealth(float Damage, Controller InstigatedBy, class<SFXDamageType> DamageType, out Vector HitLocation)
{
    local SFXGameEffect Effect;
    local float fEstimatedDamage;
    local bool bGetDamageProtection;
    bGetDamageProtection = false;
    if (DamageType.Default.bIgnoresBleedout == false)
    {
        fEstimatedDamage = Damage * DamageMultiplier.Value;
        if (CurrentHealth / MaxHealth.Value >= BleedOutEntryThreshold)
        {
            if (CurrentHealth - fEstimatedDamage / MaxHealth.Value < BleedOutEntryThreshold)
            {
                BleedOutStartTime = Owner.WorldInfo.TimeSeconds;
                Damage = CurrentHealth - MaxHealth.Value * BleedOutEntryThreshold + 1;
                bGetDamageProtection = true;
            }
        }
        else
        {
            if (Owner.TimeSince(BleedOutStartTime) < SFXGame(Owner.WorldInfo.Game).Difficulty.Default.BleedOutInterval)
            {
                Damage = 0;
            }
        }
    }
    ApplyDamageToHealth(Damage, InstigatedBy, DamageType, HitLocation);
    if (bGetDamageProtection)
    {
        Effect = BioPawn(Owner).GetModule(Class'SFXModule_GameEffectManager').CreateEffect(Class'SFXGameEffect_DamageMultiplier', 'Bleedout', SFXGame(Owner.WorldInfo.Game).Difficulty.Default.BreachDuration, 1, 0,35);
        if (Effect != None)
        {
            Effect.OnApplied();
        }
    }
}

function RecoverFromBleedout()
{
    CurrentHealth = MaxHealth.Value;
}

function Tick(float TimeDelta)
{
    local float fShields;
    local float fHealth;
    local float fCooldown;
    local SFXPower oPower;
    Tick(TimeDelta);
    if (SFXPawn_Player(Owner) != None && SFXPawn_Player(Owner).IsHumanControlled())
    {
        UpdateBleedOut(TimeDelta);
        BleedOutSoundTime += TimeDelta;
    }
    if (bDebugHealthBarEnabled)
    {
        fCooldown = 0;
        foreach BioPawn(Owner).PowerManager.Powers(oPower)
        {
            if (oPower.GetCooldownRemaining() > 0)
            {
                fCooldown = oPower.GetCooldownRemaining();
                break;
            }
        }
        fShields = BioPawn(Owner).GetMaxShields() > 0 ? BioPawn(Owner).GetCurrentShields() / BioPawn(Owner).GetMaxShields() : 0;
        fHealth = MaxHealth.Value > 0 ? CurrentHealth / MaxHealth.Value : 0;
        fDebugHealthIndicator = Lerp(fDebugHealthIndicator, fHealth, TimeDelta * 3);
        fDebugShieldIndicator = Lerp(fDebugShieldIndicator, fShields, TimeDelta * 3);
        fDebugCooldownIndicator = Lerp(fDebugCooldownIndicator, fCooldown, TimeDelta * 3);
        fDebugCooldownIndicator = fDebugCooldownIndicator < fCooldown ? fCooldown : fDebugCooldownIndicator;
        fDebugHealthIndicator = FClamp(fDebugHealthIndicator, 0, 1);
        fDebugShieldIndicator = FClamp(fDebugShieldIndicator, 0, 1);
    }
}

function float CalculateBleedoutVFXParam()
{
    fBleedOutInterpolatedValue = FClamp(fBleedOutInterpolatedValue, 0, 1);
    return fBleedOutInterpolatedValue;
}

function UpdateBleedOut(float TimeDelta)
{
    local float PercentBledOut;
    local bool bIsBleedingOutNow;
    PercentBledOut = ComputePercentBledOut();
    fBleedOutInterpolatedValue = Lerp(fBleedOutInterpolatedValue, PercentBledOut, TimeDelta / 0,25);
    bIsBleedingOutNow = fBleedOutInterpolatedValue > 0,05;
    if (WwiseComponent == None)
    {
        WwiseComponent = Class'WwiseAudioComponent'.CreateComponentFromScript(Owner, "_BleedOut");
        if (WwiseComponent != None)
        {
            WwiseComponent.SetGlobalRTPCFromScript(BleedOutRTPC, 0);
        }
    }
    if (bIsBleedingOutNow && !bIsBleedingOut)
    {
        OnBleedOutStart();
    }
    else
    {
        if (bIsBleedingOut && !bIsBleedingOutNow)
        {
            OnBleedOutStop();
        }
    }
    if (bIsBleedingOutNow)
    {
        ScaleBleedOut(fBleedOutInterpolatedValue);
    }
    bIsBleedingOut = bIsBleedingOutNow;
}

function ScaleBleedOut(float PercentBledOut)
{
    if (WwiseComponent != None)
    {
        WwiseComponent.SetGlobalRTPCFromScript(BleedOutRTPC, PercentBledOut);
    }
    if (BleedOutVFX != None)
    {
        BleedOutVFX.SetScalarParameter(BleedOutVFXParameter, CalculateBleedoutVFXParam());
    }
}

function OnBleedOutStart()
{
    if (WwiseComponent != None)
    {
        WwiseComponent.Play(BleedOutEventPair);
    }
    if (BleedOutVFXTemplate != None)
    {
        if (BleedOutVFX == None)
        {
            BleedOutVFX = Class'BioVisualEffect'.CreateVFXOnPosition(BleedOutVFXTemplate, vect(0,0,0), rot(0,0,0));
        }
        if (BleedOutVFX != None)
        {
            BleedOutVFX.bPauseOnDeath = true;
            BleedOutVFX.bLoopLife = true;
            BleedOutVFX.SetState(0, true);
        }
    }
}

function OnBleedOutStop()
{
    if (WwiseComponent != None)
    {
        WwiseComponent.SetGlobalRTPCFromScript(BleedOutRTPC, 0);
        WwiseComponent.Stop(BleedOutEventPair);
    }
    if (BleedOutVFX != None)
    {
        BleedOutVFX.SetState(2);
    }
}

function float ComputePercentBledOut()
{
    return 1 - CurrentHealth / MaxHealth.Value;
}

defaultproperties
{
    BleedOutEntryThreshold=0,4
    BleedOutRecoveryThreshold=0,45
    BleedOutSoundDurationStart=2
    BleedOutSoundDurationEnd=1
    BleedOutSoundVolumeStart=0,5
    BleedOutSoundVolumeEnd=1
    BleedOutEventPair=WwiseEventPairObject'BleedOutWwiseEventPair'
    BleedOutRTPC="Bleedout_Health"
    BleedOutVFXTemplate=BioVFXTemplate'GameOver_Screen_VFX'
    BleedOutVFXParameter=fb_Damage
    // child object BleedOutWwiseEventPair of type WwiseEventPairObject
}
