﻿class SFXModule_GameEffectManager extends SFXModule
    hidecategories(Object)
    editinlinenew;

var() array<SFXGameEffect> GameEffects;

function Tick(float DeltaSeconds)
{
    local SFXGameEffect Effect;
    local SFXGameEffect BestEffect;
    local int idx;
    foreach GameEffects(Effect)
    {
        Effect.bVisited = false;
    }
    foreach GameEffects(Effect)
    {
        if (Effect.bVisited == false)
        {
            if (Effect.bStacking == false && FindBestEffect(Effect.Class, BestEffect))
            {
                BestEffect.OnUpdate(DeltaSeconds);
            }
            else
            {
                Effect.OnUpdate(DeltaSeconds);
            }
        }
    }
    idx = GameEffects.Length - 1;
    while (idx >= 0)
    {
        Effect = GameEffects[idx];
        if (Effect.DurationType == 1)
        {
            Effect.CurrentTime += DeltaSeconds;
        }
        if (Effect.DurationType == 1 && Effect.HasExpired())
        {
            Effect.OnRemoved();
            GameEffects.Remove(idx, 1);
        }
        idx--;
    }
}

final function bool FindBestEffect(class<SFXGameEffect> EffectClass, out SFXGameEffect BestEffect)
{
    local SFXGameEffect Effect;
    foreach GameEffects(Effect)
    {
        if (Effect.bStacking == false && ClassIsChildOf(Effect.Class, EffectClass))
        {
            Effect.GetStrongerEffect(BestEffect);
            Effect.bVisited = true;
        }
    }
    if (BestEffect != None)
    {
        return true;
    }
    return false;
}

final function bool HasEffectOfType(class<SFXGameEffect> EffectClass)
{
    local SFXGameEffect Effect;
    foreach GameEffects(Effect)
    {
        if (Effect.Class == EffectClass)
        {
            return true;
        }
    }
    return false;
}

final function bool HasEffectOfCategory(Name Category)
{
    local SFXGameEffect Effect;
    foreach GameEffects(Effect)
    {
        if (Effect.Category == Category)
        {
            return true;
        }
    }
    return false;
}

final function bool HasEffectOfTypeAndCategory(class<SFXGameEffect> EffectClass, Name Category)
{
    local SFXGameEffect Effect;
    foreach GameEffects(Effect)
    {
        if (Effect.Class == EffectClass && Effect.Category == Category)
        {
            return true;
        }
    }
    return false;
}

final function SFXGameEffect CreateEffect(class<SFXGameEffect> EffectClass, Name Category, float Duration, EDurationType DurationType, float EffectValue, optional Controller Instigator)
{
    local SFXGameEffect Effect;
    Effect = new(self,EffectClass);
    Effect.Duration = Duration;
    Effect.DurationType = DurationType;
    Effect.EffectValue = EffectValue;
    Effect.Category = Category;
    Effect.Owner = Actor(Outer);
    if (Instigator == None && Pawn(Outer) != None)
    {
        Instigator = Pawn(Outer).Controller;
    }
    Effect.Instigator = Instigator;
    GameEffects.AddItem(Effect);
    return Effect;
}

final function RemoveEffect(SFXGameEffect Effect)
{
    if (Effect == None)
    {
        return;
    }
    Effect.OnRemoved();
    GameEffects.RemoveItem(Effect);
}

final function RemoveEffectsByType(class<SFXGameEffect> EffectClass)
{
    local int idx;
    local SFXGameEffect Effect;
    idx = GameEffects.Length - 1;
    while (idx >= 0)
    {
        Effect = GameEffects[idx];
        if (Effect.Class == EffectClass)
        {
            Effect.OnRemoved();
            GameEffects.Remove(idx, 1);
        }
        idx--;
    }
}

final function RemoveEffectsByCategory(Name Category)
{
    local int idx;
    local SFXGameEffect Effect;
    idx = GameEffects.Length - 1;
    while (idx >= 0)
    {
        Effect = GameEffects[idx];
        if (Effect.Category == Category)
        {
            Effect.OnRemoved();
            GameEffects.Remove(idx, 1);
        }
        idx--;
    }
}

final function UpdateEffectsByCategory(array<class<SFXGameEffect> > EffectClasses, Name Category, Controller Instigator)
{
    local int idx;
    local SFXGameEffect Effect;
    local class<SFXGameEffect> EffectClass;
    idx = GameEffects.Length - 1;
    while (idx >= 0)
    {
        Effect = GameEffects[idx];
        if (Effect.Category == Category)
        {
            if (EffectClasses.Find(Effect.Class) != -1)
            {
                EffectClasses.RemoveItem(Effect.Class);
            }
            else
            {
                Effect.OnRemoved();
                GameEffects.Remove(idx, 1);
            }
        }
        idx--;
    }
    foreach EffectClasses(EffectClass)
    {
        Effect = CreateEffect(EffectClass, Category, EffectClass.Default.Duration, EffectClass.Default.DurationType, EffectClass.Default.EffectValue, Instigator);
        Effect.OnApplied();
    }
}

final function RemoveEffectsByTypeAndCategory(class<SFXGameEffect> EffectClass, Name Category)
{
    local int idx;
    local SFXGameEffect Effect;
    idx = GameEffects.Length - 1;
    while (idx >= 0)
    {
        Effect = GameEffects[idx];
        if (Effect.Class == EffectClass && Effect.Category == Category)
        {
            Effect.OnRemoved();
            GameEffects.Remove(idx, 1);
        }
        idx--;
    }
}

function RemoveAllEffects()
{
    local int idx;
    local SFXGameEffect Effect;
    idx = GameEffects.Length - 1;
    while (idx >= 0)
    {
        Effect = GameEffects[idx];
        Effect.OnRemoved();
        GameEffects.Remove(idx, 1);
        idx--;
    }
}

function float GetEffectValue(class<SFXGameEffect> EffectClass, optional float baseValue)
{
    local SFXGameEffect Effect;
    local float StackValue;
    local float NonStackValue;
    foreach GameEffects(Effect)
    {
        if (ClassIsChildOf(Effect.Class, EffectClass))
        {
            if (EffectClass == None)
            {
                EffectClass = Effect.Class;
            }
            if (Effect.bStacking)
            {
                Effect.ComputeEffectValue(StackValue);
            }
            else
            {
                Effect.ComputeEffectValue(NonStackValue);
            }
        }
    }
    if (EffectClass != None)
    {
        return EffectClass.ComputeTotalEffectValue(baseValue, StackValue, NonStackValue);
    }
    return baseValue;
}

function OnCombatEnd()
{
    local SFXGameEffect Effect;
    foreach GameEffects(Effect)
    {
        Effect.OnCombatEnd();
    }
}

function SFXGameEffect GetFirstEffectOfType(class<SFXGameEffect> EffectClass)
{
    local SFXGameEffect Effect;
    foreach GameEffects(Effect)
    {
        if (Effect.Class == EffectClass)
        {
            return Effect;
        }
    }
    return None;
}

final function PauseEffectsByType(class<SFXGameEffect> EffectClass)
{
    local int idx;
    local SFXGameEffect Effect;
    idx = 0;
    while (idx < GameEffects.Length)
    {
        Effect = GameEffects[idx];
        if (Effect.Class == EffectClass)
        {
            Effect.OnPaused();
        }
        idx++;
    }
}

final function UnpauseEffectsByType(class<SFXGameEffect> EffectClass)
{
    local int idx;
    local SFXGameEffect Effect;
    idx = 0;
    while (idx < GameEffects.Length)
    {
        Effect = GameEffects[idx];
        if (Effect.Class == EffectClass)
        {
            Effect.OnUnpaused();
        }
        idx++;
    }
}

defaultproperties
{
}
