﻿class SFXPawn_Player extends SFXPawn
    hidecategories(Navigation)
    config(Game)
    abstract;

enum EOriginType
{
    OriginType_None,
    OriginType_Spacer,
    OriginType_Colony,
    OriginType_Earthborn,
};

enum ENotorietyType
{
    NotorietyType_None,
    NotorietyType_Survivor,
    NotorietyType_Warhero,
    NotorietyType_Ruthless,
};

enum EPlayerAppearanceType
{
    PlayerAppearanceType_Parts,
    PlayerAppearanceType_Full,
};

enum EHelmetPart
{
    HelmetPart_Helmet,
    HelmetPart_Visor,
    HelmetPart_Breather,
};

var WwiseEvent CoverEnterSound;

var ForceFeedbackWaveform CoverForceFeedback;

var transient String m_sWwiseRTPCNameUseCasual;

var transient String m_sWwiseAudioComponentLabel;

var() String FaceCode;

var() Texture2D m_GUI_Icon_Dead;

var() Texture2D m_GUI_Icon;

var() ScreenShakeStruct HitShake;

var() transient float TotalXP;

var int CosmeticSurgeryPlotID;

var transient bool bInPersonalization;

var transient bool m_bHasBreather;

var transient bool bOverrideHideScars;

var config array<ScarInfo> Scars;

var config float OutOfAmmoSwapThreshold;

var transient float OutOfAmmoTimestamp;

var() SFXCharacterClass PlayerClass;

var() String PlayerClassName;

var() int LastName;

var() String FirstName;

var() String FemaleHeadMesh;

var() String FemalePhysAsset;

var() transient int OverrideHelmetID;

var() int HelmetID;

var() int PatternColorID;

var() int PatternID;

var() int Tint2ID;

var() int Tint1ID;

var() int SpecID;

var() int LegID;

var() int ArmID;

var() int ShoulderID;

var() int TorsoID;

var() int FullBodyID;

var() int CasualID;

var() ENotorietyType Notoriety;

var() EOriginType Origin;

var() EPlayerAppearanceType CombatAppearance;

var() bool bUseCasualAppearance;

var() config int N7ArmourDescription;

var() config int N7ArmourName;

var() config array<CustomizableElement> PatternColorAppearances;

var() config array<CustomizableElement> PatternAppearances;

var() config array<CustomizableElement> Tint2Appearances;

var() config array<CustomizableElement> Tint1Appearances;

var() config array<CustomizableElement> SpecAppearances;

var() config array<CustomizableElement> HelmetAppearances;

var() config array<CustomizableElement> LegAppearances;

var() config array<CustomizableElement> ArmAppearances;

var() config array<CustomizableElement> ShoulderAppearances;

var() config array<CustomizableElement> TorsoAppearances;

var() config array<CustomizableElement> FullBodyAppearances;

var() config array<CustomizableElement> CasualAppearances;

struct ScarInfo
{
    var() config Vector2D Threshold;

    var() config String Emissive;

    var() config String Normal;

    var() config String EyeEmissive;

    var() config String FemaleEmissive;

    var() config String FemaleNormal;

    var() config String FemaleEyeEmissive;

    var() config LinearColor Color;

}

final function String GetFullName()
{
    return FirstName @ Class'SFXGame'.GetSimpleString(LastName);
}

final function LoadShaderCacheForObject(String BaseObjectName)
{
    local String sPackageName;
    local int nTokenIndex;
    nTokenIndex = InStr(BaseObjectName, ".");
    if (nTokenIndex > 0)
    {
        sPackageName = Left(BaseObjectName, nTokenIndex);
        DynamicLoadObject(sPackageName $ ".SeekFreeShaderCache", Class'ShaderCache');
    }
}

final function SkeletalMesh GetMesh(int Id, out array<CustomizableElement> Meshes, out CustomizableElement Element)
{
    local SkeletalMesh SkelMesh;
    local String MeshName;
    local int idx;
    idx = 0;
    while (idx < Meshes.Length)
    {
        if (Meshes[idx].Id == Id)
        {
            Element = Meshes[idx];
            MeshName = bIsFemale ? Element.Mesh.Female : Element.Mesh.Male;
            break;
        }
        idx++;
    }
    if (MeshName != "")
    {
        SkelMesh = SkeletalMesh(FindObject(MeshName, Class'SkeletalMesh'));
        if (SkelMesh == None)
        {
            LoadShaderCacheForObject(MeshName);
            SkelMesh = SkeletalMesh(DynamicLoadObject(MeshName, Class'SkeletalMesh'));
        }
    }
    return SkelMesh;
}

final function MaterialInstance GetMaterialOverride(int Id, out array<CustomizableElement> Meshes, out CustomizableElement Element)
{
    local MaterialInstance MaterialOverride;
    local String MaterialOverrideName;
    local int idx;
    idx = 0;
    while (idx < Meshes.Length)
    {
        if (Meshes[idx].Id == Id)
        {
            Element = Meshes[idx];
            MaterialOverrideName = bIsFemale ? Element.Mesh.FemaleMaterialOverride : Element.Mesh.MaleMaterialOverride;
            break;
        }
        idx++;
    }
    if (MaterialOverrideName != "")
    {
        MaterialOverride = MaterialInstance(FindObject(MaterialOverrideName, Class'MaterialInstance'));
        if (MaterialOverride == None)
        {
            LoadShaderCacheForObject(MaterialOverrideName);
            MaterialOverride = MaterialInstance(DynamicLoadObject(MaterialOverrideName, Class'MaterialInstance'));
        }
    }
    return MaterialOverride;
}

final function SkeletalMesh GetHelmetMesh(int Id, EHelmetPart Part, out array<CustomizableElement> Meshes, out CustomizableElement Element)
{
    local SkeletalMesh SkelMesh;
    local String MeshName;
    local int idx;
    idx = 0;
    while (idx < Meshes.Length)
    {
        if (Meshes[idx].Id == Id)
        {
            Element = Meshes[idx];
            switch (Part)
            {
                case 0:
                    MeshName = bIsFemale ? Element.Mesh.Female : Element.Mesh.Male;
                    break;
                case 1:
                    MeshName = bIsFemale ? Element.Mesh.FemaleVisor : Element.Mesh.MaleVisor;
                    break;
                case 2:
                    MeshName = bIsFemale ? Element.Mesh.FemaleFaceplate : Element.Mesh.MaleFaceplate;
                    break;
            }
            break;
        }
        idx++;
    }
    if (MeshName != "")
    {
        SkelMesh = SkeletalMesh(FindObject(MeshName, Class'SkeletalMesh'));
        if (SkelMesh == None)
        {
            LoadShaderCacheForObject(MeshName);
            SkelMesh = SkeletalMesh(DynamicLoadObject(MeshName, Class'SkeletalMesh'));
        }
    }
    return SkelMesh;
}

final private function MaterialInstanceConstant EnsureMIC(SkeletalMeshComponent MeshCmpt, optional int MatIdx)
{
    local MaterialInstanceConstant MIC;
    local MaterialInterface Parent;
    if (MeshCmpt == None || MeshCmpt.SkeletalMesh == None)
    {
        return None;
    }
    if (MatIdx < MeshCmpt.Materials.Length)
    {
        MIC = MaterialInstanceConstant(MeshCmpt.Materials[MatIdx]);
    }
    if (MIC == None || MIC.Outer != MeshCmpt)
    {
        if (MIC == None && MeshCmpt.SkeletalMesh != None && MatIdx < MeshCmpt.SkeletalMesh.Materials.Length)
        {
            Parent = MeshCmpt.SkeletalMesh.Materials[MatIdx];
        }
        if (Parent != None)
        {
            MIC = new(MeshCmpt,Class'MaterialInstanceConstant');
            MIC.SetParent(Parent);
            MeshCmpt.SetMaterial(MatIdx, MIC);
        }
    }
    return MIC;
}

final private function ValidateAppearanceIDs()
{
    if (CasualAppearances.Find('Id', CasualID) == -1)
    {
        if (CasualAppearances.Length > 0)
        {
            CasualID = CasualAppearances[0].Id;
        }
    }
    if (FullBodyAppearances.Find('Id', FullBodyID) == -1)
    {
        if (FullBodyAppearances.Length > 0)
        {
            FullBodyID = FullBodyAppearances[0].Id;
        }
    }
    if (TorsoAppearances.Find('Id', TorsoID) == -1)
    {
        if (TorsoAppearances.Length > 0)
        {
            TorsoID = TorsoAppearances[0].Id;
        }
    }
    if (ShoulderAppearances.Find('Id', ShoulderID) == -1)
    {
        if (ShoulderAppearances.Length > 0)
        {
            ShoulderID = ShoulderAppearances[0].Id;
        }
    }
    if (ArmAppearances.Find('Id', ArmID) == -1)
    {
        if (ArmAppearances.Length > 0)
        {
            ArmID = ArmAppearances[0].Id;
        }
    }
    if (LegAppearances.Find('Id', LegID) == -1)
    {
        if (LegAppearances.Length > 0)
        {
            LegID = LegAppearances[0].Id;
        }
    }
    if (HelmetAppearances.Find('Id', HelmetID) == -1)
    {
        if (HelmetAppearances.Length > 0)
        {
            HelmetID = HelmetAppearances[0].Id;
        }
    }
    if (SpecAppearances.Find('Id', SpecID) == -1)
    {
        if (SpecAppearances.Length > 0)
        {
            SpecID = SpecAppearances[0].Id;
        }
    }
    if (Tint1Appearances.Find('Id', Tint1ID) == -1)
    {
        if (Tint1Appearances.Length > 0)
        {
            Tint1ID = Tint1Appearances[0].Id;
        }
    }
    if (Tint2Appearances.Find('Id', Tint2ID) == -1)
    {
        if (Tint2Appearances.Length > 0)
        {
            Tint2ID = Tint2Appearances[0].Id;
        }
    }
    if (PatternAppearances.Find('Id', PatternID) == -1)
    {
        if (PatternAppearances.Length > 0)
        {
            PatternID = PatternAppearances[0].Id;
        }
    }
    if (PatternColorAppearances.Find('Id', PatternColorID) == -1)
    {
        if (PatternColorAppearances.Length > 0)
        {
            PatternColorID = PatternColorAppearances[0].Id;
        }
    }
}

function GetGameEffects(int Id, out array<CustomizableElement> Meshes, out array<class<SFXGameEffect> > Effects)
{
    local int idx;
    local class<SFXGameEffect> Effect;
    local String EffectName;
    idx = 0;
    while (idx < Meshes.Length)
    {
        if (Meshes[idx].Id == Id)
        {
            foreach Meshes[idx].GameEffects(EffectName)
            {
                Effect = Class<SFXGameEffect>(FindObject(EffectName, Class'Class'));
                if (Effect != None)
                {
                    Effects.AddItem(Effect);
                }
            }
            return;
        }
        idx++;
    }
}

function UpdateGameEffects()
{
    local array<class<SFXGameEffect> > Effects;
    local SFXModule_GameEffectManager Manager;
    Manager = GetModule(Class'SFXModule_GameEffectManager');
    if (Manager == None)
    {
        ClientMessage("Unable to give effect - Player does not have an SFXModule_GameEffectManager");
        return;
    }
    if (bUseCasualAppearance)
    {
        GetGameEffects(CasualID, CasualAppearances, Effects);
    }
    else
    {
        if (CombatAppearance == 1)
        {
            GetGameEffects(FullBodyID, FullBodyAppearances, Effects);
        }
        else
        {
            GetGameEffects(OverrideHelmetID >= 0 ? OverrideHelmetID : HelmetID, HelmetAppearances, Effects);
            GetGameEffects(ShoulderID, ShoulderAppearances, Effects);
            GetGameEffects(ArmID, ArmAppearances, Effects);
            GetGameEffects(LegID, LegAppearances, Effects);
            GetGameEffects(TorsoID, TorsoAppearances, Effects);
        }
    }
    Manager.UpdateEffectsByCategory(Effects, 'PersonalizationEffects', Controller);
}

function UpdateHairAppearance()
{
    if (MorphHead != None && MorphHead.m_oHairMesh != None)
    {
        if (None == m_oHairMesh)
        {
            m_oHairMesh = new(self,Class'SkeletalMeshComponent');
        }
        m_oHairMesh.SetSkeletalMesh(MorphHead.m_oHairMesh);
        m_oHairMesh.bTransformFromAnimParent = 1;
        m_oHairMesh.SetParentAnimComponent(Mesh);
        m_oHairMesh.SetShadowParent(Mesh);
        m_oHairMesh.SetLightEnvironment(LightEnvironment);
    }
    else
    {
        m_oHairMesh = None;
    }
}

function UpdateAppearance()
{
    ValidateAppearanceIDs();
    UpdateHairAppearance();
    UpdateBodyAppearance();
    UpdateAppearance();
    UpdateParameters();
    UpdateWeaponVisibility();
    ForceUpdateComponents(true, false);
    UpdateGameEffects();
}

function ClearMaterialInstances();

function RefreshMaterialInstances()
{
    local int idx;
    BodyMaterialInstances[0] = EnsureMIC(Mesh);
    idx = 0;
    while (idx < HeadMesh.GetNumElements())
    {
        HeadMaterialInstances[idx] = EnsureMIC(HeadMesh, idx);
        idx++;
    }
    EnsureMIC(m_oHairMesh);
}

final function UpdateBodyAppearance()
{
    local SkeletalMesh SkelMesh;
    local MaterialInterface MaterialOverride;
    local bool bUpdatePhysicsAsset;
    local PhysicsAsset PhysAsset;
    local array<SkeletalMesh> MeshParts;
    local int NewHelmetID;
    local int I;
    local bool bHideHead;
    local bool bHideHair;
    local CustomizableElement Element;
    local SFXCompositeSkeletalMesh CompositeMesh;
    local WwiseAudioComponent PawnAudioComponent;
    local bool bPreserveAnimation;
    local bool bDoMergeMeshes;
    local Name nmActiveEffectsMaterial;
    if (MorphHead == None)
    {
        MorphHead = BioPawnType(ActorType).m_oMorphFace;
    }
    if (MorphHead != None)
    {
        SkelMesh = MorphHead.m_oBaseHead;
    }
    if (bIsFemale && SkelMesh == None)
    {
        SkelMesh = SkeletalMesh(FindObject(FemaleHeadMesh, Class'SkeletalMesh'));
        if (SkelMesh == None)
        {
            SkelMesh = SkeletalMesh(DynamicLoadObject(FemaleHeadMesh, Class'SkeletalMesh'));
        }
    }
    HeadMesh.bOverrideParentSkeleton = true;
    if (SkelMesh != None)
    {
        HeadMesh.SetSkeletalMesh(SkelMesh);
        I = 0;
        while (I < HeadMesh.Materials.Length)
        {
            HeadMesh.SetMaterial(I, None);
            I++;
        }
        HeadMaterialInstances.Length = 0;
    }
    SkelMesh = None;
    bHideHead = false;
    bHideHair = false;
    bDoMergeMeshes = false;
    if (bUseCasualAppearance)
    {
        SkelMesh = GetMesh(CasualID, CasualAppearances, Element);
        MaterialOverride = GetMaterialOverride(CasualID, CasualAppearances, Element);
        m_bHasBreather = false;
    }
    else
    {
        if (CombatAppearance == 1)
        {
            SkelMesh = GetMesh(FullBodyID, FullBodyAppearances, Element);
            bHideHead = Element.Mesh.bHideHead;
            bHideHair = Element.Mesh.bHideHair;
            m_bHasBreather = true;
        }
        else
        {
            NewHelmetID = OverrideHelmetID >= 0 ? OverrideHelmetID : HelmetID;
            SkelMesh = GetMesh(ShoulderID, ShoulderAppearances, Element);
            if (SkelMesh != None)
            {
                MeshParts.AddItem(SkelMesh);
            }
            SkelMesh = GetHelmetMesh(NewHelmetID, 0, HelmetAppearances, Element);
            if (SkelMesh != None)
            {
                MeshParts.AddItem(SkelMesh);
                m_bHasBreather = Element.Mesh.bHasBreather;
            }
            else
            {
                m_bHasBreather = false;
            }
            bHideHead = Element.Mesh.bHideHead;
            bHideHair = Element.Mesh.bHideHair;
            SkelMesh = GetHelmetMesh(NewHelmetID, 1, HelmetAppearances, Element);
            if (SkelMesh != None)
            {
                MeshParts.AddItem(SkelMesh);
            }
            SkelMesh = GetHelmetMesh(NewHelmetID, 2, HelmetAppearances, Element);
            if (SkelMesh != None)
            {
                MeshParts.AddItem(SkelMesh);
            }
            SkelMesh = GetMesh(ArmID, ArmAppearances, Element);
            if (SkelMesh != None)
            {
                MeshParts.AddItem(SkelMesh);
            }
            SkelMesh = GetMesh(LegID, LegAppearances, Element);
            if (SkelMesh != None)
            {
                MeshParts.AddItem(SkelMesh);
            }
            SkelMesh = GetMesh(TorsoID, TorsoAppearances, Element);
            if (SkelMesh != None)
            {
                bDoMergeMeshes = true;
            }
        }
    }
    if (!bHideHead)
    {
        AttachComponent(HeadMesh);
        if (m_oHairMesh != None)
        {
            if (!bHideHair)
            {
                if (m_oHairMesh.bAttached == false)
                {
                    AttachComponent(m_oHairMesh);
                }
            }
            else
            {
                if (m_oHairMesh.bAttached == true)
                {
                    DetachComponent(m_oHairMesh);
                }
            }
        }
    }
    else
    {
        HeadMesh.bOverrideParentSkeleton = false;
        DetachComponent(HeadMesh);
        DetachComponent(m_oHairMesh);
    }
    if (bDoMergeMeshes)
    {
        CompositeMesh = SFXCompositeSkeletalMesh(Mesh.SkeletalMesh);
        if (CompositeMesh == None || !CompositeMesh.MatchesMerge(SkelMesh, MeshParts, bHideHead ? None : HeadMesh))
        {
            SkelMesh = MergeMeshes(SkelMesh, MeshParts, HeadMesh);
        }
        else
        {
            SkelMesh = CompositeMesh;
        }
    }
    PawnAudioComponent = Class'WwiseAudioComponent'.CreateComponentFromScript(self, m_sWwiseAudioComponentLabel);
    if (PawnAudioComponent != None)
    {
        PawnAudioComponent.SetWwiseRTPC(m_sWwiseRTPCNameUseCasual, bUseCasualAppearance);
    }
    if (SkelMesh != None && Mesh.SkeletalMesh == None || Mesh.SkeletalMesh != SkelMesh)
    {
        nmActiveEffectsMaterial = Mesh.GetEffectsMaterial();
        Mesh.ClearEffectsMaterial();
        I = 0;
        while (I < Mesh.GetNumElements())
        {
            Mesh.SetMaterial(I, None);
            ++I;
        }
        bPreserveAnimation = false;
        if (bInPersonalization && Mesh.SkeletalMesh != None && Mesh.SkeletalMesh.RefSkeleton.Length == SkelMesh.RefSkeleton.Length && Mesh.SkeletalMesh.Class == SkelMesh.Class)
        {
            bPreserveAnimation = true;
        }
        if (bUseCasualAppearance || CombatAppearance == 1 && Mesh.SkeletalMesh != SkelMesh)
        {
            bPreserveAnimation = false;
        }
        Mesh.SetSkeletalMesh(SkelMesh, bPreserveAnimation);
        Mesh.SetEffectsMaterial(nmActiveEffectsMaterial);
        bUpdatePhysicsAsset = true;
    }
    Mesh.SetMaterial(0, MaterialOverride);
    PhysAsset = Default.Mesh.PhysicsAsset;
    if (bIsFemale)
    {
        PhysAsset = PhysicsAsset(FindObject(FemalePhysAsset, Class'PhysicsAsset'));
        if (PhysAsset == None)
        {
            PhysAsset = PhysicsAsset(DynamicLoadObject(FemalePhysAsset, Class'PhysicsAsset'));
        }
        if (PhysAsset != None && PhysAsset != Mesh.PhysicsAsset)
        {
            bUpdatePhysicsAsset = true;
        }
    }
    if (bUpdatePhysicsAsset)
    {
        Mesh.SetPhysicsAsset(PhysAsset, true);
    }
}

final static function ApplyScarParameters(const array<ScarInfo> InScars, int InNetAlignment, BioPawn Pawn)
{
    local ScarInfo Scar;
    local Texture2D Emissive;
    local Texture2D Normal;
    local Texture2D Eyes;
    local String EmissiveString;
    local String NormalString;
    local String EyesString;
    local LinearColor ColorValue;
    local MaterialInstanceConstant MIC;
    local MaterialInterface Material;
    local SkeletalMeshComponent InHead;
    local int nScarBucket;
    if (Pawn == None)
    {
        return;
    }
    InHead = Pawn.HeadMesh;
    nScarBucket = 0;
    nScarBucket = 0;
    while (nScarBucket < InScars.Length)
    {
        Scar = InScars[nScarBucket];
        if (InNetAlignment >= Scar.Threshold.X && InNetAlignment <= Scar.Threshold.Y)
        {
            break;
        }
        ++nScarBucket;
    }
    if (nScarBucket >= InScars.Length)
    {
        nScarBucket = InScars.Length - 1;
    }
    Scar = InScars[nScarBucket];
    if (Pawn.bIsFemale)
    {
        EmissiveString = Scar.FemaleEmissive;
        NormalString = Scar.FemaleNormal;
        EyesString = Scar.FemaleEyeEmissive;
    }
    else
    {
        EmissiveString = Scar.Emissive;
        NormalString = Scar.Normal;
        EyesString = Scar.EyeEmissive;
    }
    Emissive = Texture2D(FindObject(EmissiveString, Class'Texture2D'));
    if (Emissive == None)
    {
        Emissive = Texture2D(DynamicLoadObject(EmissiveString, Class'Texture2D'));
    }
    Normal = Texture2D(FindObject(NormalString, Class'Texture2D'));
    if (Normal == None)
    {
        Normal = Texture2D(DynamicLoadObject(NormalString, Class'Texture2D'));
    }
    Eyes = Texture2D(FindObject(EyesString, Class'Texture2D'));
    if (Eyes == None)
    {
        Eyes = Texture2D(DynamicLoadObject(EyesString, Class'Texture2D'));
    }
    ColorValue = Scar.Color;
    foreach InHead.Materials(Material)
    {
        MIC = MaterialInstanceConstant(Material);
        if (MIC != None && Emissive != None && Normal != None)
        {
            MIC.SetTextureParameterValue('HED_Face_Alignment_Emis', Emissive);
            MIC.SetTextureParameterValue('HED_Face_Alignment_Norm', Normal);
            MIC.SetTextureParameterValue('EYE_Diff', Eyes);
            MIC.SetVectorParameterValue('DummyParam', ColorValue);
        }
    }
}

final function GetParagonRenegadeNetPips(out int ParagonPips, out int RenegadePips)
{
    local int ParagonPoints;
    local int RenegadePoints;
    local float PipValue;
    local float TotalParagonPipValue;
    local float TotalRenegadePipValue;
    if (m_oBehavior == None)
    {
        return;
    }
    ParagonPoints = m_oBehavior.GetCharmSkill();
    RenegadePoints = m_oBehavior.GetIntimidateSkill();
    foreach Class'BioSFHandler_CharacterRecord2'.Default.m_ParagonRenegadeBarValues(PipValue)
    {
        TotalParagonPipValue += PipValue * Class'BioSFHandler_CharacterRecord2'.Default.m_nMaxParagon;
        TotalRenegadePipValue += PipValue * Class'BioSFHandler_CharacterRecord2'.Default.m_nMaxRenegade;
        if (ParagonPoints >= TotalParagonPipValue)
        {
            ParagonPips++;
        }
        if (RenegadePoints >= TotalRenegadePipValue)
        {
            RenegadePips++;
        }
    }
}

final function UpdateParameters()
{
    local SkeletalMeshComponent MeshCmpt;
    local MaterialInstanceConstant MIC;
    local ScalarParameterValue SParam;
    local VectorParameterValue VParam;
    local int MeshElementID;
    local MaterialInterface CurrentMaterial;
    local int ParagonPips;
    local int RenegadePips;
    local BioGlobalVariableTable VarTable;
    local int NetPips;
    GetParagonRenegadeNetPips(ParagonPips, RenegadePips);
    NetPips = ParagonPips - RenegadePips;
    if (BioWorldInfo(WorldInfo) != None)
    {
        VarTable = BioWorldInfo(WorldInfo).GetGlobalVariables();
        if (VarTable != None && VarTable.GetBool(CosmeticSurgeryPlotID) == true)
        {
            NetPips = 5;
        }
    }
    if (bOverrideHideScars)
    {
        NetPips = 5;
    }
    ApplyScarParameters(Scars, NetPips, self);
    if (MorphHead != None)
    {
        MorphHead.ApplyMaterialOverridesToPawn(self);
    }
    if (bUseCasualAppearance || CombatAppearance == 1)
    {
        return;
    }
    foreach ComponentList(Class'SkeletalMeshComponent', MeshCmpt)
    {
        MeshElementID = 0;
        while (MeshElementID < MeshCmpt.GetNumElements())
        {
            MIC = None;
            CurrentMaterial = MeshCmpt.GetBaseMaterial(MeshElementID);
            if (CurrentMaterial == None)
            {
            }
            else
            {
                if (CurrentMaterial.Outer == self)
                {
                    MIC = MaterialInstanceConstant(CurrentMaterial);
                }
            }
            if (MIC == None)
            {
                MIC = new(self,Class'MaterialInstanceConstant');
                MIC.SetParent(CurrentMaterial);
                MeshCmpt.SetMaterial(MeshElementID, MIC);
            }
            if (SpecID >= 0 && SpecID < SpecAppearances.Length)
            {
                SParam = SpecAppearances[SpecID].Spec.SpecParam;
                MIC.SetScalarParameterValue(SParam.ParameterName, SParam.ParameterValue);
                SParam = SpecAppearances[SpecID].Spec.SpecPwrParam;
                MIC.SetScalarParameterValue(SParam.ParameterName, SParam.ParameterValue);
            }
            if (Tint1ID >= 0 && Tint1ID < Tint1Appearances.Length)
            {
                VParam = Tint1Appearances[Tint1ID].Tint.TintParam;
                MIC.SetVectorParameterValue(VParam.ParameterName, VParam.ParameterValue);
                VParam = Tint1Appearances[Tint1ID].Tint.PhongParam;
                MIC.SetVectorParameterValue(VParam.ParameterName, VParam.ParameterValue);
            }
            if (Tint2ID >= 0 && Tint2ID < Tint2Appearances.Length)
            {
                VParam = Tint2Appearances[Tint2ID].Tint.TintParam;
                MIC.SetVectorParameterValue(VParam.ParameterName, VParam.ParameterValue);
            }
            if (PatternID >= 0 && PatternID < PatternAppearances.Length)
            {
                VParam = PatternAppearances[PatternID].Pattern.Stripe1Param;
                MIC.SetVectorParameterValue(VParam.ParameterName, VParam.ParameterValue);
                VParam = PatternAppearances[PatternID].Pattern.Stripe2Param;
                MIC.SetVectorParameterValue(VParam.ParameterName, VParam.ParameterValue);
                VParam = PatternAppearances[PatternID].Pattern.Stripe3Param;
                MIC.SetVectorParameterValue(VParam.ParameterName, VParam.ParameterValue);
            }
            if (PatternColorID >= 0 && PatternColorID < PatternColorAppearances.Length)
            {
                VParam = PatternColorAppearances[PatternColorID].Pattern.Stripe1Param;
                MIC.SetVectorParameterValue(VParam.ParameterName, VParam.ParameterValue);
                VParam = PatternColorAppearances[PatternColorID].Pattern.Stripe2Param;
                MIC.SetVectorParameterValue(VParam.ParameterName, VParam.ParameterValue);
                VParam = PatternColorAppearances[PatternColorID].Pattern.Stripe3Param;
                MIC.SetVectorParameterValue(VParam.ParameterName, VParam.ParameterValue);
            }
            ++MeshElementID;
        }
    }
}

function bool CreateWeapon(class<SFXWeapon> WeaponClass, optional bool bEquipWeapon = false)
{
    local bool bRetval;
    bRetval = CreateWeapon(WeaponClass, bEquipWeapon);
    UpdateWeaponVisibility();
    return bRetval;
}

function UpdateWeaponVisibility()
{
    local SFXWeapon ChkWeapon;
    if (InvManager != None)
    {
        foreach InvManager.InventoryActors(Class'SFXWeapon', ChkWeapon)
        {
            if (ChkWeapon != None && ChkWeapon.Mesh != None)
            {
                ChkWeapon.Mesh.SetHidden(bUseCasualAppearance ? true : false);
            }
        }
    }
}

function PlayHit(float Damage, Controller InstigatedBy, Vector HitLocation, class<DamageType> DamageType, Vector Momentum, TraceHitInfo HitInfo)
{
    local SFXPlayerCamera Camera;
    PlayHit(Damage, InstigatedBy, HitLocation, DamageType, Momentum, HitInfo);
    if (InstigatedBy != None && InstigatedBy.Pawn != None && BioPlayerController(Controller) != None)
    {
        BioPlayerController(Controller).QueueDamageIndicator(InstigatedBy.Pawn);
        Camera = SFXPlayerCamera(BioPlayerController(Controller).PlayerCamera);
        Camera.AddScreenShake(HitShake);
    }
}

final function ForceHideHelmet(bool bHideHelmet)
{
    if (bHideHelmet)
    {
        OverrideHelmetID = 0;
    }
    else
    {
        OverrideHelmetID = -1;
    }
}

final function ForceShowHelmet(bool bShowHelmet)
{
    local int idx;
    local CustomizableElement Helmet;
    local String HelmetName;
    local String ChkName;
    if (bShowHelmet)
    {
        idx = HelmetAppearances.Find('Id', HelmetID);
        if (idx != -1)
        {
            Helmet = HelmetAppearances[idx];
            HelmetName = bIsFemale ? Helmet.Mesh.Female : Helmet.Mesh.Male;
            if (HelmetName != "")
            {
                if (Helmet.Mesh.bHasBreather)
                {
                    OverrideHelmetID = HelmetID;
                    return;
                }
                else
                {
                    idx = 0;
                    while (idx < HelmetAppearances.Length)
                    {
                        ChkName = bIsFemale ? HelmetAppearances[idx].Mesh.Female : HelmetAppearances[idx].Mesh.Male;
                        if (ChkName == HelmetName && HelmetAppearances[idx].Mesh.bHasBreather)
                        {
                            OverrideHelmetID = HelmetAppearances[idx].Id;
                            return;
                        }
                        idx++;
                    }
                }
            }
        }
        idx = 0;
        while (idx < HelmetAppearances.Length)
        {
            ChkName = bIsFemale ? HelmetAppearances[idx].Mesh.Female : HelmetAppearances[idx].Mesh.Male;
            if (ChkName != "" && HelmetAppearances[idx].Mesh.bHasBreather)
            {
                OverrideHelmetID = HelmetAppearances[idx].Id;
                return;
            }
            idx++;
        }
        if (HelmetAppearances.Length > 0)
        {
            OverrideHelmetID = HelmetAppearances[0].Id;
            return;
        }
    }
    OverrideHelmetID = -1;
}

function ForceHelmetVisibility(bool bHelmetVisible)
{
    if (bHelmetVisible)
    {
        ForceShowHelmet(true);
    }
    else
    {
        ForceHideHelmet(true);
    }
}

simulated function ProcessViewRotation(float DeltaTime, out Rotator out_ViewRotation, out Rotator out_DeltaRot)
{
    local Vector CamRot;
    local Vector CamCross;
    local Vector PlayerRot;
    local float CamDot;
    local float Angle;
    ProcessViewRotation(DeltaTime, out_ViewRotation, out_DeltaRot);
    if (IsInCover())
    {
        CamRot = out_ViewRotation;
        PlayerRot = Rotation;
        CamDot = CamRot Dot PlayerRot;
        CamCross = PlayerRot Cross CamRot;
        Angle = CoverType == 1 ? 0,9 : 0,97;
        if (IsAtLeftEdgeSlot() && CoverAction == 3)
        {
            if (CamDot < Angle && CamCross.Z > 0)
            {
                if (CoverType == 2)
                {
                    CoverAction = 7;
                }
                else
                {
                    CoverAction = 0;
                }
            }
        }
        else
        {
            if (IsAtRightEdgeSlot() && CoverAction == 4)
            {
                if (CamDot < Angle && CamCross.Z < 0)
                {
                    if (CoverType == 2)
                    {
                        CoverAction = 7;
                    }
                    else
                    {
                        CoverAction = 0;
                    }
                }
            }
        }
    }
}

function CopyPawnAppearance(BioPawn SourcePawn)
{
    local SFXPawn_Player SourcePlayer;
    local SFXPower Power;
    local SFXPower Power2;
    CopyPawnAppearance(SourcePawn);
    SourcePlayer = SFXPawn_Player(SourcePawn);
    if (SourcePlayer != None)
    {
        CombatAppearance = SourcePlayer.CombatAppearance;
        CasualID = SourcePlayer.CasualID;
        FullBodyID = SourcePlayer.FullBodyID;
        TorsoID = SourcePlayer.TorsoID;
        ShoulderID = SourcePlayer.ShoulderID;
        ArmID = SourcePlayer.ArmID;
        LegID = SourcePlayer.LegID;
        SpecID = SourcePlayer.SpecID;
        Tint1ID = SourcePlayer.Tint1ID;
        Tint2ID = SourcePlayer.Tint2ID;
        PatternID = SourcePlayer.PatternID;
        PatternColorID = SourcePlayer.PatternColorID;
        HelmetID = SourcePlayer.HelmetID;
        if (SourcePawn.PowerManager != None && PowerManager != None)
        {
            foreach SourcePawn.PowerManager.Powers(Power)
            {
                if (SFXPower_PassivePower(Power) != None)
                {
                    foreach PowerManager.Powers(Power2)
                    {
                        if (Power.Class == Power2.Class)
                        {
                            Power2.Rank = Power.Rank;
                        }
                    }
                }
            }
        }
        UpdateAppearance();
    }
}

function int GetScaledLevel()
{
    return 1;
}

function AddPlayerInventory()
{
    local SFXInventoryManager InventoryManager;
    local SFXLoadoutData ChkLoadout;
    InventoryManager = SFXInventoryManager(InvManager);
    if (InventoryManager != None && InventoryManager.FindInventoryType(Class'SFXWeapon', true) == None)
    {
        ChkLoadout = GetPawnLoadout();
        CreateWeapons(ChkLoadout);
        ScaleWeapons(ChkLoadout, GetScaledLevel());
        InventoryManager.CurrentWeaponSelection = ChkLoadout.Weapons[0];
        WeaponOnDeck = SFXWeapon(InventoryManager.FindInventoryType(ChkLoadout.Weapons[1]));
    }
}

function PostBeginPlay()
{
    local Object DynamicLoadedClassObject;
    local class<SFXCharacterClass> CharacterClass;
    DynamicLoadedClassObject = Class'SFXEngine'.LoadSeekFreeObject(PlayerClassName, Class'Class');
    CharacterClass = Class<SFXCharacterClass>(DynamicLoadedClassObject);
    Class'SFXEngine'.ReleaseSeekFreeObject(PlayerClassName);
    PlayerClass = new(self,CharacterClass);
    Loadout = PlayerClass.Loadout;
    SetTimer(0,25, false, 'CheckEnterCombat');
    SetTimer(1, false, 'AutoMap');
    SetTimer(1,25, false, 'ApplyBonuses');
    PostBeginPlay();
}

function PossessedBy(Controller C, bool bVehicleTransition)
{
    local SFXModule_GameEffectManager Manager;
    PossessedBy(C, bVehicleTransition);
    Manager = GetModule(Class'SFXModule_GameEffectManager');
    if (Manager != None)
    {
        Manager.RemoveEffectsByCategory('PersonalizationEffects');
    }
    UpdateGameEffects();
}

function SFXPower FindPower(Name nmPower)
{
    local BioPower oPower;
    if (nmPower == 'None' || nmPower == 'None')
    {
        return None;
    }
    oPower = None;
    foreach m_oBehavior.m_powers(oPower)
    {
        if (oPower.NewPower.Rank > 0 && oPower.NewPower.IsA(nmPower))
        {
            return oPower.NewPower;
        }
    }
    return None;
}

function AutoMap()
{
    if (WorldInfo.IsConsoleBuild())
    {
        AutoMapXbox();
    }
    else
    {
        AutoMapPC();
    }
}

function ApplyBonuses()
{
    local BioWorldInfo World;
    local SFXGame Game;
    local int Index;
    local BioPawn Member;
    local SFXShield_Base Shield;
    World = BioWorldInfo(WorldInfo);
    if (World != None)
    {
        Game = SFXGame(World.Game);
        if (Game != None && Game.BonusList != None)
        {
            Game.BonusList.UpdateCache(World);
        }
        if (World.m_playerSquad != None)
        {
            Index = 0;
            while (Index < World.m_playerSquad.GetSquadSize())
            {
                Member = BioPawn(World.m_playerSquad.GetMember(Index));
                Shield = Member.GetShields();
                if (Shield != None)
                {
                    Shield.CurrentShields = Shield.MaxShields.Value;
                }
                Index++;
            }
        }
    }
}

function AutoMapXbox()
{
    local array<Name> MappedPowers;
    local BioPlayerInput BPI;
    local int idx;
    local SFXPower Power;
    BPI = BioPlayerInput(BioPlayerController(Controller).PlayerInput);
    MappedPowers = PlayerClass.Default.MappedPowers;
    idx = 0;
    while (idx < MappedPowers.Length)
    {
        Power = FindPower(MappedPowers[idx]);
        if (Power != None)
        {
            MappedPowers[idx] = Power.Class.Name;
            idx++;
        }
        else
        {
            MappedPowers.Remove(idx, 1);
        }
    }
    Power = FindPower(BPI.m_nmMappedPower);
    if (Power != None)
    {
        BPI.m_nmMappedPower = Power.Class.Name;
    }
    else
    {
        BPI.m_nmMappedPower = 'None';
    }
    Power = FindPower(BPI.m_nmMappedPower2);
    if (Power != None)
    {
        BPI.m_nmMappedPower2 = Power.Class.Name;
    }
    else
    {
        BPI.m_nmMappedPower2 = 'None';
    }
    Power = FindPower(BPI.m_nmMappedPower3);
    if (Power != None)
    {
        BPI.m_nmMappedPower3 = Power.Class.Name;
    }
    else
    {
        BPI.m_nmMappedPower3 = 'None';
    }
    MappedPowers.RemoveItem(BPI.m_nmMappedPower);
    MappedPowers.RemoveItem(BPI.m_nmMappedPower2);
    MappedPowers.RemoveItem(BPI.m_nmMappedPower3);
    if (BPI.m_nmMappedPower3 == 'None' && MappedPowers.Length > 0)
    {
        BPI.m_nmMappedPower3 = MappedPowers[0];
        MappedPowers.Remove(0, 1);
    }
    if (BPI.m_nmMappedPower2 == 'None' && MappedPowers.Length > 0)
    {
        BPI.m_nmMappedPower2 = MappedPowers[0];
        MappedPowers.Remove(0, 1);
    }
    if (BPI.m_nmMappedPower == 'None' && MappedPowers.Length > 0)
    {
        BPI.m_nmMappedPower = MappedPowers[0];
        MappedPowers.Remove(0, 1);
    }
}

function AutoMapPC()
{
    local MassEffectGuiManager oGM;
    local SFXSFHandler_PCPowerWheel oPowerWheel;
    local BioPlayerController BPC;
    local SFXPower Power;
    local SFXPower Power2;
    local Name nmPawn;
    local int I;
    local int J;
    BPC = BioPlayerController(Controller);
    oGM = MassEffectGuiManager(BPC.GetScaleFormManager());
    oPowerWheel = SFXSFHandler_PCPowerWheel(oGM.m_oSFPowerWheelHandler);
    if (oGM == None || oPowerWheel == None)
    {
        return;
    }
    oPowerWheel.SetupPlayerPowers();
    I = 0;
    while (I < BPC.8)
    {
        Power = PowerManager.Powers[BPC.m_aHotKeyDefines[I].nPowerID];
        nmPawn = BPC.m_aHotKeyDefines[I].nmPawn;
        J = 0;
        while (Power != None && J < oPowerWheel.m_aPowerIcons.Length)
        {
            if (oPowerWheel.m_aPowerIcons[J].pPawn != None && oPowerWheel.m_aPowerIcons[J].pPawn.Tag == nmPawn)
            {
                Power2 = oPowerWheel.m_aPowerIcons[J].pPower;
                if (Power2 != None && Power.EvolvedPowerClass1 == Power2.Class || Power.EvolvedPowerClass2 == Power2.Class)
                {
                    oPowerWheel.NewSetQuickSlotPower(I, J, true, true);
                    break;
                }
            }
            J++;
        }
        I++;
    }
    I = 0;
    while (I < oPowerWheel.m_aPowerIcons.Length)
    {
        if (oPowerWheel.m_aPowerIcons[I].pPawn == self)
        {
            Power = oPowerWheel.m_aPowerIcons[I].pPower;
            J = 0;
            while (Power != None && J < BPC.8)
            {
                if (BPC.m_aHotKeyDefines[J].nmPawn == 'Player' && oPowerWheel.m_aPowerIcons[I].nPowerID == BPC.m_aHotKeyDefines[J].nPowerID)
                {
                    Power = None;
                }
                J++;
            }
            if (Power != None)
            {
                J = 0;
                while (J < BPC.8)
                {
                    if (BPC.m_aHotKeyDefines[J].nmPawn == 'None')
                    {
                        oPowerWheel.NewSetQuickSlotPower(J, I, true, true);
                        Power = None;
                        break;
                    }
                    J++;
                }
            }
        }
        I++;
    }
}

function CheckEnterCombat()
{
    local BioPlayerController PC;
    local BioPlayerInput Input;
    PC = BioPlayerController(Controller);
    if (PC != None)
    {
        Input = BioPlayerInput(PC.PlayerInput);
        if (Input != None && ActionState != 1 && Input.IsCombatEnabled())
        {
            SetCombatState(true);
        }
    }
}

state Dying
{
    function BeginState(Name PreviousStateName);

    event Timer();

}

state Ragdoll
{
    event BeginState(Name PreviousStateName)
    {
        BeginState(PreviousStateName);
        if (BioPlayerController(Controller) != None)
        {
            BioPlayerController(Controller).SetZoomed(false);
        }
    }

}

function bool PlayAnimatedReaction(EAICustomAction CustomAction, Controller InstigatedBy, optional bool bPlayImpact, optional Vector HitLocation, optional Vector HitNormal, optional out TraceHitInfo HitInfo, optional bool bDeathReaction)
{
    local SFXPlayerController PC;
    local SFXCustomAction_DamageReaction Action;
    PC = SFXPlayerController(Controller);
    if (Physics != 10 && PC != None && PC.CurrentCustomAction == 0 && PC.CanDoCustomAction(CustomAction))
    {
        PC.DoCustomAction(CustomAction);
        LastAnimatedReactionTime = WorldInfo.TimeSeconds;
        Action = SFXCustomAction_DamageReaction(PC.CustomActions[CustomAction]);
        Action.Init(HitLocation, HitNormal, HitInfo, bPlayImpact, InstigatedBy);
        return true;
    }
    return false;
}

function PlayShieldHit(float Damage, Controller InstigatedBy, Vector HitLocation, class<DamageType> DamageType, Vector Momentum, TraceHitInfo HitInfo)
{
    local class<SFXDamageType> SFXDT;
    local PlayerController PC;
    SFXDT = Class<SFXDamageType>(DamageType);
    PlayShieldHit(Damage, InstigatedBy, HitLocation, DamageType, Momentum, HitInfo);
    PC = PlayerController(Controller);
    if (PC != None)
    {
        PC.ClientPlayForceFeedbackWaveform(SFXDT.Default.ShieldHitFFWaveform);
    }
}

simulated function PlayDying(class<DamageType> DamageType, Vector HitLoc)
{
    local PlayerController PC;
    PlayDying(DamageType, HitLoc);
    PC = PlayerController(Controller);
    if (PC != None)
    {
        PC.ClientPlayForceFeedbackWaveform(DamageType.Default.KilledFFWaveform);
    }
}

function Texture2D GetGUIIcon(optional bool bDeadIcon = false)
{
    return bDeadIcon ? m_GUI_Icon_Dead : m_GUI_Icon;
}

function PlayerCoverAcquired(CovPosInfo CovInfo)
{
    PlayerCoverAcquired(CovInfo);
    PlaySound(CoverEnterSound);
}

function bool SwitchToBackupWeapon(optional SFXWeapon PreferredWeapon = None)
{
    local SFXInventoryManager InventoryManager;
    local SFXWeapon BestWeapon;
    local SFXWeapon Weap;
    local BioPlayerController PC;
    local BioPlayerInput BPInput;
    if (Weapon == None || Weapon.IsInState('WeaponEquipping') || Weapon.IsInState('WeaponPuttingDown'))
    {
        return false;
    }
    PC = BioPlayerController(Controller);
    if (PC != None)
    {
        BPInput = BioPlayerInput(PC.PlayerInput);
        if (BPInput == None || BPInput.IsCombatEnabled() == false)
        {
            return false;
        }
    }
    if (IsPerformingBlockingAction())
    {
        return false;
    }
    if (IsInCoverLeaning())
    {
        return false;
    }
    if (ActionState != 1)
    {
        SetCombatState_Active();
    }
    InventoryManager = SFXInventoryManager(InvManager);
    foreach InventoryManager.InventoryActors(Class'SFXWeapon', Weap)
    {
        if (Weap == Weapon)
        {
            continue;
        }
        if (PreferredWeapon != None && Weap == PreferredWeapon)
        {
            BestWeapon = Weap;
            break;
        }
        if (Weap.OutOfAmmo() || Weap.SwitchPriority <= 0)
        {
            continue;
        }
        if (BestWeapon == None)
        {
            BestWeapon = Weap;
        }
        else
        {
            if (BestWeapon.SwitchPriority < Weap.SwitchPriority)
            {
                BestWeapon = Weap;
            }
        }
    }
    if (BestWeapon != None)
    {
        BioPlayerController(Controller).SetZoomed(false);
        BioPlayerController(Controller).OrderWeaponSwitch(self, BestWeapon);
        return true;
    }
    return false;
}

event SetRTPCHelmetIsEnabled(WwiseAudioComponent WwiseComponent)
{
    if (WwiseComponent != None)
    {
        if (m_bHasBreather)
        {
            WwiseComponent.SetWwiseRTPC("Helmet", 1);
        }
        else
        {
            WwiseComponent.SetWwiseRTPC("Helmet", 0);
        }
    }
}

function SetCombatState(bool bDraw)
{
    ClearTimer('SetCombatState_Active');
    ClearTimer('SetCombatState_Inactive');
    if (bDraw)
    {
        SetTimer(0,25, false, 'SetCombatState_Active');
    }
    else
    {
        SetTimer(0,25, false, 'SetCombatState_Inactive');
    }
}

function SetCombatState_Active()
{
    ClearTimer('SetCombatState_Active');
    ClearTimer('SetCombatState_Inactive');
    if (BioPlayerSquad(m_oBehavior.Squad).m_bCombatEnabled == false)
    {
        return;
    }
    if (BioPlayerController(Controller).GameModeManager2.IsActive(6) || BioPlayerController(Controller).GameModeManager2.IsActive(5))
    {
        return;
    }
    m_oBehavior.SetActionState(1);
    BioPlayerController(Controller).PlayerSquadSetActionState(1);
}

function SetCombatState_Inactive()
{
    ClearTimer('SetCombatState_Active');
    ClearTimer('SetCombatState_Inactive');
    if (BioPlayerController(Controller).GameModeManager2.IsActive(6) || BioPlayerController(Controller).GameModeManager2.IsActive(5))
    {
        return;
    }
    m_oBehavior.SetActionState(0);
    BioPlayerController(Controller).PlayerSquadSetActionState(0);
}

function CreateWeapons(SFXLoadoutData ChkLoadout)
{
    local class<SFXWeapon> WeaponClass;
    local SFXPlayerSquadLoadoutData PlayerLoadout;
    local SFXEngine Engine;
    Engine = SFXEngine(BioPlayerController(Controller).Player.Outer);
    PlayerLoadout = SFXPlayerSquadLoadoutData(ChkLoadout);
    if (PlayerLoadout != None)
    {
        CreatePlayerSquadWeapons(PlayerLoadout, Engine.PlayerLoadoutWeapons);
    }
    foreach ChkLoadout.Weapons(WeaponClass)
    {
        CreateWeapon(WeaponClass);
    }
}

defaultproperties
{
    CasualAppearances=System.Byte[]
    TorsoAppearances=System.Byte[]
    ShoulderAppearances=System.Byte[]
    ArmAppearances=System.Byte[]
    LegAppearances=System.Byte[]
    HelmetAppearances=System.Byte[]
    SpecAppearances=System.Byte[]
    Tint1Appearances=System.Byte[]
    Tint2Appearances=System.Byte[]
    PatternAppearances=System.Byte[]
    PatternColorAppearances=System.Byte[]
    OverrideHelmetID=-1
    FemalePhysAsset="BIOG_PHY_Humanoid_F.PHY_HMF_HVYa"
    FemaleHeadMesh="BIOG_HMF_HED_PROMorph_R.Custom.HMF_HED_PROCustom_MDL"
    LastName=125303
    OutOfAmmoSwapThreshold=0,4
    Scars=System.Byte[]
    CosmeticSurgeryPlotID=5978
    m_GUI_Icon=Texture2D'Icon_N7'
    m_GUI_Icon_Dead=Texture2D'Icon_N7'
    m_sWwiseAudioComponentLabel="_PlayerPawn"
    m_sWwiseRTPCNameUseCasual="Use_Casual"
    CoverForceFeedback=ForceFeedbackWaveform'CoverEnterSound0'
    CoverEnterSound=WwiseEvent'_Play_cinedesign_foley_movement_human_hard_fast'
    PrettyName=System.Byte[]
    bShouldSpawnWeapons=false
    bIconicAppearance=true
    bScalePowers=false
    HeadMesh=SkeletalMeshComponent'HeadMesh0'
    m_pAnimSetMgr=BioGestureAnimSetMgr'oAnimSetMgr'
    LightEnvironment=BioDynamicLightEnvironmentComponent'BioLightEnvComponent0'
    PowerManager=SFXPowerManager'PowerMgr'
    bCanPickupInventory=true
    bDontPossess=true
    Mesh=None
    CylinderComponent=None
    ViewPitchMin=-10922
    ViewPitchMax=10922
    Components=None
    // child object HeadMesh0 of type SkeletalMeshComponent
    // child object CollisionCylinder of type CylinderComponent
    // child object BioLightEnvComponent0 of type BioDynamicLightEnvironmentComponent
    // child object BioPawnSkeletalMeshComponent of type SkeletalMeshComponent
    // child object oAnimSetMgr of type BioGestureAnimSetMgr
    // child object PowerMgr of type SFXPowerManager
    // child object GEMod0 of type SFXModule_GameEffectManager
    // child object RadarModule of type SFXModule_Radar
    // child object GPModule of type SFXGamePropertiesModule
    // child object DmgMod1 of type SFXModule_DamagePlayer
    // child object Behavior of type BioPawnBehavior
    // child object CoverEnterSound0 of type ForceFeedbackWaveform
}
