﻿class SFXProjectile extends Projectile
    hidecategories(Navigation);

var const float ProjEffectsTrailWaitTime;

var ParticleSystem ProjEffectsHeadTemplate;

var ParticleSystem ProjEffectsTrailTemplate;

var /* 00000008 */ ParticleSystemComponent ProjEffectsHead;

var /* 00000008 */ ParticleSystemComponent ProjEffectsTrail;

var() float MeshScaleTime;

var() Vector DesiredScale;

var() /* 00000008 */ MeshComponent Mesh;

var bool bPooled;

var const bool bArcing;

var bool bShuttingDown;

var(Debug) bool bSuppressAudio;

final simulated private function SpawnFlightEffects()
{
    local int idx;
    if (WorldInfo.NetMode != 1)
    {
        if (ProjEffectsTrail == None && ProjEffectsTrailTemplate != None)
        {
            ProjEffectsTrail = new(self,Class'ParticleSystemComponent');
            ProjEffectsTrail.bAutoActivate = false;
            ProjEffectsTrail.bUpdateComponentInTick = true;
            ProjEffectsTrail.SetTemplate(ProjEffectsTrailTemplate);
            idx = 0;
            while (idx < ProjEffectsTrail.EmitterInstances.Length)
            {
                ProjEffectsTrail.SetKillOnDeactivate(idx, false);
                ProjEffectsTrail.SetKillOnCompleted(idx, false);
                ++idx;
            }
            self.AttachComponent(ProjEffectsTrail);
        }
        if (ProjEffectsHead == None && ProjEffectsHeadTemplate != None)
        {
            ProjEffectsHead = new(self,Class'ParticleSystemComponent');
            ProjEffectsHead.bAutoActivate = false;
            ProjEffectsHead.bUpdateComponentInTick = true;
            ProjEffectsHead.SetTemplate(ProjEffectsHeadTemplate);
            if (ProjEffectsHead.Template.bUseFixedRelativeBoundingBox == false)
            {
                ProjEffectsHead.Template.FixedRelativeBoundingBox.IsValid = 1;
                ProjEffectsHead.Template.FixedRelativeBoundingBox.Min.X = -32;
                ProjEffectsHead.Template.FixedRelativeBoundingBox.Min.Y = -32;
                ProjEffectsHead.Template.FixedRelativeBoundingBox.Min.Z = -32;
                ProjEffectsHead.Template.FixedRelativeBoundingBox.Max.X = 32;
                ProjEffectsHead.Template.FixedRelativeBoundingBox.Max.Y = 32;
                ProjEffectsHead.Template.FixedRelativeBoundingBox.Max.Z = 32;
                ProjEffectsHead.Template.bUseFixedRelativeBoundingBox = true;
            }
            idx = 0;
            while (idx < ProjEffectsHead.EmitterInstances.Length)
            {
                ProjEffectsHead.SetKillOnDeactivate(idx, false);
                ProjEffectsHead.SetKillOnCompleted(idx, false);
                ++idx;
            }
            self.AttachComponent(ProjEffectsHead);
        }
        if (ProjEffectsTrail != None)
        {
            ProjEffectsTrail.SetAbsolute(false, false, false);
            ProjEffectsTrail.SetLODLevel(WorldInfo.bDropDetail ? 1 : 0);
            ProjEffectsTrail.__OnSystemFinished__Delegate = ProjEffectsTrailFinished;
            ProjEffectsTrail.SetActive(true);
        }
        if (ProjEffectsHead != None)
        {
            ProjEffectsHead.SetAbsolute(false, false, false);
            ProjEffectsHead.SetLODLevel(WorldInfo.bDropDetail ? 1 : 0);
            ProjEffectsHead.__OnSystemFinished__Delegate = None;
            ProjEffectsHead.SetActive(true);
        }
    }
}

simulated function Recycle()
{
    bShuttingDown = false;
    bSuppressAudio = Default.bSuppressAudio;
    GotoState(InitialState != 'None' ? InitialState : 'Auto');
    ClearTimer('ShutdownPostEffects');
    SetCollision(false, false);
    SetPhysics(7);
    SetHidden(true);
    if (ProjEffectsTrail == None && ProjEffectsTrailTemplate != None || ProjEffectsHead == None && ProjEffectsHeadTemplate != None)
    {
        SpawnFlightEffects();
    }
    if (ProjEffectsTrail != None)
    {
        ProjEffectsTrail.__OnSystemFinished__Delegate = None;
        ProjEffectsTrail.KillParticlesForced();
        ProjEffectsTrail.DeactivateSystem();
    }
    if (ProjEffectsHead != None)
    {
        ProjEffectsHead.__OnSystemFinished__Delegate = None;
        ProjEffectsHead.KillParticlesForced();
        ProjEffectsHead.DeactivateSystem();
    }
    SetOwner(None);
    self.Instigator = None;
    self.InstigatorController = None;
    LifeSpan = 0;
    bStasis = true;
}

final simulated function Reuse()
{
    if (bPooled)
    {
        SetCollision(Default.bCollideActors, Default.bBlockActors);
        SetPhysics(6);
        SetProjectileHidden(false);
        SetHidden(false);
        LifeSpan = Default.LifeSpan;
        bStasis = false;
        GotoState(InitialState != 'None' ? InitialState : 'Auto');
    }
}

final simulated private function ShutdownPostEffects()
{
    if (bPooled)
    {
        Recycle();
    }
    else
    {
        Destroy();
    }
}

function Init(Vector Direction)
{
    Init(Direction);
    SpawnFlightEffects();
}

simulated function ShutDown()
{
    bShuttingDown = true;
    SetCollision(false, false);
    SetProjectileHidden(true);
    SetPhysics(7);
    if (ProjEffectsHead != None)
    {
        ProjEffectsHead.SetActive(false);
    }
    if (ProjEffectsTrail != None && ProjEffectsTrail.bIsActive)
    {
        ProjEffectsTrail.SetActive(false);
        SetTimer(ProjEffectsTrailWaitTime, false, 'ShutdownPostEffects');
        LifeSpan = 0;
    }
    else
    {
        ShutdownPostEffects();
    }
}

final function bool IsShuttingDown()
{
    return bShuttingDown;
}

simulated event OutsideWorldBounds()
{
    ShutdownPostEffects();
}

singular simulated event HitWall(Vector HitNormal, Actor Wall, PrimitiveComponent WallComp)
{
    ImpactedActor = Wall;
    if (!Wall.bStatic && !Wall.bWorldGeometry)
    {
        if (DamageRadius == 0)
        {
            Wall.TakeDamage(Damage, InstigatorController, Location, MomentumTransfer * Normal(Velocity), MyDamageType, self);
        }
    }
    Explode(Location, HitNormal);
    ImpactedActor = None;
    ShutDown();
}

simulated function Explode(Vector HitLocation, Vector HitNormal)
{
    SpawnImpactEffect(HitLocation, HitNormal);
}

final simulated private function SetProjectileHidden(bool bHide)
{
    if (Mesh != None)
    {
        Mesh.SetHidden(bHide);
    }
}

final simulated private function ProjEffectsTrailFinished(ParticleSystemComponent PSC)
{
    if (PSC == ProjEffectsTrail)
    {
        if (bShuttingDown)
        {
            SetTimer(0,001, false, 'ShutdownPostEffects');
        }
    }
}

simulated function Tick(float DeltaTime)
{
    Tick(DeltaTime);
    if (bArcing)
    {
        Velocity.Z += WorldInfo.GetGravityZ() * DeltaTime;
    }
}

simulated function ProcessTouch(Actor Other, Vector HitLocation, Vector HitNormal)
{
    if (Other != Instigator)
    {
        if (SFXPawn_Player(Instigator) == None || SFXPawn_Henchman(Other) == None)
        {
            Explode(HitLocation, HitNormal);
            ShutDown();
        }
    }
}

function float GetDamage()
{
    local SFXWeapon Weapon;
    Weapon = SFXWeapon(Outer);
    if (Weapon == None)
    {
        Weapon = SFXWeapon(Owner);
    }
    if (Weapon != None)
    {
        return Weapon.GetFireModeBaseDamage();
    }
    return 0;
}

function float GetDamageRadius()
{
    local SFXWeapon Weapon;
    Weapon = SFXWeapon(Outer);
    if (Weapon == None)
    {
        Weapon = SFXWeapon(Owner);
    }
    return Weapon.GetDamageType(0).Default.DamageRadius;
}

simulated function SpawnImpactEffect(Vector HitLocation, Vector HitNormal);

defaultproperties
{
    ProjEffectsTrailWaitTime=10
    CylinderComponent=None
    Components=None
    // child object CollisionCylinder of type CylinderComponent
}
