﻿class SFXSFHandler_Save extends BioSFHandler
    config(UI)
    transient
    native;

const SaveOption_Initialize = 1;
const SaveOption_SaveGame = 2;
const SaveOption_DeleteGame = 4;
const SaveOption_ChangeStorageDevice = 10;
const SaveOption_NewSaveGame = 11;
enum ESaveGuiRecordType
{
    SaveRecordType_Default,
    SaveRecordType_QuickSave,
    SaveRecordType_AutoSave,
    SaveRecordType_Chapter,
    SaveRecordType_Legacy,
};

struct native SaveGUIAreaInfo
{
    var() String AreaName;

// AreaStrRef: StringRefProperty[44]
    var() String ImageName;

}

var() String CareerNameOverride;

// CancelDeleteGameText: StringRefProperty[44]
// ConfirmDeleteGameText: StringRefProperty[44]
// DeleteGameText: StringRefProperty[44]
// CancelOverwriteSaveGameText: StringRefProperty[44]
// ConfirmOverwriteSaveGameText: StringRefProperty[44]
// OverwriteSaveGameText: StringRefProperty[44]
// LastPlayedTimeText: StringRefProperty[44]
// LastPlayedText: StringRefProperty[44]
// TimePlayedText: StringRefProperty[44]
// NewGameText: StringRefProperty[44]
// SaveDisplayText: StringRefProperty[44]
// SaveGameText: StringRefProperty[44]
// QuickSaveText: StringRefProperty[44]
// ChapterSaveText: StringRefProperty[44]
// AutoSaveText: StringRefProperty[44]
// DefaultAreaNameText: StringRefProperty[44]
var() ESaveGuiMode GuiMode;

var() transient bool bWaitingOnMsgBox;

var() transient String ImagePackageName;

var() transient int MaxSaves;

var() array<SaveGUIRecord> SaveList;

var() String DefaultImageName;

var() Texture2D DefaultAreaImage;

var() config array<SaveGUIAreaInfo> AreaData;

struct native SaveGUIRecord
{
    var() SFXSaveGame SaveGame;

    var() String FriendlyName;

    var() Texture2D AreaImage;

    var() String ImagePath;

    var() int SaveIndex;

    var() ESaveGuiRecordType Type;

}

enum ESaveGuiMode
{
    SaveGuiMode_BrowserWheel,
    SaveGuiMode_MainMenu,
    SaveGuiMode_GameOver,
};

native function InitializeSaveList();

function Callback_ConfirmOverwrite(bool bAPressed, int Context)
{
    local MassEffectGuiManager GuiMan;
    local BioPlayerController PC;
    if (bAPressed)
    {
        GuiMan = MassEffectGuiManager(oPanel.oParentManager);
        if (GuiMan != None)
        {
            PC = BioPlayerController(GuiMan.Player);
            if (PC != None)
            {
                if (PC.OverwriteGame(SaveList[Context].SaveIndex, GetNewSaveIdx()))
                {
                    PlayGuiSound('ReturnToBrowser');
                    GuiMan.ReturnToBrowserWheel(oPanel, true);
                }
            }
        }
    }
    bWaitingOnMsgBox = false;
}

function Callback_ConfirmDelete(bool bAPressed, int Context)
{
    local BioPlayerController PC;
    if (bAPressed)
    {
        PC = BioPlayerController(oPanel.oParentManager.GetPlayerController());
        if (PC != None)
        {
            if (SaveList[Context].Type == 2)
            {
                PC.DeleteGame(true, 0, CareerNameOverride);
            }
            else
            {
                PC.DeleteGame(false, SaveList[Context].SaveIndex, CareerNameOverride);
            }
            PC.SetTimer(0,1, false, 'ResetGui', self);
        }
    }
    bWaitingOnMsgBox = false;
}

function ResetGui()
{
    InitializeSaveList();
}

function Callback_DeviceSelectionComplete(bool bWasSuccessful)
{
    local BioPlayerController PC;
    if (bWasSuccessful)
    {
        InitializeSaveList();
    }
    PC = BioPlayerController(MassEffectGuiManager(oPanel.oParentManager).Player);
    if (PC != None)
    {
        PlayerIntEx = PC.OnlineSub.PlayerInterfaceEx;
        PlayerIntEx.ClearDeviceSelectionDoneDelegate(LocalPlayer(PC.Player).ControllerId, Callback_DeviceSelectionComplete);
    }
}

function HandleEvent(byte Command, const out array<String> lstArguments)
{
    local MassEffectGuiManager GuiMan;
    local BioPlayerController PC;
    local array<ASParams> args;
    local BioSFHandler_MessageBox MessageBox;
    local BioMessageBoxOptionalParams Params;
    local int SaveIdx;
    if (bWaitingOnMsgBox)
    {
        return;
    }
    GuiMan = MassEffectGuiManager(oPanel.oParentManager);
    if (GuiMan == None)
    {
        return;
    }
    PC = BioPlayerController(GuiMan.Player);
    if (PC == None)
    {
        return;
    }
    switch (Command)
    {
        case 1:
            args.Length = 6;
            args[0].nVar = 1;
            args[0].Type = 0;
            args[1].bVar = false;
            args[1].Type = 3;
            args[2].bVar = false;
            args[2].Type = 3;
            args[3].nVar = 0;
            args[3].Type = 0;
            args[4].bVar = false;
            args[4].Type = 3;
            args[5].nVar = ScreenLayout;
            args[5].Type = 0;
            oPanel.InvokeMethodArgs("loadSave.SetLoadSave", args);
            if (Class'Engine'.IsShip() == false)
            {
                MaxSaves = 1000;
            }
            InitializeSaveList();
            break;
        case 2:
            SaveIdx = lstArguments[0];
            if (SaveIdx >= 0 && SaveIdx < SaveList.Length)
            {
                MessageBox = GuiMan.CreateMessageBox();
                MessageBox.SetInputDelegate(Callback_ConfirmOverwrite, SaveIdx);
                Params.srAText = ConfirmOverwriteSaveGameText;
                Params.srBText = CancelOverwriteSaveGameText;
                Params.bNoFade = true;
                MessageBox.DisplayMessageBox(OverwriteSaveGameText, Params);
                bWaitingOnMsgBox = true;
            }
            break;
        case 11:
            if (PC.SaveGame(GetNewSaveIdx()))
            {
                PlayGuiSound('ReturnToBrowser');
                GuiMan.ReturnToBrowserWheel(oPanel, true);
            }
            break;
        case 4:
            SaveIdx = lstArguments[0];
            if (SaveIdx >= 0 && SaveIdx < SaveList.Length)
            {
                MessageBox = GuiMan.CreateMessageBox();
                MessageBox.SetInputDelegate(Callback_ConfirmDelete, SaveIdx);
                Params.srAText = ConfirmDeleteGameText;
                Params.srBText = CancelDeleteGameText;
                Params.bNoFade = true;
                MessageBox.DisplayMessageBox(DeleteGameText, Params);
                bWaitingOnMsgBox = true;
            }
            break;
    }
}

final event bool GetPlayerData(out String FirstName, out int Level)
{
    local MassEffectGuiManager GuiMan;
    local SFXPawn_Player Player;
    GuiMan = MassEffectGuiManager(oPanel.oParentManager);
    if (GuiMan != None)
    {
        Player = SFXPawn_Player(GuiMan.Player.Pawn);
        if (Player != None)
        {
            FirstName = Player.FirstName;
            LastName = Player.LastName;
            ClassName = Player.PlayerClass.srClassName;
            Level = Player.CharacterLevel;
            return true;
        }
    }
    return false;
}

final function int GetNewSaveIdx()
{
    local int idx;
    local int NewIdx;
    NewIdx = -1;
    if (SaveList.Length == 0)
    {
        return 1;
    }
    idx = 0;
    while (idx < SaveList.Length)
    {
        if (SaveList[idx].SaveIndex > NewIdx)
        {
            NewIdx = SaveList[idx].SaveIndex;
        }
        idx++;
    }
    NewIdx++;
    return NewIdx;
}

defaultproperties
{
    AreaData=System.Byte[]
    DefaultImageName="Images_UNC93_I1"
    MaxSaves=30
    ImagePackageName="GUI_SF_SaveLoad_Images"
    DefaultAreaNameText=System.Byte[]
    AutoSaveText=System.Byte[]
    ChapterSaveText=System.Byte[]
    QuickSaveText=System.Byte[]
    SaveGameText=System.Byte[]
    SaveDisplayText=System.Byte[]
    NewGameText=System.Byte[]
    TimePlayedText=System.Byte[]
    LastPlayedText=System.Byte[]
    LastPlayedTimeText=System.Byte[]
    OverwriteSaveGameText=System.Byte[]
    ConfirmOverwriteSaveGameText=System.Byte[]
    CancelOverwriteSaveGameText=System.Byte[]
    DeleteGameText=System.Byte[]
    ConfirmDeleteGameText=System.Byte[]
    CancelDeleteGameText=System.Byte[]
    nHandlerID=17
}
