﻿class SFXSaveGame extends Object
    native;

enum EEndGameState
{
    EGS_NotFinished,
    EGS_OutInABlazeOfGlory,
    EGS_LivedToFightAgain,
};

struct native ME1PlotTableRecord
{
    var() array<int> BoolVariables;

    var() array<int> IntVariables;

    var() array<float> FloatVariables;

}

struct native GalaxyMapSaveRecord
{
    var() array<PlanetSaveRecord> Planets;

    struct native PlanetSaveRecord
{
        var() int PlanetID;

        var() bool bVisited;

        var() array<Vector2D> Probes;

    }

}

struct native DependentDLCRecord
{
    var() int ModuleID;

    var() String Name;

}

var() int CurrentLoadingTip;

var() ME1PlotTableRecord ME1PlotRecord;

var() PlotTableSaveRecord PlotRecord;

var() GalaxyMapSaveRecord GalaxyMapRecord;

var() array<HenchmanSaveRecord> HenchmanRecords;

var() PlayerSaveRecord PlayerRecord;

var() array<Guid> PawnRecords;

var() array<DoorSaveRecord> DoorRecords;

var() array<KismetBoolSaveRecord> KismetRecords;

var() array<StreamingStateSaveRecord> StreamingRecords;

var() array<LevelSaveRecord> LevelRecords;

var() Rotator SaveRotation;

var() Vector SaveLocation;

var() int EndGameState;

var() EDifficultyOptions Difficulty;

var() String BaseLevelName;

var() int Disc;

var() SaveTimeStamp TimeStamp;

var() float SecondsPlayed;

var() transient String MaleMorphHead;

var() transient String FemaleMorphHead;

var() transient bool bIsValid;

var() transient String DebugName;

var() transient String Filename;

var() array<DependentDLCRecord> DependentDLC;

struct native LevelSaveRecord
{
    var() String LevelName;

    var() bool bShouldBeLoaded;

    var() bool bShouldBeVisible;

}

struct native StreamingStateSaveRecord
{
    var() String Name;

    var() bool bActive;

}

struct native KismetBoolSaveRecord
{
    var() Guid BoolGUID;

    var() bool bValue;

}

struct native DoorSaveRecord
{
    var() Guid DoorGUID;

    var() byte CurrentState;

    var() byte OldState;

}

struct native MorphHeadSaveRecord
{
    struct native TextureParameterSaveRecord
{
        var() String Name;

        var() String Texture;

    }

    struct native VectorParameterSaveRecord
{
        var() String Name;

        var() LinearColor Value;

    }

    struct native ScalarParameterSaveRecord
{
        var() String Name;

        var() float Value;

    }

    struct native OffsetBoneSaveRecord
{
        var() String Name;

        var() Vector Offset;

    }

    struct native MorphFeatureSaveRecord
{
        var() String Feature;

        var() float Offset;

    }

    var() String HairMesh;

    var() array<String> AccessoryMeshes;

    var() array<MorphFeatureSaveRecord> MorphFeatures;

    var() array<OffsetBoneSaveRecord> OffsetBones;

    var() array<Vector> LOD0Vertices;

    var() array<Vector> LOD1Vertices;

    var() array<Vector> LOD2Vertices;

    var() array<Vector> LOD3Vertices;

    var() array<ScalarParameterSaveRecord> ScalarParameters;

    var() array<VectorParameterSaveRecord> VectorParameters;

    var() array<TextureParameterSaveRecord> TextureParameters;

}

struct native AppearanceSaveRecord
{
    var() EPlayerAppearanceType CombatAppearance;

    var() int CasualID;

    var() int FullBodyID;

    var() int TorsoID;

    var() int ShoulderID;

    var() int ArmID;

    var() int LegID;

    var() int SpecID;

    var() int Tint1ID;

    var() int Tint2ID;

    var() int Tint3ID;

    var() int PatternID;

    var() int PatternColorID;

    var() int HelmetID;

    var() bool bHasMorphHead;

    var() MorphHeadSaveRecord MorphHead;

}

struct native WeaponSaveRecord
{
    var() String WeaponClassName;

    var() int AmmoUsedCount;

    var() int TotalAmmo;

    var() bool bLastWeapon;

    var() bool bCurrentWeapon;

    var() String AmmoPowerName;

}

struct native PowerSaveRecord
{
    var() String PowerName;

    var() float CurrentRank;

    var() String PowerClassName;

    var() int WheelDisplayIndex;

}

struct native HotKeySaveRecord
{
    var() String PawnName;

    var() int PowerID;

}

struct native PlayerSaveRecord
{
    var() bool bIsFemale;

    var() String PlayerClassName;

    var() transient class<SFXPawn_Player> PlayerClass;

    var() int srClassFriendlyName;

    var() int Level;

    var() float CurrentXP;

    var() String FirstName;

    var() int LastName;

    var() EOriginType Origin;

    var() ENotorietyType Notoriety;

    var() int TalentPoints;

    var() String MappedPower1;

    var() String MappedPower2;

    var() String MappedPower3;

    var() AppearanceSaveRecord Appearance;

    var() array<PowerSaveRecord> Powers;

    var() array<WeaponSaveRecord> Weapons;

    var String LoadoutWeapons[6];

    var() array<HotKeySaveRecord> HotKeys;

    var() int Credits;

    var() int Medigel;

    var() int Eezo;

    var() int Iridium;

    var() int Palladium;

    var() int Platinum;

    var() int Probes;

    var() float CurrentFuel;

    var() String FaceCode;

}

struct native HenchmanSaveRecord
{
    var() String Tag;

    var() array<PowerSaveRecord> Powers;

    var() int CharacterLevel;

    var() int TalentPoints;

    var String LoadoutWeapons[6];

    var String MappedPower;

}

struct native PlotTableSaveRecord
{
    struct native PlotCodex
{
        var() array<PlotCodexPage> Pages;

        struct native PlotCodexPage
{
            var() int Page;

            var() bool bNew;

        }

    }

    struct native PlotQuest
{
        var() int QuestCounter;

        var() bool bQuestUpdated;

        var() array<int> History;

    }

    var() array<int> BoolVariables;

    var() array<int> IntVariables;

    var() array<float> FloatVariables;

    var() int QuestProgressCounter;

    var() array<PlotQuest> QuestProgress;

    var() array<int> QuestIDs;

    var() array<PlotCodex> CodexEntries;

    var() array<int> CodexIDs;

}

struct native SaveTimeStamp
{
    var() int SecondsSinceMidnight;

    var() int Day;

    var() int Month;

    var() int Year;

}

const MaxSaveSize = 1048576;
final function bool GetPlayerRecord(out PlayerSaveRecord Record)
{
    Record = PlayerRecord;
    return true;
}

final event SavePlayer(int PlayerID)
{
    local WorldInfo WorldInfo;
    local SFXGame Game;
    local SFXEngine Engine;
    local SFXPawn_Player Player;
    local SFXInventoryManager InvMan;
    local BioPlayerInput Input;
    local int idx;
    local BioSaveGame SaveGame;
    WorldInfo = Class'Engine'.GetCurrentWorldInfo();
    if (WorldInfo != None)
    {
        Engine = SFXEngine(Outer);
        Game = SFXGame(WorldInfo.Game);
        Player = Game != None ? Game.GetPlayer(PlayerID) : None;
        if (Player != None)
        {
            bIsValid = true;
            if (Player.bIsCrouched)
            {
                SaveLocation.Z += Class'SFXPawn_Player'.Default.CylinderComponent.CollisionHeight - Player.CylinderComponent.CollisionHeight;
            }
            SaveRotation = Player.Rotation;
            PlayerRecord.bIsFemale = Player.bIsFemale;
            PlayerRecord.PlayerClassName = PathName(Player.Class);
            PlayerRecord.PlayerClass = Player.Class;
            PlayerRecord.srClassFriendlyName = Player.PlayerClass.srClassName;
            PlayerRecord.Level = Player.CharacterLevel;
            PlayerRecord.CurrentXP = Player.TotalXP;
            PlayerRecord.FirstName = Player.FirstName;
            PlayerRecord.LastName = Player.LastName;
            PlayerRecord.Origin = Player.Origin;
            PlayerRecord.Notoriety = Player.Notoriety;
            PlayerRecord.TalentPoints = Player.TalentPoints;
            PlayerRecord.FaceCode = Player.FaceCode;
            idx = 0;
            while (idx < 6)
            {
                PlayerRecord.LoadoutWeapons[idx] = Engine.PlayerLoadoutWeapons[idx];
                idx++;
            }
            if (PlayerController(Player.Controller) != None)
            {
                Input = BioPlayerInput(PlayerController(Player.Controller).PlayerInput);
                if (Input != None)
                {
                    PlayerRecord.MappedPower1 = Input.m_nmMappedPower;
                    PlayerRecord.MappedPower2 = Input.m_nmMappedPower2;
                    PlayerRecord.MappedPower3 = Input.m_nmMappedPower3;
                }
            }
            SaveAppearance(Player, PlayerRecord.Appearance);
            SavePowers(Player, PlayerRecord.Powers);
            SaveWeapons(Player, PlayerRecord.Weapons);
            if (BioPlayerController(Player.Controller) != None)
            {
                SaveHotKeys(BioPlayerController(Player.Controller), PlayerRecord.HotKeys);
            }
            InvMan = SFXInventoryManager(Player.InvManager);
            if (InvMan != None)
            {
                PlayerRecord.Credits = InvMan.Credits;
                PlayerRecord.Medigel = InvMan.Medigel;
                PlayerRecord.Eezo = InvMan.Eezo;
                PlayerRecord.Iridium = InvMan.Iridium;
                PlayerRecord.Palladium = InvMan.Palladium;
                PlayerRecord.Platinum = InvMan.Platinum;
                PlayerRecord.Probes = InvMan.Probes;
                PlayerRecord.CurrentFuel = InvMan.CurrentFuel;
            }
            SaveGame = BioWorldInfo(Player.WorldInfo).CurrentGame;
            if (SaveGame != None && SaveGame.ImportedDataManager != None)
            {
                ME1PlotRecord.BoolVariables = SaveGame.ImportedDataManager.m_oImportedGlobalVariables.BoolVariables;
                ME1PlotRecord.IntVariables = SaveGame.ImportedDataManager.m_oImportedGlobalVariables.IntVariables;
                ME1PlotRecord.FloatVariables = SaveGame.ImportedDataManager.m_oImportedGlobalVariables.FloatVariables;
            }
        }
    }
}

final function EnsureHenchmanRecordExists(SFXPawn_Henchman Hench)
{
    local int HenchIdx;
    local HenchmanSaveRecord SaveInfo;
    local SFXEngine Engine;
    Engine = SFXEngine(Outer);
    if (Engine == None)
    {
        return;
    }
    HenchIdx = Engine.HenchmanRecords.Find('Tag', Hench.Tag);
    if (HenchIdx == -1)
    {
        SaveInfo.Tag = Hench.Tag;
        SavePowers(Hench, SaveInfo.Powers);
        SaveInfo.TalentPoints = Hench.TalentPoints;
        SaveInfo.CharacterLevel = Hench.CharacterLevel;
        Class'SFXPawn_Henchman'.GetDefaultLoadout(Hench.Tag, SaveInfo.LoadoutWeapons);
        SaveInfo.MappedPower = Hench.m_nmMappedPower;
        Engine.HenchmanRecords.AddItem(SaveInfo);
    }
}

final event SaveHenchmen(int PlayerID)
{
    local WorldInfo WorldInfo;
    local SFXGame Game;
    local SFXPawn_Player Player;
    local SFXEngine Engine;
    local BioBaseSquad Squad;
    local int idx;
    local int HenchIdx;
    local SFXPawn_Henchman Hench;
    local HenchmanSaveRecord SaveInfo;
    local array<PowerSaveRecord> Powers;
    Engine = SFXEngine(Outer);
    if (Engine == None)
    {
        return;
    }
    WorldInfo = Class'Engine'.GetCurrentWorldInfo();
    if (WorldInfo != None)
    {
        Game = SFXGame(WorldInfo.Game);
        Player = Game != None ? Game.GetPlayer(PlayerID) : None;
        if (Player != None)
        {
            HenchmanRecords = Engine.HenchmanRecords;
            Squad = Player.Squad;
            if (Squad != None)
            {
                idx = 0;
                while (idx < Squad.GetSquadSize())
                {
                    Hench = SFXPawn_Henchman(Squad.GetMember(idx));
                    if (Hench != None)
                    {
                        HenchIdx = HenchmanRecords.Find('Tag', Hench.Tag);
                        if (HenchIdx != -1)
                        {
                            Powers = HenchmanRecords[HenchIdx].Powers;
                            SavePowers(Hench, Powers);
                            HenchmanRecords[HenchIdx].Powers = Powers;
                            HenchmanRecords[HenchIdx].TalentPoints = Hench.TalentPoints;
                            HenchmanRecords[HenchIdx].CharacterLevel = Hench.CharacterLevel;
                            HenchmanRecords[HenchIdx].MappedPower = Hench.m_nmMappedPower;
                        }
                        else
                        {
                            SaveInfo.Tag = Hench.Tag;
                            SavePowers(Hench, SaveInfo.Powers);
                            SaveInfo.TalentPoints = Hench.TalentPoints;
                            SaveInfo.CharacterLevel = Hench.CharacterLevel;
                            Class'SFXPawn_Henchman'.GetDefaultLoadout(Hench.Tag, SaveInfo.LoadoutWeapons);
                            SaveInfo.MappedPower = Hench.m_nmMappedPower;
                            HenchmanRecords.AddItem(SaveInfo);
                        }
                    }
                    idx++;
                }
            }
            Engine.HenchmanRecords = HenchmanRecords;
        }
    }
}

final function SaveAppearance(SFXPawn_Player Player, out AppearanceSaveRecord Record)
{
    Record.CombatAppearance = Player.CombatAppearance;
    Record.CasualID = Player.CasualID;
    Record.FullBodyID = Player.FullBodyID;
    Record.TorsoID = Player.TorsoID;
    Record.ShoulderID = Player.ShoulderID;
    Record.ArmID = Player.ArmID;
    Record.LegID = Player.LegID;
    Record.SpecID = Player.SpecID;
    Record.Tint1ID = Player.Tint1ID;
    Record.Tint2ID = Player.Tint2ID;
    Record.Tint3ID = 0;
    Record.PatternID = Player.PatternID;
    Record.PatternColorID = Player.PatternColorID;
    Record.HelmetID = Player.HelmetID;
    Record.bHasMorphHead = false;
    if (SaveMorphHead(Player.MorphHead, Record.MorphHead))
    {
        Record.bHasMorphHead = true;
    }
}

final function bool SaveMorphHead(BioMorphFace Morph, out MorphHeadSaveRecord Record)
{
    local int idx;
    local BioMaterialOverride MatOverride;
    if (Morph != None)
    {
        Record.HairMesh = PathName(Morph.m_oHairMesh);
        Record.AccessoryMeshes.Length = Morph.m_oOtherMeshes.Length;
        idx = 0;
        while (idx < Morph.m_oOtherMeshes.Length)
        {
            Record.AccessoryMeshes[idx] = PathName(Morph.m_oOtherMeshes[idx]);
            idx++;
        }
        Record.MorphFeatures.Length = Morph.m_aMorphFeatures.Length;
        idx = 0;
        while (idx < Morph.m_aMorphFeatures.Length)
        {
            Record.MorphFeatures[idx].Feature = Morph.m_aMorphFeatures[idx].sFeatureName;
            Record.MorphFeatures[idx].Offset = Morph.m_aMorphFeatures[idx].Offset;
            idx++;
        }
        Record.OffsetBones.Length = Morph.m_aFinalSkeleton.Length;
        idx = 0;
        while (idx < Morph.m_aFinalSkeleton.Length)
        {
            Record.OffsetBones[idx].Name = Morph.m_aFinalSkeleton[idx].nName;
            Record.OffsetBones[idx].Offset = Morph.m_aFinalSkeleton[idx].vPos;
            idx++;
        }
        Record.LOD0Vertices.Length = Morph.GetNumVerts(0);
        idx = 0;
        while (idx < Morph.GetNumVerts(0))
        {
            Record.LOD0Vertices[idx] = Morph.GetPosition(0, idx);
            idx++;
        }
        MatOverride = Morph.m_oMaterialOverrides;
        if (MatOverride != None)
        {
            Record.ScalarParameters.Length = MatOverride.m_aScalarOverrides.Length;
            idx = 0;
            while (idx < MatOverride.m_aScalarOverrides.Length)
            {
                Record.ScalarParameters[idx].Name = MatOverride.m_aScalarOverrides[idx].nName;
                Record.ScalarParameters[idx].Value = MatOverride.m_aScalarOverrides[idx].sValue;
                idx++;
            }
            Record.VectorParameters.Length = MatOverride.m_aColorOverrides.Length;
            idx = 0;
            while (idx < MatOverride.m_aColorOverrides.Length)
            {
                Record.VectorParameters[idx].Name = MatOverride.m_aColorOverrides[idx].nName;
                Record.VectorParameters[idx].Value = MatOverride.m_aColorOverrides[idx].cValue;
                idx++;
            }
            Record.TextureParameters.Length = MatOverride.m_aTextureOverrides.Length;
            idx = 0;
            while (idx < MatOverride.m_aTextureOverrides.Length)
            {
                Record.TextureParameters[idx].Name = MatOverride.m_aTextureOverrides[idx].nName;
                Record.TextureParameters[idx].Texture = PathName(MatOverride.m_aTextureOverrides[idx].m_pTexture);
                idx++;
            }
        }
        return true;
    }
    return false;
}

final function SavePowers(SFXPawn Pawn, out array<PowerSaveRecord> Records)
{
    local array<PowerSaveInfo> Powers;
    local int idx;
    local SFXPowerManager PowerMan;
    PowerMan = Pawn.PowerManager;
    if (PowerMan != None)
    {
        PowerMan.SavePowers(Powers);
        Records.Length = Powers.Length;
        idx = 0;
        while (idx < Powers.Length)
        {
            Records[idx].PowerName = Powers[idx].PowerName;
            Records[idx].CurrentRank = Powers[idx].CurrentRank;
            Records[idx].PowerClassName = Powers[idx].PowerClassName;
            Records[idx].WheelDisplayIndex = Powers[idx].WheelDisplayIndex;
            idx++;
        }
    }
}

final function SaveWeapons(SFXPawn_Player Player, out array<WeaponSaveRecord> Records)
{
    local SFXInventoryManager InvManager;
    local SFXWeapon Weapon;
    local WeaponSaveRecord SaveInfo;
    InvManager = SFXInventoryManager(Player.InvManager);
    if (InvManager != None)
    {
        foreach InvManager.InventoryActors(Class'SFXWeapon', Weapon)
        {
            SaveInfo.WeaponClassName = Weapon.Class;
            SaveInfo.AmmoUsedCount = Weapon.AmmoUsedCount;
            SaveInfo.TotalAmmo = Weapon.GetCurrentSpareAmmo();
            SaveInfo.bLastWeapon = false;
            if (Player.WeaponOnDeck == Weapon)
            {
                SaveInfo.bLastWeapon = true;
            }
            SaveInfo.bCurrentWeapon = false;
            if (InvManager.CurrentWeaponSelection == Weapon.Class)
            {
                SaveInfo.bCurrentWeapon = true;
            }
            SaveInfo.AmmoPowerName = Weapon.AmmoPowerName;
            Records.AddItem(SaveInfo);
        }
    }
}

final function SaveHotKeys(BioPlayerController PC, out array<HotKeySaveRecord> Records)
{
    local int idx;
    if (!Class'WorldInfo'.IsConsoleBuild())
    {
        Records.Length = PC.8;
        idx = 0;
        while (idx < PC.8)
        {
            Records[idx].PawnName = PC.m_aHotKeyDefines[idx].nmPawn;
            Records[idx].PowerID = PC.m_aHotKeyDefines[idx].nPowerID;
            idx++;
        }
    }
}

final function GetSpawnData(out int IsFemale, out class<SFXPawn_Player> CharacterClass, out String FirstName, out EOriginType Origin, out ENotorietyType Notoriety)
{
    IsFemale = PlayerRecord.bIsFemale;
    CharacterClass = PlayerRecord.PlayerClass;
    FirstName = PlayerRecord.FirstName;
    Origin = PlayerRecord.Origin;
    Notoriety = PlayerRecord.Notoriety;
}

final event LoadPlayer(int PlayerID)
{
    local WorldInfo WorldInfo;
    local SFXGame Game;
    local SFXPawn_Player Player;
    local SFXInventoryManager InvMan;
    local SFXEngine Engine;
    local BioPlayerInput Input;
    local BioSaveGame SaveGame;
    WorldInfo = Class'Engine'.GetCurrentWorldInfo();
    if (WorldInfo != None)
    {
        Engine = SFXEngine(Outer);
        Game = SFXGame(WorldInfo.Game);
        Player = Game != None ? Game.GetPlayer(PlayerID) : None;
        if (Player != None)
        {
            Engine = SFXEngine(Outer);
            if (Engine != None && Engine.bPlayerLoadPosition)
            {
                Player.bCollideWorld = false;
                Player.SetLocation(SaveLocation + vect(0,0,2));
                Player.bCollideWorld = true;
                Player.SetRotation(SaveRotation);
                Player.Controller.SetRotation(SaveRotation);
                Engine.bPlayerLoadPosition = false;
            }
            Player.CharacterLevel = PlayerRecord.Level;
            Player.TotalXP = PlayerRecord.CurrentXP;
            Player.FirstName = PlayerRecord.FirstName;
            Player.Origin = PlayerRecord.Origin;
            Player.Notoriety = PlayerRecord.Notoriety;
            Player.TalentPoints = PlayerRecord.TalentPoints;
            Player.FaceCode = PlayerRecord.FaceCode;
            if (PlayerController(Player.Controller) != None)
            {
                Input = BioPlayerInput(PlayerController(Player.Controller).PlayerInput);
                if (Input != None)
                {
                    Input.m_nmMappedPower = PlayerRecord.MappedPower1;
                    Input.m_nmMappedPower2 = PlayerRecord.MappedPower2;
                    Input.m_nmMappedPower3 = PlayerRecord.MappedPower3;
                }
            }
            LoadAppearance(Player, PlayerRecord.Appearance);
            LoadPowers(Player, PlayerRecord.Powers);
            if (BioPlayerController(Player.Controller) != None)
            {
                LoadHotKeys(BioPlayerController(Player.Controller), PlayerRecord.HotKeys);
            }
            InvMan = SFXInventoryManager(Player.InvManager);
            if (InvMan != None)
            {
                LoadWeapons(Player, PlayerRecord.Weapons);
                InvMan.Credits = PlayerRecord.Credits;
                InvMan.Medigel = PlayerRecord.Medigel;
                InvMan.Eezo = PlayerRecord.Eezo;
                InvMan.Iridium = PlayerRecord.Iridium;
                InvMan.Palladium = PlayerRecord.Palladium;
                InvMan.Platinum = PlayerRecord.Platinum;
                InvMan.Probes = PlayerRecord.Probes;
                InvMan.CurrentFuel = PlayerRecord.CurrentFuel;
            }
            Player.UpdateAppearance();
            SaveGame = BioWorldInfo(Player.WorldInfo).CurrentGame;
            if (SaveGame != None && SaveGame.ImportedDataManager != None)
            {
                SaveGame.ImportedDataManager.m_oImportedGlobalVariables.BoolVariables = ME1PlotRecord.BoolVariables;
                SaveGame.ImportedDataManager.m_oImportedGlobalVariables.IntVariables = ME1PlotRecord.IntVariables;
                SaveGame.ImportedDataManager.m_oImportedGlobalVariables.FloatVariables = ME1PlotRecord.FloatVariables;
                jump 1369;
            }
        }
    }
    return;
}

final event LoadHenchman(SFXPawn_Henchman Hench)
{
    local SFXEngine Engine;
    local int idx;
    local array<PowerSaveRecord> Powers;
    Engine = SFXEngine(Outer);
    if (Engine == None)
    {
        return;
    }
    idx = Engine.HenchmanRecords.Find('Tag', Hench.Tag);
    if (idx >= 0)
    {
        Powers = HenchmanRecords[idx].Powers;
        LoadPowers(Hench, Powers);
        Hench.TalentPoints = Engine.HenchmanRecords[idx].TalentPoints;
        Hench.CharacterLevel = Engine.HenchmanRecords[idx].CharacterLevel;
        Hench.CreateWeapons(Hench.Loadout);
        Hench.m_nmMappedPower = Engine.HenchmanRecords[idx].MappedPower;
    }
}

final function LoadAppearance(SFXPawn_Player Player, out AppearanceSaveRecord Record)
{
    Player.CombatAppearance = Record.CombatAppearance;
    Player.CasualID = Record.CasualID;
    Player.FullBodyID = Record.FullBodyID;
    Player.TorsoID = Record.TorsoID;
    Player.ShoulderID = Record.ShoulderID;
    Player.ArmID = Record.ArmID;
    Player.LegID = Record.LegID;
    Player.SpecID = Record.SpecID;
    Player.Tint1ID = Record.Tint1ID;
    Player.Tint2ID = Record.Tint2ID;
    Player.PatternID = Record.PatternID;
    Player.PatternColorID = Record.PatternColorID;
    Player.HelmetID = Record.HelmetID;
    if (Record.bHasMorphHead)
    {
        Player.MorphHead = LoadMorphHead();
    }
}

final function BioMorphFace LoadMorphHead()
{
    local BioMorphFace MorphHead;
    local MorphHeadSaveRecord Record;
    local String PackageName;
    local int idx;
    local MorphFeature Feature;
    local OffsetBonePos Offset;
    local ScalarParameter ScalarParam;
    local ColorParameter ColorParam;
    local TextureParameter TextureParam;
    local array<int> BuffersToRefresh;
    if (PlayerRecord.Appearance.bHasMorphHead)
    {
        Record = PlayerRecord.Appearance.MorphHead;
        PackageName = Left(PlayerRecord.bIsFemale ? FemaleMorphHead : MaleMorphHead, InStr(PlayerRecord.bIsFemale ? FemaleMorphHead : MaleMorphHead, ".", false));
        Class'SFXGame'.LoadPackage(PackageName);
        MorphHead = BioMorphFace(FindObject(PlayerRecord.bIsFemale ? FemaleMorphHead : MaleMorphHead, Class'BioMorphFace'));
        if (MorphHead != None)
        {
            if (PathName(MorphHead.m_oHairMesh) != Record.HairMesh)
            {
                MorphHead.m_oHairMesh = SkeletalMesh(DynamicLoadObject(Record.HairMesh, Class'SkeletalMesh'));
            }
            if (MorphHead.m_oOtherMeshes.Length != Record.AccessoryMeshes.Length)
            {
                MorphHead.m_oOtherMeshes.Length = Record.AccessoryMeshes.Length;
                idx = 0;
                while (idx < Record.AccessoryMeshes.Length)
                {
                    MorphHead.m_oOtherMeshes[idx] = SkeletalMesh(DynamicLoadObject(Record.AccessoryMeshes[idx], Class'SkeletalMesh'));
                    idx++;
                }
            }
            MorphHead.m_aMorphFeatures.Length = Record.MorphFeatures.Length;
            idx = 0;
            while (idx < Record.MorphFeatures.Length)
            {
                Feature.sFeatureName = Record.MorphFeatures[idx].Feature;
                Feature.Offset = Record.MorphFeatures[idx].Offset;
                MorphHead.m_aMorphFeatures[idx] = Feature;
                idx++;
            }
            MorphHead.m_aFinalSkeleton.Length = Record.OffsetBones.Length;
            idx = 0;
            while (idx < Record.OffsetBones.Length)
            {
                Offset.nName = Record.OffsetBones[idx].Name;
                Offset.vPos = Record.OffsetBones[idx].Offset;
                MorphHead.m_aFinalSkeleton[idx] = Offset;
                idx++;
            }
            idx = 0;
            while (idx < Record.LOD0Vertices.Length)
            {
                MorphHead.SetPosition(0, idx, Record.LOD0Vertices[idx]);
                idx++;
            }
            BuffersToRefresh.AddItem(0);
            MorphHead.RefreshBuffers(BuffersToRefresh);
            MorphHead.m_oMaterialOverrides.m_aScalarOverrides.Length = Record.ScalarParameters.Length;
            idx = 0;
            while (idx < Record.ScalarParameters.Length)
            {
                ScalarParam.nName = Record.ScalarParameters[idx].Name;
                ScalarParam.sValue = Record.ScalarParameters[idx].Value;
                MorphHead.m_oMaterialOverrides.m_aScalarOverrides[idx] = ScalarParam;
                idx++;
            }
            MorphHead.m_oMaterialOverrides.m_aColorOverrides.Length = Record.VectorParameters.Length;
            idx = 0;
            while (idx < Record.VectorParameters.Length)
            {
                ColorParam.nName = Record.VectorParameters[idx].Name;
                ColorParam.cValue = Record.VectorParameters[idx].Value;
                MorphHead.m_oMaterialOverrides.m_aColorOverrides[idx] = ColorParam;
                idx++;
            }
            MorphHead.m_oMaterialOverrides.m_aTextureOverrides.Length = Record.TextureParameters.Length;
            idx = 0;
            while (idx < Record.TextureParameters.Length)
            {
                TextureParam.nName = Record.TextureParameters[idx].Name;
                TextureParam.m_pTexture = Texture2D(DynamicLoadObject(Record.TextureParameters[idx].Texture, Class'Texture2D'));
                MorphHead.m_oMaterialOverrides.m_aTextureOverrides[idx] = TextureParam;
                idx++;
            }
            return MorphHead;
        }
    }
    return None;
}

final function LoadPowers(SFXPawn Pawn, out array<PowerSaveRecord> Records)
{
    local array<PowerSaveInfo> Powers;
    local int idx;
    local SFXPowerManager PowerMan;
    PowerMan = Pawn.PowerManager;
    if (PowerMan != None)
    {
        Powers.Length = Records.Length;
        idx = 0;
        while (idx < Records.Length)
        {
            Powers[idx].PowerName = Records[idx].PowerName;
            Powers[idx].CurrentRank = Records[idx].CurrentRank;
            Powers[idx].PowerClassName = Records[idx].PowerClassName;
            Powers[idx].WheelDisplayIndex = Records[idx].WheelDisplayIndex;
            idx++;
        }
        PowerMan.LoadPowers(Powers);
    }
}

final function LoadWeapons(SFXPawn_Player Player, out array<WeaponSaveRecord> Records)
{
    local SFXInventoryManager InvManager;
    local SFXWeapon Weapon;
    local int idx;
    local SFXPower Power;
    local BioPowerScript PowerScript;
    Player.CreateWeapons(Player.Loadout);
    InvManager = SFXInventoryManager(Player.InvManager);
    if (InvManager != None)
    {
        foreach InvManager.InventoryActors(Class'SFXWeapon', Weapon)
        {
            idx = Records.Find('WeaponClassName', Weapon.Class);
            if (idx != -1)
            {
                if (Records[idx].AmmoUsedCount <= Weapon.GetMagazineSize())
                {
                    Weapon.AmmoUsedCount = Records[idx].AmmoUsedCount;
                }
                else
                {
                    Weapon.AmmoUsedCount = Weapon.GetMagazineSize();
                }
                Weapon.CurrentSpareAmmo = Records[idx].TotalAmmo;
                if (Records[idx].bLastWeapon)
                {
                    Player.WeaponOnDeck = Weapon;
                }
                if (Records[idx].bCurrentWeapon)
                {
                    InvManager.CurrentWeaponSelection = Weapon.Class;
                }
                Weapon.AmmoPowerName = Records[idx].AmmoPowerName;
                if (Weapon.AmmoPowerName != "")
                {
                    Power = Player.PowerManager.GetPower(Weapon.AmmoPowerName);
                    if (Power != None)
                    {
                        PowerScript = Power.GetPowerScript();
                        if (PowerScript != None)
                        {
                            PowerScript.ReloadAmmoPower(Player, Weapon);
                        }
                    }
                }
            }
        }
    }
}

final function LoadHotKeys(BioPlayerController PC, out array<HotKeySaveRecord> Records)
{
    local int idx;
    if (!Class'WorldInfo'.IsConsoleBuild())
    {
        idx = 0;
        while (idx < PC.8 && idx < Records.Length)
        {
            PC.m_aHotKeyDefines[idx].nmPawn = Records[idx].PawnName;
            PC.m_aHotKeyDefines[idx].nPowerID = Records[idx].PowerID;
            idx++;
        }
    }
}

final function class<SFXPower> LoadPower(String PowerClassName)
{
    local class<SFXPower> PowerClass;
    PowerClass = Class<SFXPower>(FindObject(PowerClassName, Class'Class'));
    if (PowerClass == None)
    {
        if (Class'SFXEngine'.IsSeekFreeObjectSupported(PowerClassName))
        {
            PowerClass = Class<SFXPower>(Class'SFXEngine'.LoadSeekFreeObject(PowerClassName, Class'Class'));
        }
    }
    return PowerClass;
}

defaultproperties
{
    FemaleMorphHead="BIOG_MORPH_FACE.Player_Base_Female"
    MaleMorphHead="BIOG_MORPH_FACE.Player_Base_Male"
    Difficulty=DO_Level2
}
