﻿class SFXTracer extends SFXProjectilePooled
    hidecategories(Navigation);

enum ESFXTracerScaling
{
    eSFXTracerScaling_Idle,
    eSFXTracerScaling_Up,
    eSFXTracerScaling_Down,
};

var Vector DesiredEndPoint;

var float ScaleTimeCount;

var float ScaleTimeFactor;

var ESFXTracerScaling ScaleMode;

function InitTracer(Vector Start, Vector End)
{
    local Vector Direction;
    local float Lifetime;
    Direction = End - Start;
    Init(Normal(Direction));
    DesiredEndPoint = End;
    SetLocation(Start);
    SetRotation(Direction);
    Lifetime = FMax(VSize(Direction) / VSize(Velocity), 0,01);
    SetTimer(Lifetime, false, 'ShutDown');
    ScaleUpStart();
}

simulated function ScaleUpStart()
{
    local Vector NewScale;
    local BoxSphereBounds UnscaledBounds;
    local float MeshLength;
    local float MinSpeed;
    local float CurSpeed;
    if (Mesh != None)
    {
        NewScale = DesiredScale;
        NewScale.X = 0;
        Mesh.SetScale3D(NewScale);
    }
    ScaleTimeCount = 0;
    if (Mesh != None)
    {
        Mesh.GetUnscaledBounds(UnscaledBounds);
        MeshLength = UnscaledBounds.BoxExtent.X * 2 * DesiredScale.X;
        MinSpeed = MeshLength / MeshScaleTime * 2;
        CurSpeed = VSize(Velocity);
        ScaleTimeFactor = FMin(1, CurSpeed / MinSpeed);
    }
    else
    {
        ScaleTimeFactor = 1;
    }
    ScaleMode = 1;
    Enable('Tick');
}

simulated function ScaleUpFinishedNotify()
{
    if (Mesh != None)
    {
        Mesh.SetScale3D(DesiredScale);
    }
    ScaleTimeCount = 0;
    ScaleMode = 0;
    Disable('Tick');
}

simulated function ScaleDownStart()
{
    if (ScaleMode != 1)
    {
        ScaleTimeCount = 0;
    }
    else
    {
        ScaleTimeCount = FMax(MeshScaleTime - ScaleTimeCount, 0);
    }
    ScaleTimeFactor = 1;
    ScaleMode = 2;
    Enable('Tick');
}

simulated function ScaleDownFinishedNotify()
{
    Disable('Tick');
    SetTimer(ProjEffectsTrailWaitTime, false, 'Recycle');
    ProjEffectsTrail.SetActive(false);
    Mesh.SetHidden(true);
}

simulated function ShutDown()
{
    bShuttingDown = true;
    SetPhysics(7);
    SetLocation(DesiredEndPoint);
    ScaleDownStart();
}

simulated function Tick(float DeltaTime)
{
    local float Pct;
    local Vector NewScale;
    Tick(DeltaTime);
    Pct = ScaleTimeCount / MeshScaleTime;
    ScaleTimeCount = FMin(ScaleTimeCount + DeltaTime * ScaleTimeFactor, MeshScaleTime);
    if (Pct ~= 1)
    {
        switch (ScaleMode);
        case 1;
        ScaleUpFinishedNotify();
        jump 122;
        case 2;
        ScaleDownFinishedNotify();
        jump 122;
        default;
    }
    else
    {
        NewScale = DesiredScale;
        switch (ScaleMode)
        {
            case 1:
                NewScale.X = DesiredScale.X * Pct;
                break;
            case 2:
                NewScale.X = DesiredScale.X * 1 - Pct;
                break;
        }
        if (Mesh != None)
        {
            Mesh.SetScale3D(NewScale);
        }
    }
    return;
}

defaultproperties
{
    MeshScaleTime=0,05
    CylinderComponent=None
    Components=None
}
