﻿class SFXTreasureUseModule extends SFXSimpleUseModule
    hidecategories(Object)
    editinlinenew;

enum ETreasureType
{
    AMMO_TREASURE,
    MEDIGEL_TREASURE,
    CREDITS_TREASURE,
    EEZO_TREASURE,
    PALLADIUM_TREASURE,
    IRIDIUM_TREASURE,
    PLATINUM_TREASURE,
    PROBES_TREASURE,
};

var() ETreasureType TreasureType;

var() int ResourcePercent;

function OnUsed(Actor User)
{
    local bool bGaveResources;
    bGaveResources = AwardResource(TreasureType, BioWorldInfo(Owner.WorldInfo), ResourcePercent, false);
    if (!bGaveResources)
    {
        return;
    }
    ResourcePercent = 0;
    SetTargetable(false);
}

function PostBeginPlay()
{
    PostBeginPlay();
    if (ResourcePercent <= 0)
    {
        SetTargetable(false);
    }
}

function SetTargetable(bool bTargetable)
{
    local ActorComponent Comp;
    local StaticMeshComponent oStaticMesh;
    local LinearColor EmissiveColor;
    local int nIndex;
    local MaterialInstanceConstant M;
    local MaterialInstanceConstant Instance;
    SetTargetable(bTargetable);
    if (bTargetable == false)
    {
        ResourcePercent = 0;
        foreach Owner.AllOwnedComponents(Class'ActorComponent', Comp)
        {
            oStaticMesh = StaticMeshComponent(Comp);
            if (oStaticMesh != None)
            {
                nIndex = 0;
                while (nIndex < oStaticMesh.Materials.Length)
                {
                    M = MaterialInstanceConstant(oStaticMesh.Materials[nIndex]);
                    if (M != None)
                    {
                        M.GetVectorParameterValue('Emissive_color', EmissiveColor);
                        if (M.Outer != Owner && EmissiveColor.A > 1)
                        {
                            Instance = new(Owner,Class'MaterialInstanceConstant');
                            Instance.SetParent(M);
                            EmissiveColor.A = EmissiveColor.A / 10;
                            Instance.SetVectorParameterValue('Emissive_color', EmissiveColor);
                            oStaticMesh.SetMaterial(nIndex, Instance);
                        }
                    }
                    nIndex++;
                }
            }
        }
    }
}

static function class<BioBaseSaveObject> GetSaveObjectClass()
{
    return Class'SFXTreasureUseModuleSaveObject';
}

static function bool AwardResource(ETreasureType ttype, BioWorldInfo oWorldInfo, int nBudgetPercent, optional bool bAbsoluteAmount = false, optional bool bIsSalvage = false)
{
    local bool bResult;
    bResult = GiveCash(ttype, oWorldInfo, nBudgetPercent, bAbsoluteAmount, bIsSalvage);
    bResult = bResult || GiveAmmo(ttype, oWorldInfo, nBudgetPercent);
    bResult = bResult || GiveMediGel(ttype, oWorldInfo, nBudgetPercent, bAbsoluteAmount);
    return bResult;
}

static function bool GiveCash(ETreasureType ttype, BioWorldInfo oWorldInfo, int nBudgetPercent, bool bAbsoluteAmount, bool bIsSalvage)
{
    local SFXPlotTreasure oTreasure;
    local SResourceBudget stBudget;
    local int nAmount;
    local bool bResult;
    if (bAbsoluteAmount == false)
    {
        nBudgetPercent = FClamp(nBudgetPercent * 1, 0, 100);
    }
    oTreasure = oWorldInfo.m_oTreasure;
    stBudget = oTreasure.Budget(oTreasure.CurrentLevel());
    nAmount = nBudgetPercent;
    switch (ttype)
    {
        case 2:
            if (bAbsoluteAmount == false)
            {
                nAmount = nBudgetPercent * stBudget.nCredits * 0,01;
            }
            bResult = GiveInventoryTreasure(0, nAmount, oWorldInfo, bIsSalvage);
            break;
        case 3:
            if (bAbsoluteAmount == false)
            {
                nAmount = nBudgetPercent * stBudget.nEezo * 0,01;
            }
            bResult = GiveInventoryTreasure(4, nAmount, oWorldInfo);
            break;
        case 5:
            if (bAbsoluteAmount == false)
            {
                nAmount = nBudgetPercent * stBudget.nIridium * 0,01;
            }
            bResult = GiveInventoryTreasure(5, nAmount, oWorldInfo);
            break;
        case 4:
            nAmount = nBudgetPercent * stBudget.nPalladium * 0,01;
            bResult = GiveInventoryTreasure(6, nAmount, oWorldInfo);
            break;
        case 6:
            if (bAbsoluteAmount == false)
            {
                nAmount = nBudgetPercent * stBudget.nPlatinum * 0,01;
            }
            bResult = GiveInventoryTreasure(7, nAmount, oWorldInfo);
            break;
        case 7:
            bResult = GiveInventoryTreasure(8, nAmount, oWorldInfo);
            break;
        default:
            bResult = false;
    }
    return bResult;
}

static function bool GiveMediGel(ETreasureType ttype, BioWorldInfo oWorldInfo, int ResourceAmount, bool bAbsoluteAmount)
{
    local bool bGaveMediGel;
    local SFXGame Game;
    if (ttype != 1 || ResourceAmount <= 0)
    {
        return false;
    }
    if (bAbsoluteAmount == false)
    {
        ResourceAmount = ResourceAmount * 5 * 0,01;
        ResourceAmount = ResourceAmount <= 0 ? 1 : ResourceAmount;
    }
    if (oWorldInfo != None)
    {
        Game = SFXGame(oWorldInfo.Game);
        if (Game != None && Game.Difficulty == Class'SFXDifficulty_Level1')
        {
            ResourceAmount = ResourceAmount * 2;
        }
    }
    bGaveMediGel = GiveInventoryTreasure(1, ResourceAmount, oWorldInfo, false, false, true);
    if (!bGaveMediGel)
    {
        GiveInventoryTreasure(0, 100, oWorldInfo, true, true);
        return true;
    }
    return bGaveMediGel;
}

static function bool GiveInventoryTreasure(EInventoryResourceTypes InvType, int Amount, BioWorldInfo oWorldInfo, optional bool bIsSalvage, optional bool bIgnoreBudget = false, optional bool bExtraIfLow = false)
{
    local BioPlayerController oController;
    local SFXInventoryManager oInventory;
    local int nCurrentAmount;
    local int nGivenAmount;
    local bool bTickerResource;
    bTickerResource = InvType != 0 || bIsSalvage == false;
    bTickerResource = bTickerResource && Amount > 0;
    oController = oWorldInfo.GetLocalPlayerController();
    oInventory = SFXInventoryManager(oController.Pawn.InvManager);
    nCurrentAmount = oInventory.GetResource(InvType);
    if (bExtraIfLow && nCurrentAmount < Amount * 2)
    {
        Amount = Amount * 2 - nCurrentAmount;
    }
    oInventory.AdjustResource(InvType, Amount, bTickerResource, bIgnoreBudget);
    nGivenAmount = oInventory.GetResource(InvType) - nCurrentAmount;
    if (bIsSalvage && Amount > 0 && InvType == 0)
    {
        BioHintSystem(oController.HintSystem).AddNotification_SalvageRecovery(nGivenAmount);
        SFXPawn(oWorldInfo.GetLocalPlayerController().Pawn).PlayAnimatedReaction(77, oWorldInfo.GetLocalPlayerController());
    }
    return nGivenAmount > 0;
}

function ScaleAmmo(SFXHeavyWeapon HWeapon)
{
    local BioWorldInfo BWI;
    local SFXGame Game;
    if (ResourcePercent < 1 || HWeapon == None)
    {
        return;
    }
    BWI = BioWorldInfo(Owner.WorldInfo);
    if (BWI != None)
    {
        Game = SFXGame(BWI.Game);
        if (Game != None && Game.Difficulty == Class'SFXDifficulty_Level1')
        {
            ResourcePercent = ResourcePercent * 2;
        }
    }
    if (HWeapon.AmmoUsedCount >= HWeapon.MagSize.Value)
    {
        ResourcePercent += 10;
    }
    ResourcePercent = ResourcePercent * HWeapon.MagSize.Value / 100;
    ResourcePercent = FClamp(ResourcePercent * 1, 1, HWeapon.MagSize.Value);
}

static function bool GiveAmmo(ETreasureType ttype, BioWorldInfo oWorldInfo, int ResourceAmount)
{
    local SFXHeavyWeapon HWeapon;
    local SFXHeavyWeapon FoundWeapon;
    local SFXInventoryManager InvManager;
    local BioPlayerController Controller;
    local int nCurrentAmmo;
    local bool bGaveAmmo;
    if (ttype != 0 || ResourceAmount <= 0)
    {
        return false;
    }
    Controller = oWorldInfo.GetLocalPlayerController();
    InvManager = SFXInventoryManager(Controller.Pawn.InvManager);
    foreach InvManager.InventoryActors(Class'SFXHeavyWeapon', HWeapon)
    {
        if (HWeapon.AmmoUsedCount > 0)
        {
            FoundWeapon = HWeapon;
        }
    }
    HWeapon = FoundWeapon;
    Class'SFXDroppedAmmo'.GiveAmmo(InvManager, 1);
    if (HWeapon != None && ResourceAmount > 0)
    {
        ResourceAmount = HWeapon.MagSize.Value * ResourceAmount * 0,01;
        if (HWeapon.IsA('SFXHeavyWeapon_NukeLauncher'))
        {
            ResourceAmount = ResourceAmount * 0,65;
        }
        nCurrentAmmo = HWeapon.MagSize.Value - HWeapon.AmmoUsedCount;
        if (nCurrentAmmo < ResourceAmount)
        {
            ResourceAmount = ResourceAmount * 2 - nCurrentAmmo;
        }
        ResourceAmount = ResourceAmount <= 0 ? 1 : ResourceAmount;
        ResourceAmount = HWeapon.AmmoUsedCount < ResourceAmount ? HWeapon.AmmoUsedCount * 1 : ResourceAmount * 1;
        HWeapon.AddHeavyAmmo(ResourceAmount);
        BioHintSystem(Controller.HintSystem).AddNotification_HeavyWeaponAmmoRecovery(ResourceAmount);
        bGaveAmmo = true;
    }
    if (!bGaveAmmo)
    {
        GiveInventoryTreasure(0, 100, oWorldInfo, true, true);
        return true;
    }
    return bGaveAmmo;
}

event bool IsDefaultActionPossible()
{
    return m_srTargetTipText != 0 && ResourcePercent > 0;
}

defaultproperties
{
    TreasureType=MEDIGEL_TREASURE
}
