﻿class SFXVehicleHover extends SVehicle
    hidecategories(Navigation)
    config(Game)
    native
    placeable;

var transient Vector m_vPauseVelocity;

var config float RadarFOV;

var config float RadarRange;

var(Lighting) /* 00000008 */ LightEnvironmentComponent LightEnvironment;

var /* 00000008 */ ParticleSystemComponent RightAfterburnerPSC;

var /* 00000008 */ ParticleSystemComponent LeftAfterburnerPSC;

var ParticleSystem m_oAfterburnerVisual;

var /* 00000008 */ ParticleSystemComponent MiningPSC;

var ParticleSystem m_oMiningVFXTemplate;

var float MiningFlashDuration;

var float MiningFlashDelay;

var transient float MiningIndicatorColor;

var transient float TimeToFlashMiningIndicator;

var transient float TimeToNextMiningFlash;

var ForceFeedbackWaveform MiningFeedbackWaveForm;

var transient bool m_bIsPaused;

var transient bool bMiningFailed;

var transient float TimeToNextMiningImpulse;

var transient float TotalMiningTime;

var transient SFXMiningNode CurrentMiningNode;

var config float MaxPitchAngle;

var float HitDamage;

var transient float LastHitTime;

var config float SelfRepairRate;

var config float SelfRepairDelay;

var config Vector OffGroundForce;

var transient float TimeLeftToStartRegen;

var transient float ForcedRegenAmount;

var config float BurnOutPercentage;

var config float ThrustRegenerationDelay;

var config float VerticalThrustBurnRate;

var config float ForwardThrustBurnRate;

var() config float OnGroundJumpMultiplier;

var transient float PreviousCameraPitch;

var ForceFeedbackWaveform ThrustFeedback;

var() BioVFXTemplate m_oDamageVFXTemplate;

var BioVFXTemplate m_oDeathVFXTemplate;

var transient BioVisualEffect m_oDamageVisualEffect;

var transient float LastRightSuspension;

var transient float LastLeftSuspension;

var WwiseEventPairObject VehicleTurretMovementStartEventPair;

var WwiseEventPairObject VehicleMiningEventPair;

var WwiseEvent HitSomething;

var WwiseEvent RightSuspensionEvent;

var WwiseEvent LeftSuspensionEvent;

var WwiseEvent VehicleShutdownEvent;

var WwiseEvent VehicleStartupEvent;

var WwiseEvent VehicleMiningFailureEvent;

var WwiseEvent VehicleMiningSuccessEvent;

var WwiseEvent VehicleBottomOutEvent;

var WwiseEvent VehicleForwardBoostEndEvent;

var WwiseEvent VehicleVerticalBoostEndEvent;

var WwiseEventPairObject VehicleForwardBoostEventPair;

var WwiseEventPairObject VehicleVerticalBoostEventPair;

var WwiseEventPairObject VehicleMotorEventPair;

var /* 00000008 */ transient WwiseAudioComponent AudioComponent;

var() config float ThrustRegenerationFactor;

var() config float MaxThrustJuice;

var transient float CurrentThrustJuice;

var() config Vector BoostForce;

var() config Vector JumpForce;

var() class<BioFaction> FactionClass;

var() SFXLoadoutData Loadout;

var bool bWeaponFiring;

function AddDefaultInventory()
{
    local SFXLoadoutData ChkLoadout;
    local SFXShield_Base Shields;
    local class<SFXWeapon> WeaponClass;
    local ShieldLoadout ShieldLoadout;
    AudioComponent = Class'WwiseAudioComponent'.CreateComponentFromScript(self);
    if (AudioComponent != None)
    {
        AudioComponent.Play(VehicleMotorEventPair);
    }
    ChkLoadout = Loadout;
    if (ChkLoadout == None)
    {
        return;
    }
    foreach ChkLoadout.Weapons(WeaponClass)
    {
        Weapon = SFXWeapon(CreateInventory(WeaponClass));
        Weapon.AttachWeaponTo(Mesh, 'Turret_Muzzle');
    }
    foreach ChkLoadout.ShieldLoadouts(ShieldLoadout)
    {
        Shields = SFXShield_Base(CreateInventory(ShieldLoadout.Shields));
        if (Shields != None)
        {
            Shields.ShieldScale = ChkLoadout.ShieldScale;
            Shields.ShieldOffset = ChkLoadout.ShieldOffset;
        }
    }
    CurrentThrustJuice = MaxThrustJuice;
}

function bool Died(Controller Killer, class<DamageType> DamageType, Vector HitLocation)
{
    StopFire(0);
    if (AudioComponent != None)
    {
        AudioComponent.Stop(VehicleMotorEventPair);
    }
    m_oDamageVisualEffect = Class'BioVisualEffect'.CreateCrustEffect(m_oDeathVFXTemplate, self, 0, false);
    if (m_oDamageVisualEffect != None)
    {
        m_oDamageVisualEffect.bPauseOnDeath = true;
        m_oDamageVisualEffect.SetState(0);
    }
    SetHidden(true);
    SetPhysics(0);
    return BioPawn(Driver).Squad.Died(self, Killer);
}

final simulated function bool IsPendingFire(byte InFiringMode)
{
    return InvManager != None && InvManager.PendingFire[InFiringMode] == 1;
}

event TakeDamage(float Damage, Controller EventInstigator, Vector HitLocation, Vector Momentum, class<DamageType> DamageType, optional TraceHitInfo HitInfo, optional Actor DamageCauser)
{
    local SFXModule_Damage DmgModule;
    TakeDamage(Damage, EventInstigator, HitLocation, Momentum, DamageType, HitInfo);
    DmgModule = GetModule(Class'SFXModule_Damage');
    if (AudioComponent != None && DmgModule != None)
    {
        AudioComponent.SetWwiseRTPC("Health", DmgModule.CurrentHealth);
    }
    LastHitTime = WorldInfo.TimeSeconds;
}

function Tick(float DeltaTime)
{
    local BioPlayerController BPC;
    local Vector ForceVector;
    local SkelControlLookAt oSkelControl;
    local Vector CamLoc;
    local Rotator CamRot;
    local SFXModule_Damage DmgModule;
    local float fHealthPercent;
    local MaterialInstanceConstant DamageMaterial;
    local SFXMiningNode Node;
    local Rotator RandomRotation;
    local Vector RandomImpulse;
    local int idx;
    local SFXSeqEvt_VehicleMiningSuccess SuccessEvent;
    local SFXSeqEvt_VehicleMiningFailure FailureEvent;
    local float fBottomOutPercentage;
    if (m_bIsPaused)
    {
        return;
    }
    fBottomOutPercentage = 0,85;
    bWeaponFiring = IsPendingFire(0);
    if (AudioComponent != None)
    {
        AudioComponent.SetWwiseRTPC("Velocity", VSize(Velocity));
        if (Abs(LastLeftSuspension - Wheels[0].SuspensionPosition) > 3 && !AudioComponent.IsPlaying(LeftSuspensionEvent))
        {
            AudioComponent.Play(LeftSuspensionEvent);
        }
        LastLeftSuspension = Wheels[0].SuspensionPosition;
        if (Abs(LastRightSuspension - Wheels[3].SuspensionPosition) > 3 && !AudioComponent.IsPlaying(RightSuspensionEvent))
        {
            AudioComponent.Play(RightSuspensionEvent);
        }
        LastRightSuspension = Wheels[3].SuspensionPosition;
        if (Wheels[0].SuspensionPosition > Wheels[0].SuspensionTravel * fBottomOutPercentage || Wheels[3].SuspensionPosition > Wheels[3].SuspensionTravel * fBottomOutPercentage && !AudioComponent.IsPlaying(VehicleBottomOutEvent))
        {
            AudioComponent.Play(VehicleBottomOutEvent);
        }
    }
    if (Controller == None)
    {
        return;
    }
    BPC = BioPlayerController(Controller);
    if (BPC != None)
    {
        if (BPC.bBoost == 1 && CurrentThrustJuice > 0 && ForcedRegenAmount <= 0 && Rotation.Pitch < MaxPitchAngle)
        {
            CurrentThrustJuice -= DeltaTime * ForwardThrustBurnRate;
            ForceVector = QuatRotateVector(QuatFromRotator(Rotation), BoostForce);
            Mesh.AddForce(ForceVector);
            BPC.ClientPlayForceFeedbackWaveform(ThrustFeedback);
            if (AudioComponent != None && !AudioComponent.IsPlaying(VehicleForwardBoostEventPair.Play))
            {
                AudioComponent.Play(VehicleForwardBoostEventPair);
            }
            TimeLeftToStartRegen = ThrustRegenerationDelay;
            if (m_oAfterburnerVisual != None)
            {
                if (LeftAfterburnerPSC != None && !LeftAfterburnerPSC.bIsActive)
                {
                    LeftAfterburnerPSC.SetTemplate(m_oAfterburnerVisual);
                    LeftAfterburnerPSC.SetAbsolute(false, false, false);
                    LeftAfterburnerPSC.SetLODLevel(WorldInfo.bDropDetail ? 1 : 0);
                    LeftAfterburnerPSC.SetTranslation(vect(-164,4395,-57,16,5));
                    LeftAfterburnerPSC.SetRotation(rot(32768,0,0));
                    LeftAfterburnerPSC.ActivateSystem();
                }
                if (RightAfterburnerPSC != None && !RightAfterburnerPSC.bIsActive)
                {
                    RightAfterburnerPSC.SetTemplate(m_oAfterburnerVisual);
                    RightAfterburnerPSC.SetAbsolute(false, false, false);
                    RightAfterburnerPSC.SetLODLevel(WorldInfo.bDropDetail ? 1 : 0);
                    RightAfterburnerPSC.SetTranslation(vect(-164,4395,57,16,5));
                    RightAfterburnerPSC.SetRotation(rot(32768,0,0));
                    RightAfterburnerPSC.ActivateSystem();
                }
            }
        }
        else
        {
            if (AudioComponent != None && AudioComponent.IsPlaying(VehicleForwardBoostEventPair.Play))
            {
                AudioComponent.Stop(VehicleForwardBoostEventPair);
                AudioComponent.Play(VehicleForwardBoostEndEvent);
            }
            if (LeftAfterburnerPSC != None)
            {
                LeftAfterburnerPSC.DeactivateSystem();
            }
            if (RightAfterburnerPSC != None)
            {
                RightAfterburnerPSC.DeactivateSystem();
            }
        }
        if (BPC.bJump == 1 && CurrentThrustJuice > 0 && ForcedRegenAmount <= 0)
        {
            CurrentThrustJuice -= DeltaTime * VerticalThrustBurnRate;
            ForceVector = QuatRotateVector(QuatFromRotator(Rotation), JumpForce);
            if (bVehicleOnGround == true)
            {
                Mesh.AddForce(ForceVector + OffGroundForce * OnGroundJumpMultiplier);
            }
            else
            {
                Mesh.AddForce(ForceVector);
            }
            if (AudioComponent != None && !AudioComponent.IsPlaying(VehicleVerticalBoostEventPair.Play))
            {
                AudioComponent.Play(VehicleVerticalBoostEventPair);
            }
            TimeLeftToStartRegen = ThrustRegenerationDelay;
        }
        else
        {
            if (AudioComponent != None && AudioComponent.IsPlaying(VehicleVerticalBoostEventPair.Play))
            {
                AudioComponent.Stop(VehicleVerticalBoostEventPair);
                AudioComponent.Play(VehicleVerticalBoostEndEvent);
            }
        }
        if (bVehicleOnGround == true)
        {
            TimeLeftToStartRegen = 0;
        }
        if (BPC.bJump == 0 && BPC.bBoost == 0 || ForcedRegenAmount > 0 && TimeLeftToStartRegen <= 0)
        {
            if (ForcedRegenAmount > 0)
            {
                ForcedRegenAmount -= DeltaTime * ThrustRegenerationFactor;
            }
            if (CurrentThrustJuice < MaxThrustJuice)
            {
                CurrentThrustJuice = CurrentThrustJuice + DeltaTime * ThrustRegenerationFactor;
                if (CurrentThrustJuice > MaxThrustJuice)
                {
                    CurrentThrustJuice = MaxThrustJuice;
                }
            }
        }
        else
        {
            if (TimeLeftToStartRegen > 0)
            {
                TimeLeftToStartRegen -= DeltaTime;
            }
            if (BPC.bJump == 1 || BPC.bBoost == 1 && CurrentThrustJuice <= 0)
            {
                ForcedRegenAmount = MaxThrustJuice * BurnOutPercentage;
            }
        }
    }
    if (bVehicleOnGround == false)
    {
        Mesh.AddForce(OffGroundForce);
    }
    DmgModule = GetModule(Class'SFXModule_Damage');
    fHealthPercent = DmgModule.CurrentHealth / DmgModule.MaxHealth.Value;
    if (DmgModule != None && WorldInfo.TimeSeconds - LastHitTime > SelfRepairDelay && DmgModule.CurrentHealth < DmgModule.MaxHealth.Value)
    {
        DmgModule.CurrentHealth = FMin(DmgModule.CurrentHealth + DeltaTime * SelfRepairRate, DmgModule.MaxHealth.Value);
        if (AudioComponent != None)
        {
            AudioComponent.SetWwiseRTPC("Health", DmgModule.CurrentHealth);
        }
    }
    if (Mesh.SkeletalMesh.Materials.Length > 0)
    {
        DamageMaterial = MaterialInstanceConstant(Mesh.SkeletalMesh.Materials[0]);
        if (DamageMaterial != None)
        {
            DamageMaterial.SetScalarParameterValue('VEH_ROVb_Damage_Scalar', 1,5 - fHealthPercent * 1,5);
            if (fHealthPercent <= 0,2)
            {
                DamageMaterial.SetScalarParameterValue('Engine_Damaged', 1);
            }
            else
            {
                if (fHealthPercent <= 0,3)
                {
                    DamageMaterial.SetScalarParameterValue('Engine_Damaged', 1 - fHealthPercent - 0,2 / 0,1);
                }
                else
                {
                    DamageMaterial.SetScalarParameterValue('Engine_Damaged', 0);
                }
            }
            DamageMaterial.SetScalarParameterValue('Fuel_Time', CurrentThrustJuice / MaxThrustJuice * 0,25 + 0,35);
            DamageMaterial.SetScalarParameterValue('Fuel_Gauge', 1 - CurrentThrustJuice / MaxThrustJuice * 0,6);
            if (CurrentThrustJuice / MaxThrustJuice < BurnOutPercentage && BurnOutPercentage > 0)
            {
                DamageMaterial.SetScalarParameterValue('Engine_Heat', 1 - CurrentThrustJuice / MaxThrustJuice / BurnOutPercentage);
            }
            else
            {
                DamageMaterial.SetScalarParameterValue('Engine_Heat', 0);
            }
            if (CurrentMiningNode != None && BPC.bMine != 0 && bVehicleOnGround == true && CurrentMiningNode.bAlreadyMined != true)
            {
                DamageMaterial.SetScalarParameterValue('mining_color', 1 - TotalMiningTime / CurrentMiningNode.Duration);
                DamageMaterial.SetScalarParameterValue('Mining_Time', TotalMiningTime / CurrentMiningNode.Duration * 0,8);
                AudioComponent.SetWwiseRTPC("mining_progress", TotalMiningTime / CurrentMiningNode.Duration);
            }
            else
            {
                if (TimeToFlashMiningIndicator > 0)
                {
                    TimeToFlashMiningIndicator -= DeltaTime;
                    TimeToNextMiningFlash -= DeltaTime;
                    DamageMaterial.SetScalarParameterValue('mining_color', MiningIndicatorColor);
                    if (TimeToNextMiningFlash <= 0)
                    {
                        DamageMaterial.SetScalarParameterValue('Mining_Time', 0);
                        TimeToNextMiningFlash = MiningFlashDelay;
                    }
                    else
                    {
                        DamageMaterial.SetScalarParameterValue('Mining_Time', TimeToNextMiningFlash / MiningFlashDelay);
                    }
                }
                else
                {
                    DamageMaterial.SetScalarParameterValue('Mining_Time', 0);
                }
            }
        }
        if (Mesh.SkeletalMesh.Materials.Length > 1)
        {
            DamageMaterial = MaterialInstanceConstant(Mesh.SkeletalMesh.Materials[1]);
            if (DamageMaterial != None)
            {
                if (CurrentThrustJuice / MaxThrustJuice < BurnOutPercentage && BurnOutPercentage > 0)
                {
                    DamageMaterial.SetScalarParameterValue('Engine_Heat', 1 - CurrentThrustJuice / MaxThrustJuice / BurnOutPercentage);
                }
                else
                {
                    DamageMaterial.SetScalarParameterValue('Engine_Heat', 0);
                }
            }
        }
    }
    if (fHealthPercent <= 0,75)
    {
        if (m_oDamageVisualEffect == None && m_oDamageVFXTemplate != None)
        {
            m_oDamageVisualEffect = Class'BioVisualEffect'.CreateCrustEffect(m_oDamageVFXTemplate, self, 0, false);
            if (m_oDamageVisualEffect != None)
            {
                m_oDamageVisualEffect.bPauseOnDeath = true;
                m_oDamageVisualEffect.SetState(0);
            }
        }
        if (m_oDamageVisualEffect != None)
        {
            if (m_oDamageVisualEffect.bPaused)
            {
                m_oDamageVisualEffect.SetPaused(false);
            }
            if (fHealthPercent <= 0,3)
            {
                m_oDamageVisualEffect.SetScalarParameter('Mako_Fire', 1);
                m_oDamageVisualEffect.SetScalarParameter('Mako_Smoke', 1);
                m_oDamageVisualEffect.SetScalarParameter('Mako_Smoke_Pre', 0);
            }
            else
            {
                m_oDamageVisualEffect.SetScalarParameter('Mako_Smoke_Pre', 1);
                m_oDamageVisualEffect.SetScalarParameter('Mako_Fire', 0);
                m_oDamageVisualEffect.SetScalarParameter('Mako_Smoke', 0);
            }
        }
    }
    else
    {
        if (m_oDamageVisualEffect != None && !m_oDamageVisualEffect.bPaused)
        {
            m_oDamageVisualEffect.SetPaused(true);
        }
    }
    oSkelControl = SkelControlLookAt(Mesh.FindSkelControl('Turret_01'));
    if (oSkelControl != None)
    {
        BPC.GetPlayerViewPoint(CamLoc, CamRot);
        if (BPC != None && AudioComponent != None)
        {
            if (CamRot.Pitch != PreviousCameraPitch)
            {
                if (!AudioComponent.IsPlaying(VehicleTurretMovementStartEventPair.Play))
                {
                    AudioComponent.Play(VehicleTurretMovementStartEventPair);
                }
                AudioComponent.SetWwiseRTPC("turret_movement", CamRot.Pitch - PreviousCameraPitch);
            }
            else
            {
                if (AudioComponent.IsPlaying(VehicleTurretMovementStartEventPair.Play))
                {
                    AudioComponent.Stop(VehicleTurretMovementStartEventPair);
                }
            }
            PreviousCameraPitch = CamRot.Pitch;
        }
        oSkelControl.TargetLocation = CamLoc + Normal(CamRot) * 100000;
    }
    if (bWeaponFiring == false && Controller.bFire == 1 && DmgModule.bIsDead == false)
    {
        StartFire(0);
    }
    else
    {
        if (bWeaponFiring && Controller.bFire == 0)
        {
            StopFire(0);
        }
    }
    if (m_oMiningVFXTemplate != None)
    {
        if (BPC.bMine != 0 && bVehicleOnGround == true)
        {
            if (MiningPSC != None && !MiningPSC.bIsActive)
            {
                MiningPSC.SetTemplate(m_oMiningVFXTemplate);
                MiningPSC.SetAbsolute(false, false, false);
                MiningPSC.SetLODLevel(WorldInfo.bDropDetail ? 1 : 0);
                MiningPSC.SetTranslation(vect(100,0,-200));
                MiningPSC.ActivateSystem();
            }
        }
        else
        {
            if (MiningPSC != None)
            {
                MiningPSC.DeactivateSystem();
            }
        }
    }
    if (BPC.bMine != 0 && bMiningFailed == false && CurrentMiningNode == None || CurrentMiningNode.bAlreadyMined == false)
    {
        if (!AudioComponent.IsPlaying(VehicleMiningEventPair))
        {
            AudioComponent.Play(VehicleMiningEventPair);
        }
        if (CurrentMiningNode == None)
        {
            foreach AllActors(Class'SFXMiningNode', Node)
            {
                if (VSize(Node.Location - Location) <= Node.Radius)
                {
                    if (Node != CurrentMiningNode)
                    {
                        TimeToNextMiningImpulse = 0;
                        TotalMiningTime = 0;
                        CurrentMiningNode = Node;
                        break;
                    }
                }
            }
        }
        if (CurrentMiningNode == None || bVehicleOnGround != true || VSize(CurrentMiningNode.Location - Location) > CurrentMiningNode.Radius && TotalMiningTime > 0)
        {
            idx = 0;
            while (idx < CurrentMiningNode.GeneratedEvents.Length)
            {
                FailureEvent = SFXSeqEvt_VehicleMiningFailure(CurrentMiningNode.GeneratedEvents[idx]);
                if (FailureEvent != None)
                {
                    FailureEvent.CheckActivate(CurrentMiningNode, CurrentMiningNode);
                }
                idx++;
            }
            AudioComponent.Play(VehicleMiningFailureEvent);
            Class'BioTelemetry'.SendEventInt('VEHICLE_MINING_FAILED', 1);
            CurrentMiningNode = None;
            TotalMiningTime = 0;
            TimeToNextMiningImpulse = 0;
            bMiningFailed = true;
            MiningFeedbackWaveForm.Samples[0].Duration = 1;
            BPC.ClientPlayForceFeedbackWaveform(MiningFeedbackWaveForm);
            TimeToFlashMiningIndicator = MiningFlashDuration;
            MiningIndicatorColor = 2;
            BPC.bMine = 0;
        }
        else
        {
            if (CurrentMiningNode != None && CurrentMiningNode.bAlreadyMined != true)
            {
                AudioComponent.SetWwiseRTPC("distance_to_node", VSize(CurrentMiningNode.Location - Location) / CurrentMiningNode.Radius);
                if (TotalMiningTime == 0)
                {
                    TimeToNextMiningImpulse = FRand() * CurrentMiningNode.ImpulseMaxFrequency - CurrentMiningNode.ImpulseMinFrequency + CurrentMiningNode.ImpulseMinFrequency;
                }
                TimeToNextMiningImpulse -= DeltaTime;
                TotalMiningTime += DeltaTime;
                if (TotalMiningTime > CurrentMiningNode.Duration)
                {
                    MiningFeedbackWaveForm.Samples[0].Duration = 2;
                    BPC.ClientPlayForceFeedbackWaveform(MiningFeedbackWaveForm);
                    CurrentMiningNode.bAlreadyMined = true;
                    MiningIndicatorColor = 0;
                    TimeToFlashMiningIndicator = MiningFlashDuration;
                    CurrentMiningNode.PSC_GroundReticle.DeactivateSystem();
                    BPC.bMine = 0;
                    idx = 0;
                    while (idx < CurrentMiningNode.GeneratedEvents.Length)
                    {
                        SuccessEvent = SFXSeqEvt_VehicleMiningSuccess(CurrentMiningNode.GeneratedEvents[idx]);
                        if (SuccessEvent != None)
                        {
                            SuccessEvent.CheckActivate(CurrentMiningNode, CurrentMiningNode);
                        }
                        idx++;
                    }
                    AudioComponent.Play(VehicleMiningSuccessEvent);
                    Class'BioTelemetry'.SendEventInt('VEHICLE_MINING_SUCCESS', 1);
                }
                else
                {
                    if (TimeToNextMiningImpulse <= 0)
                    {
                        RandomRotation.Yaw = FRand() * 65536;
                        RandomImpulse.X = FRand() * CurrentMiningNode.ImpulseMaxStrength - CurrentMiningNode.ImpulseMinStrength + CurrentMiningNode.ImpulseMinStrength;
                        MiningFeedbackWaveForm.Samples[0].LeftAmplitude = RandomImpulse.X / CurrentMiningNode.ImpulseMaxStrength * 100;
                        MiningFeedbackWaveForm.Samples[0].RightAmplitude = RandomImpulse.X / CurrentMiningNode.ImpulseMaxStrength * 100;
                        RandomImpulse.Y = 0;
                        RandomImpulse.Z = 0;
                        RandomImpulse = QuatRotateVector(QuatFromRotator(RandomRotation), RandomImpulse);
                        Mesh.AddImpulse(RandomImpulse);
                        TimeToNextMiningImpulse = FRand() * CurrentMiningNode.ImpulseMaxFrequency - CurrentMiningNode.ImpulseMinFrequency + CurrentMiningNode.ImpulseMinFrequency;
                        MiningFeedbackWaveForm.Samples[0].Duration = TimeToNextMiningImpulse;
                        BPC.ClientPlayForceFeedbackWaveform(MiningFeedbackWaveForm);
                    }
                }
            }
        }
    }
    else
    {
        if (AudioComponent.IsPlaying(VehicleMiningEventPair))
        {
            AudioComponent.Stop(VehicleMiningEventPair);
        }
        if (CurrentMiningNode != None)
        {
            CurrentMiningNode = None;
            TotalMiningTime = 0;
        }
        if (BPC.bMine == 0)
        {
            bMiningFailed = false;
        }
    }
}

simulated function Rotator GetViewRotation()
{
    local Rotator ViewRotation;
    local Rotator ControlRotation;
    local Rotator MaxDelta;
    ViewRotation.Yaw = Controller.Rotation.Yaw;
    MaxDelta.Yaw = Controller.Rotation.Yaw;
    if (!ClampRotation(ViewRotation, Rotation, MaxDelta, MaxDelta))
    {
        ControlRotation.Yaw = Controller.Rotation.Yaw;
        if (!ClampRotation(ControlRotation, ViewRotation, rot(0,16384,0), rot(0,16384,0)))
        {
            ControlRotation.Pitch = Controller.Rotation.Pitch;
            ControlRotation.Roll = Controller.Rotation.Roll;
            Controller.SetRotation(ControlRotation);
        }
    }
    ViewRotation.Pitch = Controller.Rotation.Pitch;
    ViewRotation.Roll = Controller.Rotation.Roll;
    return ViewRotation;
}

function bool IsHostile(Pawn Other)
{
    if (FactionClass != None && Other != None)
    {
        return FactionClass.IsHostile(GetFaction(Other));
    }
    return false;
}

function bool IsFriendly(Pawn Other)
{
    if (FactionClass != None && Other != None)
    {
        return FactionClass.IsFriendly(GetFaction(Other));
    }
    return true;
}

function EBioFactionTypes GetFaction(Pawn Other)
{
    local BioPawn BioPawn;
    BioPawn = BioPawn(Other);
    if (BioPawn == None)
    {
        return FactionClass.Default.SquadFaction;
    }
    if (BioPawn != None)
    {
        if (BioPawn.FactionClass == None && BioPawn.Squad != None)
        {
            return BioPawn.Squad.SquadFaction.Default.SquadFaction;
        }
        else
        {
            if (BioPawn.FactionClass != None)
            {
                return BioPawn.FactionClass.Default.SquadFaction;
            }
        }
    }
    return 1;
}

event RanInto(Actor Other);

function bool NotifyBump(Actor Other, Vector HitNormal)
{
    local BioPawn BPawn;
    BPawn = BioPawn(Other);
    if (BPawn != None)
    {
        BPawn.AddRagdollImpulse(Velocity * HitNormal Dot Normal(Velocity), Controller, BPawn.Location, true);
        Other.TakeDamage(HitDamage, BPawn.Controller, vect(0,0,0), vect(0,0,0), Class'SFXDamageType');
    }
    return true;
}

simulated event RigidBodyCollision(PrimitiveComponent HitComponent, PrimitiveComponent OtherComponent, const out CollisionImpactData RigidCollisionData, int ContactIndex)
{
    local float ContactMag;
    if (OtherComponent.Owner == self)
    {
        OtherComponent = HitComponent;
    }
    ContactMag = VSize(RigidCollisionData.TotalNormalForceVector);
    if (ContactMag > 20)
    {
        AudioComponent.Play(HitSomething);
        BioPlayerController(Controller).CameraShake(0,45, vect(0,002,0,002,0,002) * ContactMag, vect(10,10,10), vect(0,0002,0,0002,0,0002) * ContactMag, vect(100,100,100), 0,0002 * ContactMag, 10);
    }
}

defaultproperties
{
    JumpForce=(X=200,Y=0,Z=2070)
    BoostForce=(X=1000,Y=0,Z=0)
    MaxThrustJuice=0,6
    ThrustRegenerationFactor=0,55
    ThrustFeedback=ForceFeedbackWaveform'ThrustShakeFF0'
    OnGroundJumpMultiplier=1
    ForwardThrustBurnRate=0,1
    VerticalThrustBurnRate=1
    ThrustRegenerationDelay=0,5
    BurnOutPercentage=0,2
    OffGroundForce=(X=0,Y=0,Z=-400)
    SelfRepairDelay=5
    SelfRepairRate=100
    HitDamage=350
    MaxPitchAngle=32
    MiningPSC=ParticleSystemComponent'MiningPSC'
    LeftAfterburnerPSC=ParticleSystemComponent'AfterburnerPSC'
    RightAfterburnerPSC=ParticleSystemComponent'AfterburnerPSC'
    LightEnvironment=BioDynamicLightEnvironmentComponent'BioLightEnvComponent0'
    RadarRange=10000
    RadarFOV=45
    SimObj=None
    Wheels=None
    // child object BioLightEnvComponent0 of type BioDynamicLightEnvironmentComponent
    // child object CollisionCylinder of type CylinderComponent
    // child object ThrustShakeFF0 of type ForceFeedbackWaveform
    // child object AfterburnerPSC of type ParticleSystemComponent
    // child object AfterburnerPSC of type ParticleSystemComponent
    // child object MiningPSC of type ParticleSystemComponent
    // child object MyStayUprightConstraintInstance of type RB_ConstraintInstance
    // child object MyStayUprightSetup of type RB_StayUprightSetup
    // child object SimObject of type SFXVehicleSimHover
    // child object SVehicleMesh of type SkeletalMeshComponent
    // child object LFWheel of type SVehicleWheel
    // child object LRWheel of type SVehicleWheel
    // child object RFWheel of type SVehicleWheel
    // child object RRWheel of type SVehicleWheel
}
